@@ -3,15 +3,15 @@
 agentclient.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Client-side interface to the SSH authentication agent.
+  Client-side interface to the SSH authentication agent.
 
 */
 
@@ -195,8 +195,8 @@ void ssh_agent_received_packet(SshCrossPacketType type,
         }
       ssh_cancel_timeouts(ssh_agent_timeout, (void *)agent);
       if (ssh_decode_array(data, len,
-                               SSH_FORMAT_UINT32, &temp,
-                               SSH_FORMAT_END) != len)
+                           SSH_FORMAT_UINT32, &temp,
+                           SSH_FORMAT_END) != len)
         {
           ssh_debug("ssh_agent_received_packet: VERSION_RESPONSE bad data");
           return;
@@ -400,7 +400,11 @@ void ssh_agent_open(SshAgentOpenCallback callback, void *context)
   c->context = context;
 
   /* Connect to the agent socket. */
+#ifndef SSH_WIN_CLIENT
   ssh_agenti_connect(ssh_agent_open_complete, FALSE, (void *)c);
+#else
+  ssh_agent_open_complete(NULL, (void *)c);
+#endif /* SSH_WIN_CLIENT */
 }
 
 /* Closes the connection to the authentication agent.  If a command is
@@ -435,8 +439,7 @@ void ssh_agent_add_generic(SshAgent agent,
 {
   unsigned char *blob;
   size_t blob_len;
-  SshRandomState dummy_random_state;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
 
   if (agent->state != SSH_AC_IDLE)
     {
@@ -446,25 +449,17 @@ void ssh_agent_add_generic(SshAgent agent,
       return;
     }
   
-  /* Allocate a dummy random state.  This is needed to export a private
-     key.  However, we export without encryption, so it doesn't matter
-     whether the random state is initialized.  For that reason, we don't
-     require it to be externally supplied nor try to add any entropy. */
   if (key)
     {
-      dummy_random_state = ssh_random_allocate();
       if (ssh_private_key_export(key, "none", (unsigned char *)"", 
-                                 0, dummy_random_state, &blob,
+                                 0, &blob,
                                  &blob_len) != SSH_CRYPTO_OK)
         {
-          ssh_random_free(dummy_random_state);
           ssh_debug("ssh_agent_add: export failed");
           if (callback)
             (*callback)(SSH_AGENT_FAILURE, context);
           return;
         }
-      /* Free the dummy random state.  It is no longer needed. */
-      ssh_random_free(dummy_random_state);
     }
   else
     {
@@ -719,13 +714,25 @@ void ssh_agent_op(SshAgent agent, SshAgentOp op,
   agent->op_callback = callback;
   agent->context = context;
 
-  /* Send the request packet. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_PRIVATE_KEY_OP,
-                             SSH_FORMAT_UINT32_STR, name, strlen(name),
-                             SSH_FORMAT_UINT32_STR, certs, certs_len,
-                             SSH_FORMAT_UINT32_STR, data, len,
-                             SSH_FORMAT_END);
+  if (op != SSH_AGENT_SSH1_CHALLENGE_RESPONSE)
+    {
+      /* Send the request packet. */
+      ssh_cross_down_send_encode(agent->down,
+                                 (SshCrossPacketType)SSH_AGENT_PRIVATE_KEY_OP,
+                                 SSH_FORMAT_UINT32_STR, name, strlen(name),
+                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                 SSH_FORMAT_UINT32_STR, data, len,
+                                 SSH_FORMAT_END);
+    }
+  else
+    {
+      ssh_cross_down_send_encode(agent->down,
+                                 (SshCrossPacketType)SSH_AGENT_PRIVATE_KEY_OP,
+                                 SSH_FORMAT_UINT32_STR, name, strlen(name),
+                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                 SSH_FORMAT_DATA, data, len,
+                                 SSH_FORMAT_END);
+    }
 }
 
 
