@@ -3,12 +3,12 @@
   authc-passwd.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Password authentication, client side.
@@ -23,7 +23,7 @@
 void ssh_client_auth_passwd(SshAuthClientOperation op,
                             const char *user,
                             unsigned int packet_type,
-                            SshBuffer *packet_in,
+                            SshBuffer packet_in,
                             const unsigned char *session_id,
                             size_t session_id_len,
                             void **state_placeholder,
