@@ -2,13 +2,14 @@
 
 auths-passwd.c
 
-Author: Tatu Ylonen <ylo@cs.hut.fi>
+  Author: Tatu Ylonen <ylo@ssh.com>
 
-Copyright (c) 1997 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
-                   All rights reserved
+  Copyright (C) 1997 Tatu Ylonen <ylo@ssh.com>, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  All rights reserved.
 
-Password authentication, server-side.  This calls functions in machine-specific
-files to perform the actual authentication.
+  Password authentication, server-side.  This calls functions in 
+  machine-specific files to perform the actual authentication.
 
 */
 
@@ -16,11 +17,11 @@ files to perform the actual authentication.
 #include "sshencode.h"
 #include "sshauth.h"
 #include "sshmsgs.h"
-#include "auths-passwd.h"
 #include "sshuser.h"
 #include "sshserver.h"
 #include "sshconfig.h"
 #include "auths-common.h"
+#include "auths-passwd.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthPasswdServer"
 
@@ -29,7 +30,7 @@ files to perform the actual authentication.
 
 SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
                                            const char *user,
-                                           SshBuffer *packet,
+                                           SshBuffer packet,
                                            const unsigned char *session_id,
                                            size_t session_id_len,
                                            void **state_placeholder,
@@ -42,32 +43,27 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
   Boolean change_request;
   char *password, *prompt;
   int disable_method = 0;
-
-  SSH_DEBUG(6, ("auth_passwd op = %d  user = %s", op, user));
+  unsigned long pass_len = 0L;
+  
+  SSH_TRACE(6, ("password auth."));
+  
+  SSH_DEBUG(6, ("op = %d  user = %s", op, user));
   
   switch (op)
     {
     case SSH_AUTH_SERVER_OP_START:
-      /* Check whether user's login is allowed */
-      if (ssh_server_auth_check_user(&uc, user, config))
+      if (ssh_server_auth_check(&uc, user, config, server->common,
+                                SSH_AUTH_PASSWD))
         {
-          /* User does not exist or is not allowed to log in. */
-          return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-        }
-      else
-        {
-          *longtime_placeholder = (void *)uc;
-        }
-
-      if (ssh_server_auth_check_host(server->common))
-        {
-          /* logins from remote host are not allowed. */
-          ssh_log_event(config->log_facility, SSH_LOG_WARNING,
-                        "Connection from %s denied. Authentication as user "
-                        "%s was attempted.", server->common->remote_host,
-                        ssh_user_name(uc));
-          return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
+          config->password_guesses--;
+          if (config->password_guesses <= 0)
+            {
+              /* If this attempt is not succesful, disable this method. */
+              disable_method = 1;
+            }
+          goto password_bad;
         }
+      *longtime_placeholder = (void *)uc;
       
       {
         config->password_guesses--;
@@ -76,57 +72,61 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
             /* If this attempt is not succesful, disable this method. */
             disable_method = 1;
           }
-
-        /* If password authentication is denied in the configuration
-           file, deny it here too. */
-        if (config->password_authentication == FALSE )
-          {
-            ssh_warning("Password authentication denied. (user '%s' not"
-                        " allowed to log in)", ssh_user_name(uc));
-            ssh_log_event(config->log_facility, SSH_LOG_WARNING,
-                          "Password authentication denied. (user '%s' not"
-                          " allowed to log in)", ssh_user_name(uc));
-            /* XXX should be
-             SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED, but that
-             disconnects (incorrectly so) now */
-            goto password_bad;            
-          }
-        
-        else if(ssh_user_uid(uc) == SSH_UID_ROOT &&
-                config->permit_root_login == FALSE)
+#ifndef SSHDIST_WINDOWS
+        if(ssh_user_uid(uc) == SSH_UID_ROOT &&
+           (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
+            config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
           {
             /* XXX Add client addresses etc. */
+            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
+                          "not permitted."));
             ssh_log_event(config->log_facility,
                           SSH_LOG_WARNING,
                           "root logins are not permitted.");
-            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
-                      "not permitted."));
-            return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
+
+            if (config->password_guesses > 0)
+              {
+                return SSH_AUTH_SERVER_REJECTED;
+              }
+            else
+              {
+                return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
+              }
           }
+#endif /* SSHDIST_WINDOWS */
       }
       
       /* Parse the password authentication request. */
       if (ssh_decode_buffer(packet,
                             SSH_FORMAT_BOOLEAN, &change_request,
-                            SSH_FORMAT_UINT32_STR, &password, NULL,
+                            SSH_FORMAT_UINT32_STR, &password, &pass_len,
                             SSH_FORMAT_END) == 0)
         {
-          SSH_DEBUG(2, ("ssh_server_auth_passwd: bad packet"));
+          SSH_DEBUG(2, ("bad packet"));
           goto password_bad;
         }
 
+      if (!config->permit_empty_passwords && pass_len == 0L)
+        {
+          char *s = "login with empty passwords not permitted.";
+          ssh_log_event(config->log_facility,
+                        SSH_LOG_WARNING,
+                        "%s", s);
+          SSH_DEBUG(2, ("%s", s));
+          goto password_bad;
+        }
+      
       /* Password changing requests should only be received as continuation
          messages. */
       if (change_request)
         {
-          SSH_DEBUG(2 ,("ssh_server_auth_passwd: changing password " \
-                        "cannot start."));
+          SSH_DEBUG(2 ,("changing password cannot start."));
           goto password_bad;
         }
       
       /* Sanity check: do not pass excessively long passwords to system
          functions to avoid buffer overflows in operating system code. */
-      if (strlen(password) > 64)
+      if (strlen(password) > 256)
         {
           SSH_DEBUG(2, ("ssh_server_auth_passwd: password too long."));
           ssh_xfree(password);
@@ -191,8 +191,8 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
         {
           ssh_buffer_clear(packet);
           ssh_encode_buffer(packet,
-                            SSH_FORMAT_UINT32,
-                            SSH_MSG_USERAUTH_PASSWD_CHANGEREQ,
+                            SSH_FORMAT_CHAR,
+                            (unsigned int) SSH_MSG_USERAUTH_PASSWD_CHANGEREQ,
                             SSH_FORMAT_UINT32_STR, prompt, strlen(prompt),
                             SSH_FORMAT_END);
           ssh_xfree(prompt);
