@@ -3,12 +3,12 @@
 auth-passwd.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
 */
@@ -25,15 +25,15 @@ auth-passwd.h
      `session_id_len' length of session identifier
      `state_placeholder' place to store context data between packets
      `longtime_placeholder'  can hold data between authentications
-     		    (this is per-method)
+                    (this is per-method)
      `method_context'   passed to the method function */
 SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
-					   const char *user,
-					   SshBuffer *packet,
-					   const unsigned char *session_id,
-					   size_t session_id_len,
-					   void **state_placeholder,
-					   void **longtime_placeholder,
-					   void *method_context);
+                                           const char *user,
+                                           SshBuffer packet,
+                                           const unsigned char *session_id,
+                                           size_t session_id_len,
+                                           void **state_placeholder,
+                                           void **longtime_placeholder,
+                                           void *method_context);
 
 #endif /* AUTH_PASSWD_H */
