@@ -2,12 +2,12 @@
   auths-pubkey.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
                   
   Public key authentication, server-side.
@@ -22,7 +22,7 @@
 
 SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
                                            const char *user,
-                                           SshBuffer *packet,
+                                           SshBuffer packet,
                                            const unsigned char *session_id,
                                            size_t session_id_len,
                                            void **state_placeholder,
