@@ -3,16 +3,16 @@
 readpass.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Functions for reading a passphrase from the user.  The terminal must be
-in normal, blocking and cooked mode.
+  Functions for reading a passphrase from the user.  The terminal must be
+  in normal, blocking and cooked mode.
 
 */
 
@@ -23,10 +23,10 @@ in normal, blocking and cooked mode.
    passphrase (allocated with ssh_xmalloc).  Exits if EOF is encountered. 
    The passphrase if read from stdin if from_stdin is true (as is the
    case with ssh-keygen).  */
-char *ssh_read_passphrase(const char *prompt, int from_stdin);
+char *ssh_read_passphrase(const char *prompt, Boolean from_stdin);
 
 /* Reads a yes/no confirmation from /dev/tty.  Returns TRUE if "yes" is
    received.  Otherwise returns FALSE (also if EOF is encountered). */
-Boolean ssh_read_confirmation(const char *prompt);
+Boolean ssh_read_confirmation(const char *prompt, Boolean from_stdin);
 
 #endif /* READPASS_H */
