@@ -3,21 +3,21 @@
   sftp_server.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   File server that is executed by sshd.
-  
+
 */
 
 #include "ssh2includes.h"
-#include "sshunixeloop.h"
-#include "sshunixfdstream.h"
+#include "ssheloop.h"
+#include "sshfdstream.h"
 #include "sshfilexfer.h"
 #include "sshsignals.h"
 
@@ -29,15 +29,15 @@ int deny_severity = SSH_LOG_WARNING;
 #endif /* HAVE_LIBWRAP */
 
 int main(void)
-{       
+{
   SshFileServer server;
-    
+
   ssh_event_loop_initialize();
   ssh_signals_prevent_core(TRUE, NULL);
   server = ssh_file_server_wrap(ssh_stream_fd_stdio());
-  ssh_event_loop_run();      
+  ssh_event_loop_run();
   ssh_event_loop_uninitialize();
   ssh_signals_reset();
-  
+
   return 0;
 }
