@@ -1,17 +1,17 @@
 /*
 
-ssh-agent.c
+  ssh-agent.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-The ssh authentication agent.
+  The ssh authentication agent.
 
 */
 
@@ -20,16 +20,20 @@ The ssh authentication agent.
 #include "sshcross.h"
 #include "sshencode.h"
 #include "sshtcp.h"
-#include "match.h"
+#include "sshmatch.h"
 #include "sshtimeouts.h"
 #include "sshagent.h"
 #include "sshagentint.h"
 #include "sshuser.h"
 #include "sshuserfiles.h"
-#include "sshunixeloop.h"
+#include "ssheloop.h"
 #include "sshgetopt.h"
-#include "pubkeyencode.h"
-#include "gmp.h"
+#include "ssh2pubkeyencode.h"
+#ifdef WITH_SSH_AGENT1_COMPAT
+#include "ssh1encode.h"
+#include "ssh1keydecode.h"
+#endif /* WITH_SSH_AGENT1_COMPAT */
+#include "sshmp.h" /* was "gmp.h" */
 
 #define SSH_DEBUG_MODULE "SshAgent"
 
@@ -67,9 +71,9 @@ typedef struct SshAgentKeyRec {
 #ifdef WITH_SSH_AGENT1_COMPAT
 typedef struct SshAgentSsh1KeyRec {
   struct SshAgentSsh1KeyRec *next;
-  MP_INT n;
-  MP_INT e;
-  MP_INT d;
+  SshInt n;
+  SshInt e;
+  SshInt d;
   char *description;
 } *SshAgentSsh1Key;
 #endif /* WITH_SSH_AGENT1_COMPAT */
@@ -81,7 +85,6 @@ struct SshAgentImplRec {
   char *socket_name;
   char *socket_dir_name;
   char *lock_password;
-  SshRandomState random_state;
 #ifdef WITH_SSH_AGENT1_COMPAT
   /* Is `-1' flag on? */
   Boolean ssh1_compat;
@@ -180,10 +183,7 @@ Boolean ssh_agenti_invalid_key(SshAgentConnection conn,
 #ifdef WITH_SSH_AGENT1_COMPAT
 
 /* Make ssh1 style (length in two bytes) encoding of long integer */
-void ssh_agenti_ssh1_encode_mp(SshBuffer *buffer, MP_INT *n);
-
-/* Decode ssh1 style encoded long integer */
-Boolean ssh_agenti_ssh1_decode_mp(SshBuffer *buffer, MP_INT *n);
+void ssh_agenti_ssh1_encode_mp(SshBuffer buffer, SshInt *n);
 
 /* Reply to the list query from ssh-agent1 */
 void ssh_agenti_ssh1_list_keys(SshAgentConnection conn);
@@ -214,20 +214,20 @@ void ssh_agenti_handle_ssh1_packet(SshAgentConnection conn,
 
 /* Find ssh1 key with the given public part */
 SshAgentSsh1Key ssh_agenti_ssh1key_find(SshAgentImpl agent,
-                                         MP_INT *n, 
-                                         MP_INT *e);
+                                         SshInt *n, 
+                                         SshInt *e);
 
 /* Adds the given ssh1 private key to be managed by the agent. */
 Boolean ssh_agenti_ssh1key_add(SshAgentImpl agent,
-                               MP_INT *n, 
-                               MP_INT *e,
-                               MP_INT *d,
+                               SshInt *n, 
+                               SshInt *e,
+                               SshInt *d,
                                char *description);
 
 /* Delete a ssh1 key with given public key */
 Boolean ssh_agenti_ssh1key_delete(SshAgentImpl agent,
-                                   MP_INT *n, 
-                                   MP_INT *e);
+                                   SshInt *n, 
+                                   SshInt *e);
 
 /* Delete all ssh1 keys */
 void ssh_agenti_ssh1key_delete_all(SshAgentImpl agent);
@@ -253,7 +253,7 @@ void ssh_agenti_send(SshAgentConnection conn, unsigned int packet_type, ...)
 void ssh_agenti_send_error(SshAgentConnection conn, unsigned int err)
 {
   ssh_agenti_send(conn, SSH_AGENT_FAILURE,
-                  SSH_FORMAT_UINT32, (long)err,
+                  SSH_FORMAT_UINT32, (SshUInt32) err,
                   SSH_FORMAT_END);
 }
 
@@ -335,7 +335,7 @@ size_t ssh_split_comma_list(const char *str, char ***arr)
 Boolean ssh_agent_path_match(const char *rule, const char *path)
 {
   char **r, **p;
-  size_t nr, np, x, y;
+  size_t nr = 0L, np = 0L, x = 0L, y = 0L;
 
   /* Empty rule allows everything. */
   if ((rule == NULL) || (*rule == '\000'))
@@ -531,8 +531,8 @@ Boolean ssh_agenti_delete_key(SshAgentImpl agent,
 void ssh_agenti_list_keys(SshAgentConnection conn)
 {
   SshAgentKey key;
-  unsigned long num_keys;
-  SshBuffer buffer;
+  SshUInt32 num_keys;
+  SshBufferStruct buffer;
 
   if (conn->agent->lock_password)
     {
@@ -583,9 +583,12 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
   SshAgentKey key;
   const unsigned char *arg;
   size_t arg_len;
-  unsigned char *outputbuf;
-  size_t outputlen;
+  const unsigned char *session_id;
+  size_t session_id_len;
+  unsigned char *outputbuf = NULL, *tmp_buf = NULL;
+  size_t outputlen, tmp_buf_len;
   SshPrivateKey privkey;
+  SshHash hash;
 
   SSH_DEBUG(7, ("op=%s", op_name));
   key = ssh_agenti_find_key(conn->agent, public_blob, public_len, TRUE);
@@ -620,7 +623,7 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
       outputbuf = ssh_xmalloc(outputlen);
       if (ssh_private_key_sign_digest(privkey, arg, arg_len,
                                       outputbuf, outputlen,
-                                      &outputlen, conn->agent->random_state) !=
+                                      &outputlen) !=
           SSH_CRYPTO_OK)
         {
           SSH_DEBUG(6, ("sign failed"));
@@ -644,7 +647,7 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
       outputlen = ssh_private_key_max_signature_output_len(privkey);
       outputbuf = ssh_xmalloc(outputlen);
       if (ssh_private_key_sign(privkey, arg, arg_len, outputbuf, outputlen,
-                               &outputlen, conn->agent->random_state) !=
+                               &outputlen) !=
           SSH_CRYPTO_OK)
         {
           SSH_DEBUG(6, ("hash-and-sign failed"));
@@ -679,6 +682,49 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
                       SSH_FORMAT_UINT32_STR, outputbuf, outputlen,
                       SSH_FORMAT_END);
     }
+  else if (strcmp(op_name, "ssh1-challenge-response") == 0)
+    {
+      if (ssh_decode_array(data, len,
+                           SSH_FORMAT_UINT32_STR_NOCOPY, 
+                           &arg, &arg_len,
+                           SSH_FORMAT_UINT32_STR_NOCOPY, 
+                           &session_id, &session_id_len,
+                           SSH_FORMAT_END) != len)
+        {
+          SSH_DEBUG(3, ("ssh1-challenge-response: bad data"));
+          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_UNSUPPORTED_OP);
+          goto cleanup_and_return;
+        }
+      outputlen = ssh_private_key_max_decrypt_output_len(privkey);
+      outputbuf = ssh_xmalloc(outputlen);
+      if (ssh_private_key_decrypt(privkey, arg, arg_len,
+                                  outputbuf, outputlen,
+                                  &outputlen) != SSH_CRYPTO_OK)
+        {
+          SSH_DEBUG(6, ("decrypt failed"));
+          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_FAILURE);
+          goto cleanup_and_return;
+        }
+      if (ssh_hash_allocate("md5", &hash) != SSH_CRYPTO_OK)
+        {
+          SSH_DEBUG(6, ("hash allocate failed"));
+          ssh_agenti_send_error(conn, SSH_AGENT_ERROR_FAILURE);
+          goto cleanup_and_return;
+        }
+      ssh_hash_update(hash, 
+                      outputbuf, 
+                      (outputlen < 32) ? outputlen : 32);
+      ssh_hash_update(hash, 
+                      session_id, 
+                      session_id_len);
+      tmp_buf_len = ssh_hash_digest_length(hash);
+      tmp_buf = ssh_xmalloc(tmp_buf_len);
+      ssh_hash_final(hash, tmp_buf);
+      ssh_hash_free(hash);
+      ssh_agenti_send(conn, SSH_AGENT_OPERATION_COMPLETE,
+                      SSH_FORMAT_UINT32_STR, tmp_buf, tmp_buf_len,
+                      SSH_FORMAT_END);
+    }
   else
     {
       SSH_DEBUG(3, ("unknown op '%.50s'", op_name));
@@ -686,6 +732,7 @@ void ssh_agenti_private_key_op(SshAgentConnection conn, char *op_name,
     }
 
  cleanup_and_return:
+  ssh_xfree(outputbuf);
   return;
 }
 
@@ -701,8 +748,8 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
   size_t bytes, private_len, public_len, password_len;
   char *description, *op_name, *forwarding_host, *password;
   SshUInt32 timeout_time;
-  time_t current_time;
-  SshBuffer buffer;
+  SshTime current_time;
+  SshBufferStruct buffer;
   unsigned int byte;
   struct SshAgentKeyAttrsRec attrs;
 
@@ -740,7 +787,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
              versions.  //tri */
           /* Send our version number. */
           ssh_agenti_send(conn, SSH_AGENT_VERSION_RESPONSE,
-                          SSH_FORMAT_UINT32, 2L,
+                          SSH_FORMAT_UINT32, (SshUInt32) 2,
                           SSH_FORMAT_END);
         }
       break;
@@ -776,7 +823,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
           break;
         }
       ssh_agent_init_key_attrs(&attrs);
-      current_time = time(NULL);
+      current_time = ssh_time();
 
       ssh_buffer_init(&buffer);
       ssh_buffer_append(&buffer, &(data[bytes]), len - bytes);
@@ -793,6 +840,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
             }
           switch (byte)
             {
+            case SSH_AGENT_CONSTRAINT_OLD_TIMEOUT:
             case SSH_AGENT_CONSTRAINT_TIMEOUT:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_TIMEOUT"));
               if (ssh_decode_buffer(&buffer,
@@ -804,9 +852,10 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
                                         SSH_AGENT_ERROR_UNSUPPORTED_OP);
                   goto end_of_attrs;
                 }
-              attrs.timeout_time = (time_t)timeout_time + current_time;
+              attrs.timeout_time = (SshTime)timeout_time + current_time;
               break;
 
+            case SSH_AGENT_CONSTRAINT_OLD_USE_LIMIT:
             case SSH_AGENT_CONSTRAINT_USE_LIMIT:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_USE_LIMIT"));
               if (ssh_decode_buffer(&buffer,
@@ -820,6 +869,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
                 }
               break;
 
+            case SSH_AGENT_CONSTRAINT_OLD_FORWARDING_STEPS:
             case SSH_AGENT_CONSTRAINT_FORWARDING_STEPS:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_FORWARDING_STEPS"));
               if (ssh_decode_buffer(&buffer,
@@ -834,6 +884,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
                 }
               break;
 
+            case SSH_AGENT_CONSTRAINT_OLD_FORWARDING_PATH:
             case SSH_AGENT_CONSTRAINT_FORWARDING_PATH:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_FORWARDING_PATH"));
               if (ssh_decode_buffer(&buffer,
@@ -848,6 +899,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
                 }
               break;
 
+            case SSH_AGENT_CONSTRAINT_OLD_COMPAT:
             case SSH_AGENT_CONSTRAINT_COMPAT:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_COMPAT"));
               if (ssh_decode_buffer(&buffer,
@@ -862,6 +914,7 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
                 }
               break;
 
+            case SSH_AGENT_CONSTRAINT_OLD_STATUS:
             case SSH_AGENT_CONSTRAINT_STATUS:
               SSH_DEBUG(7, ("got attr SSH_AGENT_CONSTRAINT_STATUS"));
               if (ssh_decode_buffer(&buffer,
@@ -1039,6 +1092,12 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
       ssh_xfree(password);
       break;
 
+    case SSH_AGENT_PING:
+      ssh_agenti_send(conn, SSH_AGENT_ALIVE,
+                      SSH_FORMAT_DATA, data, len,
+                      SSH_FORMAT_END);
+      break;
+
     case SSH_AGENT_FORWARDING_NOTICE:
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32_STR, &forwarding_host, NULL,
@@ -1079,6 +1138,8 @@ void ssh_agenti_received_packet(SshCrossPacketType type,
       break;
 #endif /* WITH_SSH_AGENT1_COMPAT */
     default:
+      if ((((int)type) >= 130) && (((int)type) <= 199))
+        break; /* Unknown notification or request is ignored. */
       ssh_agenti_send_error(conn, SSH_AGENT_ERROR_UNSUPPORTED_OP);
       break;
     }
@@ -1098,7 +1159,7 @@ void ssh_agenti_received_eof(void *context)
   if (!*connp)
     ssh_fatal("ssh_agenti_received_eof: connection %lx not found",
               (unsigned long)conn);
-  assert(*connp == conn);
+  SSH_ASSERT(*connp == conn);
   *connp = conn->next;
 
   /* Destroy and free the object.  This also closes the stream. */
@@ -1135,7 +1196,6 @@ SshAgentImpl ssh_agenti_create(char **path_return)
 
   agent = ssh_xcalloc(1, sizeof(*agent));
   agent->connections = NULL;
-  agent->random_state = NULL;
   agent->lock_password = NULL; /* Not locked */
   agent->listener = ssh_agenti_create_listener(getuid(), path_return,
                                                ssh_agenti_connection,
@@ -1162,11 +1222,11 @@ void ssh_agenti_check_timeout_keys(void *context)
   SshAgentKey key;
   SshAgentKey tmpkey = NULL;
   SshAgentKey newkey = NULL;
-  time_t ct;
+  SshTime ct;
 
   SSH_DEBUG(7, ("checking for expired keys"));
 
-  ct = time(NULL);
+  ct = ssh_time();
 
   /* Remove the key with given certs */
   for (tmpkey = agent->keys; tmpkey; /*NOTHING*/)
@@ -1235,7 +1295,7 @@ void ssh_agenti_check_parent(void *context)
 Boolean ssh_agenti_invalid_key(SshAgentConnection conn, 
                                SshAgentKeyAttrs attrs)
 {
-  time_t current_time;
+  SshTime current_time;
 
   if ((attrs->path_len_limit != 0xffffffff) &&
       (attrs->path_len_limit < conn->forwarding_path_len))
@@ -1248,7 +1308,7 @@ Boolean ssh_agenti_invalid_key(SshAgentConnection conn,
       SSH_DEBUG(7, ("key path len ok"));
     }
   
-  current_time = time(NULL);
+  current_time = ssh_time();
   if ((attrs->timeout_time != 0) &&
       (attrs->timeout_time <= current_time))
     {
@@ -1538,7 +1598,7 @@ int main(int argc, char **argv)
   agent->socket_name = ssh_xstrdup(socket_name);
 
   /* Load the random seed file. */
-  agent->random_state = ssh_randseed_open(user, NULL);
+  ssh_randseed_open(user, NULL);
 
   /* Register a timeout to periodically check whether the parent has
      exited (which means we should exit too). */
@@ -1562,13 +1622,8 @@ int main(int argc, char **argv)
   /* Uninitialize the event loop. */
   ssh_event_loop_uninitialize();
 
-  if (agent->random_state)
-    {
-      /* Update the random seed file. */
-      ssh_randseed_update(user, agent->random_state, NULL);
-      /* Free the random seed. */
-      ssh_random_free(agent->random_state);
-    }
+  /* Update the random seed file. */
+  ssh_randseed_update(user, NULL);
 
   /* Free user database information about the current user. */
   ssh_user_free(user, FALSE);
@@ -1584,7 +1639,7 @@ int main(int argc, char **argv)
 }
 
 #ifdef WITH_SSH_AGENT1_COMPAT
-void ssh_agenti_ssh1_encode_mp(SshBuffer *buffer, MP_INT *n)
+void ssh_agenti_ssh1_encode_mp(SshBuffer buffer, SshInt *n)
 {
   SshUInt32 len;
   unsigned char len_buf[2];
@@ -1606,30 +1661,12 @@ void ssh_agenti_ssh1_encode_mp(SshBuffer *buffer, MP_INT *n)
   return;
 }
 
-Boolean ssh_agenti_ssh1_decode_mp(SshBuffer *buffer, MP_INT *n)
-{
-  SshUInt32 len;
-  unsigned char *len_buf;
-  size_t num_buf_len;
-
-  if (ssh_buffer_len(buffer) < 2)
-    return FALSE;
-  len_buf = ssh_buffer_ptr(buffer);
-  len = (len_buf[0] * 0x100) + len_buf[1];
-  num_buf_len = ((len + 7) >> 3) & 0xffff;
-  if (ssh_buffer_len(buffer) < (2 + num_buf_len))
-    return FALSE;
-  ssh_mp_set_buf(n, &(len_buf[2]), num_buf_len);
-  ssh_buffer_consume(buffer, num_buf_len + 2);
-  return TRUE;
-}
-
 void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
 {
   SshAgentKey key;
   SshAgentSsh1Key ssh1_key;
   unsigned long num_keys;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   char *key_name;
   SshPublicKey public_key;
   size_t bits;
@@ -1685,10 +1722,10 @@ void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
               public_key = ssh_private_key_derive_public_key(key->private_key);
               if (public_key)
                 {
-                  MP_INT n, e;
+                  SshInt n, e;
                   
-                  mpz_init(&n);
-                  mpz_init(&e);
+                  ssh_mp_init(&n);
+                  ssh_mp_init(&e);
                   if (ssh_public_key_get_info(public_key,
                                               SSH_PKF_MODULO_N, &n,
                                               SSH_PKF_PUBLIC_E, &e,
@@ -1711,8 +1748,8 @@ void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
                                             SSH_FORMAT_END);
                         }
                     }
-                  mpz_clear(&n);
-                  mpz_clear(&e);
+                  ssh_mp_clear(&n);
+                  ssh_mp_clear(&e);
                   ssh_public_key_free(public_key);
                 }
             }
@@ -1737,13 +1774,13 @@ void ssh_agenti_ssh1_list_keys(SshAgentConnection conn)
 }
 
 void ssh_agenti_ssh1_challenge(SshAgentConnection conn, 
-                                   const unsigned char *data, 
-                                   size_t len)
+                               const unsigned char *data, 
+                               size_t len)
 {
-  MP_INT e, n, challenge, output;
-  SshBuffer buffer;
-  SshBuffer reply;
-  size_t bits;
+  SshInt e, n, challenge, output;
+  SshBufferStruct buffer;
+  SshBufferStruct reply;
+  SshUInt32 bits;
   SshPublicKey public_key = NULL;
   unsigned char *public_blob = NULL;
   unsigned char *output_buf = NULL;
@@ -1761,10 +1798,10 @@ void ssh_agenti_ssh1_challenge(SshAgentConnection conn,
   ssh_buffer_init(&buffer);
   ssh_buffer_init(&reply);
   ssh_buffer_append(&buffer, data, len);
-  mpz_init(&n);
-  mpz_init(&e);
-  mpz_init(&challenge);
-  mpz_init(&output);
+  ssh_mp_init(&n);
+  ssh_mp_init(&e);
+  ssh_mp_init(&challenge);
+  ssh_mp_init(&output);
 
   SSH_DEBUG(7, ("got ssh1 challenge from the server"));
 
@@ -1776,9 +1813,9 @@ void ssh_agenti_ssh1_challenge(SshAgentConnection conn,
       SSH_DEBUG(3, ("failed to decode challenge key"));
       goto cleanup_reply_and_return;
     }
-  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &n) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &challenge)))
+  if (! (ssh1_decode_mp(&buffer, &e) &&
+         ssh1_decode_mp(&buffer, &n) &&
+         ssh1_decode_mp(&buffer, &challenge)))
     {
       SSH_DEBUG(3, ("failed to decode challenge"));
       goto cleanup_reply_and_return;
@@ -1808,7 +1845,7 @@ void ssh_agenti_ssh1_challenge(SshAgentConnection conn,
     {
       SSH_DEBUG(7, ("Using ssh1 key to decrypt challenge."));
       /* Decrypt the challenge. */
-      mpz_powm(&output, &challenge, &(ssh1_key->d), &(ssh1_key->n));
+      ssh_mp_powm(&output, &challenge, &(ssh1_key->d), &(ssh1_key->n));
       /* Linearize the output */
       output_len = ((ssh_mp_get_size(&output, 2) + 7) >> 3) & 0xffff;
       if (output_len == 0)
@@ -1937,10 +1974,10 @@ void ssh_agenti_ssh1_challenge(SshAgentConnection conn,
 
  cleanup_reply_and_return:
   ssh_buffer_uninit(&buffer);
-  mpz_clear(&n);
-  mpz_clear(&e);
-  mpz_clear(&challenge);
-  mpz_clear(&output);
+  ssh_mp_clear(&n);
+  ssh_mp_clear(&e);
+  ssh_mp_clear(&challenge);
+  ssh_mp_clear(&output);
   if (public_key)
     ssh_public_key_free(public_key);
   ssh_xfree(public_blob);
@@ -1960,8 +1997,8 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
                              const unsigned char *data, 
                              size_t len)
 {
-  SshBuffer buffer;
-  MP_INT n, e, d, u, p, q;
+  SshBufferStruct buffer;
+  SshInt n, e, d, u, p, q;
   SshUInt32 bits;
   SshPrivateKey private_key = NULL;
   SshPublicKey public_key = NULL;
@@ -1972,12 +2009,12 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
 
   ssh_buffer_init(&buffer);
   ssh_buffer_append(&buffer, data, len);
-  mpz_init(&n);
-  mpz_init(&e);
-  mpz_init(&d);
-  mpz_init(&u);
-  mpz_init(&p);
-  mpz_init(&q);
+  ssh_mp_init(&n);
+  ssh_mp_init(&e);
+  ssh_mp_init(&d);
+  ssh_mp_init(&u);
+  ssh_mp_init(&p);
+  ssh_mp_init(&q);
 
   /* Get length of the key (not needed) */
   if (ssh_decode_buffer(&buffer, 
@@ -1986,12 +2023,12 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
     goto cleanup_reply_and_return;
 
   /* Get key numbers */
-  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &n) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &d) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &u) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &p) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &q)))
+  if (! (ssh1_decode_mp(&buffer, &n) &&
+         ssh1_decode_mp(&buffer, &e) &&
+         ssh1_decode_mp(&buffer, &d) &&
+         ssh1_decode_mp(&buffer, &u) &&
+         ssh1_decode_mp(&buffer, &p) &&
+         ssh1_decode_mp(&buffer, &q)))
     goto cleanup_reply_and_return;
 
   /* Get key comment */
@@ -2007,7 +2044,7 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
    * be only visible for ssh-agent1.
    */
   /* Create the SSH private key from the extracted numbers */
-  if (ssh_private_key_generate(conn->agent->random_state, &private_key,
+  if (ssh_private_key_generate(&private_key,
                                SSH_CRYPTO_RSA,
                                SSH_PKF_SIZE, bits,
                                SSH_PKF_MODULO_N, &n,
@@ -2042,12 +2079,12 @@ void ssh_agenti_ssh1_add_key(SshAgentConnection conn,
   public_blob = NULL;
 
  cleanup_reply_and_return:
-  mpz_clear(&n);
-  mpz_clear(&e);
-  mpz_clear(&d);
-  mpz_clear(&u);
-  mpz_clear(&p);
-  mpz_clear(&q);
+  ssh_mp_clear(&n);
+  ssh_mp_clear(&e);
+  ssh_mp_clear(&d);
+  ssh_mp_clear(&u);
+  ssh_mp_clear(&p);
+  ssh_mp_clear(&q);
   ssh_buffer_uninit(&buffer);
   ssh_xfree(comment);
   ssh_xfree(public_blob);
@@ -2065,8 +2102,8 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
                                 const unsigned char *data, 
                                 size_t len)
 {
-  SshBuffer buffer;
-  MP_INT n, e;
+  SshBufferStruct buffer;
+  SshInt n, e;
   SshUInt32 bits;
   SshPublicKey key = NULL;
   unsigned char *blob = NULL;
@@ -2075,8 +2112,8 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
 
   ssh_buffer_init(&buffer);
   ssh_buffer_append(&buffer, data, len);
-  mpz_init(&n);
-  mpz_init(&e);
+  ssh_mp_init(&n);
+  ssh_mp_init(&e);
 
   /* Get length of the key (not needed) */
   if (ssh_decode_buffer(&buffer, 
@@ -2085,8 +2122,8 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
     goto cleanup_reply_and_return;
 
   /* Get public key numbers */
-  if (! (ssh_agenti_ssh1_decode_mp(&buffer, &e) &&
-         ssh_agenti_ssh1_decode_mp(&buffer, &n)))
+  if (! (ssh1_decode_mp(&buffer, &e) &&
+         ssh1_decode_mp(&buffer, &n)))
     goto cleanup_reply_and_return;
 
   success = ssh_agenti_ssh1key_delete(conn->agent, &n, &e);
@@ -2112,8 +2149,8 @@ void ssh_agenti_ssh1_remove_key(SshAgentConnection conn,
   blob = NULL;
 
  cleanup_reply_and_return:
-  mpz_clear(&n);
-  mpz_clear(&e);
+  ssh_mp_clear(&n);
+  ssh_mp_clear(&e);
   if (key)
     ssh_public_key_free(key);
   ssh_buffer_uninit(&buffer);
@@ -2165,15 +2202,15 @@ void ssh_agenti_handle_ssh1_packet(SshAgentConnection conn,
 
 /* Find ssh1 key with the given public part */
 SshAgentSsh1Key ssh_agenti_ssh1key_find(SshAgentImpl agent,
-                                        MP_INT *n, 
-                                        MP_INT *e)
+                                        SshInt *n, 
+                                        SshInt *e)
 {
   SshAgentSsh1Key key;
 
   key = agent->ssh1_keys;
   while (key)
     {
-      if ((mpz_cmp(n, &(key->n)) == 0) && (mpz_cmp(e, &(key->e)) == 0))
+      if ((ssh_mp_cmp(n, &(key->n)) == 0) && (ssh_mp_cmp(e, &(key->e)) == 0))
         return key;
       key = key->next;
     }
@@ -2182,9 +2219,9 @@ SshAgentSsh1Key ssh_agenti_ssh1key_find(SshAgentImpl agent,
 
 /* Adds the given ssh1 private key to be managed by the agent. */
 Boolean ssh_agenti_ssh1key_add(SshAgentImpl agent,
-                               MP_INT *n, 
-                               MP_INT *e,
-                               MP_INT *d,
+                               SshInt *n, 
+                               SshInt *e,
+                               SshInt *d,
                                char *description)
 {
   SshAgentSsh1Key key;
@@ -2194,9 +2231,9 @@ Boolean ssh_agenti_ssh1key_add(SshAgentImpl agent,
   (void)ssh_agenti_ssh1key_delete(agent, n, e);
   key = ssh_xcalloc(1, sizeof (*key));
   key->description = ssh_xstrdup(description);
-  mpz_init_set(&(key->n), n);
-  mpz_init_set(&(key->e), e);
-  mpz_init_set(&(key->d), d);
+  ssh_mp_init_set(&(key->n), n);
+  ssh_mp_init_set(&(key->e), e);
+  ssh_mp_init_set(&(key->d), d);
   key->next = agent->ssh1_keys;
   agent->ssh1_keys = key;
   return TRUE;
@@ -2204,8 +2241,8 @@ Boolean ssh_agenti_ssh1key_add(SshAgentImpl agent,
 
 /* Delete a ssh1 key with given public key */
 Boolean ssh_agenti_ssh1key_delete(SshAgentImpl agent,
-                                  MP_INT *n, 
-                                  MP_INT *e)
+                                  SshInt *n, 
+                                  SshInt *e)
 {
   SshAgentSsh1Key key, prev;
   Boolean success = FALSE;
@@ -2214,15 +2251,15 @@ Boolean ssh_agenti_ssh1key_delete(SshAgentImpl agent,
   key = agent->ssh1_keys;
   while (key)
     {
-      if ((mpz_cmp(n, &(key->n)) == 0) && (mpz_cmp(e, &(key->e)) == 0))
+      if ((ssh_mp_cmp(n, &(key->n)) == 0) && (ssh_mp_cmp(e, &(key->e)) == 0))
         {
           if (prev)
             prev->next = key->next;
           else
             agent->ssh1_keys = key->next;
-          mpz_clear(&key->n);
-          mpz_clear(&key->e);
-          mpz_clear(&key->d);
+          ssh_mp_clear(&key->n);
+          ssh_mp_clear(&key->e);
+          ssh_mp_clear(&key->d);
           ssh_xfree(key->description);
           ssh_xfree(key);
           success = TRUE;
@@ -2243,9 +2280,9 @@ void ssh_agenti_ssh1key_delete_all(SshAgentImpl agent)
 
   for (key = agent->ssh1_keys; key; key = key->next)
     {
-      mpz_clear(&key->n);
-      mpz_clear(&key->e);
-      mpz_clear(&key->d);
+      ssh_mp_clear(&key->n);
+      ssh_mp_clear(&key->e);
+      ssh_mp_clear(&key->d);
       ssh_xfree(key->description);
       ssh_xfree(key);
     }
