@@ -2,12 +2,12 @@
   ssh-keygen.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   A tool for generating and manipulating private keys.
@@ -21,15 +21,21 @@
 
 
 #include "sshincludes.h"
-#include "sshreadline.h"
-#include "readpass.h"
+#include "ssh2includes.h"
 #include "sshuserfiles.h"
+#include "sshreadline.h"
+#include "sshfileio.h"
+#include "readpass.h"
 #include "sshuser.h"
-#include "ssh2version.h"
+#include "sshcrypt.h"
 #include "sshcipherlist.h"
+#include "ssh2pubkeyencode.h"
 #include "sshgetopt.h"
+#include "sshappcommon.h"
 #include <sys/types.h>
 #include <pwd.h>
+#include "sshmp.h"
+#include "sshfingerprint.h"
 
 #define SSH_DEBUG_MODULE "SshKeyGen"
 
@@ -39,32 +45,33 @@
 #define KEYGEN_ASSUMED_PUBKEY_LEN 1024
 #endif /* KEYGEN_ASSUMED_PUBKEY_LEN */
 
+#ifdef HAVE_LIBWRAP
+int allow_severity = SSH_LOG_INFORMATIONAL;
+int deny_severity = SSH_LOG_WARNING;
+#endif /* HAVE_LIBWRAP */
+
 /* helptext */
 
 const char keygen_helptext[] =
-  "Usage: ssh-keygen [options]\n"
+  "Usage: ssh-keygen [options] [key1 key2 ...]\n"
   "\n"
   "Where `options' are:\n"
   " -b nnn         Specify key strength in bits (e.g. 1024)\n"
-#ifdef SSHDIST_CRYPT_RSA
-
-
-
-
-
-#else /* SSHDIST_CRYPT_RSA */
   " -t dsa         Choose the key type (only dsa available).\n"
-#endif /* SSHDIST_CRYPT_RSA */
-  " -h             Print this help text.\n"
-  " -e file        Edit the comment/passphrase of the key.\n"
   " -c comment     Provide the comment.\n"
+  " -e file        Edit the comment/passphrase of the key.\n"
   " -p passphrase  Provide passphrase.\n"
   " -P             Assume empty passphrase.\n"
+  " -?\n"
+  " -h             Print this help text.\n"
   " -q             Suppress the progress indicator.\n"
   " -1             Convert a SSH 1.x key.  (not implemented)\n"
-  " -i file        Load and display information on `file'.  (not implemented)\n"
-  " -v             Print ssh-keygen version number.\n"
-  " -r             Stir data from stdin to random pool.\n";
+  " -i file        Load and display information on `file'.\n"
+  " -D file        Derive the private key given in 'file' to public key.\n"
+  " -B number      The number base for displaying key information (default 10).\n"
+  " -V             Print ssh-keygen version number.\n"
+  " -r file        Stir data from file to random pool.\n"
+  " -F file        Dump fingerprint of file.\n\n";
 
 /* A context structure -- we don't like global variables. */
 
@@ -75,7 +82,6 @@ typedef struct
   Boolean convert;
   Boolean status;
   Boolean edit_key;
-  Boolean read_stdin;
   
   char *keytype;
   char *keytypecommon;
@@ -83,9 +89,12 @@ typedef struct
   char *in_filename;
   char *out_filename;
   char *passphrase;
+  char *info_file;
+  char *derive_file;
   Boolean pass_empty;
 
-  SshRandomState random_state;
+  unsigned int base;
+  
   SshPrivateKey private_key;
   SshPublicKey public_key;
   SshUser user;
@@ -93,6 +102,10 @@ typedef struct
   Boolean prog_ind;
   Boolean have_prog_ind;
 
+  Boolean read_file;
+  Boolean dump_fingerprint;
+  SshFingerPrintType fingerprint_type;
+  char *entropy_file_name;
 } KeyGenCtx;
 
 /* mapping of common names and CryptoLib names. Common names are
@@ -107,12 +120,6 @@ const char *keygen_common_names[][2] =
   { "dsa", SSH_CRYPTO_DSS },
   { "dss", SSH_CRYPTO_DSS },
 
-#ifdef SSHDIST_CRYPT_RSA
-
-
-
-
-#endif /* SSHDIST_CRYPT_RSA */
 
   /* Last entry */
   { NULL, NULL }
@@ -120,7 +127,7 @@ const char *keygen_common_names[][2] =
 
 /* allocate the context */
 
-KeyGenCtx *keygen_init_ctx()
+KeyGenCtx *keygen_init_ctx(void)
 {
   KeyGenCtx *kgc;
 
@@ -129,7 +136,7 @@ KeyGenCtx *keygen_init_ctx()
   kgc->newkey = FALSE;
   kgc->convert = FALSE;
   kgc->status = FALSE;
-  kgc->read_stdin = FALSE;
+  kgc->read_file = FALSE;
   kgc->edit_key = FALSE;
   
   kgc->keytype = NULL;
@@ -139,15 +146,22 @@ KeyGenCtx *keygen_init_ctx()
   kgc->in_filename = NULL;
   kgc->passphrase = NULL;
   kgc->pass_empty = FALSE;
-
+  kgc->info_file = NULL;
+  kgc->derive_file = NULL;
+  kgc->dump_fingerprint = FALSE;
+  kgc->fingerprint_type = SSH_FINGERPRINT_BABBLE;
+  
+  kgc->base = 10;
+  
   kgc->user = ssh_user_initialize(NULL, FALSE);  
-  kgc->random_state = ssh_randseed_open(kgc->user, NULL);
+  ssh_randseed_open(kgc->user, NULL);
   kgc->public_key = NULL;
   kgc->private_key = NULL;
 
   kgc->prog_ind = FALSE;
   kgc->have_prog_ind = TRUE;
 
+  kgc->entropy_file_name = NULL;
   return kgc;
 }
 
@@ -172,13 +186,15 @@ void keygen_free_ctx(KeyGenCtx *kgc)
   keygen_free_str(kgc->in_filename);
   keygen_free_str(kgc->out_filename);
   keygen_free_str(kgc->passphrase);
+  keygen_free_str(kgc->info_file);
+  keygen_free_str(kgc->derive_file);
+  keygen_free_str(kgc->entropy_file_name);
 
   if (kgc->public_key != NULL)
     ssh_public_key_free(kgc->public_key);
   if (kgc->private_key != NULL)
     ssh_private_key_free(kgc->private_key);
-  ssh_randseed_update(kgc->user, kgc->random_state, NULL);
-  ssh_random_free(kgc->random_state);
+  ssh_randseed_update(kgc->user, NULL);
   if (kgc->user != NULL)
     ssh_user_free(kgc->user, FALSE);
   if (kgc->prog_ind)
@@ -315,6 +331,290 @@ done:
   ssh_xfree(udir);
 }
 
+/* Read private key. */
+
+SshPrivateKey keygen_read_privkey(KeyGenCtx *kgc, char *filename)
+{
+  SshPrivateKey prvkey = NULL;
+  /* Guess that it is a private key. */
+  
+  if (kgc->passphrase)
+    {
+      prvkey = ssh_privkey_read(kgc->user,
+                                filename,
+                                kgc->passphrase, NULL, NULL);
+    }
+
+  if (prvkey == NULL)
+    {
+      prvkey = ssh_privkey_read(kgc->user, filename,
+                                "", NULL, NULL);
+      if (prvkey == NULL &&
+          !kgc->passphrase && kgc->pass_empty == FALSE)
+        {
+          char *pass;
+          pass = ssh_read_passphrase("Passphrase : ", FALSE);
+
+          prvkey = ssh_privkey_read(kgc->user, filename,
+                                    pass, NULL, NULL);
+          ssh_xfree(pass);
+        }
+    }
+
+  return prvkey;
+}
+
+Boolean keygen_out_number(KeyGenCtx *kgc, SshInt *number)
+{
+  char *buf;
+  buf = ssh_mp_get_str(NULL, kgc->base, number);
+  printf("%s\n", buf);
+  ssh_xfree(buf);
+  return FALSE;
+}
+
+Boolean keygen_view_prvkey(KeyGenCtx *kgc, SshPrivateKey prv)
+{
+  const char *name;
+
+  if (ssh_private_key_get_info(prv, SSH_PKF_KEY_TYPE, &name,
+                               SSH_PKF_END) != SSH_CRYPTO_OK)
+    {
+      printf("Private key has no type (corrupted)\n");
+      goto failed;
+    }
+
+  if (strcmp(name, "if-modn") == 0)
+    {
+      SshInt n, e, d, p, q, u;
+
+      /* RSA? */
+      ssh_mp_init(&n);
+      ssh_mp_init(&e);
+      ssh_mp_init(&d);
+      ssh_mp_init(&p);
+      ssh_mp_init(&q);
+      ssh_mp_init(&u);
+
+      /* Get the necessary information of the SSH style RSA key. */
+      if (ssh_private_key_get_info(prv,
+                                   SSH_PKF_MODULO_N,  &n,
+                                   SSH_PKF_PUBLIC_E,  &e,
+                                   SSH_PKF_SECRET_D,  &d,
+                                   SSH_PKF_PRIME_P,   &p,
+                                   SSH_PKF_PRIME_Q,   &q,
+                                   SSH_PKF_INVERSE_U, &u,
+                                   SSH_PKF_END) != SSH_CRYPTO_OK)
+        {
+          ssh_mp_clear(&n);
+          ssh_mp_clear(&e);
+          ssh_mp_clear(&d);
+          ssh_mp_clear(&p);
+          ssh_mp_clear(&q);
+          ssh_mp_clear(&u);
+          printf("RSA private key does not have all the fields.");
+          goto failed;
+        }
+
+      printf("RSA Private Key");
+      printf("  e = [Encryption exponent]\n");
+      keygen_out_number(kgc, &e);
+
+      printf("  n = [Large composite moduli]\n");
+      keygen_out_number(kgc, &n);
+
+      printf("  d = [Inverse of e modulo (p-1)*(q-1) (Euler's phi function of n)]\n");
+      keygen_out_number(kgc, &d);
+
+      printf("  p = [Large random prime]\n");
+      keygen_out_number(kgc, &p);
+
+      printf("  q = [Large random prime]\n");
+      keygen_out_number(kgc, &q);
+
+      printf("  u = [Multiplicative inverse of p modulo q, used to speed RSA]\n");
+      keygen_out_number(kgc, &u);
+
+      ssh_mp_clear(&n);
+      ssh_mp_clear(&e);
+      ssh_mp_clear(&d);
+      ssh_mp_clear(&p);
+      ssh_mp_clear(&q);
+      ssh_mp_clear(&u);
+
+      return FALSE;
+    }
+
+  if (strcmp(name, "dl-modp") == 0)
+    {
+      SshInt p, q, g, y, x;
+
+      ssh_mp_init(&p);
+      ssh_mp_init(&q);
+      ssh_mp_init(&g);
+      ssh_mp_init(&y);
+      ssh_mp_init(&x);
+
+      if (ssh_private_key_get_info(prv,
+                                   SSH_PKF_PRIME_P, &p,
+                                   SSH_PKF_PRIME_Q, &q,
+                                   SSH_PKF_GENERATOR_G, &g,
+                                   SSH_PKF_PUBLIC_Y, &y,
+                                   SSH_PKF_SECRET_X, &x,
+                                   SSH_PKF_END) != SSH_CRYPTO_OK)
+        {
+          ssh_mp_clear(&p);
+          ssh_mp_clear(&g);
+          ssh_mp_clear(&q);
+          ssh_mp_clear(&y);
+          ssh_mp_clear(&x);
+
+          printf("DSA private key does not have all the fields.");
+          goto failed;
+        }
+
+      /* DSA */
+      printf("DSA Private Key\n");
+
+      printf("  p = [Large prime, characteristic of the finite field]\n");
+      keygen_out_number(kgc, &p);
+
+      printf("  g = [Generator of a cyclic subgroup of the multiplicative group F_p]\n");
+      keygen_out_number(kgc, &g);
+
+      printf("  q = [Prime number, order of the cyclic group]\n");
+      keygen_out_number(kgc, &q);
+
+      printf("  y = [Random power of the generator, the 'actual' public key]\n");
+      keygen_out_number(kgc, &y);
+
+      printf("  x = [A random number, the 'actual' secret key]\n");
+      keygen_out_number(kgc, &x);
+
+      ssh_mp_clear(&p);
+      ssh_mp_clear(&g);
+      ssh_mp_clear(&q);
+      ssh_mp_clear(&y);
+      ssh_mp_clear(&x);
+
+      return FALSE;
+    }
+
+failed:
+  return TRUE;
+}
+
+Boolean keygen_view_pubkey(KeyGenCtx *kgc, SshPublicKey prv)
+{
+  const char *name;
+
+  if (ssh_public_key_get_info(prv, SSH_PKF_KEY_TYPE, &name,
+                               SSH_PKF_END) != SSH_CRYPTO_OK)
+    {
+      printf("Public key has no type (corrupted)\n");
+      goto failed;
+    }
+
+  if (strcmp(name, "if-modn") == 0)
+    {
+      SshInt n, e, d, p, q, u;
+
+      /* RSA? */
+      ssh_mp_init(&n);
+      ssh_mp_init(&e);
+      ssh_mp_init(&d);
+      ssh_mp_init(&p);
+      ssh_mp_init(&q);
+      ssh_mp_init(&u);
+
+      /* Get the necessary information of the SSH style RSA key. */
+      if (ssh_public_key_get_info(prv,
+                                  SSH_PKF_MODULO_N,  &n,
+                                  SSH_PKF_PUBLIC_E,  &e,
+                                  SSH_PKF_END) != SSH_CRYPTO_OK)
+        {
+          ssh_mp_clear(&n);
+          ssh_mp_clear(&e);
+          ssh_mp_clear(&d);
+          ssh_mp_clear(&p);
+          ssh_mp_clear(&q);
+          ssh_mp_clear(&u);
+          printf("RSA public key does not have all the fields.");
+          goto failed;
+        }
+
+      printf("RSA Public Key");
+      printf("  e = [Encryption exponent]\n");
+      keygen_out_number(kgc, &e);
+
+      printf("  n = [Large composite moduli]\n");
+      keygen_out_number(kgc, &n);
+
+      ssh_mp_clear(&n);
+      ssh_mp_clear(&e);
+      ssh_mp_clear(&d);
+      ssh_mp_clear(&p);
+      ssh_mp_clear(&q);
+      ssh_mp_clear(&u);
+
+      return FALSE;
+    }
+
+  if (strcmp(name, "dl-modp") == 0)
+    {
+      SshInt p, q, g, y, x;
+
+      ssh_mp_init(&p);
+      ssh_mp_init(&q);
+      ssh_mp_init(&g);
+      ssh_mp_init(&y);
+      ssh_mp_init(&x);
+
+      if (ssh_public_key_get_info(prv,
+                                  SSH_PKF_PRIME_P, &p,
+                                  SSH_PKF_PRIME_Q, &q,
+                                  SSH_PKF_GENERATOR_G, &g,
+                                  SSH_PKF_PUBLIC_Y, &y,
+                                  SSH_PKF_END) != SSH_CRYPTO_OK)
+        {
+          ssh_mp_clear(&p);
+          ssh_mp_clear(&g);
+          ssh_mp_clear(&q);
+          ssh_mp_clear(&y);
+          ssh_mp_clear(&x);
+
+          printf("DSA public key does not have all the fields.");
+          goto failed;
+        }
+
+      /* DSA */
+      printf("DSA Public Key\n");
+
+      printf("  p = [Large prime, characteristic of the finite field]\n");
+      keygen_out_number(kgc, &p);
+
+      printf("  g = [Generator of a cyclic subgroup of the multiplicative group F_p]\n");
+      keygen_out_number(kgc, &g);
+
+      printf("  q = [Prime number, order of the cyclic group]\n");
+      keygen_out_number(kgc, &q);
+
+      printf("  y = [Random power of the generator, the 'actual' public key]\n");
+      keygen_out_number(kgc, &y);
+
+      ssh_mp_clear(&p);
+      ssh_mp_clear(&g);
+      ssh_mp_clear(&q);
+      ssh_mp_clear(&y);
+      ssh_mp_clear(&x);
+
+      return FALSE;
+    }
+
+failed:
+  return TRUE;
+}
+
 
 /* Generate the key. this is done when kgc->newkey is TRUE. */
 
@@ -339,8 +639,7 @@ int keygen_keygen(KeyGenCtx *kgc)
          kgc->keybits,
          kgc->keytypecommon);
 
-  if ((code = ssh_private_key_generate(kgc->random_state, 
-                                       &(kgc->private_key),
+  if ((code = ssh_private_key_generate(&(kgc->private_key),
                                        kgc->keytype,
                                        SSH_PKF_SIZE, kgc->keybits,
                                        SSH_PKF_END)) != SSH_CRYPTO_OK)
@@ -372,14 +671,21 @@ int keygen_keygen(KeyGenCtx *kgc)
 
   if (!(*(kgc->passphrase)))
     {
-      ssh_warning("Key is stored with NULL passphrase.");      
+      ssh_informational("Key is stored with NULL passphrase.\n");
+      ssh_informational("(This is not recommended. \n");
+      ssh_informational(" Don't do this unless you know what you're doing.\n");
+      ssh_informational(" If file system protections fail (someone can "
+                        " access the keyfile), \n");
+      ssh_informational(" or if the super-user is malicious, your key can "
+                        " be used without \n");
+      ssh_informational(" the deciphering effort.)\n");
     }
 
   if (ssh_privkey_write(kgc->user,
                         kgc->out_filename, 
                         kgc->passphrase, 
                         kgc->comment,
-                        kgc->private_key, kgc->random_state, NULL))
+                        kgc->private_key, NULL))
     {
       ssh_warning("Private key not written !");
       r++;
@@ -393,7 +699,12 @@ int keygen_keygen(KeyGenCtx *kgc)
 
   snprintf(buf, sizeof (buf), "%s.pub", kgc->out_filename);
   kgc->public_key = ssh_private_key_derive_public_key(kgc->private_key);
-
+  if (kgc->public_key == NULL)
+    {
+      ssh_warning("Could not derive public key from private key.");
+      return r + 1;
+    }
+  
   if (ssh_pubkey_write(kgc->user, buf, kgc->comment, kgc->public_key, NULL))
     {
       ssh_warning("Public key not written !");
@@ -406,26 +717,24 @@ int keygen_keygen(KeyGenCtx *kgc)
   return r;
 }
 
-/* Stir in data from stdin */
+/* Stir in data from file */
 
-void stir_stdin(KeyGenCtx *kgc)
+void stir_from_file(KeyGenCtx *kgc)
 {
-  unsigned char buffer[64];
-  size_t n, bytes;
-  
-  bytes = 0;
-  
-  while (!feof(stdin))
+  unsigned char *buffer;
+  size_t buffer_len;
+
+  if (FALSE == ssh_read_file(kgc->entropy_file_name, &buffer, &buffer_len))
     {
-      n = fread(buffer, 1, sizeof (buffer), stdin);
-      if (n > 0)
-        ssh_random_add_noise(kgc->random_state, buffer, n);
-      bytes += n;
+      ssh_fatal("Cannot read file %s.", kgc->entropy_file_name);
+      return;
     }
-  memset(buffer, 0, sizeof (buffer));
-  
+  ssh_random_add_noise(buffer, buffer_len);
+  memset(buffer, 0, buffer_len);
+  ssh_xfree(buffer);
+  ssh_random_stir();
   if (kgc->have_prog_ind)
-    printf("Stirred in %lu bytes.\n", (unsigned long) bytes);
+    printf("Stirred in %lu bytes.\n", (unsigned long) buffer_len);
 }
 
 int keygen_convert_key(KeyGenCtx *kgc)
@@ -522,7 +831,7 @@ int keygen_edit_key(KeyGenCtx *kgc)
     }
 
   printf("Do you want to edit key \"%s\" ", oldcomment);
-  if (! ssh_read_confirmation("(yes or no)? "))
+  if (! ssh_read_confirmation("(yes or no)? ", FALSE))
     {
       printf("Key unedited and unsaved.\n");
       keygen_free_str(oldcomment);
@@ -537,11 +846,11 @@ int keygen_edit_key(KeyGenCtx *kgc)
       if (! newcomment)
         newcomment = ssh_xstrdup(oldcomment);
       printf("Your key comment is \"%s\". ", newcomment);
-      if (ssh_read_confirmation("Do you want to edit it (yes or no)? "))
+      if (ssh_read_confirmation("Do you want to edit it (yes or no)? ", FALSE))
         {
-          if (ssh_readline("New key comment: ", 
+          if (ssh_readline((unsigned char *)"New key comment: ", 
                            (unsigned char **)&newcomment, 
-                           TRUE) < 0)
+                           fileno(stdin), fileno(stdout)) < 0)
             {
               fprintf(stderr, "Abort!  Key unedited and unsaved.\n");
               keygen_free_str(newpass);
@@ -555,7 +864,7 @@ int keygen_edit_key(KeyGenCtx *kgc)
         newpass = ssh_xstrdup(kgc->passphrase);
       if (! kgc->pass_empty)
         {
-          if (ssh_read_confirmation("Do you want to edit passphrase (yes or no)? "))
+          if (ssh_read_confirmation("Do you want to edit passphrase (yes or no)? ", FALSE))
             {
               keygen_free_str(newpass);
               newpass = ssh_read_passphrase_twice("New passphrase : ", 
@@ -571,7 +880,7 @@ int keygen_edit_key(KeyGenCtx *kgc)
             }
         }
       printf("Do you want to continue editing key \"%s\" ", newcomment);
-      if (ssh_read_confirmation("(yes or no)? "))
+      if (ssh_read_confirmation("(yes or no)? ", FALSE))
         ok = 0;
       else
         ok = 1;
@@ -586,7 +895,7 @@ int keygen_edit_key(KeyGenCtx *kgc)
       return 0;
     }
   printf("Do you want to save key \"%s\" to file %s ", newcomment, outsecfn);
-  if (! ssh_read_confirmation("(yes or no)? "))
+  if (! ssh_read_confirmation("(yes or no)? ", FALSE))
     {
       printf("Key unsaved.\n");
       keygen_free_str(newpass);
@@ -628,7 +937,6 @@ int keygen_edit_key(KeyGenCtx *kgc)
                         newpass, 
                         newcomment,
                         seckey, 
-                        kgc->random_state, 
                         NULL))
     {
       ssh_warning("Unable to write private key.!");
@@ -669,6 +977,56 @@ int keygen_edit_key(KeyGenCtx *kgc)
   return 0;
 }
 
+Boolean keygen_dump_pubkey_fingerprint(KeyGenCtx *kgc, SshPublicKey pubkey)
+{
+  char *hash_name = "sha1";
+  unsigned char *digest;
+  SshHash hash_ctx;
+  size_t digest_len;
+  unsigned char *blob;
+  size_t blob_len;
+  char *fingerprint;
+  
+  if (!(blob_len = ssh_encode_pubkeyblob(pubkey, &blob)))
+    {
+      ssh_warning("keygen_dump_pubkey_fingerprint: Couldn't encode publickey "
+                  "to blob.");
+      return FALSE;
+    }
+  
+  if (!ssh_hash_supported(hash_name))
+    {
+      ssh_warning("keygen_dump_pubkey_fingerprint: Hash \"%s\" not supported, "
+                  "even though it is required in the draft. Contact "
+                  "ssh2-bugs@ssh.com.", hash_name);
+      return FALSE;
+    }
+
+  if (ssh_hash_allocate(hash_name, &hash_ctx) != SSH_CRYPTO_OK)
+    {
+      ssh_warning("keygen_dump_pubkey_fingerprint: Couldn't initialize hash "
+                  "function.", hash_name);
+      return FALSE;
+    }  
+
+  digest_len = ssh_hash_digest_length(hash_ctx);
+  digest = ssh_xcalloc(digest_len, sizeof(unsigned char));
+  ssh_hash_update(hash_ctx, blob, blob_len);
+  ssh_hash_final(hash_ctx, digest);
+  ssh_hash_free(hash_ctx);
+
+  fingerprint =
+    ssh_fingerprint(digest, digest_len, kgc->fingerprint_type);
+
+  fprintf(stderr, "Fingerprint for key:\r\n");
+  printf("%s\r\n", fingerprint);
+
+  ssh_xfree(blob);
+  ssh_xfree(fingerprint);
+  ssh_xfree(digest);
+  return TRUE;
+}
+
 /* main for ssh-keygen */
 
 int main(int argc, char **argv)
@@ -677,15 +1035,22 @@ int main(int argc, char **argv)
   KeyGenCtx *kgc;
   struct passwd *pw;
   char *t;
-  time_t now;
+  SshTime now;
   int r = 0, rr = 0;
-  
+  char *progname;
+
+  if (strchr(argv[0], '/'))
+    progname = strrchr(argv[0], '/') + 1;
+  else
+    progname = argv[0];
+
   /* Initialize the context */
   kgc = keygen_init_ctx();
 
   /* try to find the user's home directory */
 
-  while ((ch = ssh_getopt(argc, argv, "vhqr?P1:t:b:p:o:i:e:c:", NULL)) != EOF)
+  while ((ch = ssh_getopt(argc, argv, "vhqr:?P1:t:b:p:i:D:B:e:c:VF:", NULL))
+         != EOF)
     {
       if (!ssh_optval)
         {
@@ -701,7 +1066,7 @@ int main(int argc, char **argv)
           if (kgc->keybits != 0)
             keygen_error(kgc, "Multiple -b parameters");
           kgc->keybits = atoi(ssh_optarg);
-          if (kgc->keybits < 128 || kgc->keybits > 0x10000)
+          if (kgc->keybits < 512 || kgc->keybits > 0x10000)
             keygen_error(kgc, "Illegal key size.");
           break;
 
@@ -753,20 +1118,14 @@ int main(int argc, char **argv)
           kgc->in_filename = ssh_xstrdup(ssh_optarg);
           break;
 
-          /* -o: Provide the output filename. */
-        case 'o':
-          kgc->out_filename = ssh_xstrdup(ssh_optarg);
-          break;
-
-          /* -v: print the version number */
-        case 'v':
-          printf("ssh2-keygen version " SSH2_VERSION
-                 ", compiled "__DATE__".\n");
+          /* -V: print the version number */
+        case 'V':
+          printf("%s version " SSH2_VERSION
+                 ", compiled "__DATE__".\n", progname);
           /* XXX more stuff here possibly */ 
           keygen_free_ctx(kgc);
           exit(0);
 
-          /* -h: print a short help text */
         case '?':
         case 'h':
           printf("%s", keygen_helptext);
@@ -778,25 +1137,136 @@ int main(int argc, char **argv)
           kgc->have_prog_ind = FALSE;
           break;
 
-          /* -r: stir in data from stdin to the random pool */
+          /* -r: stir in data from file to the random pool */
         case 'r':
-          kgc->read_stdin = TRUE;
+          kgc->read_file = TRUE;
+          kgc->entropy_file_name = ssh_xstrdup(ssh_optarg);
           break;
+
+          /* XXX Following are experimental options. */
           
           /* -i: display (all) information about a key */
         case 'i':
-          ssh_fatal("-i not yet implemented XXX");
+          kgc->info_file = ssh_xstrdup(ssh_optarg);
+          break;
+        case 'D':
+          kgc->derive_file = ssh_xstrdup(ssh_optarg);
+          break;
+        case 'B':
+          kgc->base = atoi(ssh_optarg);
+          break;
+        case 'F':
+          kgc->dump_fingerprint = TRUE;
+          ssh_xfree(kgc->info_file);
+          kgc->info_file = ssh_xstrdup(ssh_optarg);
+          break;
+        default:
+          keygen_error(kgc, "Invalid option.");
+          break;
         }
     }
 
-  
-  /* Stir in random data from stdin, if requested */
-
-  if (kgc->read_stdin)
+  /* Stir in random data from file, if requested */
+  if (kgc->read_file)
     {
-      stir_stdin(kgc);
-      keygen_free_ctx(kgc);      
-      return 0;
+      stir_from_file(kgc);
+    }
+
+  /* Dump fingerprint */
+  if (kgc->dump_fingerprint)
+    {
+      SshPublicKey pubkey = NULL;
+
+      if (!kgc->info_file)
+        ssh_fatal("-F option needs public key file as argument.");
+      
+      pubkey = ssh_pubkey_read(kgc->user, kgc->info_file,
+                               NULL, NULL);
+      if (pubkey != NULL)
+        {
+          if (keygen_dump_pubkey_fingerprint(kgc, pubkey))
+            r = 0;
+          else
+            r = 1;
+          
+          goto finish;
+        }
+      else
+        {
+          ssh_fatal("Couldn't read public key \"%s\".", kgc->info_file);
+        }
+    }
+  
+  /* Perhaps we should derive a key? */
+  if (kgc->derive_file)
+    {
+      SshPrivateKey prvkey = NULL;
+      SshPublicKey  pubkey = NULL;
+      unsigned char buf[256];
+
+      pubkey = ssh_pubkey_read(kgc->user, kgc->derive_file,
+                               NULL, NULL);
+      if (pubkey != NULL)
+        {
+          /* Attempt at deriving the private key? */
+          printf("\n  Derivation of a private key from the public key.\n"
+                 "   Please wait...\n");
+          sleep(1);
+          printf("\n  Ho ho, just kidding.\n"
+                 "   Cryptographically strong keys cannot be broken!\n");
+          goto finish;
+        }
+
+      /* Perhaps its a private key? */
+      prvkey = keygen_read_privkey(kgc, kgc->derive_file);
+      
+      if (prvkey != NULL)
+        {
+          /* Derive the appropriate public key. */
+          snprintf(buf, sizeof (buf), "%s.pub", kgc->derive_file);
+          pubkey = ssh_private_key_derive_public_key(prvkey);
+          if (pubkey == NULL)
+            {
+              keygen_error(kgc, "Could not derive public key from private key.");
+            }
+
+          if (ssh_pubkey_write(kgc->user, buf, kgc->comment, pubkey, NULL))
+            {
+              keygen_error(kgc, "Public key not written !");
+            }
+          printf("Public key saved to %s\n", buf);
+
+          ssh_public_key_free(pubkey);
+          ssh_private_key_free(prvkey);
+          goto finish;
+        }
+
+      keygen_error(kgc, "Cannot determine the type of the key.");
+    }
+
+  if (kgc->info_file)
+    {
+      SshPrivateKey prvkey = NULL;
+      SshPublicKey  pubkey = NULL;
+
+      pubkey = ssh_pubkey_read(kgc->user, kgc->info_file,
+                               NULL, NULL);
+      if (pubkey != NULL)
+        {
+          keygen_view_pubkey(kgc, pubkey);
+          goto finish;
+        }
+
+      /* Perhaps its a private key? */
+      prvkey = keygen_read_privkey(kgc, kgc->info_file);
+      
+      if (prvkey != NULL)
+        {
+          keygen_view_prvkey(kgc, prvkey);
+          goto finish;
+        }
+
+      keygen_error(kgc, "Cannot determine the type of the key.");
     }
         
   if (kgc->convert)
@@ -824,17 +1294,20 @@ int main(int argc, char **argv)
 
       if (kgc->comment == NULL)
         {
+          char *time_str;
+
           pw = getpwuid(getuid());
           if (!pw)
             keygen_error(kgc, "Could not get user's password structure.");
           t = ssh_xmalloc(64);
           gethostname(t, 64);
           kgc->comment = ssh_xmalloc(256);
-          time(&now);
-          snprintf(kgc->comment, 256, "%d-bit %s, created by %s@%s %s", 
+          now = ssh_time();
+          time_str = ssh_readable_time_string(now, TRUE);
+          snprintf(kgc->comment, 256, "%d-bit %s, %s@%s, %s", 
                    kgc->keybits, kgc->keytypecommon,
-                   pw->pw_name, t, ctime(&now));
-          kgc->comment[strlen(kgc->comment) - 1] = '\0';
+                   pw->pw_name, t, time_str);
+          ssh_xfree(time_str);
           ssh_xfree(t);
         }
 
@@ -869,6 +1342,8 @@ int main(int argc, char **argv)
         }
     }
 
+finish:
+  
   keygen_free_ctx(kgc);
 
   return r;
