@@ -3,16 +3,16 @@
 sshagent.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Interface to the SSH agent.  This header defines the functions that
-applications using the agent can call.
+  Interface to the SSH agent.  This header defines the functions that
+  applications using the agent can call.
 
 */
 
@@ -53,7 +53,7 @@ struct SshAgentKeyAttrsRec
   SshUInt32 use_limit;
   SshUInt32 path_len_limit; 
   char *path_constraint;
-  time_t timeout_time;
+  SshTime timeout_time;
   Boolean compat_allowed; 
 };
 typedef struct SshAgentKeyAttrsRec *SshAgentKeyAttrs;
@@ -105,7 +105,7 @@ void ssh_agent_add_with_attrs(SshAgent agent,
                               char *path_constraint,
                               SshUInt32 use_limit, 
                               Boolean compat_forbidden, 
-                              time_t timeout_time,
+                              SshTime timeout_time,
                               SshAgentCompletion callback,
                               void *context);
 
