@@ -3,27 +3,32 @@
 sshchsession.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Public interface to SSH2 session channels.
+  Public interface to SSH2 session channels.
 
 */
 
 #ifndef SSHCHSESSION_H
 #define SSHCHSESSION_H
 
+#include "sshconn.h"
+#include "sshcommon.h"
+
 /* This environment variable is used to store the original command, given
    to the client in the command line, to the server end's environment.*/
 #ifndef SSH_ORIGINAL_COMMAND
 #define SSH_ORIGINAL_COMMAND "SSH2_ORIGINAL_COMMAND"
 #endif
 
+typedef struct SshChannelSessionRec *SshChannelSession;
+
 /* This function is called whenever an open request is received for a
    session channel. */
 void ssh_channel_session_open(const char *type,
@@ -62,8 +67,23 @@ void ssh_channel_start_session(SshCommon common, SshStream stdio_stream,
                                const char **env,
                                Boolean forward_x11, Boolean forward_agent,
                                void (*completion)(Boolean success,
+                                                  SshChannelSession session,
                                                   void *context),
-                               void (*close_notify)(void *context),
+                               void (*close_notify)(SshUInt32 exit_status,
+                                                    void *context),
                                void *context);
 
+/* Set window size information for the channel, and send the information
+   to the server.  Width and height are in characters; *_pixels can be
+   zero if the information is not available. */
+void ssh_channel_session_send_window_change(SshChannelSession session,
+                                            unsigned int width,
+                                            unsigned int height,
+                                            unsigned int width_pixels,
+                                            unsigned int height_pixels);
+
+/* Closes the given session channel. This will generate a callback to
+   close_notify-function. */
+void ssh_channel_session_close(SshChannelSession session);
+
 #endif /* SSHCHSESSION_H */
