@@ -3,16 +3,16 @@
 sshchssh1agent.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Code for ssh1 authentication agent forwarding channels for SSH2 
-servers and clients.
+  Code for ssh1 authentication agent forwarding channels for SSH2 
+  servers and clients.
 
 */
 
@@ -197,7 +197,7 @@ void ssh_channel_open_ssh1_agent_connected(SshStream stream,
                                           void *context)
 {
   SshAgentConnection a = (SshAgentConnection)context;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   unsigned char *cp;
 
   if (stream == NULL)
@@ -222,13 +222,15 @@ void ssh_channel_open_ssh1_agent_connected(SshStream stream,
       ssh_buffer_init(&buffer);
       ssh_encode_buffer(&buffer,
                         SSH_FORMAT_DATA, "1234", (size_t)4,
-                        SSH_FORMAT_CHAR, SSH_AGENT_FORWARDING_NOTICE,
+                        SSH_FORMAT_CHAR,
+                        (unsigned int) SSH_AGENT_FORWARDING_NOTICE,
                         SSH_FORMAT_UINT32_STR,
                         a->common->server_host_name,
                         strlen(a->common->server_host_name),
                         SSH_FORMAT_UINT32_STR,
                         a->common->remote_ip, strlen(a->common->remote_ip),
-                        SSH_FORMAT_UINT32, atol(a->common->remote_port),
+                        SSH_FORMAT_UINT32,
+                        (SshUInt32) atol(a->common->remote_port),
                         SSH_FORMAT_END);
       cp = ssh_buffer_ptr(&buffer);
       SSH_PUT_32BIT(cp, ssh_buffer_len(&buffer) - 4);
@@ -324,9 +326,19 @@ Boolean ssh_channel_ssh1_agent_process_request(void *session_placeholder,
   SshChannelTypeSsh1Agent ct;
   SshChannelSsh1AgentSession session;
 
-  SSH_DEBUG(5, ("request to start agent forwarding received"));
+  SSH_DEBUG(5, ("request to start ssh1 agent forwarding received"));
 
   session = (SshChannelSsh1AgentSession)session_placeholder;
+
+  if (!session->common->config->allow_agent_forwarding)
+    {
+      ssh_log_event(session->common->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "SSH1 agent forwarding request "
+                    "denied in configuration.");
+      return FALSE;
+    }
+
   ct = ssh_channel_ssh1_agent_ct(session->common);
   
   if (ct->agent_path != NULL)
