@@ -3,15 +3,15 @@
 sshchtcpfwd.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Code implementing TCP/IP forwarding channels for SSH2 servers and clients.
+  Code implementing TCP/IP forwarding channels for SSH2 servers and clients.
 
 */
 
@@ -23,20 +23,22 @@ Code implementing TCP/IP forwarding channels for SSH2 servers and clients.
 #include "sshconn.h"
 #include "sshuser.h"
 #include "sshcommon.h"
+#ifndef SSH_WIN_CLIENT
+#include <syslog.h>
+#ifdef NEED_SYS_SYSLOG_H
+#include <sys/syslog.h>
+#endif /* NEED_SYS_SYSLOG_H */
+#endif /* SSH_WIN_CLIENT */
 
 #ifdef SSH_CHANNEL_TCPFWD
-
 #include "sshchtcpfwd.h"
 
 #ifdef HAVE_LIBWRAP
 #include <netdb.h>
 #include <tcpd.h>
-#include <syslog.h>
-#include "sshunixfdstream.h"
-#ifdef NEED_SYS_SYSLOG_H
-#include <sys/syslog.h>
-#endif /* NEED_SYS_SYSLOG_H */
+#include "sshfdstream.h"
 #endif /* HAVE_LIBWRAP */
+#include "auths-common.h"
 
 #define SSH_DEBUG_MODULE "Ssh2ChannelTcpFwd"
 
@@ -51,6 +53,84 @@ typedef struct SshChannelTcpFwdConnectRec
   void *completion_context;
 } *SshChannelTcpFwdConnect;
 
+#ifndef SSH_WIN_CLIENT
+
+/* returns FALSE if user is allowed to forward. */
+Boolean is_tcp_forwarding_denied_for_user(SshCommon common)
+{
+  SshUser uc = NULL;
+  uid_t uid;
+  gid_t gid;
+  const char *group, *user;
+  Boolean ret = FALSE;
+  SshRegexContext rex_ctx = NULL;
+  SshConfig config = common->config;
+
+  SSH_PRECOND(common);
+  SSH_PRECOND(config);
+
+  user = common->user;
+  uc = ssh_user_initialize(user, TRUE);
+  if (!uc)
+    return TRUE;
+  
+  uid = ssh_user_uid(uc);
+  gid = ssh_user_gid(uc);
+  group = ssh_group_name(uc);
+  rex_ctx = ssh_regex_create_context();
+  
+  if (config->denied_tcp_forwarding_users)
+    if (ssh_match_user_in_list(user, uid, common->remote_host,
+                               common->remote_ip,
+                               config->denied_tcp_forwarding_users,
+                               rex_ctx))
+      {
+        ret = TRUE;
+        goto free_and_return;
+      }
+  
+  if (config->allowed_tcp_forwarding_users)
+    {
+      ret = TRUE;
+      if (ssh_match_user_in_list(user, uid, common->remote_host,
+                                 common->remote_ip,
+                                 config->allowed_tcp_forwarding_users,
+                                 rex_ctx))
+        ret = FALSE;
+
+      goto free_and_return;
+    }
+  
+  if (config->denied_tcp_forwarding_groups)
+    if (!ssh_match_string_or_number_in_list
+        (group, gid,
+         config->denied_tcp_forwarding_groups,
+         rex_ctx))
+      {
+        ret = TRUE;
+        goto free_and_return;
+      }
+  
+  if (config->allowed_tcp_forwarding_groups)
+    {
+      ret = TRUE;
+      if (!ssh_match_string_or_number_in_list
+          (group, gid,
+           config->allowed_tcp_forwarding_groups,
+           rex_ctx))
+        ret = FALSE;
+      
+      goto free_and_return;
+    }
+
+ free_and_return:
+  ssh_user_free(uc, FALSE);
+  ssh_regex_free_context(rex_ctx);
+return ret;
+}
+
+#endif /* SSH_WIN_CLIENT */
+
 /***********************************************************************
  * Glue functions for creating/destroying channel type and session
  * contexts.
@@ -88,6 +168,7 @@ void ssh_channel_ftcp_destroy(void *context)
         ssh_tcp_destroy_listener(remote_fwd->listener);
       ssh_xfree(remote_fwd->address_to_bind);
       ssh_xfree(remote_fwd->port);
+      ssh_xfree(remote_fwd->protocol);
       ssh_xfree(remote_fwd->connect_to_host);
       ssh_xfree(remote_fwd->connect_to_port);
       memset(remote_fwd, 'F', sizeof(*remote_fwd));
@@ -198,6 +279,24 @@ void ssh_channel_ftcp_open_connected(SshIpError error,
       return;
     }
 
+  if (strcasecmp(c->fwd->protocol, "tcp") != 0)
+    {
+      if (strcasecmp(c->fwd->protocol, "http") == 0)
+        {
+          ssh_warning("forwarding protocol \"http\" defaults to \"tcp\"");
+        }
+      else 
+        {
+          ssh_warning("forwarding protocol \"%s\" defaults to \"tcp\"",
+                      c->fwd->protocol);
+        }
+    }
+
+  SSH_DEBUG(0, ("remote open host=\"%s\" port=\"%s\" proto=\"%s\"",
+                c->fwd->connect_to_host,
+                c->fwd->connect_to_port,
+                c->fwd->protocol));
+
   /* Record that we now have a new channel. */
   ssh_common_new_channel(c->fwd->common);
   
@@ -324,6 +423,7 @@ void ssh_channel_dtcp_connected(SshIpError error,
   ssh_xfree(tcp);
 }
 
+#ifndef SSH_WIN_CLIENT
 /* Processes an open request for a TCP/IP forwarding to given address. */
 
 void ssh_channel_dtcp_open_request(const char *type, int channel_id,
@@ -337,7 +437,7 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
   SshDirectTcp tcp;
 
   SSH_DEBUG(5, ("direct TCP/IP channel open request"));
-  
+
   /* Parse packet data. */
   if (ssh_decode_array(data, len,
                        SSH_FORMAT_UINT32_STR, &connect_to_host, NULL,
@@ -348,7 +448,7 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
     {
       /* XXX disconnect? */
       SSH_DEBUG(0, ("bad data"));
-      (*completion)(SSH_OPEN_RESOURCE_SHORTAGE,
+      (*completion)(SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
                     NULL, FALSE, FALSE, 0, NULL, 0, NULL, NULL, NULL,
                     completion_context);
       return;
@@ -369,6 +469,31 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
       return;
     }
 
+  if (!common->config->allow_tcp_forwarding)
+    {
+      ssh_log_event(common->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "Direct TCP/IP forwarding request "
+                    "denied in configuration.");
+      (*completion)(SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
+                    NULL, FALSE, FALSE, 0, NULL, 0, NULL, NULL, NULL,
+                    completion_context);
+      return;
+    }
+
+  if (is_tcp_forwarding_denied_for_user(common))
+    {
+      /* Denied. */
+      ssh_log_event(common->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "Direct TCP/IP forwarding request "
+                    "denied for user in configuration.");
+      (*completion)(SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
+                    NULL, FALSE, FALSE, 0, NULL, 0, NULL, NULL, NULL,
+                    completion_context);
+      return;
+    }
+  
 
   /* Convert port number to string. */
   snprintf(connect_to_port, sizeof(connect_to_port), "%ld", (long) port);
@@ -394,6 +519,8 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
   ssh_xfree(connect_to_host);
 }
 
+#endif /* !SSH_WIN_CLIENT */
+
 /***********************************************************************
  * Processing an incoming connection to a remotely forwarded socket.
  ***********************************************************************/
@@ -407,7 +534,7 @@ void ssh_channel_ftcp_incoming_connection(SshIpError error, SshStream stream,
 {
   SshRemoteTcpForward fwd = (SshRemoteTcpForward)context;
   char ip[20], port[20];
-  SshBuffer buffer;
+  SshBufferStruct buffer;
 
   SSH_DEBUG(5, ("connection to forwarded TCP/IP port"));
   
@@ -438,6 +565,12 @@ void ssh_channel_ftcp_incoming_connection(SshIpError error, SshStream stream,
     
     old_handler = signal(SIGCHLD, SIG_DFL);
 
+    if (old_handler == SIG_ERR)
+      {
+        ssh_warning("ssh_channel_ftcp_incoming_connection: Could not set SIGCHLD signal handler.");
+        old_handler = SIG_IGN;
+      }
+    
     /* try to find port's name in /etc/services */
     serv = getservbyport(atoi(fwd->port), "tcp");
     if (serv == NULL)
@@ -466,7 +599,10 @@ void ssh_channel_ftcp_incoming_connection(SshIpError error, SshStream stream,
     
         return;
       }
-    signal(SIGCHLD, old_handler);
+    if (signal(SIGCHLD, old_handler) == SIG_ERR && old_handler != SIG_IGN)
+      {
+        ssh_warning("ssh_channel_ftcp_incoming_connection: Could not reset SIGCHLD signal handler.");
+      }
         
     ssh_log_event(fwd->common->config->log_facility, SSH_LOG_INFORMATIONAL,
                   "Remote fwd connect from %.500s to local port %s",
@@ -496,6 +632,8 @@ void ssh_channel_ftcp_incoming_connection(SshIpError error, SshStream stream,
   ssh_buffer_uninit(&buffer);
 }  
 
+#ifndef SSH_WIN_CLIENT
+
 /***********************************************************************
  * Processes a request to set up TCP/IP forwarding.  This is typically
  * used in the server.
@@ -531,7 +669,26 @@ Boolean ssh_channel_remote_tcp_forward_request(const char *type,
       ssh_warning("Remote TCP/IP forwarding request from server denied.");
       return FALSE;
     }
-  
+
+  if (!common->config->allow_tcp_forwarding)
+    {
+      ssh_log_event(common->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "Remote TCP/IP forwarding request "
+                    "denied in configuration.");
+      return FALSE;
+    }
+
+  if (is_tcp_forwarding_denied_for_user(common))
+    {
+      /* Denied. */
+      ssh_log_event(common->config->log_facility,
+                    SSH_LOG_INFORMATIONAL,
+                    "Remote TCP/IP forwarding request "
+                    "denied for user in configuration.");
+      return FALSE;
+    }
+
   /* Parse the request. */
   if (ssh_decode_array(data, len,
                        SSH_FORMAT_UINT32_STR, &address_to_bind, NULL,
@@ -611,6 +768,7 @@ Boolean ssh_channel_remote_tcp_forward_request(const char *type,
   fwd->port = ssh_xstrdup(port_string);
   fwd->connect_to_host = NULL;
   fwd->connect_to_port = NULL;
+  fwd->protocol = NULL;
 
   /* Add to list of forwardings. */
   fwd->next = ct->remote_forwards;
@@ -620,9 +778,12 @@ Boolean ssh_channel_remote_tcp_forward_request(const char *type,
                 SSH_LOG_INFORMATIONAL,
                 "Port %ld set up for remote forwarding.",
                 (unsigned long) port);
-  
   return TRUE;
 }  
+#endif /* !SSH_WIN_CLIENT */
+
+
+#ifndef SSH_WIN_CLIENT
 
 /* Processes a received request to cancel remote TCP/IP forwarding. */
 
@@ -671,6 +832,7 @@ Boolean ssh_channel_tcp_forward_cancel(const char *type,
           ssh_tcp_destroy_listener(fwd->listener);
           ssh_xfree(fwd->address_to_bind);
           ssh_xfree(fwd->port);
+          ssh_xfree(fwd->protocol);
           *fwdp = fwd->next;
           ssh_xfree(fwd);
           ssh_xfree(address_to_bind);
@@ -684,6 +846,8 @@ Boolean ssh_channel_tcp_forward_cancel(const char *type,
   return FALSE;
 }
 
+#endif /* !SSH_WIN_CLIENT */
+
 /***********************************************************************
  * Sending a request to start remote TCP/IP forwarding.
  ***********************************************************************/
@@ -696,12 +860,13 @@ void ssh_channel_start_remote_tcp_forward(SshCommon common,
                                           const char *port,
                                           const char *connect_to_host,
                                           const char *connect_to_port,
+                                          const char *protocol,
                                           void (*completion)(Boolean ok,
                                                              void *context),
                                           void *context)
 {
   SshRemoteTcpForward fwd;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   SshChannelTypeTcpForward ct;
 
   SSH_DEBUG(5, ("requesting remote forwarding for port %s", port));
@@ -715,7 +880,7 @@ void ssh_channel_start_remote_tcp_forward(SshCommon common,
   fwd->port = ssh_xstrdup(port);
   fwd->connect_to_host = ssh_xstrdup(connect_to_host);
   fwd->connect_to_port = ssh_xstrdup(connect_to_port);
-
+  fwd->protocol = ssh_xstrdup(protocol ? protocol : "tcp");
   /* Add it to the list of remote forwardings. */
   fwd->next = ct->remote_forwards;
   ct->remote_forwards = fwd;
@@ -740,7 +905,7 @@ void ssh_channel_start_remote_tcp_forward(SshCommon common,
 /* This function is called whenever a locally forwarded TCP/IP port is
    connected. */
 
-void ssh_channel_dtcp_incoming_connection(SshStreamNotification op,
+void ssh_channel_dtcp_incoming_connection(SshIpError op,
                                           SshStream stream, void *context)
 {
   SshLocalTcpForward fwd = (SshLocalTcpForward)context;
@@ -765,6 +930,12 @@ void ssh_channel_dtcp_incoming_connection(SshStreamNotification op,
     
     old_handler = signal(SIGCHLD, SIG_DFL);
     
+    if (old_handler == SIG_ERR)
+      {
+        ssh_warning("ssh_channel_dtcp_incoming_connection: Could not set SIGCHLD signal handler.");
+        old_handler = SIG_IGN;
+      }
+
     /* try to find port's name in /etc/services */
     serv = getservbyport(atoi(fwd->port), "tcp");
     if (serv == NULL)
@@ -797,13 +968,35 @@ void ssh_channel_dtcp_incoming_connection(SshStreamNotification op,
 
         return;
       }
-    signal(SIGCHLD, old_handler);
 
+    if (signal(SIGCHLD, old_handler) == SIG_ERR && old_handler != SIG_IGN)
+      {
+        ssh_warning("ssh_channel_dtcp_incoming_connection: Could not reset SIGCHLD signal handler.");
+      }
+    
     ssh_log_event(fwd->common->config->log_facility, SSH_LOG_INFORMATIONAL,
                   "direct fwd connect from %.500s to local port %s",
                   eval_client(&req), fwdportname);
   }
 #endif /* HAVE_LIBWRAP */
+  
+  if (strcasecmp(fwd->protocol, "tcp") != 0)
+    {
+      if (strcasecmp(fwd->protocol, "http") == 0)
+        {
+          ssh_warning("forwarding protocol \"http\" defaults to \"tcp\"");
+        }
+      else 
+        {
+          ssh_warning("forwarding protocol \"%s\" defaults to \"tcp\"",
+                      fwd->protocol);
+        }
+    }
+
+  SSH_DEBUG(0, ("local open host=\"%s\" port=\"%s\" proto=\"%s\"",
+                fwd->connect_to_host,
+                fwd->connect_to_port,
+                fwd->protocol));
 
   /* Send a request to open a channel and connect it to the given port. */
   ssh_channel_dtcp_open_to_remote(fwd->common, stream,
@@ -823,7 +1016,8 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
                                             const char *address_to_bind,
                                             const char *port,
                                             const char *connect_to_host,
-                                            const char *connect_to_port)
+                                            const char *connect_to_port,
+                                            const char *protocol)
 {
   SshLocalTcpForward fwd;
   SshChannelTypeTcpDirect ct;
@@ -834,6 +1028,7 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
                 port, connect_to_host, connect_to_port));
 
   portnumber = atol(port);
+#ifndef SSH_WIN_CLIENT
   user = ssh_user_initialize(NULL, FALSE);
     /* If user is not logged in as a privileged user, don't allow
      forwarding of privileged ports. */
@@ -847,7 +1042,7 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
           return FALSE;
         }
     }
-
+#endif /* SSH_WIN_CLIENT */
   if (portnumber >= 65536)
     {
       ssh_warning("Tried to forward " \
@@ -873,6 +1068,7 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
   fwd->port = ssh_xstrdup(port);
   fwd->connect_to_host = ssh_xstrdup(connect_to_host);
   fwd->connect_to_port = ssh_xstrdup(connect_to_port);
+  fwd->protocol = ssh_xstrdup(protocol ? protocol : "tcp");
 
   fwd->next = ct->local_forwards;
   ct->local_forwards = fwd;
@@ -898,7 +1094,7 @@ void ssh_channel_dtcp_open_to_remote(SshCommon common, SshStream stream,
                                      const char *originator_ip,
                                      const char *originator_port)
 {
-  SshBuffer buffer;
+  SshBufferStruct buffer;
 
   SSH_DEBUG(5, ("opening direct TCP/IP connection to %s:%s originator %s:%s",
                 connect_to_host, connect_to_port,
