@@ -3,15 +3,15 @@
 sshchx11.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Code for X11 forwarding channels for SSH2 servers and clients.
+  Code for X11 forwarding channels for SSH2 servers and clients.
 
 */
 
@@ -24,7 +24,8 @@ Code for X11 forwarding channels for SSH2 servers and clients.
 #include "sshmsgs.h"
 #include "sshlocalstream.h"
 #include "sshcommon.h"
-#include "sshunixfdstream.h"
+#include "sshfdstream.h"
+#include "sshdsprintf.h"
 
 #ifdef SSH_CHANNEL_X11
 
@@ -52,12 +53,16 @@ Code for X11 forwarding channels for SSH2 servers and clients.
 #include <sys/utsname.h>
 #endif /* HAVE_SYS_UTSNAME_H */
 #endif /* XAUTH_PATH || HPSUX_NONSTANDARD_X11_KLUDGE */
-#ifdef HAVE_LIBWRAP
-#include <tcpd.h>
+
+#ifndef SSH_WIN_CLIENT
 #include <syslog.h>
+#endif /* SSH_WIN_CLIENT */
+
 #ifdef NEED_SYS_SYSLOG_H
 #include <sys/syslog.h>
 #endif /* NEED_SYS_SYSLOG_H */
+#ifdef HAVE_LIBWRAP
+#include <tcpd.h>
 #endif /* HAVE_LIBWRAP */
 
 #define SSH_DEBUG_MODULE "Ssh2ChannelX11"
@@ -81,7 +86,7 @@ typedef struct SshChannelX11SessionRec
 {
   /* Controlling SshCommon object. */
   SshCommon common;
-  
+
   /* X11-related data for the server. */
   Boolean forward_x11;          /* True if forwarding has been requested. */
   char *auth_protocol;          /* The received auth protocol. */
@@ -158,7 +163,7 @@ void ssh_channel_x11_session_create(SshCommon common,
 
   *session_placeholder = (void *)session;
 }
-                                        
+
 /* This function is called once whenever a session channel is destroyed.
    This should free any X11 forwarding state related to the session; however,
    this should typically not close forwarded X11 channels. */
@@ -199,25 +204,27 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
   unsigned int data_len;
   unsigned int value;
   char *new_data;
-  int i, screen_number;
+  int i;
+  SshUInt32 screen_number;
   const char *cp;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   SshChannelTypeX11 ct;
 
   /* Find the channel type context. */
   ct = ssh_channel_x11_ct(common);
-  
+
+#ifndef SSH_WIN_CLIENT
   /* If DISPLAY isn't set, no sense in requesting X11 forwarding. */
   if (getenv("DISPLAY") == NULL)
     {
       ssh_debug("DISPLAY not set; X11 forwarding disabled.");
       return;
     }
-      
+
   /* Got local authentication reasonable information.  Request forwarding
      with authentication spoofing. */
   ssh_debug("Requesting X11 forwarding with authentication spoofing.");
-      
+
   /* Extract screen number from the display. */
   cp = getenv("DISPLAY");
   if (cp)
@@ -228,6 +235,9 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
     screen_number = atoi(cp + 1);
   else
     screen_number = 0;
+#else /* SSH_WIN_CLIENT */
+    screen_number = 0;
+#endif /* SSH_WIN_CLIENT */
 
   /* Generate a fake cookie only if this is the first X11 request in this
      connection. */
@@ -235,10 +245,10 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
     {
 #ifdef XAUTH_PATH
       /* Try to get Xauthority information for the display. */
-      snprintf(line, sizeof(line), "%s list %s 2>/dev/null", 
+      snprintf(line, sizeof(line), "%s list %s 2>/dev/null",
                XAUTH_PATH, getenv("DISPLAY"));
       f = popen(line, "r");
-      if (f && fgets(line, sizeof(line), f) && 
+      if (f && fgets(line, sizeof(line), f) &&
           sscanf(line, "%*s %s %s", proto, data) == 2)
         got_data = TRUE;
       else
@@ -258,9 +268,9 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
           strcpy(proto, "MIT-MAGIC-COOKIE-1");
           for (i = 0; i < 16; i++)
             snprintf(data + 2 * i, 3, "%02x",
-                     ssh_random_get_byte(common->random_state));
+                     ssh_random_get_byte());
         }
-      
+
       /* Save protocol name. */
       ct->x11_real_proto = ssh_xstrdup(proto);
       ct->x11_real_proto_len = strlen(proto);
@@ -289,9 +299,9 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
 
       /* Generate fake cookie. */
       for (i = 0; i < data_len; i++)
-        ct->x11_fake_cookie[i] = ssh_random_get_byte(common->random_state);
+        ct->x11_fake_cookie[i] = ssh_random_get_byte();
     }
-  
+
   /* Convert the fake data into hex for transmission. */
   new_data = ssh_xmalloc(2 * ct->x11_fake_cookie_len + 1);
   for (i = 0; i < ct->x11_fake_cookie_len; i++)
@@ -306,6 +316,7 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
                     SSH_FORMAT_UINT32_STR, new_data, strlen(new_data),
                     SSH_FORMAT_UINT32, screen_number,
                     SSH_FORMAT_END);
+  /* XXX We want reply. (set callback and context to non-NULL. */
   ssh_conn_send_channel_request(common->conn, session_channel_id,
                                 "x11-req", ssh_buffer_ptr(&buffer),
                                 ssh_buffer_len(&buffer), NULL, NULL);
@@ -331,17 +342,20 @@ void ssh_channel_x11_connection_destroy(void *context)
   ssh_common_destroy_channel(common);
 }
 
+#ifndef SSH_WIN_CLIENT
+
 /* This function is called whenever a new X11 connection is received. */
 
 void ssh_channel_x11_connection(SshIpError error, SshStream stream,
                                 void *context)
 {
   SshChannelX11Session session = (SshChannelX11Session)context;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   char ip[50], port[50], msg[256];
-
-  SSH_DEBUG(6, ("X11 connection from an X11 client"));
+  SshUInt32 port_number = 0L;
   
+  SSH_DEBUG(6, ("X11 connection from an X11 client"));
+
   /* We should only receive new connections. */
   if (error != SSH_IP_NEW_CONNECTION)
     ssh_fatal("ssh_channel_x11_connection: error %d", (int)error);
@@ -352,13 +366,15 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
       ssh_tcp_destroy_listener(session->x11_listener);
       session->x11_listener = NULL;
     }
-  
+
   /* Get remote IP address and port. */
   if (!ssh_tcp_get_remote_address(stream, ip, sizeof(ip)))
     strcpy(ip, "UNKNOWN");
   if (!ssh_tcp_get_remote_port(stream, port, sizeof(port)))
     strcpy(port, "UNKNOWN");
 
+  port_number = strtol(port, NULL, 0);
+  
   /* Format the initiator string for describing where the connection
      came from. */
   snprintf(msg, sizeof(msg), "X11 connection from %s:%s", ip, port);
@@ -371,9 +387,15 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
   {
     struct request_info req;
     void *old_handler;
-    
+
     old_handler = signal(SIGCHLD, SIG_DFL);
-                
+
+    if (old_handler == SIG_ERR)
+      {
+        ssh_warning("ssh_channel_x11_connection: Could not set SIGCHLD signal handler.");
+        old_handler = SIG_IGN;
+      }
+
     /* Fill req struct with port name and fd number */
     request_init(&req, RQ_DAEMON, "sshdfwd-X11",
                  RQ_FILE, ssh_stream_fd_get_readfd(stream), NULL);
@@ -393,20 +415,43 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
         return;
       }
 
-    signal(SIGCHLD, old_handler);
-
-    ssh_log_event(session->common->config->log_facility, SSH_LOG_INFORMATIONAL,
-                  "fwd X11 connect from %.500s",
-                  eval_client(&req));
+    if (signal(SIGCHLD, old_handler) == SIG_ERR && old_handler != SIG_IGN)
+      {
+        ssh_warning("ssh_channel_x11_connection: Could not reset SIGCHLD signal handler.");
+      }
   }
 #endif /* HAVE_LIBWRAP */
-  /* XXX Logging*/
+  /* Logging */
+  {
+    char *remote_host_name;
+    remote_host_name = ssh_tcp_get_host_by_addr_sync(ip);
+    if (!remote_host_name)
+      remote_host_name = "UNKNOWN";
+    
+    ssh_log_event(session->common->config->log_facility, SSH_LOG_INFORMATIONAL,
+                  "%s (%s)", msg,
+                  remote_host_name);
+    if (session->common->client)
+      SSH_TRACE(0, ("%s (%s)", msg, remote_host_name));
+        
+  }
+  
+  /* Set socket to nodelay mode if configuration suggests this. */
+  ssh_socket_set_nodelay(stream, session->common->config->no_delay);
 
   /* Send a channel open request to the other side. */
   ssh_buffer_init(&buffer);
-  ssh_encode_buffer(&buffer,
-                    SSH_FORMAT_UINT32_STR, msg, strlen(msg),
-                    SSH_FORMAT_END);
+
+  if (!(*session->common->compat_flags->x11_channel_open_draft_incompatibility))
+    ssh_encode_buffer(&buffer,
+                      SSH_FORMAT_UINT32_STR, ip, strlen(ip),
+                      SSH_FORMAT_UINT32, port_number,
+                      SSH_FORMAT_END);
+  else
+    ssh_encode_buffer(&buffer,
+                      SSH_FORMAT_UINT32_STR, msg, strlen(msg),
+                      SSH_FORMAT_END);
+  
   ssh_conn_send_channel_open(session->common->conn, "x11", stream, TRUE, TRUE,
                              X11_WINDOW_SIZE, X11_PACKET_SIZE,
                              ssh_buffer_ptr(&buffer), ssh_buffer_len(&buffer),
@@ -416,6 +461,7 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
   ssh_buffer_uninit(&buffer);
 }
 
+
 /***********************************************************************
  * Processing a request to start X11 forwarding at server end.
  ***********************************************************************/
@@ -432,16 +478,29 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
   struct stat st;
   SshChannelX11Session session;
   SshChannelTypeX11 ct;
-  
+  SshUInt32 temp;
+
   SSH_DEBUG(6, ("request_x11"));
 
   session = (SshChannelX11Session)session_placeholder;
   ct = ssh_channel_x11_ct(session->common);
 
+  if (session->common->client)
+    {
+      ssh_warning("X11 forwarding request from server denied.");
+      return FALSE;
+    }
+
+  if (!session->common->config->x11_forwarding)
+    {
+      SSH_DEBUG(1, ("forwarding denied in configuration"));
+      return FALSE;      
+    }
+  
   /* If already forwarding X11, fail. */
   if (session->forward_x11)
     {
-      SSH_DEBUG(1, ("request_x11: already forwarding"));
+      SSH_DEBUG(1, ("already forwarding"));
       return FALSE;
     }
 
@@ -452,22 +511,27 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
                          &session->auth_protocol, NULL,
                        SSH_FORMAT_UINT32_STR,
                          &session->auth_cookie, NULL,
-                       SSH_FORMAT_UINT32, &session->screen_number,
+                       SSH_FORMAT_UINT32, &temp,
                        SSH_FORMAT_END) != len)
     {
-      SSH_DEBUG(0, ("request_x11: bad data"));
+      SSH_DEBUG(0, ("bad data"));
       return FALSE;
     }
 
+  session->screen_number = temp;
+
   /* Check whether we have xauth installed on this machine (in case
      the binary was moved from elsewhere). */
+#ifdef XAUTH_PATH
   if (stat(XAUTH_PATH, &st) < 0)
+#else
+  if (TRUE)
+#endif
     {
-      SSH_DEBUG(1, ("request_x11: no X11 installed"));
+      SSH_DEBUG(1, ("no X11 installed"));
       return FALSE;
     }
 
-
   /* Open the first available display, starting at number 10. */
   for (display_number = 10; display_number < X11_MAX_DISPLAYS;
        display_number++)
@@ -523,7 +587,7 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
   snprintf(buf, sizeof(buf), "%s:%d.%lu",
            hostname, display_number, session->screen_number);
 #endif /* HPSUX_NONSTANDARD_X11_KLUDGE */
-  
+
   /* Save the display name. */
   session->display = ssh_xstrdup(buf);
 
@@ -531,6 +595,8 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
   session->forward_x11 = TRUE;
   return TRUE;
 }
+#endif /* !SSH_WIN_CLIENT */
+
 
 /***********************************************************************
  * Functions that are used at the SSH client end for connecting to the
@@ -544,15 +610,20 @@ typedef struct SshX11ConnectionRec
   int channel_id;
   SshConnOpenCompletionProc completion;
   void *context;
-  char *originator;
+  char *originator_ip;
+  SshUInt32 originator_port;
+  char *originator_string;
 } *SshX11Connection;
 
 /* Frees the X11 stream context. */
 
 void ssh_channel_x11_x_destroy(SshX11Connection x)
 {
-  ssh_xfree(x->originator);
+  ssh_xfree(x->originator_ip);
+  ssh_xfree(x->originator_string);
+
   memset(x, 'F', sizeof(*x));
+
   ssh_xfree(x);
 }
 
@@ -561,7 +632,7 @@ void ssh_channel_x11_x_destroy(SshX11Connection x)
    been entirely received, at which point this will replace the authentication
    cookie in the packet by the real cookie, and shortcircuit the filter. */
 
-int ssh_channel_x11_filter(SshBuffer *data,
+int ssh_channel_x11_filter(SshBuffer data,
                            size_t offset,
                            Boolean eof_received,
                            void *context)
@@ -572,7 +643,7 @@ int ssh_channel_x11_filter(SshBuffer *data,
 
   SSH_DEBUG(6, ("filter: data len %d, offset %d",
                 (int)ssh_buffer_len(data), (int)offset));
-  
+
   /* Compute the start and length of data that we have in the filter. */
   ucp = ssh_buffer_ptr(data);
   ucp += offset;
@@ -613,7 +684,7 @@ int ssh_channel_x11_filter(SshBuffer *data,
   /* Sanity check: it must not be larger than we are willing to receive. */
   if (desired_len > X11_MAX_AUTH_PACKET_SIZE)
     return SSH_FILTER_DISCONNECT;
-  
+
   /* Check if the whole packet is in buffer. */
   if (received_len < desired_len)
     {
@@ -646,7 +717,7 @@ int ssh_channel_x11_filter(SshBuffer *data,
 
   /* Received authentication protocol and data match our fake data.
      Substitute the fake data with real data. */
-  assert(x->ct->x11_fake_cookie_len == x->ct->x11_real_cookie_len);
+  SSH_ASSERT(x->ct->x11_fake_cookie_len == x->ct->x11_real_cookie_len);
   memcpy(ucp + 12 + ((proto_len + 3) & ~3),
          x->ct->x11_real_cookie, x->ct->x11_real_cookie_len);
 
@@ -683,13 +754,13 @@ void ssh_channel_x11_open_connected(SshStream stream,
       ssh_channel_x11_x_destroy(x);
       return;
     }
-  
-  
-  SSH_DEBUG(6, ("open_connected: %s", x->originator));
+
+
+  SSH_DEBUG(6, ("open_connected: %s", x->originator_string));
 
   /* Increment the number of channels. */
   ssh_common_new_channel(x->common);
-  
+
   /* Wrap the stream to the X server into a filter stream that is used
      to replace the fake authentication cookie with the real one. */
   stream = ssh_stream_filter_create(stream, X11_MAX_AUTH_PACKET_SIZE,
@@ -721,7 +792,7 @@ void ssh_channel_x11_open_connected_tcp(SshIpError error,
       ssh_channel_x11_x_destroy(x);
       return;
     }
-  
+
   /* Complete processing of the channel open request. */
   ssh_channel_x11_open_connected(stream, context);
 }
@@ -743,29 +814,48 @@ void ssh_channel_x11_open(const char *type, int channel_id,
 {
   SshCommon common = (SshCommon)context;
   SshX11Connection x;
-  char *originator;
+  char *originator_ip;
+  SshUInt32 originator_port = 0L;
   int display_number;
   const char *display;
   char buf[256], port[20], *cp;
   SshChannelTypeX11 ct;
-  
+
   SSH_DEBUG(6, ("X11 channel open received"));
 
   ct = (SshChannelTypeX11)ssh_common_get_channel_type_context(common, "x11");
-  
-  if (ssh_decode_array(data, len,
-                       SSH_FORMAT_UINT32_STR, &originator, NULL,
-                       SSH_FORMAT_END) != len)
-    {
-      SSH_DEBUG(0, ("x11_open: bad data"));
-      goto fail;
-    }
 
+  if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
+    {
+      if (ssh_decode_array(data, len,
+                           SSH_FORMAT_UINT32_STR, &originator_ip, NULL,
+                           SSH_FORMAT_UINT32, &originator_port,
+                           SSH_FORMAT_END) != len)
+        {
+          SSH_DEBUG(0, ("x11_open: bad data"));
+          goto fail;
+        }
+    }
+  else
+    {    
+      /* Remote end has x11 channel open draft inconsistence bug. */
+      if (ssh_decode_array(data, len,
+                           SSH_FORMAT_UINT32_STR, &originator_ip, NULL,
+                           SSH_FORMAT_END) != len)
+        {
+          SSH_DEBUG(0, ("x11_open: bad data"));
+          goto fail;
+        }
+    }
+  
   if (!ct->x11_requested)
     {
       ssh_warning("Received X11 connection without requesting forwarding!");
-      ssh_warning("Originator string: '%s'", originator);
-      ssh_xfree(originator);
+      if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
+        ssh_warning("Originator IP: '%s', port: '%s'", originator_ip, originator_port);
+      else
+        ssh_warning("Originator message: '%s'", originator_ip);
+      ssh_xfree(originator_ip);
       goto fail;
     }
 
@@ -775,13 +865,19 @@ void ssh_channel_x11_open(const char *type, int channel_id,
   x->channel_id = channel_id;
   x->completion = completion;
   x->context = completion_context;
-  x->originator = originator;
-
+  x->originator_ip = originator_ip;
+  x->originator_port = originator_port;
+  if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
+    ssh_dsprintf(&x->originator_string, "%s:%ld", originator_ip, originator_port);
+  else
+    x->originator_string = ssh_xstrdup(originator_ip);
+  
   /* Try to open a socket for the local X server. */
+#ifndef SSH_WIN_CLIENT
   display = getenv("DISPLAY");
   if (!display)
     ssh_fatal("ssh_channel_x11_open: DISPLAY not set.");
-  
+
   /* Now we decode the value of the DISPLAY variable and make a connection
      to the real X server. */
 
@@ -824,7 +920,7 @@ void ssh_channel_x11_open(const char *type, int channel_id,
               {
                 struct utsname utsbuf;
                 /* HPSUX stores unix-domain sockets in
-                   /tmp/.X11-unix/`hostname`0 
+                   /tmp/.X11-unix/`hostname`0
                    instead of the normal /tmp/.X11-unix/X0. */
                 if (uname(&utsbuf) < 0)
                   ssh_fatal("uname: %.100s", strerror(errno));
@@ -841,7 +937,10 @@ void ssh_channel_x11_open(const char *type, int channel_id,
       ssh_local_connect(buf, ssh_channel_x11_open_connected, (void *)x);
       return;
     }
-  
+#else /* SSH_WIN_CLIENT */
+  display = "localhost:0";
+#endif /* SSH_WIN_CLIENT */
+
   /* Connect to an inet socket.  The DISPLAY value is supposedly
      hostname:d[.s], where hostname may also be numeric IP address. */
   strncpy(buf, display, sizeof(buf));
@@ -871,10 +970,12 @@ void ssh_channel_x11_open(const char *type, int channel_id,
   snprintf(port, sizeof(port), "%d", 6000 + display_number);
 
   ssh_tcp_connect_with_socks(buf, port, NULL, 1,
-                             ssh_channel_x11_open_connected_tcp, 
+                             ssh_channel_x11_open_connected_tcp,
                              (void *)x);
 }
 
+#ifndef SSH_WIN_CLIENT
+
 /* Returns the value of DISPLAY in the server. */
 
 const char *ssh_channel_x11_get_display(void *session_placeholder)
@@ -901,5 +1002,7 @@ const char *ssh_channel_x11_get_auth_cookie(void *session_placeholder)
 
   return session->auth_cookie;
 }
+#endif /* !SSH_WIN_CLIENT */
+
 
 #endif /* SSH_CHANNEL_X11 */
