@@ -2,12 +2,12 @@
   sshcommon.h
 
   Authors:
-        Tatu Ylnen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylnen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Interface to the SSH2 channel protocols.  This interface is shared
@@ -23,24 +23,40 @@
 #include "sshconfig.h"
 #include "sshcrypt.h"
 #include "sshconn.h"
+#include "sshtrans.h"
 
-/* XXX temporarily defined here. */
-#define SSH_CHANNEL_SESSION
-#ifndef DISABLE_PORT_FORWARDING
-#  define SSH_CHANNEL_AGENT
-#  define SSH_CHANNEL_SSH1_AGENT
+
+#ifndef SSH_WIN_CLIENT
+#  ifndef DISABLE_PORT_FORWARDING
+#    define SSH_CHANNEL_AGENT
+#    define SSH_CHANNEL_SSH1_AGENT
+#    define SSH_CHANNEL_TCPFWD
+#  endif /* DISABLE_PORT_FORWARDING */
+#  if !defined (X_DISPLAY_MISSING) && defined (XAUTH_PATH)
+#    ifndef DISABLE_X11_FORWARDING
+#      define SSH_CHANNEL_X11
+#    endif /* DISABLE_X11_FORWARDING */
+#  endif /* X_DISPLAY_MISSING */
+#else /* !SSH_WIN_CLIENT */
 #  define SSH_CHANNEL_TCPFWD
-#endif /* DISABLE_PORT_FORWARDING */
-#if !defined (X_DISPLAY_MISSING) && defined (XAUTH_PATH)
-#  ifndef DISABLE_X11_FORWARDING
-#    define SSH_CHANNEL_X11
-#  endif /* DISABLE_X11_FORWARDING */
-#endif /* X_DISPLAY_MISSING */
+#endif /* !SSH_WIN_CLIENT */
+
+#define SSH_CHANNEL_SESSION
 
 /* Data type for representing the common protocol object for both server and
    client. */
 typedef struct SshCommonRec *SshCommon;
 
+/* A function of this type will be called to notify the application
+   that the user has been authenticated (or, the authentication
+   process has failed, in which case `successful' will be FALSE). */
+typedef void (*SshCommonAuthenticationNotify)(const char *user,
+                                              Boolean successful,
+                                              void *context);
+/* A function of this type will be called to notify the application
+   when a SshCommon object is being destroyed. */
+typedef void (*SshCommonDestroyNotify)(void *context);
+
 struct SshCommonRec
 {
   /* TRUE if we are a client, FALSE if we are a server. */
@@ -62,17 +78,17 @@ struct SshCommonRec
   /* Function to call when a debug message is received, or NULL. */
   SshConnDebugProc debug;
 
-  /* Function to call when the user has been authenticated.  This may be
-     NULL. */
-  void (*authenticated_notify)(const char *user, void *context);
+  /* Function to call when the user has been authenticated (or, the
+     authentication process has failed).  This may be NULL. */
+  SshCommonAuthenticationNotify authentication_notify;
 
+  /* Function to call when common object is being destroyed (may be
+     NULL) */
+  SshCommonDestroyNotify destroy_notify;
+  
   /* Context argument to ``disconnect'' and ``debug'' callbacks. */
   void *context;
 
-  /* An initialized random state.  This is not automatically freed on
-     destruction. */
-  SshRandomState random_state;
-
   /* Name of the server host in client (NULL in server). */
   char *server_host_name;
 
@@ -94,7 +110,11 @@ struct SshCommonRec
   /* Authentication protocol object, needed here because of
      ssh_common_finalize */
   SshStream auth;
-  
+
+  /* Contains the application level compatibility flags needed to
+     interoperate with other/older versions. */
+  SshTransportCompat compat_flags;
+
   /* Local ip address and port. */
   char *local_ip;
   char *local_port;
@@ -107,6 +127,9 @@ struct SshCommonRec
   /* Authenticated host name, or empty if none. */
   char *authenticated_client_host;
 
+  /* TRUE if authentication has been successful, FALSE if not. */
+  Boolean authenticated;
+  
   /* An array of contexts returned by the type create function for each
      channel type. */
   void **type_contexts;
@@ -125,33 +148,39 @@ struct SshCommonRec
   Boolean being_destroyed;
 };
 
-/* A function of this type will be called to notify the application that
-   the user has been authenticated. */
-typedef void (*SshCommonAuthenticatedNotify)(const char *user, void *context);
-
 /* Creates the common processing object for the SSH server/client connection.
    This also creates the connection protocol object.
      `connection'   the connection to the other side
      `auth'         authentication protocol object
      `client'       TRUE if we are a client, FALSE if a server
      `config'       configuration data
-     `random_state' initialized random state
      `server_host_name' name of server host, or NULL in server
-     `disconnect'    function to call on disconnect
-     `debug'         function to call on debug message (may be NULL)
-     `authenticated_notify' function to call when authenticated (may be NULL)
-     `context'       context to pass to ``destroy''
+     `disconnect'   function to call on disconnect
+     `debug'        function to call on debug message (may be NULL)
+     `authentication_notify'
+                    function to call when result of
+                    authentication is found out (may be NULL)
+     `destroy_notify'
+                    function to call when common object is being
+                    destroyed (may be NULL).
+     `compat_flags' compatibility flags, stored to the common
+                    struct. Should be a ssh_xmallocated object, which
+                    is freed by ssh_common_destroy. Can also be NULL,
+                    whence default values are used.
+     `context'      context to pass to ``disconnect'', ``debug''
+                    and ``authentication_notify''
    The object should be destroyed from the ``disconnect'' callback or from
    a ``close_notify'' callback (see below).  */
 SshCommon ssh_common_wrap(SshStream connection,
                           SshStream auth,
                           Boolean client,
                           SshConfig config,
-                          SshRandomState random_state,
                           const char *server_host_name,
                           SshConnDisconnectProc disconnect,
                           SshConnDebugProc debug,
-                          SshCommonAuthenticatedNotify authenticated_notify,
+                          SshCommonAuthenticationNotify authentication_notify,
+                          SshCommonDestroyNotify destroy_notify,
+                          SshTransportCompat compat_flags,
                           void *context);
 
 /* Destroys the common protocol object.  This will not call the disconnect
