@@ -3,72 +3,31 @@
 sshconfig.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Processing configuration data in SSH (both client and server).
+  Processing configuration data in SSH (both client and server).
 
 */
 
 #include "ssh2includes.h"
 #include "sshconfig.h"
 #include "sshuser.h"
-#include "sshuserfile.h"
 #include "sshuserfiles.h"
 #include "sshcipherlist.h"
 #include "namelist.h"
 #include "sshdllist.h"
+#include "sshappcommon.h"
+#include "sshglob.h"
+#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "SshConfig"
 
-/* separate (commandline)options from their parameters */
-void ssh_split_arguments(int argc, char **argv, int *dest_ac, char ***dest_av)
-{
-  int temp_ac = 0, i;
-  char **temp_av;
-  
-  int alloc = argc + 1;
-
-  temp_av = ssh_xcalloc(alloc, sizeof(char*));
-  
-  /* count possible options and parameters */
-  for (i = 0; i < argc ; i++)
-    {
-      if ( alloc < temp_ac + 3)
-        {
-          alloc = temp_ac + 3;
-          temp_av = ssh_xrealloc(temp_av, alloc*sizeof(char*));
-        }
-      
-      if (argv[i][0] == '-' || argv[i][0] == '+')
-        {
-          if(argv[i][1] && argv[i][2])
-            {
-              temp_av[temp_ac] = ssh_xstrdup(argv[i]);
-              temp_av[temp_ac][2] = '\0';
-              temp_av[++temp_ac] = ssh_xstrdup(argv[i] + 2);
-            }
-          else
-            {
-              temp_av[temp_ac] = ssh_xstrdup(argv[i]);
-            }
-        }
-      else
-        {
-          temp_av[temp_ac] = ssh_xstrdup(argv[i]);        
-        }
-      temp_ac++;
-    }
-  temp_av[ temp_ac ] = NULL;
-  (*dest_ac) = temp_ac;
-  (*dest_av) = temp_av;
-}
-
 /* Free the "vars" and "vals" arrays */
 
 void ssh_free_varsvals(int n, char **vars, char **vals)
@@ -84,7 +43,6 @@ void ssh_free_varsvals(int n, char **vars, char **vals)
   ssh_xfree(vals); 
 }
 
-
 /* Parses a given comma-separated list to tokens, which are stored in
    a SshDlList. The list is allocated and returned by this
    function. On error, returns NULL. */
@@ -102,7 +60,8 @@ SshDlList ssh_config_parse_list(char *string,
 
   rest = string;
 
-  while (strlen(rest) != 0 && (current = ssh_name_list_get_name(rest)) != NULL)
+  while (strlen(rest) != 0 &&
+         (current = ssh_app_param_list_get_next(rest)) != NULL)
     {
       char *temp;
       int i, j;
@@ -123,21 +82,23 @@ SshDlList ssh_config_parse_list(char *string,
       
       temp[j] = '\0';
       ssh_xfree(current);
-      current = temp;
-
+      current = ssh_glob_strip_escapes(temp);
+      ssh_xfree(temp);
+      
       /* If validity function is given, invoke it to check the current
          parameter.*/
       if (function)
         if ((*function)(current, context))
           {
             ssh_xfree(current);
-            continue;
+            ssh_app_free_list(list, NULL);
+            return NULL;
           }
       
       if (ssh_dllist_add_item(list, (void *)current, SSH_DLLIST_END) !=
           SSH_DLLIST_OK)
         {
-          SSH_DEBUG(0, ("list operation gave error."));
+          SSH_TRACE(0, ("list operation gave error."));
           ssh_dllist_rewind(list);
           while (ssh_dllist_is_empty(list))
             {
@@ -152,6 +113,12 @@ SshDlList ssh_config_parse_list(char *string,
         }      
     }
 
+  if (ssh_dllist_length(list) < 1)
+    {
+      ssh_dllist_free(list);
+      return NULL;
+    }
+  
   return list;
 }
 
@@ -174,6 +141,9 @@ SshConfig ssh_config_init(Boolean client)
 
   config->forward_agent = TRUE;
   config->forward_x11 = TRUE;
+  config->x11_forwarding = TRUE;
+  config->allow_tcp_forwarding = TRUE;
+  config->allow_agent_forwarding = TRUE;
   config->password_authentication = -1;
   config->rhosts_authentication = TRUE;
   config->rhosts_pubkey_authentication = TRUE;
@@ -181,13 +151,15 @@ SshConfig ssh_config_init(Boolean client)
   config->force_ptty_allocation = FALSE;
   config->verbose_mode = FALSE;
   config->compression = FALSE;
-
+  config->compression_level = -1;
+  
   config->allowed_authentications = NULL;
   config->required_authentications = NULL;
 
   config->user_known_hosts = TRUE;
-  config->port = ssh_xstrdup("22");
+  config->port = ssh_xstrdup(SSH_DEFAULT_PORT);
   config->ciphers = NULL;
+  config->macs = NULL;
   config->user_conf_dir = ssh_xstrdup(SSH_USER_CONFIG_DIRECTORY);
   config->identity_file = ssh_xstrdup(SSH_IDENTIFICATION_FILE);
   config->authorization_file = ssh_xstrdup(SSH_AUTHORIZATION_FILE);
@@ -195,7 +167,11 @@ SshConfig ssh_config_init(Boolean client)
   config->password_prompt = ssh_xstrdup("%U's password: ");
   config->password_guesses = 3;
 
+#ifndef SSH2_MAX_CONNECTIONS
   config->max_connections = 0;
+#else /* ! SSH2_MAX_CONNECTIONS */
+  config->max_connections = SSH2_MAX_CONNECTIONS;
+#endif /* ! SSH2_MAX_CONNECTIONS */
   
   config->host_to_connect = NULL;
   config->login_as_user = NULL;
@@ -206,6 +182,12 @@ SshConfig ssh_config_init(Boolean client)
   config->denied_hosts = NULL;
   config->allowed_shosts = NULL;
   config->denied_shosts = NULL;
+
+  config->allowed_users = NULL;
+  config->denied_users = NULL;
+  config->allowed_groups = NULL;
+  config->denied_groups = NULL;
+
   config->require_reverse_mapping = FALSE;
 
   config->log_facility = SSH_LOGFACILITY_AUTH;
@@ -213,21 +195,26 @@ SshConfig ssh_config_init(Boolean client)
   config->fall_back_to_rsh = TRUE;
   config->use_rsh = TRUE;
   config->batch_mode = FALSE;
-  config->strict_host_key_checking = FALSE;
+  config->authentication_notify = FALSE;
+  config->strict_host_key_checking = SSH_STRICT_HOSTKEY_CHECKING_ASK;
   config->escape_char = ssh_xstrdup("~");
   config->go_background = FALSE;
+  config->one_shot_forwarding = FALSE;
   config->dont_read_stdin = FALSE;
   config->gateway_ports = FALSE;
   
   config->ignore_rhosts = FALSE;
   config->ignore_root_rhosts = -1;
   config->permit_root_login = SSH_ROOTLOGIN_TRUE;
-  config->permit_empty_passwords = FALSE;
+  config->permit_empty_passwords = TRUE;
+  config->try_empty_password = FALSE;
   config->strict_modes = TRUE;
   config->quiet_mode = FALSE;
   config->fascist_logging = FALSE;
   config->print_motd = TRUE;
+  config->allow_cshrc_sourcing_with_subsystems = FALSE;
   config->check_mail = TRUE;
+  config->auth_success_msg = TRUE;
   config->keep_alive = TRUE;
   config->no_delay = FALSE;
   config->listen_address = ssh_xstrdup("0.0.0.0");
@@ -235,11 +222,23 @@ SshConfig ssh_config_init(Boolean client)
   config->host_key_file = ssh_xstrdup(SSH_HOSTKEY_FILE);
   config->public_host_key_file = ssh_xstrdup(SSH_PUBLIC_HOSTKEY);
   config->forced_command = NULL;
-
+  config->chroot_users = NULL;
+  config->chroot_groups = NULL;
+  config->allowed_tcp_forwarding_users = NULL;
+  config->denied_tcp_forwarding_users = NULL;
+  config->denied_tcp_forwarding_groups = NULL;
+  config->allowed_tcp_forwarding_groups = NULL;
+  config->socks_server = NULL;
+  
   config->no_subsystems = 0;
   config->subsystems_allocated = 0;
   config->subsystems = NULL;
 
+  /* This is overridden in default config file.  However it is not
+     nice to start new service as default, if user has old config
+     file template. */
+  config->broadcasts_allowed_per_second = 0; 
+
 #ifdef SSH1_COMPATIBILITY
   config->ssh1_path = ssh_xstrdup(client ? SSH1_PATH : SSHD1_PATH);
   config->ssh1compatibility = TRUE;
@@ -248,8 +247,9 @@ SshConfig ssh_config_init(Boolean client)
   config->ssh1compatibility = FALSE;  
 #endif /* SSH1_COMPATIBILITY */
   config->ssh_agent_compat = SSH_AGENT_COMPAT_NONE;
-
+  
   config->signer_path = ssh_xstrdup(SSH_SIGNER_PATH);
+  config->default_domain = NULL;
   return config;
 }
 
@@ -261,8 +261,9 @@ void ssh_config_init_finalize(SshConfig config)
   /* Common. */
   /* "hostbased"-authentication method is not enabled by default. */
   if (!config->allowed_authentications)
-    ssh_config_parse_list((char *)SSH_AUTH_PASSWD "," SSH_AUTH_PUBKEY,
-                          NULL, NULL);
+    config->allowed_authentications =
+      ssh_config_parse_list((char *)SSH_AUTH_PUBKEY "," SSH_AUTH_PASSWD,
+                            NULL, NULL);
   
   /* Client. */
   if (config->client)
@@ -279,25 +280,6 @@ void ssh_config_init_finalize(SshConfig config)
     }  
 }
 
-/* Helper function, that destroys a list, and frees it's contents,
-   too. */
-void ssh_config_free_list(SshDlList list)
-{
-  char *to_be_deleted;
-
-  if (list == NULL)
-    return;
-  
-  ssh_dllist_rewind(list);
-  while (!ssh_dllist_is_empty(list))
-    {
-      to_be_deleted = ssh_dllist_delete_current(list);
-      ssh_xfree(to_be_deleted);
-    }
-    
-  ssh_dllist_free(list);
-}
-
 /* Helper function, that combines two lists. dst will receive nodes
    from src, and src will be freed. (so the pointer isn't valid after
    this call.) If one or both arguments are NULL, does nothing. */
@@ -309,7 +291,8 @@ void ssh_config_combine_lists(SshDlList dst, SshDlList src)
   ssh_dllist_rewind(src);
   while (!ssh_dllist_is_empty(src))
     {
-      ssh_dllist_add_item(dst, ssh_dllist_delete_current(src), SSH_DLLIST_END);
+      ssh_dllist_add_node(dst, ssh_dllist_remove_current_node(src),
+                          SSH_DLLIST_END);
     }
 
   ssh_dllist_free(src);
@@ -327,6 +310,8 @@ void ssh_config_free(SshConfig config)
 
   ssh_xfree(config->port);
   ssh_xfree(config->ciphers);
+  ssh_xfree(config->macs);
+  ssh_xfree(config->user_conf_dir);
   ssh_xfree(config->identity_file);
   ssh_xfree(config->authorization_file);
   ssh_xfree(config->escape_char);
@@ -340,13 +325,30 @@ void ssh_config_free(SshConfig config)
   ssh_xfree(config->local_forwards);
   ssh_xfree(config->remote_forwards);
   
-  ssh_config_free_list(config->allowed_hosts);
-  ssh_config_free_list(config->denied_hosts);
-  ssh_config_free_list(config->allowed_shosts);
-  ssh_config_free_list(config->denied_shosts);
+  ssh_app_free_list(config->allowed_hosts, NULL);
+  ssh_app_free_list(config->denied_hosts, NULL);
+  ssh_app_free_list(config->allowed_shosts, NULL);
+  ssh_app_free_list(config->denied_shosts, NULL);
+
+  ssh_app_free_list(config->allowed_users , NULL);
+  ssh_app_free_list(config->denied_users , NULL);
+  ssh_app_free_list(config->allowed_groups , NULL);
+  ssh_app_free_list(config->denied_groups , NULL);
+
+  ssh_app_free_list(config->allowed_authentications, NULL);
+  ssh_app_free_list(config->required_authentications, NULL);
   
   ssh_xfree(config->forced_command);
+  ssh_app_free_list(config->chroot_users, NULL);
+  ssh_app_free_list(config->chroot_groups, NULL);
 
+  ssh_app_free_list(config->allowed_tcp_forwarding_users, NULL);
+  ssh_app_free_list(config->denied_tcp_forwarding_users, NULL);
+  ssh_app_free_list(config->denied_tcp_forwarding_groups, NULL);
+  ssh_app_free_list(config->allowed_tcp_forwarding_groups, NULL);
+
+  ssh_xfree(config->socks_server);
+  
   /* Free subsystem-strings */  
   if (config->no_subsystems > 0 && config->subsystems != NULL)
     for (i = 0; i < config->no_subsystems; i++)
@@ -362,6 +364,7 @@ void ssh_config_free(SshConfig config)
     }
 
   ssh_xfree(config->signer_path);
+  ssh_xfree(config->default_domain);
   memset(config, 0, sizeof(*config));
   ssh_xfree(config);
 }
@@ -448,35 +451,21 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   /* These configuration parameters are common for both client and 
      server */
 
-  if (strcmp(var, "forwardagent") == 0)
-    {
-      config->forward_agent = bool;
-      return FALSE;
-    }
-
-  if (strcmp(var, "forwardx11") == 0)
-    {
-      config->forward_x11 = bool;
-      return FALSE;
-    }
-
   if (strcmp(var, "allowedauthentications") == 0)
     {
       if (config->password_authentication != -1 &&
-          (!config->client /*|| config->verbose_mode XXX not yet
-                             implemented in client*/))
+          (!config->client))
         ssh_warning("Defining AllowedAuthentications. Parameter "
                     "PasswordAuthentication (already defined) will be "
                     "ignored.");
       
       if (config->pubkey_authentication != -1 &&
-          (!config->client /*|| config->verbose_mode XXX not yet
-                             implemented in client*/))
+          (!config->client))
         ssh_warning("Defining AllowedAuthentications. Parameter "
                     "PubkeyAuthentication (already defined) will be "
                     "ignored.");
         
-      ssh_config_free_list(config->allowed_authentications);
+      ssh_app_free_list(config->allowed_authentications, NULL);
 
       if ((config->allowed_authentications =
            ssh_config_parse_list(val, auth_param_validity, NULL))
@@ -489,24 +478,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
         {    
           return FALSE;
         }      
-    }
-  
-  if (strcmp(var, "requiredauthentications") == 0)
-    {
-      ssh_config_free_list(config->required_authentications);
-
-      if ((config->required_authentications =
-           ssh_config_parse_list(val, auth_param_validity, NULL))
-          == NULL)
-        {
-          ssh_warning("Parsing of value for RequiredAuthentications failed.");
-          return TRUE;
-        }
-      else
-        {    
-          return FALSE;
-        }      
-    }
+    }  
 
   if (strcmp(var, "passwordauthentication") == 0)
     {
@@ -575,25 +547,21 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       ssh_xfree(config->ciphers);
       if (strcasecmp(val, "any") == 0)
         {
-          int x;
           char *hlp1, *hlp2;
           
           hlp1 = ssh_cipher_get_supported_native();
           config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
           hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
           ssh_xfree(config->ciphers);
-          x = strlen(hlp1) + strlen(hlp2) + 2;
-          config->ciphers = ssh_xmalloc(x);
-          snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
+          ssh_dsprintf(&config->ciphers, "%s,%s", hlp2, hlp1);
           ssh_xfree(hlp1);
           ssh_xfree(hlp2);
-          hlp1 = ssh_cipher_list_canonialize(config->ciphers);
+          hlp1 = ssh_cipher_list_canonicalize(config->ciphers);
           ssh_xfree(config->ciphers);
           config->ciphers = hlp1;
         }
       else if (strcasecmp(val, "anycipher") == 0)
         {
-          int x;
           char *hlp1, *hlp2;
           
           hlp2 = ssh_cipher_get_supported_native();
@@ -602,12 +570,10 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
           hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
           ssh_xfree(config->ciphers);
-          x = strlen(hlp1) + strlen(hlp2) + 2;
-          config->ciphers = ssh_xmalloc(x);
-          snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
+          ssh_dsprintf(&config->ciphers, "%s,%s", hlp2, hlp1);
           ssh_xfree(hlp1);
           ssh_xfree(hlp2);
-          hlp1 = ssh_cipher_list_canonialize(config->ciphers);
+          hlp1 = ssh_cipher_list_canonicalize(config->ciphers);
           ssh_xfree(config->ciphers);
           config->ciphers = hlp1;
         }
@@ -628,12 +594,71 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
         }
       else
         {
-          config->ciphers = ssh_cipher_list_canonialize(val);
+          config->ciphers = ssh_cipher_list_canonicalize(val);
         }
       SSH_DEBUG(3, ("Final cipherlist \"%s\"", config->ciphers));
       return FALSE;
     }
   
+  if (strcmp(var, "macs") == 0)
+    {
+      SSH_DEBUG(3, ("Got config maclist \"%s\"", val));
+      ssh_xfree(config->macs);
+      if (strcasecmp(val, "any") == 0)
+        {
+          char *hlp1, *hlp2;
+          
+          hlp1 = ssh_mac_get_supported();
+          config->macs = ssh_name_list_intersection(SSH_STD_MACS, hlp1);
+          hlp2 = ssh_cipher_list_exclude(config->macs, "none");
+          ssh_xfree(config->macs);
+          ssh_dsprintf(&config->macs, "%s,%s", hlp2, hlp1);
+          ssh_xfree(hlp1);
+          ssh_xfree(hlp2);
+          hlp1 = ssh_hash_list_canonicalize(config->macs);
+          ssh_xfree(config->macs);
+          config->macs = hlp1;
+        }
+      else if (strcasecmp(val, "anymac") == 0)
+        {
+          char *hlp1, *hlp2;
+          
+          hlp2 = ssh_mac_get_supported();
+          hlp1 = ssh_cipher_list_exclude(hlp2, "none");
+          ssh_xfree(hlp2);
+          config->macs = ssh_name_list_intersection(SSH_STD_MACS, hlp1);
+          hlp2 = ssh_cipher_list_exclude(config->macs, "none");
+          ssh_xfree(config->macs);
+          ssh_dsprintf(&config->macs, "%s,%s", hlp2, hlp1);
+          ssh_xfree(hlp1);
+          ssh_xfree(hlp2);
+          hlp1 = ssh_hash_list_canonicalize(config->macs);
+          ssh_xfree(config->macs);
+          config->macs = hlp1;
+        }
+      else if (strcasecmp(val, "anystd") == 0)
+        {
+          char *hlp = ssh_mac_get_supported();
+          config->macs = ssh_name_list_intersection(hlp, SSH_STD_MACS);
+          ssh_xfree(hlp);
+        }
+      else if (strcasecmp(val, "anystdmac") == 0)
+        {
+          char *hlp = ssh_mac_get_supported();
+          config->macs = ssh_name_list_intersection(hlp, SSH_STD_MACS);
+          ssh_xfree(hlp);
+          hlp = config->macs;
+          config->macs = ssh_cipher_list_exclude(hlp, "none");
+          ssh_xfree(hlp);
+        }
+      else
+        {
+          config->macs = ssh_hash_list_canonicalize(val);
+        }
+      SSH_DEBUG(3, ("Final maclist \"%s\"", config->macs));
+      return FALSE;
+    }
+
   if (strcmp(var, "userconfigdirectory") == 0)
     {
       ssh_xfree(config->user_conf_dir);
@@ -741,6 +766,17 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->compression = bool;
           return FALSE;
         }
+      if (strcmp(var, "compressionlevel") == 0)
+        {
+          if (num > 9 || num < 1)
+            {
+              ssh_warning("Invalid value for CompressionLevel.");
+              return TRUE;
+            }
+          
+          config->compression = num;
+          return FALSE;
+        }
 
       if (strcmp(var, "fallbacktorsh") == 0)
         {
@@ -759,17 +795,31 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->batch_mode = bool;
           return FALSE;
         }
+
+      if (strcmp(var, "authenticationnotify") == 0)
+        {
+          config->authentication_notify = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "stricthostkeychecking") == 0)
         {
-          config->strict_host_key_checking = bool;
+          if (strcasecmp(val, "ask") == 0)
+            config->strict_host_key_checking = SSH_STRICT_HOSTKEY_CHECKING_ASK;
+          else
+            config->strict_host_key_checking = bool;
+          
           return FALSE;
         }
       
       if (strcmp(var, "escapechar") == 0)
         {
           ssh_xfree(config->escape_char);
-          config->escape_char = ssh_xstrdup(val);
+          if (strcmp(val, "none") == 0)
+            config->escape_char = ssh_xstrdup("");
+          else
+            config->escape_char = ssh_xstrdup(val);
+
           return FALSE;
         }
       
@@ -782,6 +832,12 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
 
       if (strcmp(var, "gobackground") == 0)
         {
+          if (strcasecmp(val, "oneshot") == 0)
+            {
+              config->go_background = TRUE;
+              config->one_shot_forwarding = TRUE;
+              return FALSE;
+            }
           config->go_background = bool;
           return FALSE;
         }
@@ -799,6 +855,11 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
 
+      if (strcmp(var, "ssh1internalemulation") == 0)
+        {
+          return FALSE;
+        }
+
       if (strcmp(var, "ssh1agentcompatibility") == 0)
         {
           if (strcasecmp(val, "none") == 0)
@@ -824,6 +885,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
         }
 
+#ifndef SSH_WIN_CLIENT
       if (strcmp(var, "localforward") == 0)
         {
           if(ssh_parse_forward(&(config->local_forwards), val))
@@ -843,6 +905,19 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
           return FALSE;
         }  
+#endif /* !SSH_WIN_CLIENT */
+      if (strcmp(var, "forwardx11") == 0)
+        {
+          config->forward_x11 = bool;
+          return FALSE;
+        }
+
+      if (strcmp(var, "forwardagent") == 0)
+        {
+          config->forward_agent = bool;
+          return FALSE;
+        }
+      
       if (strcmp(var, "sshsignerpath") == 0)
         {
           ssh_xfree(config->signer_path);
@@ -855,6 +930,32 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->gateway_ports = bool;
           return FALSE;
         }
+
+      if (strcmp(var, "tryemptypassword") == 0)
+        {
+          config->try_empty_password = bool;
+          return FALSE;
+        }
+
+      if (strcmp(var, "socksserver") == 0)
+        {
+          ssh_xfree(config->socks_server);
+          config->socks_server = ssh_xstrdup(val);
+          return FALSE;
+        }
+
+      if (strcmp(var, "defaultdomain") == 0)
+        {
+          ssh_xfree(config->default_domain);
+          config->default_domain = ssh_xstrdup(val);
+          return FALSE;
+        }
+
+      if (strcmp(var, "authenticationsuccessmsg") == 0)
+        {
+          config->auth_success_msg = bool;
+          return FALSE;
+        }      
     }
   else
     {
@@ -904,6 +1005,194 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
         }
       
+      if (strcmp(var, "allowusers") == 0)
+        {
+          SshDlList temp;
+
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowUsers failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_users)
+                /* Allow the use of multiple AllowUsers directives. */
+                ssh_config_combine_lists(config->allowed_users, temp);
+              else
+                config->allowed_users = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denyusers") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenyUsers failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_users)
+                /* Allow the use of multiple DenyUsers directives. */
+                ssh_config_combine_lists(config->denied_users, temp);
+              else
+                config->denied_users = temp;
+                  
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "allowgroups") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowGroups failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_groups)
+                /* Allow the use of multiple AllowGroups directives. */
+                ssh_config_combine_lists(config->allowed_groups, temp);
+              else
+                config->allowed_groups = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denygroups") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenyGroups failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_groups)
+                /* Allow the use of multiple DenyGroups directives. */
+                ssh_config_combine_lists(config->denied_groups, temp);
+              else
+                config->denied_groups = temp;
+                  
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "allowtcpforwardingforusers") == 0)
+        {
+          SshDlList temp;
+
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowTcpForwardingForUsers "
+                          "failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_tcp_forwarding_users)
+                /* Allow the use of multiple AllowTcpForwardingForUsers
+                   directives. */
+                ssh_config_combine_lists(config->allowed_tcp_forwarding_users,
+                                         temp);
+              else
+                config->allowed_tcp_forwarding_users = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denytcpforwardingforusers") == 0)
+        {
+          SshDlList temp;
+
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenyTcpForwardingForUsers "
+                          "failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_tcp_forwarding_users)
+                /* Allow the use of multiple DenyTcpForwardingForUsers
+                   directives. */
+                ssh_config_combine_lists(config->denied_tcp_forwarding_users,
+                                         temp);
+              else
+                config->denied_tcp_forwarding_users = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "allowtcpforwardingforgroups") == 0)
+        {
+          SshDlList temp;
+
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for AllowTcpForwardingForGroups "
+                          "failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->allowed_tcp_forwarding_groups)
+                /* Allow the use of multiple AllowTcpForwardingForGroups
+                   directives. */
+                ssh_config_combine_lists(config->allowed_tcp_forwarding_groups,
+                                         temp);
+              else
+                config->allowed_tcp_forwarding_groups = temp;
+
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "denytcpforwardingforgroups") == 0)
+        {
+          SshDlList temp;
+
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for DenyTcpForwardingForGroups "
+                          "failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->denied_tcp_forwarding_groups)
+                /* Allow the use of multiple DenyTcpForwardingForGroups
+                   directives. */
+                ssh_config_combine_lists(config->denied_tcp_forwarding_groups,
+                                         temp);
+              else
+                config->denied_tcp_forwarding_groups = temp;
+
+              return FALSE;
+            }
+        }
+
       if (strcmp(var, "allowshosts") == 0)
         {
           SshDlList temp;
@@ -948,6 +1237,23 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
         }
 
+      if (strcmp(var, "requiredauthentications") == 0)
+        {
+          ssh_app_free_list(config->required_authentications, NULL);
+          
+          if ((config->required_authentications =
+               ssh_config_parse_list(val, auth_param_validity, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for RequiredAuthentications failed.");
+              return TRUE;
+            }
+          else
+            {    
+              return FALSE;
+            }      
+        }
+      
       if (strcmp(var, "requirereversemapping") == 0)
         {
           config->require_reverse_mapping = bool;
@@ -996,6 +1302,48 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
       
+      if (strcmp(var, "chrootusers") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for ChRootUsers failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->chroot_users)
+                /* Allow the use of multiple ChRootUsers directives. */
+                ssh_config_combine_lists(config->chroot_users, temp);
+              else
+                config->chroot_users = temp;
+              return FALSE;
+            }
+        }
+
+      if (strcmp(var, "chrootgroups") == 0)
+        {
+          SshDlList temp;
+          
+          if ((temp = ssh_config_parse_list(val, NULL, NULL))
+              == NULL)
+            {
+              ssh_warning("Parsing of value for ChRootGroups failed.");
+              return TRUE;
+            }
+          else
+            {
+              if (config->chroot_groups)
+                /* Allow the use of multiple ChRootGroups directives. */
+                ssh_config_combine_lists(config->chroot_groups, temp);
+              else
+                config->chroot_groups = temp;
+              return FALSE;
+            }
+        }
+
       if (strcmp(var, "permitemptypasswords") == 0)
         {
           config->permit_empty_passwords = bool;
@@ -1014,11 +1362,17 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
 
+      if (strcmp(var, "allowcshrcsourcingwithsubsystems") == 0)
+        {
+          config->allow_cshrc_sourcing_with_subsystems = bool;
+          return FALSE;
+        }
+
       if (strcmp(var, "checkmail") == 0)
         {
           config->check_mail = bool;
           return FALSE;
-        }      
+        }
 
       if (strcmp(var, "listenaddress") == 0)
         {
@@ -1028,6 +1382,27 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
       
+      if (strcmp(var, "allowx11forwarding") == 0 ||
+          strcmp(var, "x11forwarding") == 0 ||
+          strcmp(var, "forwardx11") == 0)
+        {
+          config->x11_forwarding = bool;
+          return FALSE;
+        }
+
+      if (strcmp(var, "allowtcpforwarding") == 0)
+        {
+          config->allow_tcp_forwarding = bool;
+          return FALSE;
+        }
+
+      if (strcmp(var, "allowagentforwarding") == 0 ||
+          strcmp(var, "forwardagent") == 0)
+        {
+          config->allow_agent_forwarding = bool;
+          return FALSE;
+        }
+
       if (strcmp(var, "hostkeyfile") == 0)
         {
           ssh_xfree(config->host_key_file);
@@ -1070,7 +1445,19 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->login_grace_time = num;
           return FALSE;
         }
-      
+
+      if (strcmp(var, "maxbroadcastspersecond") == 0)
+        {
+          if (num < 0)
+            {
+              ssh_warning("Ignoring illegal broadcasts per second value %d",
+                          num);
+              return TRUE;
+            }
+          config->broadcasts_allowed_per_second = num;
+          return FALSE;
+        }
+
       if (strcmp(var, "passwordguesses") == 0)
         {
           config->password_guesses = num;
@@ -1079,7 +1466,30 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
 
       if (strcmp(var, "maxconnections") == 0)
         {
+#ifndef SSH2_MAX_CONNECTIONS
           config->max_connections = num;
+#else /* ! SSH2_MAX_CONNECTIONS */
+#if SSH2_MAX_CONNECTIONS == 0
+          config->max_connections = num;
+#else /* SSH2_MAX_CONNECTIONS == 0 */
+          if ((num > SSH2_MAX_CONNECTIONS) || (num <= 0))
+            {
+              if (num > 0)
+                ssh_warning(
+                 "Maximum of %d connections requested while only %d allowed.", 
+                 config->max_connections, SSH2_MAX_CONNECTIONS);
+              else
+                ssh_warning(
+                 "Unlimited connections requested while only %d allowed.", 
+                 SSH2_MAX_CONNECTIONS);
+              config->max_connections = SSH2_MAX_CONNECTIONS;
+            }
+          else
+            {
+              config->max_connections = num;
+            }
+#endif /* SSH2_MAX_CONNECTIONS == 0 */
+#endif /* ! SSH2_MAX_CONNECTIONS */
           return FALSE;
         }
 
@@ -1089,8 +1499,6 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->ssh1_path = ssh_xstrdup(val);
           return FALSE;
         }
-
-
       
       /* Parse subsystem definitions */
       if (strncmp(var, SUBSYSTEM_PREFIX, SUBSYSTEM_PREFIX_LEN) == 0)
