@@ -3,12 +3,12 @@
   sshconfig.h
 
   Authors:
-        Tatu Ylnen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylnen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Processing configuration data in SSH (both client and server).
@@ -39,6 +39,7 @@ typedef struct SshForwardRec {
   char *port;
   char *connect_to_host;
   char *connect_to_port;
+  char *protocol;
 } *SshForward;
 
 typedef enum {
@@ -61,6 +62,12 @@ typedef enum {
   SSH_ROOTLOGIN_NOPWD = 2
 } SshPermitRootLogin;
 
+typedef enum {
+  SSH_STRICT_HOSTKEY_CHECKING_NO = 0,
+  SSH_STRICT_HOSTKEY_CHECKING_YES = 1,
+  SSH_STRICT_HOSTKEY_CHECKING_ASK = 2
+} SshStrictHostKeyChecking;
+
 /* Data type for SSH server configuration data. */
 
 struct SshConfigRec
@@ -84,8 +91,14 @@ struct SshConfigRec
 
   /* common for both client and server */
 
-  Boolean forward_agent;    
+  /* For client ...*/
   Boolean forward_x11;
+  Boolean forward_agent;    
+  /* ... and for server. */
+  Boolean x11_forwarding;
+  Boolean allow_tcp_forwarding;
+  Boolean allow_agent_forwarding;
+
   Boolean password_authentication;
   Boolean rhosts_authentication;
   Boolean rhosts_pubkey_authentication;
@@ -93,12 +106,14 @@ struct SshConfigRec
   Boolean force_ptty_allocation;
   Boolean verbose_mode;
   Boolean compression;
+  int compression_level;
   SshDlList allowed_authentications;
   SshDlList required_authentications;
   Boolean user_known_hosts;
   
   char *port;      
   char *ciphers;   
+  char *macs;
   char *user_conf_dir;
   char *identity_file;
   char *authorization_file;
@@ -113,7 +128,9 @@ struct SshConfigRec
   int max_connections;
   
   char *host_to_connect;
+  char *socks_server;
   char *login_as_user;
+  char *password; /* used on Windows */
   SshForward local_forwards;
   SshForward remote_forwards;
 
@@ -121,6 +138,12 @@ struct SshConfigRec
   SshDlList denied_hosts;
   SshDlList allowed_shosts;
   SshDlList denied_shosts;
+
+  SshDlList allowed_users;
+  SshDlList denied_users;
+  SshDlList allowed_groups;
+  SshDlList denied_groups;
+  
   Boolean require_reverse_mapping;
 
   SshLogFacility log_facility;
@@ -128,8 +151,10 @@ struct SshConfigRec
   Boolean fall_back_to_rsh;
   Boolean use_rsh;
   Boolean batch_mode;
-  Boolean strict_host_key_checking;
+  Boolean authentication_notify;
+  SshStrictHostKeyChecking strict_host_key_checking;
   Boolean go_background;
+  Boolean one_shot_forwarding;
   Boolean dont_read_stdin;
   Boolean gateway_ports;
   char *escape_char;
@@ -139,11 +164,14 @@ struct SshConfigRec
   int ignore_root_rhosts;
   SshPermitRootLogin permit_root_login;
   Boolean permit_empty_passwords;
+  Boolean try_empty_password;
   Boolean strict_modes;
   Boolean quiet_mode;
   Boolean fascist_logging;
   Boolean print_motd;
+  Boolean allow_cshrc_sourcing_with_subsystems;
   Boolean check_mail;
+  Boolean auth_success_msg;
   Boolean keep_alive;
   Boolean no_delay;
   Boolean inetd_mode;
@@ -154,10 +182,21 @@ struct SshConfigRec
   
   char *forced_command;
 
+  SshDlList chroot_users;
+  SshDlList chroot_groups;
+
+  SshDlList allowed_tcp_forwarding_users;
+  SshDlList denied_tcp_forwarding_users;
+  SshDlList denied_tcp_forwarding_groups;
+  SshDlList allowed_tcp_forwarding_groups;
+
   size_t no_subsystems;
   size_t subsystems_allocated;
   SshSubsystem *subsystems;
 
+  /* How many UDP broadcast commands are handled by server each second.
+     If this is zero, no broadcasts are received at all. */
+  int broadcasts_allowed_per_second;
   
   /* Flag specifying whether to enable ssh1 compatibility. */
   Boolean ssh1compatibility;
@@ -167,7 +206,8 @@ struct SshConfigRec
 
   /* Path to ssh1/sshd1.*/
   char *ssh1_path;
-  
+
+
   /* Ssh1 arguments for compatibility. If this is NULL, there will be no
      ssh1 compatibility.  The first argument should be the program name. */
   char **ssh1_argv;
@@ -179,6 +219,10 @@ struct SshConfigRec
 
   /* Path to sshsigner2. */
   char *signer_path;
+
+  /* The default domain, which should be set if, for example
+     'hostname' returns only basepart of the FQDN. */
+  char *default_domain;
 };
 
 typedef struct SshConfigRec *SshConfig;
