@@ -3,16 +3,16 @@
 sshserver.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-SSH server functionality for processing a connection.  Most of the
-implementation is actually shared with the client (in sshcommon.c).
+  SSH server functionality for processing a connection.  Most of the
+  implementation is actually shared with the client (in sshcommon.c).
 
 */
 
@@ -39,7 +39,7 @@ Boolean ssh_server_update_transport_params(SshConfig config,
 
   if (config->ciphers != NULL)
     {
-      hlp = ssh_cipher_list_canonialize(config->ciphers);
+      hlp = ssh_cipher_list_canonicalize(config->ciphers);
 
       if (hlp)
         {
@@ -52,11 +52,25 @@ Boolean ssh_server_update_transport_params(SshConfig config,
         }
     }
 
-  hlp = ssh_public_key_list_canonialize(params->host_key_algorithms);
+  if (config->macs != NULL)
+    {
+      hlp = ssh_hash_list_canonicalize(config->macs);
+
+      if (hlp)
+        {
+          ssh_xfree(params->macs_c_to_s);
+          params->macs_c_to_s = ssh_xstrdup(hlp);
+          ssh_xfree(params->macs_s_to_c);
+          params->macs_s_to_c = ssh_xstrdup(hlp);
+          ssh_xfree(hlp);
+        }
+    }
+
+  hlp = ssh_public_key_list_canonicalize(params->host_key_algorithms);
   ssh_xfree(params->host_key_algorithms);
   params->host_key_algorithms = hlp;
 
-  hlp = ssh_hash_list_canonialize(params->hash_algorithms);
+  hlp = ssh_hash_list_canonicalize(params->hash_algorithms);
   ssh_xfree(params->hash_algorithms);
   params->hash_algorithms = hlp;
 
@@ -66,12 +80,10 @@ Boolean ssh_server_update_transport_params(SshConfig config,
 /* Takes a stream, and creates an SSH server for processing that
    connection.  This closes the stream and returns NULL (without
    calling the destroy function) if an error occurs.  This does not
-   free the given server key.  The random state is required to stay
-   valid until the server has been destroyed.  ``config'' must remain
+   free the given server key.  ``config'' must remain
    valid until the server is destroyed; it is not automatically freed.
      `stream'        the connection stream
      `config'        configuration data (not freed, must remain valid)
-     `random_state'  random number generator state
      `private_server_key'   private key that changes every hour or NULL
      `disconnect'    function to call on disconnect
      `debug'         function to call on debug message (may be NULL)
@@ -80,19 +92,19 @@ Boolean ssh_server_update_transport_params(SshConfig config,
    The object should be destroyed from the ``disconnect'' callback. */
 
 SshServer ssh_server_wrap(SshStream stream, SshConfig config,
-                          SshRandomState random_state,
                           SshPrivateKey private_server_key,
-                          SshServerDisconnectProc disconnect,
-                          SshServerDebugProc debug,
+                          SshConnDisconnectProc disconnect,
+                          SshConnDebugProc debug,
                           SshVersionCallback version_check,
                           SshAuthPolicyProc auth_policy_proc,
-                          SshCommonAuthenticatedNotify authenticated_notify,
+                          SshCommonAuthenticationNotify authenticated_notify,
                           void *context)
 {
   SshServer server;
   SshStream trans, auth;
   SshTransportParams params;
-
+  SshTransportCompat compat_flags;
+  
   /* Create parameters. */
   params = ssh_transport_create_params();
   if (!ssh_server_update_transport_params(config, params))
@@ -114,7 +126,7 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
   
   /* Create a transport layer protocol object. */
   ssh_debug("ssh_server_wrap: creating transport protocol");
-  trans = ssh_transport_server_wrap(stream, random_state, 
+  trans = ssh_transport_server_wrap(stream, 
                                     SSH2_PROTOCOL_VERSION_STRING,
                                     params, config->private_host_key,
                                     private_server_key,
@@ -124,22 +136,23 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
                                     (void *)context);
 
   
-  ssh_transport_get_compatibility_flags(trans, &server->compat_flags);
+  ssh_transport_get_compatibility_flags(trans, &compat_flags);
   
   /* Create the authentication methods array for the server. */
-  server->methods = ssh_server_authentication_initialize();
-  /* XXX config data */
+  server->methods =
+    ssh_server_authentication_initialize(config->allowed_authentications);
   
   /* Create an authentication protocol object. */
   ssh_debug("ssh_server_wrap: creating userauth protocol");
-  /* XXX policy_proc */
+
   auth = ssh_auth_server_wrap(trans, auth_policy_proc, (void *)server,
                               server->methods, (void *)server);
 
   /* Create the common part of client/server objects. */
-  server->common = ssh_common_wrap(stream, auth, FALSE, config, random_state,
+  server->common = ssh_common_wrap(stream, auth, FALSE, config, 
                                    NULL,
                                    disconnect, debug, authenticated_notify,
+                                   NULL, compat_flags,
                                    context);
 
   if (server->common == NULL)
@@ -162,8 +175,11 @@ void ssh_server_destroy(SshServer server)
   /* Cancel grace timeout. */
   ssh_cancel_timeouts(ssh_login_grace_time_exceeded, SSH_ALL_CONTEXTS);
   ssh_common_destroy(server->common);
-  ssh_xfree(server->compat_flags);
+#if 0
+  /* XXX causes SIGSEGV, as the methods are called from
+     lib/sshproto/sshauths.c after this call. */
   ssh_server_authentication_uninitialize(server->methods);
+#endif
   memset(server, 'F', sizeof(*server));
   ssh_xfree(server);
 }
