@@ -3,15 +3,15 @@
 sshserver.h
 
   Authors:
-        Tatu Ylnen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylnen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-SSH server functionality for processing a connection.
+  SSH server functionality for processing a connection.
 
 */
 
@@ -42,31 +42,6 @@ struct SshServerRec
 
 typedef struct SshServerRec *SshServer;
 
-/* Callback function called when a disconnect message or EOF is
-   received from the other side.  The server protocol should be
-   destroyed after receiving this callback (typically from within this
-   callback).  If this callback is not registered (was NULL), a
-   default callback which simply destroys the protocol context is
-   used.  If this is called because of EOF being received from the
-   other side, ``reason'' will be SSH_DISCONNECT_CONNECTION_LOST,
-   and ``msg'' an appropriate descriptive message.
-     `reason'     numeric reason for disconnection (usable for localization)
-     `msg'        disconnect message in English
-     `context'    context argument from when the callback was registered */
-typedef void (*SshServerDisconnectProc)(int reason,
-                                        const char *msg,
-                                        void *context);
-
-/* Callback function called when a debug message is received from the other
-   side.  If this is NULL, all received debug messages are ignored.
-     `type'    message type (SSH_DEBUG_DISPLAY, SSH_DEBUG_DEBUG)
-     `msg'     the debugging message (normally English)
-     `context' context argument passed when the callback was registered. */
-typedef void (*SshServerDebugProc)(int type,
-                                   const char *msg,
-                                   void *context);
-
-
 /* Takes a stream, and creates an SSH server for processing that
    connection.  This closes the stream and returns NULL (without
    calling the destroy function) if an error occurs.  This does not
@@ -83,12 +58,12 @@ typedef void (*SshServerDebugProc)(int type,
      `context'       context to pass to the callbacks
    The object should be destroyed from the ``disconnect'' callback. */
 SshServer ssh_server_wrap(SshStream stream, SshConfig config,
-                          SshRandomState random_state,
                           SshPrivateKey private_server_key,
-                          SshServerDisconnectProc disconnect,
-                          SshServerDebugProc debug,
+                          SshConnDisconnectProc disconnect,
+                          SshConnDebugProc debug,
                           SshVersionCallback version_check,
-                          SshCommonAuthenticatedNotify authenticated_notify,
+                          SshAuthPolicyProc auth_policy_proc,
+                          SshCommonAuthenticationNotify authentication_notify,
                           void *context);
 
 /* Forcibly destroys the given connection. */
