@@ -3,15 +3,15 @@
 sshunixconfig.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Processing configuration data in SSH (both client and server).
+  Processing configuration data in SSH (both client and server).
 
 */
 
@@ -220,7 +220,7 @@ Boolean ssh_server_load_host_key(SshConfig config,
       
 
   /* load the host key from (typically) /etc/ssh2/hostkey */
-  if(config->host_key_file[0] != '/')
+  if (config->host_key_file[0] != '/')
     {    
       if (ssh_user_uid(user) == 0 )
         {
@@ -256,7 +256,7 @@ Boolean ssh_server_load_host_key(SshConfig config,
   *private_host_key = privkey;
 
   /* ok, now read the public host key blob */
-  if(config->public_host_key_file[0] != '/')
+  if (config->public_host_key_file[0] != '/')
     {
       snprintf(hostkeyfile, sizeof(hostkeyfile), "%s/%s", 
                userdir, config->public_host_key_file);
@@ -269,7 +269,7 @@ Boolean ssh_server_load_host_key(SshConfig config,
   
   SSH_TRACE(1, ("Reading public host key from: %s", hostkeyfile));
 
-  if (ssh2_key_blob_read(user, hostkeyfile, NULL,
+  if (ssh2_key_blob_read(user, hostkeyfile, TRUE, NULL,
                         public_host_key_blob,
                         public_host_key_blob_len, NULL) 
       != SSH_KEY_MAGIC_PUBLIC)
@@ -388,27 +388,57 @@ Boolean ssh_config_read_file(SshUser user, SshConfig config,
   return TRUE;
 }
 
-/* Parse forwarding definitions. Format is port:remotehost:remoteport */
-Boolean ssh_parse_forward(SshForward *forwards, char *spec)
+/* Parse forwarding definitions. Format is 
+   [protocol/]port:remotehost:remoteport */
+Boolean ssh_parse_forward(SshForward *forwards, char *spec_orig)
 {
   SshForward fwd;
-  char *local_port, *host, *port;
+  char *local_port, *host, *port, *proto, *spec, *to_be_deleted;
+  char *hlp1, *hlp2;
+
+  spec = ssh_xstrdup(spec_orig);
+  to_be_deleted = spec;
+  
+  if ((hlp1 = strchr(spec, '/')) &&
+      (hlp2 = strchr(spec, ':')) &&
+      (hlp1 < hlp2))
+    {
+      *hlp1 = '\0';
+      proto = ssh_xstrdup(spec);
+      spec = hlp1 + 1;
+    }
+  else
+    {
+      proto = ssh_xstrdup("tcp");
+    }
 
   local_port = strtok(spec, ":");
   if (local_port == NULL)
-    return TRUE;
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
   host = strtok(NULL, ":");
   if (host == NULL)
-    return TRUE;
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
   port = strtok(NULL, ":");
   if (port == NULL)
-    return TRUE;
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
   fwd = ssh_xcalloc(1, sizeof(*fwd));
   fwd->local_addr = ssh_xstrdup("0.0.0.0");
   fwd->port = ssh_xstrdup(local_port);
   fwd->connect_to_host = ssh_xstrdup(host);
   fwd->connect_to_port = ssh_xstrdup(port);
+  fwd->protocol = proto; /* This is already mallocated. */
   fwd->next = *forwards;
   *forwards = fwd;
+  ssh_xfree(to_be_deleted);
+  
   return FALSE;
 }  
