@@ -3,12 +3,12 @@
   sshunixuserfiles.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Simple functions that update user's files. These are unix-spesific.
@@ -23,9 +23,9 @@
 #include "sshincludes.h"
 #include "sshuserfiles.h"
 #include "sshencode.h"
-#include "pubkeyencode.h"
+#include "ssh2pubkeyencode.h"
 #include "sshuser.h"
-#include "userfile.h"
+#include "sshuserfile.h"
 #include "sshconfig.h"
 #include "sshmiscstring.h"
 
@@ -43,24 +43,19 @@ char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed)
 
   /* create the .ssh2 directory name */
 
-  if (ssh_user_uid(user) == 0)
-    {
-      sshdir = ssh_xstrdup(SSH_SERVER_DIR);
-    }
-  else
-    {
-      sshdir = ssh_user_conf_dir(config, user);
-    } 
+  sshdir = ssh_user_conf_dir(config, user);
 
   if (stat(sshdir, &st) < 0)
     {
       if (create_if_needed)
         {
           if (mkdir(sshdir, 0755) < 0)
-            ssh_debug("ssh_userdir: could not create user's ssh" 
-                        "directory %s", sshdir);
-          ssh_xfree(sshdir);
-          return NULL;
+            {
+              SSH_DEBUG(2, ("could not create user's ssh directory %s", 
+                            sshdir));
+              ssh_xfree(sshdir);
+              return NULL;
+            }  
         }
       else
         {
@@ -82,7 +77,7 @@ char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed)
 
 char *ssh_randseed_file(SshUser user, SshConfig config)
 {
-  UserFile f;
+  SshUserFile f;
   char *sshdir, *sshseed;
   size_t sshseedlen;
   struct stat st;
@@ -99,18 +94,17 @@ char *ssh_randseed_file(SshUser user, SshConfig config)
 
   /* If it doesn't exist, create it. */
 
-  if (userfile_stat(ssh_user_uid(user), sshseed, &st) < 0)
+  if (ssh_userfile_stat(ssh_user_uid(user), sshseed, &st) < 0)
     {
-      if ((f = userfile_open(ssh_user_uid(user), sshseed, O_RDWR | O_CREAT, 
-                            0600)) == NULL)
+      if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, 
+                                 O_RDWR | O_CREAT, 0600)) == NULL)
         {
-          ssh_debug("ssh_randseed_file: Could not create random"
-                      "seed file %s.", sshseed);
+          SSH_DEBUG(2, ("Could not create random seed file %s.", sshseed));
           ssh_xfree(sshdir);
           ssh_xfree(sshseed);
           return NULL;
         }
-      userfile_close(f);
+      ssh_userfile_close(f);
     }
   
   ssh_xfree(sshdir);
@@ -121,11 +115,10 @@ char *ssh_randseed_file(SshUser user, SshConfig config)
 /* Get the random state from the file.  This loads and merges any data
    in the seed file into the generator. */
 
-void ssh_randseed_load(SshUser user, SshRandomState random_state,
-                       SshConfig config)
+void ssh_randseed_load(SshUser user, SshConfig config)
 {
   int i;
-  UserFile f;
+  SshUserFile f;
   unsigned char randbuf[16];
   char *sshseed;
   size_t nbytes;
@@ -133,21 +126,22 @@ void ssh_randseed_load(SshUser user, SshRandomState random_state,
   /* Stir a bit.  This will add a couple of bits of new randomness to the 
      pool. */
   for (i = 0; i < 3; i++)
-    ssh_random_stir(random_state);
+    ssh_random_stir();
   
   /* Stir the seed file in, if possible. */
   sshseed = ssh_randseed_file(user, config);
-  if ((f = userfile_open(ssh_user_uid(user), sshseed, O_RDONLY, 0)) != NULL)
+  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, O_RDONLY, 0)) != 
+      NULL)
     {
-      while ((nbytes = userfile_read(f, randbuf, sizeof(randbuf))) > 0)
-        ssh_random_add_noise(random_state, randbuf, nbytes);
-      userfile_close(f);
+      while ((nbytes = ssh_userfile_read(f, randbuf, sizeof(randbuf))) > 0)
+        ssh_random_add_noise(randbuf, nbytes);
+      ssh_userfile_close(f);
     }
   ssh_xfree(sshseed); 
   
   /* Stir a bit.  This will add a few bits of new randomness to the pool. */
   for (i = 0; i < 3; i++)
-    ssh_random_stir(random_state);
+    ssh_random_stir();
 }
 
 /* Updates the random seed file with information from the random
@@ -156,31 +150,31 @@ void ssh_randseed_load(SshUser user, SshRandomState random_state,
    contain traces of both the generator state and the old seed
    file. */
 
-void ssh_randseed_update(SshUser user, SshRandomState rs, SshConfig config)
+void ssh_randseed_update(SshUser user, SshConfig config)
 {
   size_t i;
-  UserFile f;
+  SshUserFile f;
   char *sshseed;
   unsigned char seed[SSH_RANDSEED_LEN];
 
   /* Load the old random seed file and mix it into the generator. */
-  ssh_randseed_load(user, rs, config);
+  ssh_randseed_load(user, config);
   
   /* Write data from the generator into the random seed file. */
   sshseed = ssh_randseed_file(user, config);
-  if ((f = userfile_open(ssh_user_uid(user), sshseed, O_CREAT | O_WRONLY, 
+  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, O_CREAT | O_WRONLY, 
                          0600)) == NULL)
     {
-      ssh_debug("ssh_randseed_close: unable to write the random seed file!");
+      SSH_DEBUG(2, ("unable to write the random seed file!"));
       goto error;
     }
   for (i = 0; i < SSH_RANDSEED_LEN; i++)
-    seed[i] = ssh_random_get_byte(rs);
-  if (userfile_write(f, seed, SSH_RANDSEED_LEN) != SSH_RANDSEED_LEN)
+    seed[i] = ssh_random_get_byte();
+  if (ssh_userfile_write(f, seed, SSH_RANDSEED_LEN) != SSH_RANDSEED_LEN)
     ssh_warning("unable to write to the random seed file %s.", sshseed);
 
   memset(seed, 0, SSH_RANDSEED_LEN);
-  userfile_close(f);
+  ssh_userfile_close(f);
 
 error:
   ssh_xfree(sshseed);
@@ -193,7 +187,7 @@ error:
 Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob, 
                       size_t *bloblen, void *context)
 {
-  UserFile f;
+  SshUserFile f;
   unsigned char *data;
   struct stat st;
   size_t datalen;
@@ -201,32 +195,32 @@ Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob,
   *bloblen = 0;
   *blob = NULL;
 
-  if (userfile_stat(ssh_user_uid(user), fname, &st) < 0)
+  if (ssh_userfile_stat(ssh_user_uid(user), fname, &st) < 0)
     {
-      ssh_debug("ssh_blob_read: file %s does not exist.", fname);
+      SSH_DEBUG(2, ("file %s does not exist.", fname));
       return TRUE;
     }
   
   datalen = st.st_size;
   data = ssh_xmalloc(datalen);
 
-  if ((f = userfile_open(ssh_user_uid(user), fname, O_RDONLY, 0)) == NULL) 
+  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, O_RDONLY, 0)) == NULL) 
     {
-      ssh_debug("ssh_blob_read: Could not open %s.", fname);
+      SSH_DEBUG(2, ("Could not open %s.", fname));
       ssh_xfree(data);
       return TRUE;
     }
 
-  if (userfile_read(f, data, datalen) != datalen)
+  if (ssh_userfile_read(f, data, datalen) != datalen)
     {
-      ssh_debug("ssh_blob_read: Error while reading %s.", fname);
+      SSH_DEBUG(2, ("Error while reading %s.", fname));
       memset(data, 0, datalen);
       ssh_xfree(data);
-      userfile_close(f); 
+      ssh_userfile_close(f); 
       return TRUE;
     }
 
-  userfile_close(f);
+  ssh_userfile_close(f);
   *blob = data;
   *bloblen = datalen;
 
@@ -237,24 +231,25 @@ Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob,
 /* Write a blob. Return TRUE on failure. */
 
 Boolean ssh_blob_write(SshUser user, const char *fname, mode_t mode,
-                       const unsigned char *blob, size_t bloblen, void *context)
+                       const unsigned char *blob, size_t bloblen, 
+                       void *context)
 {
-  UserFile f;
+  SshUserFile f;
 
-  if ((f = userfile_open(ssh_user_uid(user), fname, O_WRONLY | O_CREAT, 
+  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, O_WRONLY | O_CREAT, 
                          mode)) == NULL)
     {
-      ssh_debug("ssh_blob_write: could not open %s.", fname);
+      SSH_DEBUG(2, ("could not open %s.", fname));
       return TRUE;
     }
 
-  if(userfile_write(f, blob, bloblen) != bloblen)
+  if(ssh_userfile_write(f, blob, bloblen) != bloblen)
     {
-      ssh_debug("ssh_blob_write: failed to write %s.", fname);
+      SSH_DEBUG(2, ("failed to write %s.", fname));
       return TRUE;
     }
 
-  userfile_close(f);
+  ssh_userfile_close(f);
 
   return FALSE;
 }
@@ -264,22 +259,33 @@ Boolean ssh_blob_write(SshUser user, const char *fname, mode_t mode,
    The caller should free the array and all strings in it when no longer
    needed. */
 
-char **ssh_privkey_list(SshUser user, char *host, SshConfig config)
+struct SshConfigPrivateKey **ssh_privkey_list(SshUser user, 
+                                              char *host, 
+                                              SshConfig config)
 {
   int i, j, n;
-  char *udir, **vars, **vals, **prklist, buf[1024];
+  char *udir, **vars, **vals, buf[1024];
+  struct SshConfigPrivateKey **prkey;
+#ifdef WITH_PGP
+  char *pgp_secret_key_file;
+#endif /* WITH_PGP */
 
   if ((udir = ssh_userdir(user, config, TRUE)) == NULL)
     {
-      ssh_debug("ssh_privkey_list: no user directory.");
+      SSH_DEBUG(2, ("no user directory."));
       return NULL;
     }
 
   /* read and sort the names */
 
-  snprintf(buf, sizeof(buf)-1, "%s/%s", udir, 
-           config == NULL || config->identity_file == NULL ?
-           SSH_IDENTIFICATION_FILE : config->identity_file);
+  if (config != NULL && config->identity_file != NULL &&
+      *(config->identity_file) == '/')
+    snprintf(buf, sizeof(buf)-1, "%s", config->identity_file);
+  else
+    snprintf(buf, sizeof(buf)-1, "%s/%s", udir, 
+             config == NULL || config->identity_file == NULL ?
+             SSH_IDENTIFICATION_FILE : config->identity_file);
+  
   n = ssh2_parse_config(user, host, buf, &vars, &vals, NULL);
 
   if (n < 0)
@@ -290,23 +296,93 @@ char **ssh_privkey_list(SshUser user, char *host, SshConfig config)
 
   /* construct a name list with complete file paths */
 
-  prklist = ssh_xcalloc(n + 1, sizeof(char *));
+  prkey = ssh_xcalloc(n + 1, sizeof (struct SshConfigPrivateKey *));
+
+#ifdef WITH_PGP
+  pgp_secret_key_file = ssh_xstrdup(config->pgp_secret_key_file);
+#endif /* WITH_PGP */
 
   j = 0;
   for (i = 0; i < n; i++)
     {
       if (strcmp(vars[i], "idkey") == 0)
         {
-          snprintf(buf, sizeof(buf), "%s/%s",
-                   udir, vals[i]);
-          prklist[j++] = ssh_xstrdup(buf);
+          if (*vals[i] == '/')
+            snprintf(buf, sizeof(buf), "%s", vals[i]);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, vals[i]);
+              
+          prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
+          prkey[j]->keyfile = ssh_xstrdup(buf);
+          j++;
         }
+#ifdef WITH_PGP
+      else if (strcmp(vars[i], "pgpsecretkeyfile") == 0)
+        {
+          ssh_xfree(pgp_secret_key_file);
+          pgp_secret_key_file = ssh_xstrdup(vals[i]);
+        }
+      else if (strcmp(vars[i], "idpgpkeyid") == 0)
+        {
+          unsigned long id;
+          char *endptr = NULL;
+
+          id = strtoul(vals[i], &endptr, 0);
+          if (((*(vals[0])) != '\0') && ((*endptr) == '\0'))
+            {
+              if (*pgp_secret_key_file == '/')
+                snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+              else
+                snprintf(buf, sizeof(buf), "%s/%s",
+                         udir, pgp_secret_key_file);
+              
+              prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
+              prkey[j]->pgp_keyring = ssh_xstrdup(buf);
+              prkey[j]->pgp_id = id;
+              j++;
+            }
+          else
+            {
+              SSH_DEBUG(2, ("invalid pgp key id number \"%s\"", vals[i]));
+            }
+        }
+      else if (strcmp(vars[i], "idpgpkeyname") == 0)
+        {
+          if (*pgp_secret_key_file == '/')
+            snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, pgp_secret_key_file);
+              
+          prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
+          prkey[j]->pgp_keyring = ssh_xstrdup(buf);
+          prkey[j]->pgp_name = ssh_xstrdup(vals[i]);
+          j++;
+        }
+      else if (strcmp(vars[i], "idpgpkeyfingerprint") == 0)
+        {
+          if (*pgp_secret_key_file == '/')
+            snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, pgp_secret_key_file);
+              
+          prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
+          prkey[j]->pgp_keyring = ssh_xstrdup(buf);
+          prkey[j]->pgp_fingerprint = ssh_xstrdup(vals[i]);
+          j++;
+        }
+#endif /* WITH_PGP */
     }
-  prklist[j++] = NULL;
+  prkey[j++] = NULL;
   ssh_free_varsvals(n, vars, vals);
   ssh_xfree(udir);
+#ifdef WITH_PGP
+  ssh_xfree(pgp_secret_key_file);
+#endif /* WITH_PGP */
 
-  return prklist;
+  return prkey;
 }
 
 char *ssh_user_conf_dir(SshConfig config, SshUser user)
