@@ -3,12 +3,12 @@
   sshuserfiles.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Simple functions that update user's files. These are not platform-dependent.
@@ -20,10 +20,10 @@
 #include "sshencode.h"
 #include "ssh2pubkeyencode.h"
 #include "sshuser.h"
-#include "sshuserfile.h"
 #include "sshconfig.h"
 #include "sshbase64.h"
 #include "sshmiscstring.h"
+#include "ssh1keydecode.h"
 
 #define SSH_DEBUG_MODULE "SshUserFiles"
 
@@ -55,6 +55,53 @@ unsigned int ssh_key_blob_match(unsigned char *buf, size_t buf_size,
   size_t i, j, start, end, keep, tmp_pos;
   char tmp[1024];
 
+  /* First check out the possible ssh1 key blobs. 
+     - ssh1 public key is 3 integers + free form string
+     - ssh1 private key has an identifier string.
+     - there are no actual nonencrypted ssh1 keys but they
+       are encrypted with cipher `none'. */
+
+  /* If it begins with 3-5 ascii-digits + one space, it can't be nothing
+     else than ssh1 public key. */
+  if ((buf_size > 16) &&
+      (((isdigit(buf[0])) && 
+        (isdigit(buf[1])) &&
+        (isdigit(buf[2])) &&
+        (buf[3] == ' ')) ||
+       ((isdigit(buf[0])) && 
+        (isdigit(buf[1])) &&
+        (isdigit(buf[2])) &&
+        (isdigit(buf[3])) &&
+        (buf[4] == ' ')) ||
+       ((isdigit(buf[0])) && 
+        (isdigit(buf[1])) &&
+        (isdigit(buf[2])) &&
+        (isdigit(buf[3])) &&
+        (isdigit(buf[4])) &&
+        (buf[5] == ' '))))
+    {
+      *ret_start = 0;
+      *ret_end = buf_size;
+      return SSH_KEY_MAGIC_SSH1_PUBLIC;
+    }
+
+  /* Check ssh1 private key tag followd by nul character and cipher 
+     identifier byte. */
+  if ((buf_size > (strlen(SSH1_PRIVATE_KEY_ID_STRING) + 2)) &&
+      (strncmp(SSH1_PRIVATE_KEY_ID_STRING, 
+               (char *)buf, 
+               strlen(SSH1_PRIVATE_KEY_ID_STRING)) == 0) &&
+      (buf[strlen(SSH1_PRIVATE_KEY_ID_STRING)] == 0))
+    {
+      *ret_start = 0;
+      *ret_end = buf_size;
+      if (buf[strlen(SSH1_PRIVATE_KEY_ID_STRING) + 1] == SSH1_CIPHER_NONE)
+        return SSH_KEY_MAGIC_SSH1_PRIVATE;
+      else
+        return SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED;
+    }
+
+  /* Canonical key formats are handled with the format name list. */
   for (i = 0, keep = 0, tmp_pos = 0, start = 0, end = 0; i < buf_size; i++)
     {
       if (buf[i] == '\n')
@@ -167,7 +214,7 @@ size_t ssh_key_blob_get_string(unsigned char *buf, size_t len,
                                char **string)
 {
   unsigned int quoting, ret_quoting;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   size_t step, i, j;
 
   ssh_buffer_init(&buffer);
@@ -240,7 +287,7 @@ size_t ssh_key_blob_get_line(unsigned char *buf, size_t len,
                              char **string)
 {
   size_t i, step, keep;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
 
   ssh_buffer_init(&buffer);
   for (i = 0, step = 0, keep = 0; i < len; i++)
@@ -342,9 +389,10 @@ size_t ssh_key_blob_keywords(unsigned char *buf, size_t len,
 /* Decoding of the SSH2 ascii key blob format. */
 
 unsigned long ssh2_key_blob_read(SshUser user, const char *fname, 
-                                char **comment,
-                                unsigned char **blob,
-                                size_t *bloblen, void *context)
+                                 Boolean try_convert_ssh1_cert,
+                                 char **comment,
+                                 unsigned char **blob,
+                                 size_t *bloblen, void *context)
 {
   unsigned char *pkeypem = NULL, *tmp, *whitened;
   char *my_name, *my_comment;
@@ -368,37 +416,96 @@ unsigned long ssh2_key_blob_read(SshUser user, const char *fname,
 
   if (magic2 != magic)
     goto fail;
-  
-  /* Check. */
-  if (pkeylen - end == 0)
-    goto fail;
-    
-  /* Read the keywords. */
-  step = ssh_key_blob_keywords(pkeypem + end + 1, pkeylen - end - 1,
-                               &my_name, &my_comment);
 
-  /* XXX Ignore the name for now. We don't have any means of handling
-     them nicely. */
-  ssh_xfree(my_name);
+  if ((magic != SSH_KEY_MAGIC_SSH1_PUBLIC) &&
+      (magic != SSH_KEY_MAGIC_SSH1_PRIVATE) &&
+      (magic != SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED))
+    {
+      /* Check. */
+      if (pkeylen - end == 0)
+        goto fail;
 
-  /* If comment is available pass it up. */
-  if (comment)
-    *comment = my_comment;
+      /* Read the keywords. */
+      step = ssh_key_blob_keywords(pkeypem + end + 1, pkeylen - end - 1,
+                                   &my_name, &my_comment);
+      /* Ignore the name for now. We don't have any means of handling
+         it nicely. XXX */
+      ssh_xfree(my_name);
+      
+      /* If comment is available pass it up. */
+      if (comment)
+        *comment = my_comment;
+      else
+        ssh_xfree(my_comment);
+      
+      /* Convert the remainder to a string. */
+      tmp = ssh_xmalloc(start2 - end - step);
+      memcpy(tmp, pkeypem + end + 1 + step, start2 - end - step - 1);
+      
+      /* Remove whitespace. */
+      whitened = ssh_base64_remove_whitespace(tmp, start2 - end - step - 1);
+      ssh_xfree(tmp);
+      
+      /* Decode the base64 blob. */
+      *blob = ssh_base64_to_buf(whitened, bloblen);
+      ssh_xfree(whitened);
+      ssh_xfree(pkeypem);
+    }
+  else if ((magic == SSH_KEY_MAGIC_SSH1_PUBLIC) && 
+           (try_convert_ssh1_cert != FALSE))
+    {
+      SshPublicKey tmpkey;
+      unsigned char *tmpblob;
+      size_t tmpbloblen;
+
+      tmpkey = ssh1_decode_pubkeyblob(pkeypem, pkeylen, &my_comment);
+      if (tmpkey != NULL)
+        {
+          tmpbloblen = ssh_encode_pubkeyblob(tmpkey, &tmpblob);
+          ssh_public_key_free(tmpkey);
+          if (tmpbloblen > 0)
+            {
+              ssh_xfree(pkeypem);
+              pkeypem = tmpblob;
+              pkeylen = tmpbloblen;
+              magic = SSH_KEY_MAGIC_PUBLIC;
+              SSH_DEBUG(5, ("converted ssh1 pubkey to ssh2 certs"));
+            }
+          else
+            {
+              ssh_xfree(my_comment);
+              goto fail;
+            }
+        }
+      else
+        {
+          goto fail;
+        }
+      if (comment)
+        *comment = my_comment;
+      else
+        ssh_xfree(my_comment);
+      *blob = pkeypem;
+      if (bloblen)
+        *bloblen = pkeylen;
+    }
   else
-    ssh_xfree(my_comment);
-  
-  /* Convert the remainder to a string. */
-  tmp = ssh_xmalloc(start2 - end - step);
-  memcpy(tmp, pkeypem + end + 1 + step, start2 - end - step - 1);
+    {
+      /* Just put the filename into comment string for ssh1 keys. */
+      my_comment = ssh_xstrdup(fname);
 
-  /* Remove whitespace. */
-  whitened = ssh_base64_remove_whitespace(tmp, start2 - end - step - 1);
-  ssh_xfree(tmp);
-  
-  /* Decode the base64 blob. */
-  *blob = ssh_base64_to_buf(whitened, bloblen);
-  ssh_xfree(whitened);
-  ssh_xfree(pkeypem);
+      /* If comment is available pass it up. */
+      if (comment)
+        *comment = my_comment;
+      else
+        ssh_xfree(my_comment);
+
+      /* If it's an ssh1 key we just push entire blob up. */
+      *blob = pkeypem;
+      if (bloblen)
+        *bloblen = pkeylen;
+    }
+  SSH_DEBUG(5, ("key blob magic = 0x%08x"));
   return magic;
 
 fail:
@@ -417,7 +524,7 @@ fail:
 
    */
 
-void ssh_key_blob_dump_quoted_str(SshBuffer *buffer, size_t indend,
+void ssh_key_blob_dump_quoted_str(SshBuffer buffer, size_t indend,
                                   const char *buf)
 {
   size_t pos = indend;
@@ -438,12 +545,12 @@ void ssh_key_blob_dump_quoted_str(SshBuffer *buffer, size_t indend,
   ssh_buffer_append(buffer, (const unsigned char *)"\"", 1);
 }
 
-void ssh_key_blob_dump_line_str(SshBuffer *buffer, const char *str)
+void ssh_key_blob_dump_line_str(SshBuffer buffer, const char *str)
 {
   ssh_buffer_append(buffer, (const unsigned char *)str, strlen(str));
 }
 
-void ssh_key_blob_dump_str(SshBuffer *buffer, const char *str)
+void ssh_key_blob_dump_str(SshBuffer buffer, const char *str)
 {
   size_t pos;
   size_t i, str_len = strlen(str);
@@ -459,7 +566,7 @@ void ssh_key_blob_dump_str(SshBuffer *buffer, const char *str)
     }
 }
 
-void ssh_key_blob_dump_lf(SshBuffer *buffer)
+void ssh_key_blob_dump_lf(SshBuffer buffer)
 {
   ssh_buffer_append(buffer, (const unsigned char *)"\n", 1);
 }
@@ -469,7 +576,7 @@ Boolean ssh2_key_blob_write(SshUser user, const char *fname, mode_t mode,
                            const char *comment, const unsigned char *key,
                            size_t keylen, void *context)
 {
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   char *base64;
   unsigned int key_index;
 
@@ -538,17 +645,11 @@ Boolean ssh2_key_blob_write(SshUser user, const char *fname, mode_t mode,
 /* Creates a new random number generator, and loads the random seed file
    into the generator. */
 
-SshRandomState ssh_randseed_open(SshUser user, SshConfig config)
+void ssh_randseed_open(SshUser user, SshConfig config)
 {
-  SshRandomState random_state;
-
-  /* ok, allocate the random generator */
-  random_state = ssh_random_allocate();
-
   /* Load the random seed file and mix it into the generator. */
-  ssh_randseed_load(user, random_state, config);
-
-  return random_state;
+  ssh_randseed_load(user, config);
+  return;
 }
 
 /* Read a public key from a file. Return NULL on failure. */
@@ -562,18 +663,36 @@ SshPublicKey ssh_pubkey_read(SshUser user, const char *fname, char **comment,
   SshPublicKey pubkey;
 
   pkeybuf = NULL;
-  magic = ssh2_key_blob_read(user, fname, comment, &pkeybuf, &pkeylen, context);
+  magic = ssh2_key_blob_read(user, 
+                             fname, 
+                             FALSE,
+                             comment, 
+                             &pkeybuf, 
+                             &pkeylen, 
+                             context);
 
-  if (magic != SSH_KEY_MAGIC_PUBLIC)
-    goto fail;
-  pubkey = ssh_decode_pubkeyblob(pkeybuf, pkeylen);
-  if (pubkey == NULL)
+  if ((magic != SSH_KEY_MAGIC_PUBLIC) &&
+      (magic != SSH_KEY_MAGIC_SSH1_PUBLIC))
     goto fail;
 
+  if (magic == SSH_KEY_MAGIC_PUBLIC)
+    {
+      pubkey = ssh_decode_pubkeyblob(pkeybuf, pkeylen);
+      if (pubkey == NULL)
+        goto fail;
+    }
+  else if (magic == SSH_KEY_MAGIC_SSH1_PUBLIC)
+    {
+      if (comment)
+        ssh_xfree(*comment);
+      pubkey = ssh1_decode_pubkeyblob(pkeybuf, pkeylen, comment);
+      if (pubkey == NULL)
+        goto fail;
+    }
+
   return pubkey;
 
   /* cleanup */
-  
 fail:
   if (pkeybuf != NULL)
     {
@@ -608,8 +727,10 @@ Boolean ssh_pubkey_write(SshUser user, const char *fname, const char *comment,
 /* Read a private key from a file. Returns NULL on failure. */
 
 SshPrivateKey ssh_privkey_read(SshUser user, 
-                               const char *fname, const char *passphrase, 
-                               char **comment, void *context)
+                               const char *fname,
+                               const char *passphrase, 
+                               char **comment, 
+                               void *context)
 {
   SshCryptoStatus code;
   SshPrivateKey privkey;
@@ -618,10 +739,18 @@ SshPrivateKey ssh_privkey_read(SshUser user,
   unsigned long magic;
 
   pkeybuf = NULL;
-  magic = ssh2_key_blob_read(user, fname, comment, &pkeybuf, &pkeylen, context);
+  magic = ssh2_key_blob_read(user, 
+                             fname, 
+                             FALSE,
+                             comment, 
+                             &pkeybuf, 
+                             &pkeylen, 
+                             context);
 
-  if (magic != SSH_KEY_MAGIC_PRIVATE && 
-      magic != SSH_KEY_MAGIC_PRIVATE_ENCRYPTED)
+  if ((magic != SSH_KEY_MAGIC_PRIVATE) && 
+      (magic != SSH_KEY_MAGIC_PRIVATE_ENCRYPTED) &&
+      (magic != SSH_KEY_MAGIC_SSH1_PRIVATE) &&
+      (magic != SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED))
     goto fail;
   
   switch(magic)
@@ -640,7 +769,6 @@ SshPrivateKey ssh_privkey_read(SshUser user,
       break;
 
     case SSH_KEY_MAGIC_PRIVATE_ENCRYPTED:
-      
       if ((code = ssh_private_key_import_with_passphrase(pkeybuf,
                                                          pkeylen,
                                                          passphrase,
@@ -655,6 +783,42 @@ SshPrivateKey ssh_privkey_read(SshUser user,
         }
       break;
 
+    case SSH_KEY_MAGIC_SSH1_PRIVATE:
+      if (comment)
+        {
+          ssh_xfree(*comment);
+          *comment = NULL;
+        }
+      privkey = ssh1_decode_privkeyblob(pkeybuf, 
+                                        pkeylen,
+                                        "",
+                                        comment);
+      if (privkey == NULL)
+        {
+          if (comment)
+            *comment = NULL;
+          goto fail;
+        }
+      break;
+
+    case SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED:
+      if (comment)
+        {
+          ssh_xfree(*comment);
+          *comment = NULL;
+        }
+      privkey = ssh1_decode_privkeyblob(pkeybuf, 
+                                        pkeylen,
+                                        passphrase,
+                                        comment);
+      if (privkey == NULL)
+        {
+          if (comment)
+            *comment = NULL;
+          goto fail;
+        }
+      break;
+
     default:
       goto fail;
     }
@@ -681,7 +845,7 @@ fail:
 Boolean ssh_privkey_write(SshUser user,
                           const char *fname, const char *passphrase,
                           const char *comment,
-                          SshPrivateKey key, SshRandomState rand, 
+                          SshPrivateKey key, 
                           void *context)
 {
   unsigned char *pkeybuf;
@@ -692,7 +856,6 @@ Boolean ssh_privkey_write(SshUser user,
   if((code = ssh_private_key_export_with_passphrase(key, 
                                                     SSH_PASSPHRASE_CIPHER,
                                                     passphrase,
-                                                    rand,
                                                     &pkeybuf,
                                                     &pkeylen))
      != SSH_CRYPTO_OK)
