@@ -1,7 +1,7 @@
 /*
     Author: Mika Kojo <mkojo@ssh.fi>
 
-    Copyright (C) 1996 SSH Communications Security Oy, Espoo, Finland
+    Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
     All rights reserved.
 
     Created: Mon Oct 28 06:41:24 1996 [mkojo]
@@ -9,7 +9,7 @@
     */
 
 /*
- * $Id: genciph.c,v 1.41 1999/04/21 23:34:44 kivinen Exp $
+ * $Id: genciph.c,v 1.53 2000/05/28 16:20:34 kivinen Exp $
  * $Log: genciph.c,v $
  * $EndLog$
  */
@@ -21,18 +21,30 @@
 #include "sshbuffer.h"
 #include "sshgetput.h"
 #include "nociph.h"
+
+#ifdef SSHDIST_CRYPT_SHA
 #include "sha.h"
+#endif /* SSHDIST_CRYPT_SHA */
 
+#ifdef SSHDIST_CRYPT_DES
 #include "des.h"
+#endif /* SSHDIST_CRYPT_DES */
 
+#ifdef SSHDIST_CRYPT_BLOWFISH
 #include "blowfish.h"
+#endif /* SSHDIST_CRYPT_BLOWFISH */
 
 
 
-
+#ifdef SSHDIST_CRYPT_TWOFISH
 #include "twofish.h"
+#endif /* SSHDIST_CRYPT_TWOFISH */
+
+
+
+
+
 
- 
 
 #ifndef KERNEL
 /* These ciphers can only be used in user-mode code, not in the kernel.
@@ -40,27 +52,51 @@
    file to CRYPT_LNOBJS in src/ipsec/engine/Makefile.am, and move it
    outside the #ifndef KERNEL directive both here and later in this file. */
 
+#ifdef SSHDIST_CRYPT_ARCFOUR
 #include "arcfour.h"
+#endif /* SSHDIST_CRYPT_ARCFOUR */
 
 
 
 
 #endif /* !KERNEL */
 
+#define SSH_DEBUG_MODULE "SshGenCiph"
+
+/* Some definitions to make memory allocation more readable between
+   the user- and kernel-mode differences. Yes, when we are in usermode
+   we don't need to check the return value of allocators, but, we need
+   to do that when in kernel mode. So, we opt to lowest common
+   denominator and always check the return value */
+#ifdef _KERNEL
+#define MEM_ALLOC(S)            ssh_kmalloc(S)
+#define MEM_FREE(P)             ssh_kfree(P)
+#define MEM_REALLOC(P,O,S)      ssh_krealloc(P,O,S)
+#define MEM_STRDUP(S)           ssh_kstrdup(S)
+#else /* _KERNEL */
+#define MEM_ALLOC(S)            ssh_xmalloc(S)
+#define MEM_FREE(P)             ssh_xfree(P)
+#define MEM_REALLOC(P,O,S)      ssh_xrealloc(P,S)
+#define MEM_STRDUP(S)           ssh_xstrdup(S)
+#endif /* _KERNEL */
+
 /* Algorithm definitions */
 
 static const SshCipherDef ssh_cipher_algorithms[] =
 {
-  { "3des-ecb", 8, 24, ssh_des3_ctxsize, ssh_des3_init, ssh_des3_init,
-    ssh_des3_ecb },
-  { "3des-cbc", 8, 24, ssh_des3_ctxsize, ssh_des3_init, ssh_des3_init,
-    ssh_des3_cbc },
-  { "3des-cfb", 8, 24, ssh_des3_ctxsize, ssh_des3_init, ssh_des3_init,
-    ssh_des3_cfb },
-  { "3des-ofb", 8, 24, ssh_des3_ctxsize, ssh_des3_init, ssh_des3_init,
-    ssh_des3_ofb },
+#ifdef SSHDIST_CRYPT_DES
+  { "3des-ecb", 8, 24, ssh_des3_ctxsize, ssh_des3_init,
+    ssh_des3_init_with_key_check, ssh_des3_ecb },
+  { "3des-cbc", 8, 24, ssh_des3_ctxsize, ssh_des3_init,
+    ssh_des3_init_with_key_check, ssh_des3_cbc },
+  { "3des-cfb", 8, 24, ssh_des3_ctxsize, ssh_des3_init,
+    ssh_des3_init_with_key_check, ssh_des3_cfb },
+  { "3des-ofb", 8, 24, ssh_des3_ctxsize, ssh_des3_init,
+    ssh_des3_init_with_key_check, ssh_des3_ofb },
+#endif /* SSHDIST_CRYPT_DES */
 
-  
+
+#ifdef SSHDIST_CRYPT_BLOWFISH
   { "blowfish-ecb", 8, 0,
     ssh_blowfish_ctxsize, ssh_blowfish_init, ssh_blowfish_init,
     ssh_blowfish_ecb },
@@ -73,7 +109,9 @@ static const SshCipherDef ssh_cipher_algorithms[] =
   { "blowfish-ofb", 8, 0,
     ssh_blowfish_ctxsize, ssh_blowfish_init, ssh_blowfish_init,
     ssh_blowfish_ofb },
-    
+#endif /* SSHDIST_CRYPT_BLOWFISH */
+
+#ifdef SSHDIST_CRYPT_DES
   { "des-ecb", 8, 8, ssh_des_ctxsize, ssh_des_init,
     ssh_des_init_with_key_check, ssh_des_ecb },
   { "des-cbc", 8, 8, ssh_des_ctxsize, ssh_des_init,
@@ -82,8 +120,10 @@ static const SshCipherDef ssh_cipher_algorithms[] =
     ssh_des_init_with_key_check, ssh_des_cfb },
   { "des-ofb", 8, 8, ssh_des_ctxsize, ssh_des_init,
     ssh_des_init_with_key_check, ssh_des_ofb },
-  
+#endif /* SSHDIST_CRYPT_DES */
 
+
+#ifdef SSHDIST_CRYPT_TWOFISH
   { "twofish-ecb", 16, 0,
     ssh_twofish_ctxsize, ssh_twofish_init, ssh_twofish_init, ssh_twofish_ecb },
   { "twofish-cbc", 16, 0,
@@ -92,6 +132,10 @@ static const SshCipherDef ssh_cipher_algorithms[] =
     ssh_twofish_ctxsize, ssh_twofish_init, ssh_twofish_init, ssh_twofish_cfb },
   { "twofish-ofb", 16, 0,
     ssh_twofish_ctxsize, ssh_twofish_init, ssh_twofish_init, ssh_twofish_ofb },
+#endif /* SSHDIST_CRYPT_TWOFISH */
+
+
+
 
 
 
@@ -99,18 +143,20 @@ static const SshCipherDef ssh_cipher_algorithms[] =
 #ifndef KERNEL
   /* The ciphers below can only be used in user-level code.  See
      the comments above for adding ciphers to the kernel. */
-  
 
+
+#ifdef SSHDIST_CRYPT_ARCFOUR
   { "arcfour", 1, 0, ssh_arcfour_ctxsize, ssh_arcfour_init, ssh_arcfour_init,
     ssh_arcfour_transform },
+#endif /* SSHDIST_CRYPT_ARCFOUR */
+
 
 
 
-  
 #endif /* !KERNEL */
-  
+
   { "none", 1, 0, NULL, NULL, NULL, ssh_none_cipher },
-  
+
   { NULL }
 };
 
@@ -123,10 +169,18 @@ struct SshCipherAliasRec {
 /* Common cipher names. */
 const struct SshCipherAliasRec ssh_cipher_aliases[] =
 {
+#ifdef SSHDIST_CRYPT_DES
   { "des", "des-cbc" },
+#endif /* SSHDIST_CRYPT_DES */
+#ifdef SSHDIST_CRYPT_DES
   { "3des", "3des-cbc" },
+#endif /* SSHDIST_CRYPT_DES */
+#ifdef SSHDIST_CRYPT_BLOWFISH
   { "blowfish", "blowfish-cbc" },
+#endif /* SSHDIST_CRYPT_BLOWFISH */
+#ifdef SSHDIST_CRYPT_TWOFISH
   { "twofish", "twofish-cbc" },
+#endif /* SSHDIST_CRYPT_TWOFISH */
   { NULL, NULL }
 };
 
@@ -170,7 +224,7 @@ static const SshCipherDef *ssh_cipher_get_cipher_def_internal(const char *name)
 
 /* Get the native name of the cipher. */
 
-DLLEXPORT char * DLLCALLCONV
+char *
 ssh_cipher_get_native_name(const char *name)
 {
   const SshCipherDef *cipher_def;
@@ -180,7 +234,7 @@ ssh_cipher_get_native_name(const char *name)
   if (cipher_def == NULL)
     return NULL;
 
-  return ssh_xstrdup(cipher_def->name);
+  return MEM_STRDUP(cipher_def->name);
 }
 
 /* Check if given cipher name belongs to the set of supported ciphers
@@ -205,7 +259,7 @@ static Boolean ssh_cipher_supported_native(const char *name)
 /* Check if given cipher name belongs to the set of supported ciphers
    aliases included. */
 
-DLLEXPORT Boolean DLLCALLCONV
+Boolean
 ssh_cipher_supported(const char *name)
 {
   if (ssh_cipher_get_cipher_def_internal(name) != NULL)
@@ -216,12 +270,43 @@ ssh_cipher_supported(const char *name)
 
 /* Return a comma-separated list of supported native cipher algorithm names. */
 
-DLLEXPORT char * DLLCALLCONV
+char *
 ssh_cipher_get_supported_native(void)
 {
-  int i;
-  SshBuffer buf;
   char *list;
+  int i;
+  size_t offset, list_len;
+
+  list = NULL;
+  offset = list_len = 0;
+
+  for (i = 0; ssh_cipher_algorithms[i].name != NULL; i++)
+    {
+      size_t newsize;
+
+      newsize = offset + 1 + !!offset + strlen(ssh_cipher_algorithms[i].name);
+
+      if (list_len < newsize)
+        {
+          newsize *= 2;
+
+          /* XXX: this leaks memory if in kernel */
+          if ((list = MEM_REALLOC(list, list_len, newsize)) == NULL)
+            return NULL;
+
+          list_len = newsize;
+        }
+
+      SSH_ASSERT(list_len > 0);
+      SSH_ASSERT(list != NULL);
+
+      offset += snprintf(list + offset, list_len - offset, "%s%s",
+                         offset ? "," : "",
+                         ssh_cipher_algorithms[i].name);
+
+    }
+#if 0
+  SshBufferStruct buf;
 
   ssh_buffer_init(&buf);
   for (i = 0; ssh_cipher_algorithms[i].name != NULL; i++)
@@ -234,18 +319,55 @@ ssh_cipher_get_supported_native(void)
   ssh_buffer_append(&buf, (unsigned char *) "\0", 1);
   list = ssh_xstrdup(ssh_buffer_ptr(&buf));
   ssh_buffer_uninit(&buf);
+#endif /* _KERNEL */
   return list;
 }
 
 /* Return a comma-separated list of supported cipher algorithm names
    alias names included. */
 
-DLLEXPORT char * DLLCALLCONV
+char *
 ssh_cipher_get_supported(void)
 {
   int i;
-  SshBuffer buf;
   char *list;
+  size_t offset, list_len;
+
+  list = NULL;
+  offset = list_len = 0;
+
+  if (!(list = ssh_cipher_get_supported_native()))
+    return NULL;
+
+  offset = strlen(list);
+  list_len = offset + 1;
+
+  for (i = 0; ssh_cipher_aliases[i].name != NULL; i++)
+    {
+      size_t newsize;
+      newsize = offset + 1 + !!offset + strlen(ssh_cipher_aliases[i].name);
+
+      if (!ssh_cipher_supported_native(ssh_cipher_aliases[i].name))
+        continue;
+
+      if (list_len < newsize)
+        {
+          newsize *= 2;
+
+          /* XXX: This leaks memory if in kernel */
+          if ((list = MEM_REALLOC(list, list_len, newsize)) == NULL)
+            return NULL;
+
+          list_len = newsize;
+        }
+
+      offset += snprintf(list + offset, list_len - offset, "%s%s",
+                         offset ? "," : "",
+                         ssh_cipher_aliases[i].name);
+
+    }
+#if 0
+  SshBufferStruct buf;
 
   ssh_buffer_init(&buf);
   list = ssh_cipher_get_supported_native();
@@ -258,7 +380,7 @@ ssh_cipher_get_supported(void)
         {
           if (ssh_buffer_len(&buf) != 0)
             ssh_buffer_append(&buf, (unsigned char *) ",", 1);
-          ssh_buffer_append(&buf, 
+          ssh_buffer_append(&buf,
                             (unsigned char *) ssh_cipher_aliases[i].name,
                             strlen(ssh_cipher_aliases[i].name));
         }
@@ -266,12 +388,13 @@ ssh_cipher_get_supported(void)
   ssh_buffer_append(&buf, (unsigned char *) "\0", 1);
   list = ssh_xstrdup(ssh_buffer_ptr(&buf));
   ssh_buffer_uninit(&buf);
+#endif /* _KERNEL */
   return list;
 }
 
 /* Allocates and initializes a cipher of the specified name. */
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_allocate_internal(const char *name,
                              const unsigned char *key,
                              size_t keylen,
@@ -284,7 +407,7 @@ ssh_cipher_allocate_internal(const char *name,
   unsigned int expanded_key_len;
   const SshCipherDef *cipher_def;
   Boolean rv;
-  
+
   cipher_def = ssh_cipher_get_cipher_def_internal(name);
   if (cipher_def == NULL)
     return SSH_CRYPTO_UNSUPPORTED;
@@ -300,14 +423,17 @@ ssh_cipher_allocate_internal(const char *name,
      expansion function defined in the genhash.c. It basically just
      recomputes the hash function of choice until enough key material
      is available. */
-  
+
   if (expand)
     {
       expanded_key_len = cipher_def->key_length;
       if (expanded_key_len == 0)
         expanded_key_len = SSH_CIPHER_MINIMAL_KEY_LENGTH;
-      
-      expanded_key = ssh_xmalloc(expanded_key_len);
+
+      /* XXX: Is this proper error code to return on memory allocation failure? */
+      if (!(expanded_key = MEM_ALLOC(expanded_key_len)))        
+        return SSH_CRYPTO_OPERATION_FAILED;
+
       ssh_hash_expand_key_internal(expanded_key, expanded_key_len,
                                    key, keylen,
                                    NULL, 0,
@@ -320,7 +446,7 @@ ssh_cipher_allocate_internal(const char *name,
   else
     {
       /* No need to expand the key. */
-      
+
       expanded_key_len = keylen;
       expanded_key = (unsigned char *)key;
     }
@@ -332,13 +458,19 @@ ssh_cipher_allocate_internal(const char *name,
     {
       if (expand)
         ssh_fatal("internal error: key expansion corrupted.");
-      
+
       return SSH_CRYPTO_KEY_TOO_SHORT;
     }
 
   /* Initialize the cipher. */
 
-  *cipher = ssh_xmalloc(sizeof(**cipher));
+  if (!(*cipher = MEM_ALLOC(sizeof(**cipher))))
+    {
+      MEM_FREE(expanded_key);
+
+      /* XXX: Correct return value on memory allocation failure? */
+      return SSH_CRYPTO_OPERATION_FAILED;
+    }
 
   /* Set up the cipher definition. */
   (*cipher)->ops = cipher_def;
@@ -352,8 +484,17 @@ ssh_cipher_allocate_internal(const char *name,
   if (cipher_def->ctxsize)
     {
       /* Allocate the context of the cipher. */
-      (*cipher)->context = ssh_xmalloc((*cipher_def->ctxsize)());
-      
+      if (!((*cipher)->context = MEM_ALLOC((*cipher_def->ctxsize)())))
+        {
+          MEM_FREE(*cipher);
+
+          if (expand)
+            MEM_FREE(expanded_key);
+
+          /* XXX: again, correct code? */
+          return SSH_CRYPTO_OPERATION_FAILED;
+        }
+
       if (test_weak_keys == FALSE)
         {
           /* Initialize the cipher without weak key checks. */
@@ -367,7 +508,7 @@ ssh_cipher_allocate_internal(const char *name,
           /* Initialize the cipher with a weak key check performed first.
              Not all ciphers have key classes that are easy or practical to
              test for. For those ciphers this function may perform
-             as the plain initialization. 
+             as the plain initialization.
              */
           rv = (*cipher_def->init_with_check)((*cipher)->context,
                                               expanded_key,
@@ -380,19 +521,19 @@ ssh_cipher_allocate_internal(const char *name,
 
   /* Free memory of the expanded key if necessary. */
   if (expand)
-    ssh_xfree(expanded_key);
+    MEM_FREE(expanded_key);
 
   if (rv == FALSE)
     {
-      ssh_xfree((*cipher)->context);
-      ssh_xfree(*cipher);
+      MEM_FREE((*cipher)->context);
+      MEM_FREE(*cipher);
       *cipher = NULL;
       return SSH_CRYPTO_OPERATION_FAILED;
     }
   return SSH_CRYPTO_OK;
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_allocate(const char *name,
                     const unsigned char *key,
                     size_t keylen,
@@ -403,7 +544,7 @@ ssh_cipher_allocate(const char *name,
                                       cipher, FALSE, FALSE);
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_allocate_with_passphrase(const char *name,
                                     const char *passphrase,
                                     Boolean for_encryption,
@@ -414,7 +555,7 @@ ssh_cipher_allocate_with_passphrase(const char *name,
                                       for_encryption, cipher, TRUE, FALSE);
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_allocate_and_test_weak_keys(const char *name,
                                        const unsigned char *key,
                                        size_t keylen,
@@ -428,20 +569,20 @@ ssh_cipher_allocate_and_test_weak_keys(const char *name,
 
 /* Free the cipher context */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_cipher_free(SshCipher cipher)
 {
-  ssh_xfree(cipher->context);
-  ssh_xfree(cipher);
+  MEM_FREE(cipher->context);
+  MEM_FREE(cipher);
 }
 
-DLLEXPORT char * DLLCALLCONV
+char *
 ssh_cipher_get_name(SshCipher cipher)
 {
-  return ssh_xstrdup(cipher->ops->name);
+  return MEM_STRDUP(cipher->ops->name);
 }
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_cipher_get_key_length(const char *name)
 {
   const SshCipherDef *cipher_def;
@@ -453,20 +594,20 @@ ssh_cipher_get_key_length(const char *name)
   return cipher_def->key_length;
 }
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_cipher_get_block_length(SshCipher cipher)
 {
   return cipher->ops->block_length;
 }
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_cipher_get_iv_length(SshCipher cipher)
 {
   /* XXX Currently just returns the block length. */
   return cipher->ops->block_length;
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_set_iv(SshCipher cipher,
                   const unsigned char *iv)
 {
@@ -475,7 +616,7 @@ ssh_cipher_set_iv(SshCipher cipher,
   return SSH_CRYPTO_OK;
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_get_iv(SshCipher cipher,
                   unsigned char *iv)
 {
@@ -483,7 +624,7 @@ ssh_cipher_get_iv(SshCipher cipher,
   return SSH_CRYPTO_OK;
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_transform(SshCipher cipher,
                      unsigned char *dest,
                      const unsigned char *src,
@@ -498,7 +639,7 @@ ssh_cipher_transform(SshCipher cipher,
   return SSH_CRYPTO_OK;
 }
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_cipher_transform_with_iv(SshCipher cipher,
                              unsigned char *dest,
                              const unsigned char *src,
