@@ -5,13 +5,13 @@ genhash.c
 Author: Antti Huima <huima@ssh.fi>
         Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (C) 1996 SSH Security Communications Oy, Espoo, Finland
+Copyright (C) 1996-2000 SSH Security Communications Oy, Espoo, Finland
                    All rights reserved
-                 
+
 */
 
 /*
- * $Id: genhash.c,v 1.24 1999/01/13 19:31:10 ylo Exp $
+ * $Id: genhash.c,v 1.31 2000/05/28 16:20:35 kivinen Exp $
  * $Log: genhash.c,v $
  * $EndLog$
  */
@@ -20,29 +20,58 @@ Copyright (C) 1996 SSH Security Communications Oy, Espoo, Finland
 #include "sshcrypt.h"
 #include "sshcrypti.h"
 #include "sshbuffer.h"
+#ifdef SSHDIST_CRYPT_SHA
 #include "sha.h"
+#endif /* SSHDIST_CRYPT_SHA */
+#ifdef SSHDIST_CRYPT_MD5
 #include "md5.h"
+#endif /* SSHDIST_CRYPT_MD5 */
+#ifdef SSHDIST_CRYPT_RIPEMD160
 #include "ripemd160.h"
+#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
 #ifndef KERNEL
 
 
 #endif /* !KERNEL */
 
+/* Some definitions to make memory allocation more readable between
+   the user- and kernel-mode differences. Yes, when we are in usermode
+   we don't need to check the return value of allocators, but, we need
+   to do that when in kernel mode. So, we opt to lowest common
+   denominator and always check the return value */
+#ifdef _KERNEL
+#define MEM_ALLOC(S)            ssh_kmalloc(S)
+#define MEM_FREE(P)             ssh_kfree(P)
+#define MEM_REALLOC(P,O,S)      ssh_krealloc(P,O,S)
+#define MEM_STRDUP(S)           ssh_kstrdup(S)
+#else /* _KERNEL */
+#define MEM_ALLOC(S)            ssh_xmalloc(S)
+#define MEM_FREE(P)             ssh_xfree(P)
+#define MEM_REALLOC(P,O,S)      ssh_xrealloc(P,S)
+#define MEM_STRDUP(S)           ssh_xstrdup(S)
+#endif /* _KERNEL */
+
 /* List of supported hash algorithms. */
 
 static const SshHashDef *ssh_hash_algorithms[] =
 {
+#ifdef SSHDIST_CRYPT_MD5
   &ssh_hash_md5_def,
+#endif /* SSHDIST_CRYPT_MD5 */
+#ifdef SSHDIST_CRYPT_SHA
   &ssh_hash_sha_def,
   &ssh_hash_sha_96_def,
   &ssh_hash_sha_80_def,
+#endif /* SSHDIST_CRYPT_SHA */
+#ifdef SSHDIST_CRYPT_RIPEMD160
   &ssh_hash_ripemd160_def,
   &ssh_hash_ripemd160_96_def,
   &ssh_hash_ripemd160_80_def,
+#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
 #ifndef KERNEL
-  
+
 
 #endif /* KERNEL */
 
@@ -64,14 +93,44 @@ const SshHashDef *ssh_hash_get_definition_internal(const SshHash hash)
 /* Returns a comma-separated list of supported hash functions names.
    The caller must free the returned value with ssh_xfree(). */
 
-DLLEXPORT char * DLLCALLCONV
-ssh_hash_get_supported()
+char *
+ssh_hash_get_supported(void)
 {
   int i;
-  SshBuffer buf;
+  size_t list_len, offset;
+  char *list;
+
+  list = NULL;
+  offset = list_len = 0;
+
+  for (i = 0; ssh_hash_algorithms[i] != NULL; i++)
+    {
+      size_t newsize;
+      newsize = offset + 1 + !!offset + strlen(ssh_hash_algorithms[i]->name);
+
+      if (list_len < newsize)
+        {
+          newsize *= 2;
+
+          /* XXX: This leaks memory if in kernel */
+          if ((list = MEM_REALLOC(list, list_len, newsize)) == NULL)
+            return NULL;
+
+          list_len = newsize;
+        }
+
+      offset += snprintf(list + offset, list_len - offset, "%s%s",
+                         offset ? "," : "",
+                         ssh_hash_algorithms[i]->name);
+
+    }
+#if 0
+  int i;
+  SshBufferStruct buf;
   char *list;
 
   ssh_buffer_init(&buf);
+
   for (i = 0; ssh_hash_algorithms[i] != NULL; i++)
     {
       if (ssh_buffer_len(&buf) != 0)
@@ -79,22 +138,25 @@ ssh_hash_get_supported()
       ssh_buffer_append(&buf, (unsigned char *) ssh_hash_algorithms[i]->name,
                     strlen(ssh_hash_algorithms[i]->name));
     }
+
   ssh_buffer_append(&buf, (unsigned char *) "\0", 1);
   list = ssh_xstrdup(ssh_buffer_ptr(&buf));
   ssh_buffer_uninit(&buf);
+#endif /* _KERNEL */
+
   return list;
 }
 
 /* Check if given hash name belongs to the set of supported ciphers. */
 
-DLLEXPORT Boolean DLLCALLCONV
+Boolean
 ssh_hash_supported(const char *name)
 {
   unsigned int i;
 
   if (name == NULL)
     return FALSE;
-  
+
   for (i = 0; ssh_hash_algorithms[i] != NULL; i++)
     if (strcmp(ssh_hash_algorithms[i]->name, name) == 0)
       return TRUE;
@@ -103,7 +165,7 @@ ssh_hash_supported(const char *name)
 
 /* Allocates and initializes a hash context. */
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_hash_allocate(const char *type, SshHash *hash)
 {
   int i;
@@ -112,9 +174,14 @@ ssh_hash_allocate(const char *type, SshHash *hash)
     {
       if (strcmp(ssh_hash_algorithms[i]->name, type) == 0)
         {
-          *hash = ssh_xmalloc(sizeof(**hash));
+          if (!(*hash = MEM_ALLOC(sizeof(**hash))))
+            return SSH_CRYPTO_OPERATION_FAILED;
           (*hash)->ops = ssh_hash_algorithms[i];
-          (*hash)->context = ssh_xmalloc((*ssh_hash_algorithms[i]->ctxsize)());
+          if (!((*hash)->context = MEM_ALLOC((*ssh_hash_algorithms[i]->ctxsize)())))
+            {
+              MEM_FREE(*hash);
+              return SSH_CRYPTO_OPERATION_FAILED;
+            }
           (*ssh_hash_algorithms[i]->reset_context)((*hash)->context);
           return SSH_CRYPTO_OK;
         }
@@ -127,21 +194,51 @@ ssh_hash_allocate(const char *type, SshHash *hash)
    any "standard" hash function) with this interface. Defined in
    sshcrypti.h for internal usage only. */
 
-DLLEXPORT SshHash DLLCALLCONV
+SshHash
 ssh_hash_allocate_internal(const SshHashDef *hash_def)
 {
   SshHash hash;
 
   if (hash_def == NULL)
     return NULL;
-  hash = ssh_xmalloc(sizeof(*hash));
+  if (!(hash = MEM_ALLOC(sizeof(*hash))))
+    return NULL;
   hash->ops = hash_def;
-  hash->context = ssh_xmalloc((hash_def->ctxsize)());
+  if (!(hash->context = MEM_ALLOC((hash_def->ctxsize)())))
+    {
+      MEM_FREE(hash);
+      return NULL;
+    }
   (hash_def->reset_context)(hash->context);
 
   return hash;
 }
 
+/* Duplicate the given hash with correctly duplicating the internal state
+   of the hash function also. */
+SshHash
+ssh_hash_duplicate(const SshHash hash)
+{
+  SshHash new_hash;
+
+  if (hash == NULL)
+    return NULL;
+
+  if (!(new_hash = MEM_ALLOC(sizeof(*new_hash))))
+    return NULL;
+  new_hash->ops     = hash->ops;
+  /* Copy the state. */
+  if (!(new_hash->context = MEM_ALLOC((*new_hash->ops->ctxsize)())))
+    {
+      MEM_FREE(new_hash);
+      return NULL;
+    }
+  memcpy(new_hash->context, hash->context,
+         (*new_hash->ops->ctxsize)());
+
+  return new_hash;
+}
+
 /* Expand given key (ps, ps_len) with pseudo-random function to be of
    length ssh_buffer_len. Magic can be given to make hashing output
    different results for different operations. Although it is not
@@ -158,7 +255,7 @@ ssh_hash_allocate_internal(const SshHashDef *hash_def)
    where h_i are combined as h_0 | h_1 | h_2 ... to form the expanded
    key. The function f(i) should be some bijective function (maybe
    just f(x) = x ?).
-   
+
    This function is currently meant only for internal use. */
 
 void ssh_hash_expand_key_internal(unsigned char *buffer, size_t buffer_len,
@@ -170,7 +267,7 @@ void ssh_hash_expand_key_internal(unsigned char *buffer, size_t buffer_len,
   size_t hash_buf_len;
   void *context;
   size_t i;
-  
+
   /* Hash and expand the passphrase. Idea is to
 
        for i = 0 to r
@@ -184,7 +281,9 @@ void ssh_hash_expand_key_internal(unsigned char *buffer, size_t buffer_len,
     hash->digest_length;
 
   /* Allocate just once for simplicity in freeing memory. */
-  context = ssh_xmalloc((*hash->ctxsize)() + hash_buf_len);
+  /* XXX: How could we recover from allocation failure here? */
+  if (!(context = MEM_ALLOC((*hash->ctxsize)() + hash_buf_len)))
+    ssh_fatal("Could not allocate memory for key expansion");
   hash_buf = (unsigned char *)context + (*hash->ctxsize)();
 
   /* Iterate. */
@@ -201,21 +300,21 @@ void ssh_hash_expand_key_internal(unsigned char *buffer, size_t buffer_len,
   /* Copy and free. */
   memcpy(buffer, hash_buf, buffer_len);
   memset(hash_buf, 0, hash_buf_len);
-  ssh_xfree(context);
+  MEM_FREE(context);
 }
 
 /* Free hash context. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_hash_free(SshHash hash)
 {
-  ssh_xfree(hash->context);
-  ssh_xfree(hash);
+  MEM_FREE(hash->context);
+  MEM_FREE(hash);
 }
 
 /* Returns the ASN.1 Object Identifier of the hash function if
    known. Returns NULL if OID is not known. */
-DLLEXPORT const char * DLLCALLCONV
+const char *
 ssh_hash_asn1_oid(SshHash hash)
 {
   return hash->ops->asn1_oid;
@@ -223,7 +322,7 @@ ssh_hash_asn1_oid(SshHash hash)
 
 /* Returns the ISO/IEC dedicated hash number if available. 0 if not
    known. */
-DLLEXPORT unsigned char DLLCALLCONV
+unsigned char
 ssh_hash_iso_identifier(SshHash hash)
 {
   return hash->ops->iso_identifier;
@@ -231,7 +330,7 @@ ssh_hash_iso_identifier(SshHash hash)
 
 /* Resets the hash context to its initial state. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_hash_reset(SshHash hash)
 {
   (*hash->ops->reset_context)(hash->context);
@@ -239,7 +338,7 @@ ssh_hash_reset(SshHash hash)
 
 /* Get the digest lenght of the hash. */
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_hash_digest_length(SshHash hash)
 {
   return hash->ops->digest_length;
@@ -247,7 +346,7 @@ ssh_hash_digest_length(SshHash hash)
 
 /* Get input block size (used for hmac padding). */
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_hash_input_block_size(SshHash hash)
 {
   return hash->ops->input_block_length;
@@ -255,7 +354,7 @@ ssh_hash_input_block_size(SshHash hash)
 
 /* Updates the hash context by adding the given text. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_hash_update(SshHash hash, const void *buf, size_t len)
 {
   (*hash->ops->update)(hash->context, buf, len);
@@ -263,16 +362,36 @@ ssh_hash_update(SshHash hash, const void *buf, size_t len)
 
 /* Outputs the hash digest. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_hash_final(SshHash hash, unsigned char *digest)
 {
   (*hash->ops->final)(hash->context, digest);
 }
 
+/* Allocates space for digest and outputs the digest and digest length.
+   Caller should free the digest with ssh_xfree. */
+void
+ssh_hash_final_alloc(SshHash hash, unsigned char **digest, size_t *digest_len)
+{
+  size_t len;
+  unsigned char *buf;
+
+  len = ssh_hash_digest_length(hash);
+  buf = ssh_xmalloc(len + 1);
+  buf[len] = 0;
+  ssh_hash_final(hash, buf);
+  if (digest != NULL)
+    *digest = buf;
+  else
+    ssh_xfree(digest);
+  if (digest_len != NULL)
+    *digest_len = len;
+}
+
 /* Hashes one buffer with selected hash type and returns the digest.
    This calls ssh_fatal() if called with an invalid type. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_hash_of_buffer(const char *type,
                    const void *buf, size_t len,
                    unsigned char *digest)
