@@ -1,7 +1,7 @@
 /*
 
   keyblob.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
   Copyright (C) 1996 SSH Communications Security Oy, Espoo, Finland
@@ -14,14 +14,14 @@
   */
 
 /*
- * $Id: keyblob.c,v 1.11 1999/01/21 12:38:21 tri Exp $
+ * $Id: keyblob.c,v 1.19 2000/05/28 16:20:37 kivinen Exp $
  * $Log: keyblob.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
-#include "base64.h"
-#include "crc32.h"
+#include "sshbase64.h"
+#include "sshcrc32.h"
 #include "sshbuffer.h"
 #include "keyblob.h"
 #include "sshgetput.h"
@@ -29,7 +29,7 @@
 unsigned char *get_next_chars(unsigned char *src, unsigned char *dest,
                               size_t len)
 {
-  int max;
+  size_t max;
 
   max = strlen((char *) src);
   if (max == 0)
@@ -53,7 +53,8 @@ unsigned char *get_next_chars(unsigned char *src, unsigned char *dest,
   return src;
 }
 
-#define LEGAL_HEADER_CHAR(c) (isalnum((c)) || ((c) == '-') || ((c) == '_'))
+#define LEGAL_HEADER_CHAR(c) \
+  (isalnum((int)(c)) || ((c) == '-') || ((c) == '_'))
 
 Boolean looking_at_header(const char *s)
 {
@@ -82,7 +83,7 @@ void skip_headers(const char **s)
           escaped = ((**s == '\\') || (escaped && **s == '\r'));
           (*s)++;
         }
-      while (isspace(**s))
+      while (isspace((int)(**s)))
         (*s)++;
     }
 }
@@ -107,8 +108,8 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
   size_t base64blob_len, crc64blob_len;
   const char *p, *compare, *again, *headers_beg, *headers_end;
   unsigned char *p2;
-  Boolean public;
-  unsigned long crc32;
+  Boolean publicp;
+  SshUInt32 crc32;
 
   if (version_major != NULL)
     *version_major = 0;
@@ -116,29 +117,31 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
     *version_minor = 0;
   if (blob_len != NULL)
     *blob_len = 0;
-  
+
   p = str;
 
 #define SKIP_SPACE(p) \
-  do { while (isspace(*(p))) (p)++; } while (0)
+  do { while (isspace((int)*(p))) (p)++; } while (0)
 #define EXPECT_CHAR(p,ch) \
   do { if (*(p) != (ch)) goto error; } while (0)
 #define SKIP_CHARS(p,ch) \
-  do { while (*(p) == (ch) || isspace(*(p))) (p)++; } while (0)
+  do { while (*(p) == (ch) || isspace((int)*(p))) (p)++; } while (0)
 #define MATCH_STRING(p,str) \
-  do { for(; *(p) && *(str) == tolower(*(p));(str)++) { (p)++; SKIP_SPACE(p);}\
+  do { \
+    for(; *(p) && *(str) == tolower((int)*(p));(str)++) { (p)++; \
+    SKIP_SPACE(p);}\
   } while (0)
 #define MATCH_STRING_GOTO_ERROR(p,str) \
   do { MATCH_STRING((p),(str)); if (*(str)) goto error; } while (0)
 #define READ_NUMBER(p,n) \
-  do { (n) = 0; SKIP_SPACE(p); for(; isdigit(*(p)); ) \
+  do { (n) = 0; SKIP_SPACE(p); for(; isdigit((int)*(p)); ) \
       { (n) = ((n) * 10) + (*(p) - '0'); (p)++; SKIP_SPACE(p); } } while (0)
 
   SKIP_SPACE(p);
   /* Read begin line */
   EXPECT_CHAR(p, '-');
   SKIP_CHARS(p, '-');
-  
+
   compare = "beginssh";
   MATCH_STRING_GOTO_ERROR(p, compare);
   again = p;
@@ -149,17 +152,17 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
       p = again;
       compare = "private";
       MATCH_STRING_GOTO_ERROR(p, compare);
-      public = FALSE;
+      publicp = FALSE;
     }
   else
     {
-      public = TRUE;
+      publicp = TRUE;
     }
   compare = "keyblock";
   MATCH_STRING_GOTO_ERROR(p, compare);
   EXPECT_CHAR(p, '-');
   SKIP_CHARS(p, '-');
-  
+
   /* Read version line */
   compare = "version:";
   MATCH_STRING_GOTO_ERROR(p, compare);
@@ -181,9 +184,9 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
   headers_beg = p;
   skip_headers(&p);
   headers_end = p;
-  if ((headers_end != headers_beg) && (!(isspace(*headers_end))))
+  if ((headers_end != headers_beg) && (!(isspace((int)*headers_end))))
     headers_end--;
-  while (isspace(*headers_end) && (headers_end > headers_beg))
+  while (isspace((int)*headers_end) && (headers_end > headers_beg))
     headers_end--;
   if (headers_end != headers_beg)
     headers_end++;
@@ -210,7 +213,7 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
   SKIP_CHARS(p, '-');
   compare = "endssh";
   MATCH_STRING_GOTO_ERROR(p, compare);
-  if (public)
+  if (publicp)
     compare = "public";
   else
     compare = "private";
@@ -220,10 +223,10 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
   MATCH_STRING_GOTO_ERROR(p, compare);
   EXPECT_CHAR(p, '-');
   SKIP_CHARS(p, '-');
-  
+
   if (is_public)
-    *is_public = public;
-  
+    *is_public = publicp;
+
   blob = ssh_base64_remove_whitespace((const unsigned char *) crc64blob,
                                       crc64blob_len);
   p2 = blob;
@@ -258,7 +261,7 @@ unsigned char *ssh_key_blob_read_from_string(const char *str,
   if (blob_len)
     *blob_len = base64blob_len;
   return blob;
-  
+
 error:
   if (blob_len)
     blob_len = 0;
@@ -293,20 +296,13 @@ unsigned char *ssh_key_blob_read(FILE *fp, size_t *blob_len, char **comments)
           buffer = ssh_xrealloc(buffer, max_len);
         }
     }
-  r = ssh_key_blob_read_from_string(buffer, 
-                                    blob_len, 
+  r = ssh_key_blob_read_from_string(buffer,
+                                    blob_len,
                                     comments,
                                     NULL,
                                     NULL,
                                     NULL);
 
-  if (r)
-    if (*comments)
-      fprintf(stderr, ">>>comments>>>%s<<<\n", *comments);
-    else
-      fprintf(stderr, ">>>comments>>> *** NONE *** <<<\n");
-      
-
   ssh_xfree(buffer);
   return r;
 }
@@ -315,7 +311,7 @@ unsigned char *ssh_key_blob_read(FILE *fp, size_t *blob_len, char **comments)
  * Write blob to buffer as ascii string. Take initialized buffer and appends
  * blob there.
  */
-void ssh_key_blob_write_to_buffer(SshBuffer *buffer,
+void ssh_key_blob_write_to_buffer(SshBuffer buffer,
                                   unsigned char *blob,
                                   size_t blob_len,
                                   const char *comments,
@@ -324,16 +320,16 @@ void ssh_key_blob_write_to_buffer(SshBuffer *buffer,
   unsigned char *base64, *new_base64;
   unsigned char line[80];
   SshUInt32 crc32;
-  
+
   base64 = ssh_buf_to_base64(blob, blob_len);
 
   if (is_public)
     snprintf((char *) line, 80, "----BEGIN SSH PUBLIC KEY BLOCK----\n");
   else
     snprintf((char *) line, 80, "----BEGIN SSH PRIVATE KEY BLOCK----\n");
-  
+
   ssh_buffer_append(buffer, line, strlen((char *) line));
-  
+
   snprintf((char *) line, 80, "Version: %s\n", SSH_BLOB_VERSION);
   ssh_buffer_append(buffer, line, strlen((char *) line));
 
@@ -346,7 +342,7 @@ void ssh_key_blob_write_to_buffer(SshBuffer *buffer,
   ssh_buffer_append(buffer, line, strlen((char *) line));
 
   new_base64 = base64;
-  
+
   while ((new_base64 = get_next_chars(new_base64, line, 60)) != NULL)
     {
       ssh_buffer_append(buffer, line, strlen((char *) line));
@@ -360,16 +356,16 @@ void ssh_key_blob_write_to_buffer(SshBuffer *buffer,
   /* Compute crc of the blob */
   crc32 = crc32_buffer(blob, blob_len);
   SSH_PUT_32BIT(line, crc32);
-  
+
   /* Compute base64 representation crc. */
   base64 = ssh_buf_to_base64(line, 4);
-  
+
   snprintf((char *) line, 80, "=%s\n", base64);
   ssh_buffer_append(buffer, line, strlen((char *) line));
 
   /* Free the crc in base64. */
   ssh_xfree(base64);
-  
+
   snprintf((char *) line, 80, "\n");
   ssh_buffer_append(buffer, line, strlen((char *) line));
   if (is_public)
@@ -380,15 +376,15 @@ void ssh_key_blob_write_to_buffer(SshBuffer *buffer,
 }
 
 /*
- * Write blob to file as ascii string. 
+ * Write blob to file as ascii string.
  */
 void ssh_key_blob_write(FILE *fp, unsigned char *blob,
                                   size_t blob_len,
                                   const char *comments,
                                   Boolean is_public)
 {
-  SshBuffer buffer;
-  
+  SshBufferStruct buffer;
+
   ssh_buffer_init(&buffer);
   ssh_key_blob_write_to_buffer(&buffer, blob, blob_len, comments, is_public);
   fwrite(ssh_buffer_ptr(&buffer), 1, ssh_buffer_len(&buffer), fp);
