@@ -15,18 +15,21 @@
   */
 
 /*
- * $Id: t-cryptest.c,v 1.5 1998/10/07 11:56:30 tri Exp $
+ * $Id: t-cryptest.c,v 1.11 2000/02/15 01:11:02 mkojo Exp $
  * $Log: t-cryptest.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshcrypt.h"
-#include "psystem.h"
-#include "timeit.h"
-#include "gmp.h"
+#include "sshpsystem.h"
+#include "sshmp.h"
 
+#ifdef WINDOWS
+#define TEST_SRC_PATH "."
+#else /* WINDOWS */
 #include "testsrcpath.h"
+#endif /* WINDOWS */
 
 /* The PSystem definition. */
 
@@ -62,7 +65,6 @@ typedef struct
 typedef struct
 {
   FILE *fp;
-  SshRandomState state;
 } Passed;
 
 void fhexdump(FILE *fp, char *str, unsigned char *cp, size_t len)
@@ -166,7 +168,7 @@ SSH_PSYSTEM_HANDLER(timing_info)
       switch (aptype)
         {
         case TCR_LOOPS:
-          t->loops = mpz_get_ui((MP_INT*)data);
+          t->loops = ssh_mp_get_ui((SshInt*)data);
           return TRUE;
         default:
           break;
@@ -237,7 +239,8 @@ Boolean hash_test(void *t)
 
   if (memcmp(buf, h->digest, len) != 0)
     {
-      fprintf(h->s->fp, "hash_test: digest did not match!\n");
+      fprintf(h->s->fp, "hash_test: digest did not match! (%s)\n",
+              h->name);
       fhexdump(h->s->fp, "computed", buf, len);
       fhexdump(h->s->fp, " correct", h->digest, len);
       ssh_hash_free(hash);
@@ -284,14 +287,17 @@ SSH_PSYSTEM_HANDLER(hash_test)
         {
         case TCR_NAME:
           h->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DATA:
           h->data = data;
           h->data_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DIGEST:
           h->digest = data;
           h->digest_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_TIMING:
           h->t = data;
@@ -358,7 +364,8 @@ Boolean mac_test(void *t)
 
   if (memcmp(buf, h->digest, len) != 0)
     {
-      fprintf(h->s->fp, "mac_test: digests did not match!\n");
+      fprintf(h->s->fp, "mac_test: digests did not match! (%s)\n",
+              h->name);
       fhexdump(h->s->fp, "computed", buf, len);
       fhexdump(h->s->fp, " correct", h->digest, len);
       ssh_mac_free(mac);
@@ -407,18 +414,22 @@ SSH_PSYSTEM_HANDLER(mac_test)
         {
         case TCR_NAME:
           h->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_KEY:
           h->key = data;
           h->key_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DATA:
           h->data = data;
           h->data_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DIGEST:
           h->digest = data;
           h->digest_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_TIMING:
           h->t = data;
@@ -476,15 +487,25 @@ Boolean cipher_test(void *t)
     {
       if (c->iv)
         {
-          if (len != c->input_len || len != c->output_len || len != c->iv_len)
+          if (len != c->input_len || len != c->output_len)
             {
               fprintf(c->s->fp,
-                      "cipher_test: input = %u output = %u iv = %u\n"
+                      "cipher_test: cipher '%s'\n"
+                      "             input = %u output = %u iv = %u\n"
                       "             assumed all to be %u bytes.\n",
+                      c->name,
                       c->input_len, c->output_len, c->iv_len, len);
               ssh_cipher_free(cipher);
               goto failed;
             }
+          if (c->iv_len < c->input_len)
+            {
+              unsigned char *tmp;
+              tmp = ssh_xcalloc(1, c->input_len);
+              memcpy(tmp, c->iv, c->iv_len);
+              ssh_xfree(c->iv);
+              c->iv = tmp;
+            }
           ssh_cipher_set_iv(cipher, c->iv);
         }
       else
@@ -492,9 +513,10 @@ Boolean cipher_test(void *t)
           if (len != c->input_len || len != c->output_len)
             {
               fprintf(c->s->fp,
-                      "cipher_test: input = %u output = %u\n"
+                      "cipher_test: cipher '%s'\n"
+                      "             input = %u output = %u\n"
                       "             assumed both to be %u bytes.\n",
-                      c->input_len, c->output_len, len);
+                      c->name, c->input_len, c->output_len, len);
               ssh_cipher_free(cipher);
               goto failed;
             }
@@ -513,7 +535,7 @@ Boolean cipher_test(void *t)
 
   if (memcmp(buf, c->output, len) != 0)
     {
-      fprintf(c->s->fp, "cipher_test: matching failed!\n");
+      fprintf(c->s->fp, "cipher_test: matching failed! (%s)\n", c->name);
       fhexdump(c->s->fp, "computed", buf, len);
       fhexdump(c->s->fp, " correct", c->output, len);
       ssh_cipher_free(cipher);
@@ -563,22 +585,27 @@ SSH_PSYSTEM_HANDLER(cipher_test)
         {
         case TCR_NAME:
           c->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_KEY:
           c->key = data;
           c->key_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_IV:
           c->iv = data;
           c->iv_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_INPUT:
           c->input = data;
           c->input_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_OUTPUT:
           c->output = data;
           c->output_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         default:
           break;
@@ -695,7 +722,6 @@ int main(int ac, char *av[])
     }
   
   passed.fp = stderr;
-  passed.state = ssh_random_allocate();
   def.root = root;
   def.feeding = &passed;
   def.more_context = fp;
@@ -703,7 +729,6 @@ int main(int ac, char *av[])
   def.assign_operator = "=";
   ret = ssh_psystem_parse(&def, &error);
   fclose(fp);
-  ssh_random_free(passed.state);
   if (error.status == SSH_PSYSTEM_OK)
     return 0;
 
