@@ -23,7 +23,7 @@
   */
 
 /*
- * $Id: sshmp.h,v 1.12 1998/06/24 13:26:47 kivinen Exp $
+ * $Id: sshmp.h,v 1.21 2000/03/10 18:33:16 mkojo Exp $
  * $Log: sshmp.h,v $
  * $EndLog$
  */
@@ -31,6 +31,8 @@
 #ifndef SSHMP_H
 #define SSHMP_H
 
+#include "sshmath-types.h"
+
 /* SSH MP data structures. This information is usually best left alone. */ 
 
 /* The basic definition of a Ssh Integer. */
@@ -50,6 +52,9 @@ typedef struct SshIntRec
   SshWord *v;
 } SshInt;
 
+/* SSH Integer Cell. */
+typedef SshInt SshIntC[1];
+
 /* Definitions of a Ssh Integer Moduli. */
 typedef struct SshIntModuliRec
 {
@@ -88,6 +93,9 @@ typedef struct SshIntModQRec
 
 /* Some memory management. */
 
+SshInt *ssh_mp_malloc(void);
+void ssh_mp_free(SshInt *op);
+
 /* This function makes the integer 'op' to have new_size words of memory
    reserved, even if it doesn't need it at the moment. This cannot truncate
    the size of an allocated memory space for an integer, thus should be
@@ -161,7 +169,7 @@ void ssh_mp_out_str(FILE *fp, unsigned int base, const SshInt *op);
 /* Routines to linearize the integer into a octet string. Note, that these
    routines do not handle the signs correctly, at the moment. */
 void ssh_mp_get_buf(unsigned char *buf, size_t buf_length,
-		    const SshInt *op);
+                    const SshInt *op);
 void ssh_mp_set_buf(SshInt *ret, const unsigned char *buf, size_t buf_length);
 
 /* Handle signs. */
@@ -262,7 +270,7 @@ void ssh_mp_gcd(SshInt *d, const SshInt *a, const SshInt *b);
 
 /* Computes d = u*a + v*b, where a, b are given as input. */
 void ssh_mp_gcdext(SshInt *d, SshInt *u, SshInt *v,
-		   const SshInt *a, const SshInt *b);
+                   const SshInt *a, const SshInt *b);
 
 /* op*inv == 1 (mod m), where op and m are given as input. */
 Boolean ssh_mp_invert(SshInt *inv, const SshInt *op, const SshInt *m);
@@ -282,16 +290,16 @@ int ssh_mp_legendre(const SshInt *a, const SshInt *b);
    function returns anything sensible in those cases. */
 Boolean ssh_mp_mod_sqrt(SshInt *ret, const SshInt *op, const SshInt *p);
 
-/* Computes sqrt^2 = op, where op is given as input. */
-void ssh_mp_sqrt(SshInt *sqrt, const SshInt *op);
+/* Solves sqrt^2 = op, where op is given as input. Works with integers, and
+   the output thus is only an approximation. */
+void ssh_mp_sqrt(SshInt *sqrt_out, const SshInt *op);
 
 /* Routine to check whether a given value 'op' is perfect square, that is
    if op = t^2. Returns 1 if it is, 0 if not. */
 int ssh_mp_is_perfect_square(const SshInt *op);
 
 #if 1
-/* XXX Dump innards of bn, this is for debugging only and shall be
-   removed (or made unusable in releases). */
+/* XXX Dump structure internal data. */
 void ssh_mp_dump(const SshInt *bn);
 #endif
 
@@ -300,32 +308,83 @@ void ssh_mp_dump(const SshInt *bn);
 
    The routine which has best running times is the selected for general
    usage. However, it doesn't support even moduli.
-
-   XXX fix it with a fallback operation. 
    */
+
+/* This is a special heuristic for faster modular exponentiation. If you
+   know that you are using certain moduli and generator for many
+   exponentiations it pays to do some precomputation. This is
+   only very minor computation, and takes into account the desire to
+   keep from allocating very much. Basically it takes at most 512
+   integers for the table, which may seem like a lot.
+
+   At the moment this isn't very much faster than the other routines,
+   as it uses pretty much same building blocks. This could, however,
+   be speeded up somewhat. So the idea seems good. 
+   */
+typedef struct
+{
+  /* Wether this context is initialized or not. */
+  Boolean defined;
+
+  /* This number increases the speed, but also also the storage
+     amount. Basically speed increases "linearly", but storage requirements
+     increase exponentially. The linear increase, however, does not
+     hold in practice that well. */
+#define SSH_MP_POWM_BASE_K 6
+  
+  /* A large table for computed values. */
+  unsigned int table_size;
+  unsigned int table_bits;
+  SshIntModQ *table;
+  /* The moduli under the table was computed. */
+  SshIntModuli mod;
+  SshInt       order;
+} SshMpPowmBase;
+
+/* Initialize the base structure, performs the precomputation (which
+   is very fast). */
+void ssh_mp_powm_with_base_init(SshInt *g, SshInt *m, SshInt *order,
+                                SshMpPowmBase *base);
+/* Clears the base. */
+void ssh_mp_powm_with_base_clear(SshMpPowmBase *base);
+
+/* This is the general modular exponentiation with support for precomputed
+   base. */
+void ssh_mp_powm_with_base_dv(SshInt *ret, const SshInt *e,
+                              SshMpPowmBase *base);
+
+/* Following functions are the general modular exponentiation functions. */
 void ssh_mp_powm_naive(SshInt *op, const SshInt *g, const SshInt *e,
-		       const SshInt *m);
+                       const SshInt *m);
 void ssh_mp_powm_bsw(SshInt *op, const SshInt *g, const SshInt *e,
-		     const SshInt *m);
+                     const SshInt *m);
 void ssh_mp_powm_naive_mont(SshInt *op, const SshInt *g, const SshInt *e,
-			    const SshInt *m);
+                            const SshInt *m);
 void ssh_mp_powm_bsw_mont(SshInt *op, const SshInt *g, const SshInt *e,
-			  const SshInt *m);
+                          const SshInt *m);
+void ssh_mp_powm_naive_mont_gg(SshInt *op, const SshInt *g1, const SshInt *e1,
+                               const SshInt *g2, const SshInt *e2,
+                               const SshInt *m);
 
 /* Specialized routines for computing g^e (mod m), where g is very small. */
 void ssh_mp_powm_naive_ui(SshInt *op, SshWord g, const SshInt *e,
-			  const SshInt *m);
+                          const SshInt *m);
 void ssh_mp_powm_naive_mont_ui(SshInt *ret, SshWord g,
-			       const SshInt *e, const SshInt *m);
+                               const SshInt *e, const SshInt *m);
+void ssh_mp_powm_naive_mont_base2(SshInt *ret, const SshInt *e,
+                                  const SshInt *m);
 
 /* Exponentiation with a very small exponent. */
 void ssh_mp_powm_naive_expui(SshInt *op, const SshInt *g, SshWord e,
-			     const SshInt *m);
+                             const SshInt *m);
 
 /* Select your favourite, or fastest, routine here. */
-#define ssh_mp_powm       ssh_mp_powm_bsw_mont
-#define ssh_mp_powm_ui    ssh_mp_powm_naive_mont_ui
-#define ssh_mp_powm_expui ssh_mp_powm_naive_expui
+#define ssh_mp_powm           ssh_mp_powm_bsw_mont
+#define ssh_mp_powm_ui        ssh_mp_powm_naive_mont_ui
+#define ssh_mp_powm_base2     ssh_mp_powm_naive_mont_base2
+#define ssh_mp_powm_expui     ssh_mp_powm_naive_expui
+#define ssh_mp_powm_with_base ssh_mp_powm_with_base_dv
+#define ssh_mp_powm_gg        ssh_mp_powm_naive_mont_gg
 
 /* Intermediate arithmetic routines. */
 
@@ -347,32 +406,38 @@ Boolean ssh_mp_next_prime(SshInt *p, const SshInt *start);
 
 /* XXX prime searchers that seek for large 'safe' primes. */
 
-/* Following group of routines implement the interface to quicker modular
-   arithmetics. Indeed, all modular arithmetic code, which uses the same
-   moduli for same set of values for longer period of time should use
-   this interface. Some experimentation might be necessary.
+/* Following group of routines implements the interface for fast modular
+   arithmetic. Indeed, all modular arithmetic code, which uses the same
+   moduli for some set of values for longer period of time should use
+   this interface. Often the code under this interface is worth the
+   extra effort. 
 
-   XXX Current version doesn't work for even moduli. This is not a serious
-   problem. In fact, it is hard to find any sensible occurrence of an
-   problem which needs integers mod 2^n. And when such is found, we would
-   notice, that it can be done as easily without this interface for
-   most cases.
+   XXX This version doesn't work with even moduli. This is not a
+   serious problem. In fact, it is hard to find any sensible
+   occurrence of a problem which needs integers mod k*2^n. And when
+   such is found, we would notice, that it can be done as easily
+   without this interface for most cases.
 
    */
 
 /* This defines the use of workspace. That is, the workspace will be
    allocated (to minimize the needed allocations in modular arithmetic)
-   to the modulus structure. */
+   to the modulus structure.
+   
+   You can of course undefine it, and then most allocation will be handled
+   dynamically when doing computations. The amount of memory used is not
+   prohibitive. */
 #define SSHMATH_USE_WORKSPACE 
 
-/* Initialize the moduli. That is this translates the moduli given in
+/* Initialize the moduli. That is, this translates the moduli given in
    integer form to faster representation m. */
 Boolean ssh_mpm_init_m(SshIntModuli *m, const SshInt *op);
 
 /* Clear/free the modulus. */
 void ssh_mpm_clear_m(SshIntModuli *m);
 
-/* Initialize a new integer modulo m. */
+/* Initialize a new integer modulo m. Notice that the moduli must be known
+   when this is called. */
 void ssh_mpm_init(SshIntModQ *op, const SshIntModuli *m);
 
 /* Clear the modulo m integer. */
@@ -382,6 +447,8 @@ void ssh_mpm_clear(SshIntModQ *op);
 void ssh_mpm_set_mp(SshIntModQ *ret, const SshInt *op);
 /* Copy one value modulo m into another. I.e. ret = op. */
 void ssh_mpm_set(SshIntModQ *ret, const SshIntModQ *op);
+/* Copy unsigned int value to ret. */
+void ssh_mpm_set_ui(SshIntModQ *ret, SshWord u);
 
 /* Convert a value modulo m into SshInt. */
 void ssh_mp_set_mpm(SshInt *ret, const SshIntModQ *op);
@@ -390,20 +457,21 @@ void ssh_mp_set_m(SshInt *ret, const SshIntModuli *m);
 /* Comparison function. One should not rely on the fact that -1 and 1
    mean anything but that the inputs are different. 0 means always the
    the inputs are same. */
-int ssh_mpm_cmp(SshIntModQ *op1, SshIntModQ *op2);
+int ssh_mpm_cmp(const SshIntModQ *op1, const SshIntModQ *op2);
+int ssh_mpm_cmp_ui(const SshIntModQ *op, SshWord u);
   
 /* Basic arithmetic in modulo m representation. */
 
 /* Fast modular addition and subtraction, keeps the values always within
    the modular domain. */
 void ssh_mpm_add(SshIntModQ *ret, const SshIntModQ *op1,
-		 const SshIntModQ *op2);
+                 const SshIntModQ *op2);
 void ssh_mpm_sub(SshIntModQ *ret, const SshIntModQ *op1,
-		 const SshIntModQ *op2);
+                 const SshIntModQ *op2);
 
 /* Fast multiplication which keeps the values within modular domain. */
 void ssh_mpm_mul(SshIntModQ *ret, const SshIntModQ *op1,
-		 const SshIntModQ *op2);
+                 const SshIntModQ *op2);
 /* Fast multiplication by small integer. */
 void ssh_mpm_mul_ui(SshIntModQ *ret, const SshIntModQ *op, SshWord u);
 /* Very quick squaring operation. */
@@ -412,14 +480,14 @@ void ssh_mpm_square(SshIntModQ *ret, const SshIntModQ *op);
 /* Routines for handling modular divisions by powers of 2.
 
    These routines are meant mainly to be used for small powers and thus
-   are not fastest for larger ones. However, for very small powers there
-   doesn't probably exist faster algorithms. 
+   are not fastest for larger ones. However, for very small powers these
+   work with small amount of operations. 
    */
 void ssh_mpm_div_2exp(SshIntModQ *ret, const SshIntModQ *op,
-		      unsigned int exp);
-/* Very simple multiplication by powers of 2. */
+                      unsigned int exp);
+/* Very simple, and fast, multiplication by powers of 2. */
 void ssh_mpm_mul_2exp(SshIntModQ *ret, const SshIntModQ *op,
-		      unsigned int exp);
+                      unsigned int exp);
 
 /* This inversion is not fast, but we assume that you don't need faster
    implementation. It is possible to write faster inversion later. */
@@ -431,8 +499,11 @@ int ssh_mpm_kronecker(const SshIntModQ *ret);
 Boolean ssh_mpm_gcd(SshIntModQ *ret, const SshIntModQ *op);
 Boolean ssh_mpm_sqrt(SshIntModQ *ret, const SshIntModQ *op);
 
-/* XXX Dump the innards... */
+#if 1
+/* XXX Dump the structure internal data... */
 void ssh_mpm_dump(const SshIntModQ *op);
+#endif
+
 /**********************************************************************/
 /* Rationals? Do we need 'em? Probably SshFloat's upto reasonable precision
    does the same job. */
