@@ -1,7 +1,7 @@
 /*
 
   t-mathtest.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
   Copyright (C) 1996,1997 SSH Communications Security Oy, Espoo, Finland
@@ -21,14 +21,14 @@
   */
 
 /*
- * $Id: t-mathtest.c,v 1.18 1999/05/04 02:19:56 kivinen Exp $
+ * $Id: t-mathtest.c,v 1.30 2000/03/10 18:33:29 mkojo Exp $
  * $Log: t-mathtest.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshmp.h"
-#include "timeit.h"
+#include "sshtimemeasure.h"
 #include "sieve.h"
 
 /* Printing of different types to the screen, these are helpful when
@@ -63,7 +63,7 @@ int check_mod(SshIntModQ *b, SshInt *a)
 {
   SshInt t;
   int rv;
-  
+
   ssh_mp_init(&t);
   ssh_mp_set_mpm(&t, b);
   rv = ssh_mp_cmp(a, &t);
@@ -89,7 +89,7 @@ void true_rand(SshInt *op, int bits)
 
 void test_int(int flag, int bits)
 {
-  SshInt a, b, c, d, e, f;
+  SshInt a, b, c, d, e, f, g;
   int j, k, i, l;
 
   ssh_mp_init(&a);
@@ -98,7 +98,10 @@ void test_int(int flag, int bits)
   ssh_mp_init(&d);
   ssh_mp_init(&e);
   ssh_mp_init(&f);
+  ssh_mp_init(&g);
 
+  printf("Running integer tests (%u bits):\n", bits);
+  
   printf(" * addition/subtraction test\n");
   for (j = 0; j < 1000; j++)
     {
@@ -156,7 +159,7 @@ void test_int(int flag, int bits)
           exit(1);
         }
     }
-  
+
   printf(" * division test\n");
   for (j = 0; j < 1000; j++)
     {
@@ -217,7 +220,7 @@ void test_int(int flag, int bits)
           ssh_mp_dump(&a);
           ssh_mp_dump(&b);
           ssh_mp_dump(&c);
-          
+
           print_int("a*a = ", &b);
           ssh_mp_dump(&b);
           print_int("a^2 = ", &c);
@@ -234,11 +237,11 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp_ui(&a, 0) == 0 ||
           ssh_mp_cmp_ui(&b, 0) == 0)
         continue;
-      
+
       /* Make positive. */
       ssh_mp_abs(&a, &a);
       ssh_mp_abs(&b, &b);
-      
+
       ssh_mp_mul(&c, &a, &b);
       ssh_mp_gcd(&d, &c, &a);
       ssh_mp_gcd(&e, &c, &b);
@@ -264,7 +267,7 @@ void test_int(int flag, int bits)
       ssh_mp_sqrt(&c, &b);
 
       ssh_mp_abs(&a, &a);
-      
+
       if (ssh_mp_cmp(&a, &c) != 0)
         {
           printf("error: square root/squaring failed.\n");
@@ -290,7 +293,7 @@ void test_int(int flag, int bits)
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
       ssh_mp_set(&c, &b);
-      
+
       for (i = 1; i < 100; i++)
         {
           ssh_mp_set_ui(&e, i);
@@ -311,6 +314,35 @@ void test_int(int flag, int bits)
         }
     }
 
+  if (flag)
+    {
+      printf(" * base 2 verification\n");
+      for (j = 0; j < 100; j++)
+        {
+          true_rand(&a, bits);
+          if (ssh_mp_cmp_ui(&a, 3) < 0)
+            continue;
+          if ((ssh_mp_get_ui(&a) & 0x1) == 0)
+            ssh_mp_add_ui(&a, &a, 1);
+
+          true_rand(&b, bits);
+          ssh_mp_abs(&b, &b);
+
+          ssh_mp_powm_ui(&c, 2, &b, &a);
+          ssh_mp_powm_base2(&d, &b, &a);
+          if (ssh_mp_cmp(&c, &d) != 0)
+            {
+              printf("error: powm ui/powm base2 failed.\n");
+              print_int("mod = ", &a);
+              print_int("exp = ", &b);
+              print_int("powm ui    = ", &c);
+              print_int("powm base2 = ", &d);
+              exit(1);
+            }
+        }
+    }
+
+
   printf(" * full exponentiation test\n");
   for (j = 0; j < 10; j++)
     {
@@ -327,7 +359,7 @@ void test_int(int flag, int bits)
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
       ssh_mp_set(&c, &b);
-      
+
       for (i = 1; i < 100; i++)
         {
           ssh_mp_set_ui(&e, i);
@@ -347,7 +379,51 @@ void test_int(int flag, int bits)
           ssh_mp_mod(&c, &c, &a);
         }
     }
-  
+
+  if (flag)
+    {
+      printf(" * gg exponentiation test\n");
+      for (j = 0; j < 10; j++)
+        {
+          true_rand(&a, bits);
+          ssh_mp_abs(&a, &a);
+
+          if (ssh_mp_cmp_ui(&a, 3) < 0)
+            continue;
+
+          if ((ssh_mp_get_ui(&a) & 0x1) == 0)
+            ssh_mp_add_ui(&a, &a, 1);
+
+          true_rand(&b, bits);
+          ssh_mp_mod(&b, &b, &a);
+          true_rand(&c, bits);
+          true_rand(&d, bits);
+          ssh_mp_mod(&d, &d, &a);
+          true_rand(&e, bits);
+
+          ssh_mp_powm(&f, &b, &c, &a);
+          ssh_mp_powm(&g, &d, &e, &a);
+          ssh_mp_mul(&f, &f, &g);
+          ssh_mp_mod(&f, &f, &a);
+
+          ssh_mp_powm_gg(&g, &b, &c, &d, &e, &a);
+          if (ssh_mp_cmp(&f, &g) != 0)
+            {
+              printf("error: gg exp failed.\n");
+              print_int("mod = ", &a);
+              print_int("g   = ", &b);
+              print_int("exp = ", &c);
+              print_int("g   = ", &d);
+              print_int("exp = ", &e);
+              print_int("r1  = ", &f);
+              print_int("r2  = ", &g);
+              exit(1);
+
+            }
+        }
+
+    }
+
   for (j = 0; j < 100; j++)
     {
       true_rand(&a, bits);
@@ -362,7 +438,7 @@ void test_int(int flag, int bits)
       k = random();
       ssh_mp_set_ui(&b, k);
       true_rand(&e, bits);
-      
+
       ssh_mp_powm(&c, &b, &e, &a);
       ssh_mp_powm_ui(&d, k, &e, &a);
 
@@ -400,28 +476,30 @@ void test_int(int flag, int bits)
           exit(1);
         }
     }
-  
+
   if (flag)
     {
       printf(" * prime tests\n");
       for (j = 0; j < 10; j++)
         {
-          printf("    - searching... [%u bit prime]\n", bits);
           true_rand(&a, bits);
           ssh_mp_abs(&a, &a);
 
+          printf("    - searching... [%u bit prime]\n",
+                 ssh_mp_get_size(&a, 2));
+
           if (ssh_mp_next_prime(&a, &a) == FALSE)
             continue;
 
           printf("    - probable prime found\n");
           print_int("      =", &a);
-                  
+
           printf("    - testing modular sqrt\n");
           for (l = 0; l < 10; l++)
             {
               true_rand(&b, bits);
               ssh_mp_abs(&b, &b);
-              
+
               if (ssh_mp_mod_sqrt(&d, &b, &a) == FALSE)
                 continue;
               ssh_mp_mod(&b, &b, &a);
@@ -441,6 +519,46 @@ void test_int(int flag, int bits)
         }
     }
 
+  if (flag)
+    {
+      printf(" * full exponentiation test with base\n");
+      for (j = 0; j < 10; j++)
+        {
+          SshMpPowmBase base;
+          true_rand(&a, bits);
+          ssh_mp_abs(&a, &a);
+
+          printf("     - searching for a prime.\n");
+
+          if (ssh_mp_next_prime(&a, &a) == FALSE)
+            continue;
+
+          true_rand(&b, bits);
+          ssh_mp_abs(&b, &b);
+          ssh_mp_set_bit(&b, bits);
+
+          ssh_mp_sub_ui(&c, &a, 1);
+          ssh_mp_powm_with_base_init(&b, &a, &c, &base);
+
+          true_rand(&e, bits);
+          ssh_mp_abs(&e, &e);
+          ssh_mp_powm_with_base(&c, &e, &base);
+          ssh_mp_powm(&d, &b, &e, &a);
+          ssh_mp_powm_with_base_clear(&base);
+
+          if (ssh_mp_cmp(&c, &d) != 0)
+            {
+              printf("error: powm/multiplication failed.\n");
+              print_int("mod = ", &a);
+              print_int("g   = ", &b);
+              print_int("exp = ", &e);
+              print_int("1   = ", &d);
+              print_int("2   = ", &c);
+              exit(1);
+            }
+        }
+    }
+
   if (flag)
     {
       printf(" * square tests\n");
@@ -469,25 +587,25 @@ void test_int(int flag, int bits)
         {
           true_rand(&a, bits);
           true_rand(&b, bits);
-          
+
           if (ssh_mp_cmp_ui(&a, 0) == 0 ||
               ssh_mp_cmp_ui(&b, 0) == 0)
             continue;
-      
+
           ssh_mp_abs(&a, &a);
           ssh_mp_abs(&b, &b);
-      
+
           ssh_mp_gcd(&c, &a, &b);
           if (ssh_mp_cmp_ui(&c, 1) == 0)
             {
               ssh_mp_gcdext(&d, &e, &f, &a, &b);
-              
+
               if (ssh_mp_cmp(&d, &c) != 0)
                 {
                   printf("error: gcd/gcdext failed.\n");
                   exit(1);
                 }
-              
+
               ssh_mp_mul(&e, &a, &e);
               ssh_mp_mul(&f, &b, &f);
               ssh_mp_add(&f, &f, &e);
@@ -500,6 +618,92 @@ void test_int(int flag, int bits)
         }
     }
 
+  printf(" * testing mod 2exp.\n");
+  for (i = 0; i < 1000; i++)
+    {
+      true_rand(&a, bits/2);
+      ssh_mp_abs(&a, &a);
+      true_rand(&b, bits/2);
+      ssh_mp_abs(&b, &b);
+      ssh_mp_mul_2exp(&c, &a, bits/2);
+      ssh_mp_add(&a, &c, &b);
+
+      ssh_mp_mod_2exp(&c, &a, bits/2);
+
+      if (ssh_mp_cmp(&c, &b) != 0)
+        {
+          printf("error: mod 2exp failed (case 1).\n");
+          print_int("a = ", &a);
+          print_int("b = ", &b);
+          print_int("c = ", &c);
+          exit(1);
+        }
+
+      ssh_mp_mod_2exp(&c, &a, bits*2);
+      if (ssh_mp_cmp(&c, &a) != 0)
+        {
+          printf("error: mod 2exp failed (case 2).\n");
+          print_int("a = ", &a);
+          print_int("b = ", &b);
+          print_int("c = ", &c);
+          exit(1);
+        }
+    }
+  
+  printf(" * buffer testing.\n");
+  for (i = 0; i < 1000; i++)
+    {
+      unsigned char *buffer;
+      size_t buffer_len;
+
+      true_rand(&a, bits);
+      ssh_mp_abs(&a, &a);
+      
+      buffer_len = (bits+7)/8;
+      buffer = ssh_xmalloc(buffer_len);
+
+      ssh_mp_get_buf(buffer, buffer_len, &a);
+      ssh_mp_set_buf(&b, buffer, buffer_len);
+
+      if (ssh_mp_cmp(&a, &b) != 0)
+        {
+          printf("error: buffer <-> integer failed (%u bytes).\n", buffer_len);
+          print_int("a = ", &a);
+          ssh_mp_dump(&a);
+          print_int("b = ", &b);
+          ssh_mp_dump(&b);
+          ssh_xfree(buffer);
+          exit(1);
+        }
+
+      ssh_xfree(buffer);
+
+      /* Small buffer. */
+      buffer_len = ssh_mp_get_size(&a, 256) + 4; /* (random() % buffer_len) + 1; */
+      buffer     = ssh_xmalloc(buffer_len);
+
+      ssh_mp_get_buf(buffer, buffer_len, &a);
+      ssh_mp_set_buf(&b, buffer, buffer_len);
+
+      /* Now compute suitable matching value. */
+      ssh_mp_mod_2exp(&c, &a, buffer_len * 8);
+
+      if (ssh_mp_cmp(&c, &b) != 0)
+        {
+          printf("error: buffer <-> integer failed (%u bytes / %u).\n", buffer_len,
+                 ssh_mp_get_size(&a, 256));
+          print_int("a = ", &a);
+          ssh_mp_dump(&a);
+          print_int("c = ", &c);
+          ssh_mp_dump(&c);
+          print_int("b = ", &b);
+          ssh_mp_dump(&b);
+          ssh_xfree(buffer);
+          exit(1);
+        }
+      ssh_xfree(buffer);
+    }
+  
   printf(" * conversion testing.\n");
   for (i = 0; i < 1000; i++)
     {
@@ -511,7 +715,7 @@ void test_int(int flag, int bits)
           base = random() % 65;
         }
       while (base < 2);
-      
+
       true_rand(&a, bits);
 
       str = ssh_mp_get_str(NULL, base, &a);
@@ -532,7 +736,7 @@ void test_int(int flag, int bits)
       ssh_xfree(str);
 
       /* Test for automatic recognition. */
-      
+
       switch (random() % 3)
         {
         case 0:
@@ -545,7 +749,7 @@ void test_int(int flag, int bits)
           base = 16;
           break;
         }
-      
+
       str = ssh_mp_get_str(NULL, base, &a);
       ssh_mp_set_str(&b, str, 0);
 
@@ -561,15 +765,16 @@ void test_int(int flag, int bits)
           exit(1);
         }
       ssh_xfree(str);
-      
+
     }
-  
+
   ssh_mp_clear(&a);
   ssh_mp_clear(&b);
   ssh_mp_clear(&c);
   ssh_mp_clear(&d);
   ssh_mp_clear(&e);
   ssh_mp_clear(&f);
+  ssh_mp_clear(&g);
 }
 
 void test_mod(int flag, int bits)
@@ -627,7 +832,7 @@ void test_mod(int flag, int bits)
           exit(1);
         }
     }
-  
+
   /* Subtractions. */
   printf(" * subtraction test\n");
   for (i = 0; i < 1000; i++)
@@ -789,7 +994,7 @@ void test_mod(int flag, int bits)
     }
 
 
-  
+
   ssh_mpm_clear(&a0);
   ssh_mpm_clear(&b0);
   ssh_mpm_clear(&c0);
@@ -811,7 +1016,7 @@ void test_mod(int flag, int bits)
 void timing_int(int bits)
 {
   SshInt a, b, c, d, e, f[100];
-  TimeIt tmit;
+  struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
   unsigned int i, j;
   SshMpPowmBase base;
 
@@ -820,7 +1025,7 @@ void timing_int(int bits)
   ssh_mp_init(&c);
   ssh_mp_init(&d);
   ssh_mp_init(&e);
-  
+
   printf("Timing integer arithmetic.\n");
 
   printf("Bits = %u\n", bits);
@@ -835,75 +1040,93 @@ void timing_int(int bits)
 
   printf("Timing multiplication [%u * %u = %u] \n",
          bits, bits, bits + bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     {
       ssh_mp_rand(&b, bits);
       for (j = 0; j < 100; j++)
         ssh_mp_mul(&a, &f[j], &b);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
+
+  printf("  * %g multiplications per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
-  printf("  * %g multiplications per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
-  
   printf("Timing divisions [%u / %u = %u] \n",
          bits + bits, bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     {
       ssh_mp_rand(&b, bits*2);
       for (j = 0; j < 100; j++)
         ssh_mp_div(&a, &c, &b, &f[j]);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
+
+  printf("  * %g divisions per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
-  printf("  * %g divisions per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
 
-  
   printf("Timing modular reductions [%u %% %u = %u] \n",
          bits + bits, bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     {
       ssh_mp_rand(&b, bits*2);
       for (j = 0; j < 100; j++)
         ssh_mp_mod(&a, &b, &f[j]);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
 
-  printf("  * %g modular reductions per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
+  printf("  * %g modular reductions per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
-  
   printf("Timing squarings [%u^2 = %u] \n",
          bits, bits + bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     {
       ssh_mp_rand(&b, bits);
       for (j = 0; j < 100; j++)
         ssh_mp_square(&a, &b);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
 
-  printf("  * %g squarings per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
+  printf("  * %g squarings per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
   printf("Timing modexp [%u^%u %% %u = %u] \n",
          bits, bits, bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (j = 0, i = 0; i < 10; i++, j += 2)
     {
       ssh_mp_rand(&b, bits);
-      ssh_mp_powm(&a, &b, &f[j + 1], &f[j + 2]);
+      ssh_mp_powm(&a, &f[j + 1], &b, &f[j + 2]);
     }
-  check_timing(&tmit);
-
-  printf("  * %g modexps per sec (%g cycles)\n",
-         ((double)10)/(tmit.real_secs), (double)tmit.cycles/(10.0));
+  ssh_time_measure_stop(&tmit);
 
+  printf("  * %g modexps per sec\n",
+         ((double)10) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
   /* Generate the fixed base. */
   do
@@ -913,41 +1136,101 @@ void timing_int(int bits)
   while (ssh_mp_get_size(&b, 2) < bits-1);
 
   /* Create the base. */
-  ssh_mp_powm_with_base_init(&b, &f[2], &base);
+  ssh_mp_sub_ui(&a, &f[2], 1);
+  ssh_mp_powm_with_base_init(&b, &f[2], &a, &base);
 
   if (base.defined == FALSE)
     ssh_fatal("error: could not define base.");
-    
+
   printf("Timing modexp with fixed base [%u^%u %% %u = %u] \n",
          bits, bits, bits, bits);
 
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (j = 0, i = 0; i < 10; i++, j += 2)
     ssh_mp_powm_with_base(&a, &f[j + 1], &base);
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
 
-  printf("  * %g modexps per sec (%g cycles)\n",
-         ((double)10)/(tmit.real_secs), (double)tmit.cycles/(10.0));
+  printf("  * %g modexps per sec\n",
+         ((double)10) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
   ssh_mp_powm_with_base_clear(&base);
-  
-  
+
+
 #define ENTROPY_BITS 256
-  
+
   printf("Timing modexp [%u^%u %% %u = %u] \n",
          bits, ENTROPY_BITS, bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (j = 0, i = 0; i < 10; i++, j += 2)
     {
       ssh_mp_rand(&b, ENTROPY_BITS);
       ssh_mp_set_bit(&b, ENTROPY_BITS);
       ssh_mp_powm(&a, &f[j+1], &b, &f[j + 2]);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
 
-  printf("  * %g modexps per sec (%g cycles)\n",
-         ((double)10)/(tmit.real_secs), (double)tmit.cycles/(10.0));
+  printf("  * %g modexps per sec\n",
+         ((double)10) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
+  printf("Timing gg modexp [%u^%u %% %u = %u] \n",
+         bits, ENTROPY_BITS, bits, bits);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
+  for (j = 0, i = 0; i < 10; i++, j += 2)
+    {
+      ssh_mp_rand(&b, ENTROPY_BITS);
+      ssh_mp_rand(&c, ENTROPY_BITS);
+      ssh_mp_set_bit(&b, ENTROPY_BITS);
+      ssh_mp_set_bit(&c, ENTROPY_BITS);
+      ssh_mp_powm_gg(&a, &f[j+1], &b, &f[j+2], &c, &f[j + 3]);
+    }
+  ssh_time_measure_stop(&tmit);
+
+  printf("  * %g gg modexps per sec\n",
+         ((double)10) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
+
+
+  /* Time the buffer routines. */
+
+  printf("Timing buffer conversion [%u = %u] \n",
+         bits, bits);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
+  {
+    unsigned char *buffer;
+    size_t buffer_len;
+
+    buffer_len = (bits + 7)/8;
+    buffer     = ssh_xmalloc(buffer_len);
+    
+    for (j = 0, i = 0; i < 10000; i++, j += 2)
+      {
+        ssh_mp_get_buf(buffer, buffer_len, &f[j % 20]);
+        ssh_mp_set_buf(&f[j % 20], buffer, buffer_len);
+      }
+    ssh_xfree(buffer);
+  }
+  ssh_time_measure_stop(&tmit);
+
+  printf("  * %g buffer conversions / sec\n",
+         ((double)10000) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
+
+  
+  
   ssh_mp_clear(&a);
   ssh_mp_clear(&b);
   ssh_mp_clear(&c);
@@ -964,8 +1247,8 @@ void timing_modular(int bits)
   SshIntModuli m;
   SshInt a;
   int i, j;
-  TimeIt tmit;
-  
+  struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
+
   ssh_mp_init(&a);
 
   do
@@ -986,7 +1269,7 @@ void timing_modular(int bits)
   ssh_mpm_init(&c, &m);
   ssh_mpm_init(&d, &m);
   ssh_mpm_init(&e, &m);
-  
+
   for (i = 0; i < 100; i++)
     {
       ssh_mpm_init(&f[i], &m);
@@ -996,28 +1279,36 @@ void timing_modular(int bits)
 
   printf("Timing multiplication [%u * %u = %u] \n",
          bits, bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     {
       ssh_mpm_set(&b, &f[i]);
       for (j = 0; j < 100; j++)
         ssh_mpm_mul(&c, &f[j], &b);
     }
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
+
+  printf("  * %g multiplications per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
-  printf("  * %g multiplications per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
-  
   printf("Timing squarings [%u^2 = %u] \n",
          bits, bits);
-  start_timing(&tmit);
+  ssh_time_measure_reset(&tmit);
+  ssh_time_measure_start(&tmit);
   for (i = 0; i < 50; i++)
     for (j = 0; j < 100; j++)
       ssh_mpm_square(&b, &f[j]);
-  check_timing(&tmit);
+  ssh_time_measure_stop(&tmit);
 
-  printf("  * %g squarings per sec (%g cycles)\n",
-         ((double)50*100)/(tmit.real_secs), (double)tmit.cycles/(50*100.0));
+  printf("  * %g squarings per sec\n",
+         ((double)50*100) /
+         ((double) ssh_time_measure_get(&tmit,
+                                        SSH_TIME_GRANULARITY_MICROSECOND) /
+          1000000.0));
 
   ssh_mpm_clear(&b);
   ssh_mpm_clear(&c);
@@ -1027,9 +1318,11 @@ void timing_modular(int bits)
   for (i = 0; i < 100; i++)
     ssh_mpm_clear(&f[i]);
   ssh_mpm_clear_m(&m);
-  ssh_mp_clear(&a);  
+  ssh_mp_clear(&a);
 }
 
+
+
 /* Routines for handling the arguments etc. */
 
 typedef struct CommandRec
@@ -1057,6 +1350,8 @@ typedef struct CommandRec
 
 #define C_TIMING   30
 
+#define C_FATALCB  99
+
 const Command commands[] =
 {
   { "-h", C_HELP, 0 },
@@ -1075,12 +1370,15 @@ const Command commands[] =
 
   { "-t", C_TIMING, 0 },
   { "--timing", C_TIMING, 0 },
-  
+
+  /* Fatal callback used. */
+  { "--fatalcb", C_FATALCB, 0 },
+
   /* General classes of tests. */
   { "--integer", C_INT, 1 },
   { "--modular", C_MOD, 1 },
-  
-  
+
+
   { NULL }
 };
 
@@ -1094,7 +1392,7 @@ int check_arg(char *str, int *args)
         *args = commands[i].args;
         return commands[i].type;
       }
-  
+
   *args = 0;
   return C_NONE;
 }
@@ -1126,12 +1424,22 @@ int on_off(char *str)
   exit(1);
 }
 
+void my_fatal_cb(const char *message, void *context)
+{
+  printf("t-mathtest fatal handler called.\n"
+         "message:\n"
+         "  %s\n"
+         "Aborting.\n", message);
+  abort();
+}
+
 int main(int ac, char *av[])
 {
   int i, all, itr, type, args;
   int gf2n, mod, integer, ecp, ec2n, fec2n, poly2n, bpoly,
     bits, bits_advance, timing;
-  
+  int fatal_cb;
+
   printf("Arithmetic library test suite\n"
          "Copyright (C) 1998 SSH Communications Security, Ltd.\n"
          "              All rights reserved.\n"
@@ -1140,22 +1448,24 @@ int main(int ac, char *av[])
          "  - integer arithmetic\n"
          "  - finite field arithmetic (mod p)\n"
          "\n");
-  
+
   /* Randomize the random number generator. */
   srandom(ssh_time());
 
   /* Don't use this if you want to test the mathlibrary :) */
   /*extra_test(); */
   /*test_rsa_kphi(); */
-  
+
   all = 0;
   itr = 1;
 
+  fatal_cb = FALSE;
+
   timing = 0;
-  
+
   bits = 512;
   bits_advance = 128;
-  
+
   gf2n     = 0;
   integer  = 1;
   mod      = 0;
@@ -1164,7 +1474,7 @@ int main(int ac, char *av[])
   ec2n     = 0;
   fec2n    = 0;
   poly2n   = 0;
-  
+
   for (i = 1; i < ac; i++)
     {
       type = check_arg(av[i], &args);
@@ -1177,6 +1487,9 @@ int main(int ac, char *av[])
 
       switch (type)
         {
+        case C_FATALCB:
+          fatal_cb = TRUE;
+          break;
         case C_INT:
           integer = on_off(av[i + 1]);
           i++;
@@ -1185,7 +1498,7 @@ int main(int ac, char *av[])
           mod = on_off(av[i + 1]);
           i++;
           break;
-          
+
         case C_BITS:
           bits = atoi(av[i + 1]);
           i++;
@@ -1194,7 +1507,7 @@ int main(int ac, char *av[])
           bits_advance = atoi(av[i + 1]);
           i++;
           break;
-          
+
         case C_HELP:
           usage();
           break;
@@ -1216,6 +1529,12 @@ int main(int ac, char *av[])
         }
     }
 
+  if (fatal_cb)
+    {
+      printf("Registering a new fatal handler.\n");
+      ssh_debug_register_callbacks(my_fatal_cb, NULL, NULL, NULL);
+    }
+
   if (itr <= 0)
     itr = 1;
 
@@ -1226,14 +1545,14 @@ int main(int ac, char *av[])
     {
       if (bits < 10)
         bits = 512;
-      
+
       if (integer)
         {
           test_int(all, bits);
           if (timing)
             timing_int(bits);
         }
-      if (mod)     
+      if (mod)
         {
           test_mod(all, bits);
           if (timing)
