@@ -8,7 +8,7 @@
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   SSH User Authentication Protocol interface.
@@ -16,7 +16,7 @@
 */
 
 /*
- * $Id: sshauth.h,v 1.9 1998/07/11 12:37:00 tri Exp $
+ * $Id: sshauth.h,v 1.13 2000/05/28 16:21:02 kivinen Exp $
  * $Log: sshauth.h,v $
  * $EndLog$
  */
@@ -48,11 +48,12 @@
    The returned string should be allocated by ssh_xmalloc, and the caller
    must free the string when no longer needed. */
 typedef char *(*SshAuthPolicyProc)(const char *user,
-				   const char *service,
-				   const char *client_ip,
-				   const char *client_port,
-				   const char *completed_authentications,
-				   void *context);
+                                   const char *service,
+                                   const char *client_ip,
+                                   const char *client_port,
+                                   const char *completed_authentications,
+                                   const char *disabled_authentications,
+                                   void *context);
 
 /* Return code from an authentication method. */
 typedef enum {
@@ -148,25 +149,25 @@ typedef enum {
      `session_id_len' length of session identifier
      `state_placeholder' place to store context data between packets
      `longtime_placeholder'  can hold data between authentications
-     		    (this is per-method)
+                    (this is per-method)
      `method_context'   passed to the method function */
 typedef SshAuthServerResult (*SshAuthServerMethodProc)(
-        				SshAuthServerOperation op,
-					const char *user,
-					SshBuffer *packet,
-					const unsigned char *session_id,
-					size_t session_id_len,
-					void **state_placeholder,
-					void **longtime_placeholder,
-					void *method_context);
+                                        SshAuthServerOperation op,
+                                        const char *user,
+                                        SshBuffer packet,
+                                        const unsigned char *session_id,
+                                        size_t session_id_len,
+                                        void **state_placeholder,
+                                        void **longtime_placeholder,
+                                        void *method_context);
 
 /* This structure completely describes an authentication method.  All methods
    supported by an implementation are described by an array of these.
    The "none" method should not be listed in the array.  The array is
    terminated with an entry with NULL name. */
 typedef struct SshAuthServerMethodRec {
-  const char *name;		/* method name, e.g. "password" */
-  SshAuthServerMethodProc proc;	/* function to perform authentication */
+  const char *name;             /* method name, e.g. "password" */
+  SshAuthServerMethodProc proc; /* function to perform authentication */
 } SshAuthServerMethod;
 
 /* Wraps the transport layer stream into an authentication stream.  This will
@@ -205,17 +206,17 @@ typedef struct SshAuthServerMethodRec {
      `transport_stream'     transport layer stream
      `policy_proc'          function to control authentication policy
                             (may be NULL, in which case each method in
-			    the array is individually acceptable)
+                            the array is individually acceptable)
      `policy_context'       passed to the policy function
      `methods'              array of supported methods, terminated by
-     			    an element with NULL name.  This needs to stay
-			    valid until the stream is destroyed.
+                            an element with NULL name.  This needs to stay
+                            valid until the stream is destroyed.
      `method_context'       context to pass to methods (normally NULL) */
 SshStream ssh_auth_server_wrap(SshStream transport_stream,
-			       SshAuthPolicyProc policy_proc,
-			       void *policy_context,
-			       const SshAuthServerMethod methods[],
-			       void *method_context);
+                               SshAuthPolicyProc policy_proc,
+                               void *policy_context,
+                               SshAuthServerMethod methods[],
+                               void *method_context);
 
 /************************** client side ****************************/
 
@@ -292,6 +293,13 @@ typedef enum {
      it. */
   SSH_AUTH_CLIENT_FAIL,
 
+  /* This value should be returned if the requested authentication
+     method should be disabled and no request packet should be sent
+     for it. (for example, in hostbased-authentication we might not be
+     able to read the host's private key. If only SSH_AUTH_CLIENT_FAIL
+     was returned, it would be very possible that we looped forever.)  */
+  SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD,
+  
   /* This value indicates that the user has requested to abort authentication
      entirely.  If this is returned, the connection will be closed,
      authentication terminates, and EOF will be returned from the
@@ -321,14 +329,14 @@ typedef enum {
       `result'    result code
       `user'      user name to authenticate with (copied)
       `packet'    method-specific part of SSH_MSG_USERAUTH_REQUEST,
-      		  or NULL if empty
+                  or NULL if empty
       `completion_context'  context from the call
    This does not free the packet; data is copied from there to internal
    buffers during the call. */
 typedef void (*SshAuthClientCompletionProc)(SshAuthClientResult result,
-					    const char *user,
-					    SshBuffer *packet,
-					    void *completion_context);
+                                            const char *user,
+                                            SshBuffer packet,
+                                            void *completion_context);
 
 /* Functions of this type represent authentication methods on the client
    side.  The generic code calls these to test if the method is currently
@@ -349,16 +357,16 @@ typedef void (*SshAuthClientCompletionProc)(SshAuthClientResult result,
    code will hang and not be freed.  The completion procedure is not
    called for SSH_AUTH_CLIENT_OP_ABORT, however. */
 typedef void (*SshAuthClientMethodProc)(
-		    SshAuthClientOperation op,
-		    const char *user,
-		    unsigned int packet_type,
-		    SshBuffer *packet_in,
-		    const unsigned char *session_id,
-		    size_t session_id_len,
-		    void **state_placeholder,
-		    SshAuthClientCompletionProc completion_proc,
-		    void *completion_context,
-		    void *method_context);
+                    SshAuthClientOperation op,
+                    const char *user,
+                    unsigned int packet_type,
+                    SshBuffer packet_in,
+                    const unsigned char *session_id,
+                    size_t session_id_len,
+                    void **state_placeholder,
+                    SshAuthClientCompletionProc completion_proc,
+                    void *completion_context,
+                    void *method_context);
 
 /* This structure completely describes an authentication method for
    the client side.  All methods supported by an implementation are
@@ -366,8 +374,8 @@ typedef void (*SshAuthClientMethodProc)(
    listed in the array.  The array is terminated with an entry with
    NULL name. */
 typedef struct SshAuthClientMethodRec {
-  const char *name;		/* method name, e.g. "password" */
-  SshAuthClientMethodProc proc;	/* function to perform authentication */
+  const char *name;             /* method name, e.g. "password" */
+  SshAuthClientMethodProc proc; /* function to perform authentication */
 } SshAuthClientMethod;
 
 /* Wraps the transport layer stream into an authentication stream.  This
@@ -393,13 +401,13 @@ typedef struct SshAuthClientMethodRec {
      `transport_stream'    the transport layer stream
      `service'             service name to request
      `methods'             array of supported authentication methods,
-     			   ordered the preferred one first.  The array
-			   terminates with a NULL method name.
+                           ordered the preferred one first.  The array
+                           terminates with a NULL method name.
      `method_context'      context to pass to methods (normally NULL) */
 SshStream ssh_auth_client_wrap(SshStream transport_stream,
-			       const char *initial_user,
-			       const char *service,
-			       const SshAuthClientMethod methods[],
-			       void *method_context);
+                               const char *initial_user,
+                               const char *service,
+                               SshAuthClientMethod methods[],
+                               void *method_context);
 
 #endif /* SSHTRANS_H */
