@@ -8,11 +8,11 @@
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   SSH User authentication protocol, server-side.
-                   
+
 */
 
 #include "sshincludes.h"
@@ -22,6 +22,7 @@
 #include "sshmsgs.h"
 #include "sshtimeouts.h"
 #include "sshencode.h"
+#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthServer"
 
@@ -31,19 +32,19 @@ typedef struct {
 
   /* Cross-layer protocol context for the stream upwards. */
   SshStream up;
-  
+
   /* The policy function.  This is always a valid policy function (if the
      application passed NULL, a default function will be used here). */
   SshAuthPolicyProc policy_proc;
 
   /* Context to pass to the policy function. */
   void *policy_context;
-  
+
   /* Number of authentication methods supported. */
   unsigned int num_methods;
 
   /* Array defining the supported authentication methods. */
-  const SshAuthServerMethod *methods;
+  SshAuthServerMethod *methods;
 
   /* Context argument to pass to the authentication method functions. */
   void *method_context;
@@ -55,13 +56,13 @@ typedef struct {
   /* Number of the method that is active (valid only if waiting_continuation
      is TRUE). */
   int active_method_index;
-  
+
   /* Places where the authentication methods can store data.  There is
      one element for each authentication method.  These are allocated
      with ssh_xmalloc. */
   void **state_placeholders;     /* array of num_methods elements */
   void **longtime_placeholders;  /* array of num_methods elements */
-  
+
   /* Client host IP address, as a string (e.g., "121.23.125.98").
      This is allocated by ssh_xmalloc. */
   char *client_ip;
@@ -78,12 +79,12 @@ typedef struct {
 
   /* Copy of the startup packet received from the transport layer, or NULL
      if not yet received.  This is allocated by ssh_buffer_allocate. */
-  SshBuffer *startup_packet;
+  SshBuffer startup_packet;
 
   /* Copy of the last algorithms packet received from the transport layer,
      or NULL if not yet received.  This is allocated by ssh_buffer_allocate. */
-  SshBuffer *algorithms_packet;
-  
+  SshBuffer algorithms_packet;
+
   /* The requested user name, or NULL if no requests have yet been received.
      This is allocated by ssh_xmalloc. */
   char *requested_user;
@@ -97,6 +98,11 @@ typedef struct {
      This is allocated by ssh_xmalloc. */
   char *successful_authentications;
 
+  /* Comma-separated list of successfully performed authentications, or
+     NULL if no authentications have yet been successfully performed.
+     This is allocated by ssh_xmalloc. */
+  char *disabled_authentications;
+
   /* Comma-separated list of authentication methods that can purposefully
      continue authentication.  This is the list sent to the client in
      failure messages.  This is allocated by ssh_xmalloc. */
@@ -116,15 +122,18 @@ void ssh_auths_send_failure(SshAuthServer auth, Boolean partial_success)
   ssh_cross_down_send_encode(auth->down, SSH_CROSS_PACKET,
                              SSH_FORMAT_CHAR,
                              (unsigned int) SSH_MSG_USERAUTH_FAILURE,
-                             SSH_FORMAT_UINT32_STR, 
+                             SSH_FORMAT_UINT32_STR,
                               auth->continuations, strlen(auth->continuations),
                              SSH_FORMAT_BOOLEAN, partial_success,
                              SSH_FORMAT_END);
 }
 
 /* Sends a disconnect packet and marks the protocol as dead. */
-
-void ssh_auths_disconnect(SshAuthServer auth, const char *msg)
+/* XXX This should receive a reason code as argument. The catch-all
+   isn't always correct. It should, however, accept a default
+   argument, in which case the catch-all would be used. */
+void ssh_auths_disconnect(SshAuthServer auth, int disconnect_code,
+                          const char *msg)
 {
   /* Abort active authentication method if any. */
   if (auth->waiting_continuation)
@@ -135,15 +144,15 @@ void ssh_auths_disconnect(SshAuthServer auth, const char *msg)
 
   /* Send the disconnect message and EOF down. */
   ssh_cross_down_send_disconnect(auth->down, TRUE,
-                                 SSH_DISCONNECT_AUTHENTICATION_ERROR,
+                                 disconnect_code,
                                  "%s", msg);
   ssh_cross_down_send_eof(auth->down);
 
   /* Send the disconnect message and EOF up. */
   ssh_cross_up_send_disconnect(auth->up, TRUE,
-                               SSH_DISCONNECT_AUTHENTICATION_ERROR,
+                               disconnect_code,
                                "%s (user '%s', client address '%s:%s', "
-                               "requested service '%s')", 
+                               "requested service '%s')",
                                msg, auth->requested_user,
                                auth->client_ip,
                                auth->client_port,
@@ -179,6 +188,7 @@ Boolean ssh_auths_register_success(SshAuthServer auth, const char *method)
   cp = (*auth->policy_proc)(auth->requested_user, auth->requested_service,
                             auth->client_ip, auth->client_port,
                             auth->successful_authentications,
+                            auth->disabled_authentications,
                             auth->policy_context);
   /* XXX should remove already completed methods? */
   if (auth->continuations)
@@ -244,7 +254,7 @@ void ssh_auths_success(SshAuthServer auth)
                              auth->requested_service,
                              strlen(auth->requested_service),
                            SSH_FORMAT_END);
-  
+
   /* Shortcircuit communications between the up stream and the down stream.
      Functions in this module will no longer be called until the up stream
      is destroyed. */
@@ -264,17 +274,20 @@ void ssh_auths_call_method(SshAuthServer auth,
   int i;
   unsigned int packet_type;
   SshAuthServerResult result;
-  SshBuffer *buffer;
+  SshBuffer buffer;
 
   i = auth->active_method_index;
 
   /* Sanity checks... */
-  assert(i >= 0 && i < auth->num_methods);
-  assert(auth->requested_user != NULL);
+  SSH_ASSERT(i >= 0 && i < auth->num_methods);
+  SSH_ASSERT(auth->requested_user != NULL);
 
   /* Save the data in a buffer. */
   buffer = ssh_buffer_allocate();
   ssh_buffer_append(buffer, data, len);
+
+  if (auth->methods[i].proc == NULL)
+    return;
   
   /* Call the authentication method. */
   result = (*auth->methods[i].proc)(op, auth->requested_user, buffer,
@@ -293,8 +306,8 @@ void ssh_auths_call_method(SshAuthServer auth,
         case SSH_AUTH_SERVER_ACCEPTED:
           /* Authentication method was successful.  Should have no state
              left between packets. */
-          assert(auth->state_placeholders[i] == NULL);
-          
+          SSH_ASSERT(auth->state_placeholders[i] == NULL);
+
           /* Register the success, and determine what to do next. */
           if (ssh_auths_register_success(auth, auth->methods[i].name))
             {
@@ -310,22 +323,45 @@ void ssh_auths_call_method(SshAuthServer auth,
               ssh_auths_send_failure(auth, TRUE);
             }
           break;
-          
+
         case SSH_AUTH_SERVER_REJECTED:
           /* This authentication method was rejected (but it might have
              had side-effects that were recorded in longtime_placeholder). */
-          assert(auth->state_placeholders[i] == NULL);
+          SSH_ASSERT(auth->state_placeholders[i] == NULL);
           auth->waiting_continuation = FALSE;
           ssh_auths_send_failure(auth, FALSE);
           break;
-      
+
         case SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED:
-          /* This authentication method was rejected and cannot be 
+          /* This authentication method was rejected and cannot be
              used again in this authentication.  This can be used
              for example in limiting login attempts. */
-          assert(auth->state_placeholders[i] == NULL);
-          auth->waiting_continuation = FALSE;
-          ssh_auths_disconnect(auth, "Authentication method disabled."); 
+          {
+            char *to_be_deleted;
+            SSH_ASSERT(auth->state_placeholders[i] == NULL);
+            auth->waiting_continuation = FALSE;
+            auth->methods[auth->active_method_index].proc = NULL;
+            to_be_deleted = auth->disabled_authentications;
+            ssh_dsprintf(&auth->disabled_authentications, "%s%s%s",
+                         auth->disabled_authentications,
+                         auth->disabled_authentications[0] != '\0' ? "," : "",
+                         auth->methods[auth->active_method_index].name);
+            ssh_xfree(to_be_deleted);
+
+            /* reset continuations */
+            ssh_xfree(auth->continuations);
+            auth->continuations =
+              (*auth->policy_proc)(auth->requested_user,
+                                   auth->requested_service,
+                                   auth->client_ip,
+                                   auth->client_port,
+                                   "",
+                                   auth->disabled_authentications,
+                                   auth->policy_context);
+
+            ssh_auths_send_failure(auth, FALSE);
+          }
+          
           break;
 
         case SSH_AUTH_SERVER_REJECTED_WITH_PACKET_BACK:
@@ -349,12 +385,12 @@ void ssh_auths_call_method(SshAuthServer auth,
           ssh_cross_down_send(auth->down, SSH_CROSS_PACKET,
                               ssh_buffer_ptr(buffer), ssh_buffer_len(buffer));
           break;
-      
+
         case SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK:
           /* The authentication method wants to send out a packet, and then
              continue authentication with the same method. */
           auth->waiting_continuation = TRUE;
-       
+
           /* Sanity check the outgoing packet. */
           if (ssh_decode_array(ssh_buffer_ptr(buffer), ssh_buffer_len(buffer),
                                SSH_FORMAT_CHAR, &packet_type,
@@ -377,17 +413,17 @@ void ssh_auths_call_method(SshAuthServer auth,
 
     case SSH_AUTH_SERVER_OP_ABORT:
       /* Aborted a continued authentication method.  Sanity check. */
-      assert(auth->state_placeholders[i] == NULL);
+      SSH_ASSERT(auth->state_placeholders[i] == NULL);
       break;
     case SSH_AUTH_SERVER_OP_UNDO_LONGTIME:
       /* Aborted (undid) the entire authentication sequence.  Sanity check. */
-      assert(auth->state_placeholders[i] == NULL);
-      assert(auth->longtime_placeholders[i] == NULL);
+      SSH_ASSERT(auth->state_placeholders[i] == NULL);
+      SSH_ASSERT(auth->longtime_placeholders[i] == NULL);
       break;
     case SSH_AUTH_SERVER_OP_CLEAR_LONGTIME:
       /* Freed memory reserved for undo data. */
-      assert(auth->state_placeholders[i] == NULL);
-      assert(auth->longtime_placeholders[i] == NULL);
+      SSH_ASSERT(auth->state_placeholders[i] == NULL);
+      SSH_ASSERT(auth->longtime_placeholders[i] == NULL);
       break;
     default:
       /* Unknown operation. */
@@ -420,7 +456,8 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
                                   SSH_FORMAT_END);
   if (parsed_bytes == 0)
     {
-      ssh_auths_disconnect(auth, "Received bad authentication packet");
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_PROTOCOL_ERROR,
+                           "Received bad authentication packet");
       return;
     }
 
@@ -430,7 +467,8 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ssh_xfree(user);
       ssh_xfree(service);
       ssh_xfree(method);
-      ssh_auths_disconnect(auth, "Received bad authentication packet.");
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_PROTOCOL_ERROR,
+                           "Received bad authentication packet.");
       return;
     }
 
@@ -439,7 +477,8 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ssh_xfree(user);
       ssh_xfree(service);
       ssh_xfree(method);
-      ssh_auths_disconnect(auth, "User name too long.");
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_ILLEGAL_USER_NAME,
+                           "User name too long.");
       return;
     }
   for (i = 0; user[i]; i++)
@@ -454,7 +493,8 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ssh_xfree(user);
       ssh_xfree(service);
       ssh_xfree(method);
-      ssh_auths_disconnect(auth, "User name contains illegal characters.");
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_ILLEGAL_USER_NAME,
+                           "User name contains illegal characters.");
       return;
     }
 
@@ -467,8 +507,9 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
                                                auth->client_ip,
                                                auth->client_port,
                                                "",
+                                               auth->disabled_authentications,
                                                auth->policy_context);
-  
+
   /* If continuing authentication and method is different, abort the old
      authentication method. */
   if (auth->waiting_continuation &&
@@ -478,7 +519,7 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ssh_auths_call_method(auth, SSH_AUTH_SERVER_OP_ABORT, NULL, 0);
       auth->waiting_continuation = FALSE;
     }
-  
+
   /* If the user or service has changed, undo all side-effects by
      authentications and clear state to as if authentication was just
      starting. */
@@ -501,13 +542,14 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
                                                  auth->client_ip,
                                                  auth->client_port,
                                                  "",
+                                                 auth->disabled_authentications,
                                                  auth->policy_context);
       if (auth->successful_authentications)
         {
           ssh_xfree(auth->successful_authentications);
           auth->successful_authentications = NULL;
         }
-      assert(!auth->waiting_continuation);
+      SSH_ASSERT(!auth->waiting_continuation);
     }
 
   /* Save user and service if not already saved. */
@@ -520,7 +562,7 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
   /* Find the method. */
   auth->active_method_index = -1;
   for (i = 0; auth->methods[i].name != NULL; i++)
-    if (strcmp(auth->methods[i].name, method) == 0)
+    if (auth->methods[i].proc && strcmp(auth->methods[i].name, method) == 0)
       {
         auth->active_method_index = i;
         break;
@@ -537,7 +579,7 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ssh_auths_send_failure(auth, FALSE);
       return;
     }
-  
+
   /* Process the authentication request. */
   if (auth->waiting_continuation)
     ssh_auths_call_method(auth, SSH_AUTH_SERVER_OP_CONTINUE,
@@ -557,13 +599,13 @@ void ssh_auths_down_received_packet(SshCrossPacketType type,
 {
   SshAuthServer auth = (SshAuthServer)context;
   char *service;
-  
+
   switch (type)
     {
     case SSH_CROSS_PACKET:
       ssh_auths_process_request(auth, data, len);
       break;
-      
+
     case SSH_CROSS_DISCONNECT:
       SSH_DEBUG(6, ("received_packet: DISCONNECT from transport code."));
       ssh_cross_up_send(auth->up, SSH_CROSS_DISCONNECT, data, len);
@@ -574,26 +616,27 @@ void ssh_auths_down_received_packet(SshCrossPacketType type,
       ssh_debug("Received DEBUG packet.");
       ssh_cross_up_send(auth->up, SSH_CROSS_DEBUG, data, len);
       break;
-      
+
     case SSH_CROSS_STARTUP:
       /* Received a startup packet.  We save the entire packet for passing
          up later.  We also extract from the packet the fields that are
          relevant for authentication. */
       SSH_DEBUG(6, ("down_received_packet: Received STARTUP packet."));
-      assert(auth->client_ip == NULL);
+      SSH_ASSERT(auth->client_ip == NULL);
       auth->startup_packet = ssh_buffer_allocate();
       ssh_buffer_append(auth->startup_packet, data, len);
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32_STR, NULL, NULL,
-                           SSH_FORMAT_UINT32_STR, 
+                           SSH_FORMAT_UINT32_STR,
                              &auth->session_id, &auth->session_id_len,
                            SSH_FORMAT_UINT32_STR, NULL, NULL,
                            SSH_FORMAT_UINT32_STR, &auth->client_ip, NULL,
                            SSH_FORMAT_UINT32_STR, &auth->client_port, NULL,
                            SSH_FORMAT_END) == 0)
-        ssh_auths_disconnect(auth, "Bad startup packet");
+        ssh_auths_disconnect(auth, SSH_DISCONNECT_PROTOCOL_ERROR,
+                             "Bad startup packet");
       break;
-      
+
     case SSH_CROSS_ALGORITHMS:
       SSH_DEBUG(6, ("down_received_packet: Received ALGORITHMS packet."));
       if (auth->algorithms_packet == NULL)
@@ -609,18 +652,36 @@ void ssh_auths_down_received_packet(SshCrossPacketType type,
                            SSH_FORMAT_UINT32_STR, &service, NULL,
                            SSH_FORMAT_END) == 0)
         {
-          ssh_auths_disconnect(auth, "Bad service request packet");
+          ssh_auths_disconnect(auth, SSH_DISCONNECT_PROTOCOL_ERROR,
+                               "Bad service request packet");
           break;
         }
+
       if (strcmp(service, SSH_USERAUTH_SERVICE) == 0)
-        ssh_cross_down_send(auth->down, SSH_CROSS_SERVICE_ACCEPT, NULL, 0);
+        {
+          SshBufferStruct buffer;
+          ssh_buffer_init(&buffer);
+          ssh_encode_buffer(&buffer,
+                            SSH_FORMAT_UINT32_STR, SSH_USERAUTH_SERVICE,
+                            strlen(SSH_USERAUTH_SERVICE),
+                            SSH_FORMAT_END);
+          ssh_cross_down_send(auth->down, SSH_CROSS_SERVICE_ACCEPT,
+                              ssh_buffer_ptr(&buffer),
+                              ssh_buffer_len(&buffer));
+          ssh_buffer_uninit(&buffer);
+        }
       else
-        ssh_auths_disconnect(auth, "Wrong service name.");
+        {
+          ssh_auths_disconnect(auth, SSH_DISCONNECT_SERVICE_NOT_AVAILABLE,
+                               "Wrong service name.");
+        }
+      
       break;
 
     default:
       ssh_debug("Received unknown cross packet %d", (int)type);
-      ssh_auths_disconnect(auth, "Bad incoming cross packet");
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_BY_APPLICATION,
+                           "Bad incoming cross packet");
     }
 }
 
@@ -646,7 +707,7 @@ void ssh_auths_up_destroy(void *context)
      any authentication packets yet). */
   if (auth->requested_user != NULL)
     ssh_auths_clear_all_state(auth, SSH_AUTH_SERVER_OP_UNDO_LONGTIME);
-  
+
   /* Destroy the downward stream.  Note that its buffers will automatically
      drain before it is actually destroyed. */
   ssh_cross_down_destroy(auth->down);
@@ -654,25 +715,19 @@ void ssh_auths_up_destroy(void *context)
   /* Free any memory. */
   ssh_xfree(auth->state_placeholders);
   ssh_xfree(auth->longtime_placeholders);
-  if (auth->client_ip)
-    ssh_xfree(auth->client_ip);
-  if (auth->client_port)
-    ssh_xfree(auth->client_port);
-  if (auth->session_id)
-    ssh_xfree(auth->session_id);
+  ssh_xfree(auth->client_ip);
+  ssh_xfree(auth->client_port);
+  ssh_xfree(auth->session_id);
   if (auth->startup_packet)
     ssh_buffer_free(auth->startup_packet);
   if (auth->algorithms_packet)
     ssh_buffer_free(auth->algorithms_packet);
-  if (auth->requested_user)
-    ssh_xfree(auth->requested_user);
-  if (auth->requested_service)
-    ssh_xfree(auth->requested_service);
-  if (auth->successful_authentications)
-    ssh_xfree(auth->successful_authentications);
-  if (auth->continuations)
-    ssh_xfree(auth->continuations);
-
+  ssh_xfree(auth->requested_user);
+  ssh_xfree(auth->requested_service);
+  ssh_xfree(auth->successful_authentications);
+  ssh_xfree(auth->continuations);
+  ssh_xfree(auth->disabled_authentications);
+  
   /* Fill with known "garbage" value to ease debugging. */
   memset(auth, 'F', sizeof(*auth));
   ssh_xfree(auth);
@@ -685,10 +740,11 @@ char *ssh_auths_default_policy_proc(const char *user, const char *service,
                                     const char *client_ip,
                                     const char *client_port,
                                     const char *completed,
+                                    const char *disabled,
                                     void *context)
 {
   SshAuthServer auth = (SshAuthServer)context;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   char *cp;
   int i;
 
@@ -707,6 +763,9 @@ char *ssh_auths_default_policy_proc(const char *user, const char *service,
   ssh_buffer_init(&buffer);
   for (i = 0; i < auth->num_methods; i++)
     {
+      if (strstr(disabled, auth->methods[i].name))
+        continue;
+      
       if (i > 0)
         ssh_buffer_append(&buffer, (unsigned char *) ",", 1);
       ssh_buffer_append(&buffer, (unsigned char *) auth->methods[i].name,
@@ -767,7 +826,7 @@ char *ssh_auths_default_policy_proc(const char *user, const char *service,
 SshStream ssh_auth_server_wrap(SshStream transport,
                                SshAuthPolicyProc policy_proc,
                                void *policy_context,
-                               const SshAuthServerMethod *methods,
+                               SshAuthServerMethod *methods,
                                void *method_context)
 {
   SshAuthServer auth;
@@ -776,7 +835,7 @@ SshStream ssh_auth_server_wrap(SshStream transport,
   /* Count the number of authentication methods avaialable. */
   for (num_methods = 0; methods[num_methods].name; num_methods++)
     ;
-  
+
   /* Allocate and initialize memory. */
   auth = ssh_xcalloc(sizeof(*auth), 1);
   auth->down = ssh_cross_down_create(transport,
@@ -810,6 +869,7 @@ SshStream ssh_auth_server_wrap(SshStream transport,
   auth->requested_user = NULL;
   auth->requested_service = NULL;
   auth->successful_authentications = NULL;
+  auth->disabled_authentications = ssh_xstrdup("");
   auth->continuations = NULL;
 
   /* Start receiving packets from the down stream. */
