@@ -8,10 +8,10 @@ sshconn.h
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-SSH Connection Protocol.
+  SSH Connection Protocol.
 
 */
 
@@ -20,6 +20,7 @@ SSH Connection Protocol.
 
 #include "sshstream.h"
 #include "sshcross.h"
+#include "sshtrans.h"
 
 /* The normal service name for the SSH connection protocol. */
 #define SSH_CONNECTION_SERVICE  "ssh-connection"
@@ -41,23 +42,23 @@ SSH Connection Protocol.
    This returns TRUE if the request was successfully processed, and FALSE
    if the request failed. */
 typedef Boolean (*SshConnGlobalRequestProc)(const char *type,
-					    const unsigned char *data,
-					    size_t len,
-					    void *context);
+                                            const unsigned char *data,
+                                            size_t len,
+                                            void *context);
 
 /* Callback function for processing channel requests.  This is registered
    when channel open completes successfully, and is called whenever a channel
    request is received for that channel.
      `type'        request type, e.g. "pty"
-     `data'	   request type specific part of request packet
+     `data'        request type specific part of request packet
      `len'         length of request-specific part
      `callback_context' the context argument registered with the callback
    This returns TRUE if the request was successfully processed, and FALSE
    if the request failed. */
 typedef Boolean (*SshConnChannelRequestProc)(const char *type,
-					     const unsigned char *data,
-					     size_t len,
-					     void *callback_context);
+                                             const unsigned char *data,
+                                             size_t len,
+                                             void *callback_context);
 
 /* Callback function for destroying a channel.  The destroy function,
    if specified, always gets called for a channel, regardless of
@@ -78,28 +79,28 @@ typedef void (*SshConnChannelDestroyProc)(void *callback_context);
    callback is called just before closing ``data_stream'' if non-NULL.
    ``data_stream'' will be closed automatically by the connection protocol
    after calling ``destroy''.  ``stream' may be SSH_CONN_POSTPONE_STREAM.
-     `result'		result code (SSH_OPEN_*, e.g. SSH_RESULT_OK)
+     `result'           result code (SSH_OPEN_*, e.g. SSH_RESULT_OK)
      `data_stream'      stream to use for communication, or NULL
      `auto_close'       TRUE means close data_stream when channel closed
      `close_on_eof'     if TRUE, close channel when EOF from stream
-     `window_size'	size of incoming buffer
-     `data'		type-specific part of open confirmation reply
+     `window_size'      size of incoming buffer
+     `data'             type-specific part of open confirmation reply
      `len'              length of the type-specific data
-     `request'		callback for processing channel request, or NULL
+     `request'          callback for processing channel request, or NULL
      `destroy'          callback for destroying callback_context, or NULL
      `callback_context' context to pass to ``request'' and ``destroy''
      `completion_context' the original completion_context argument */
 typedef void (*SshConnOpenCompletionProc)(int result,
-					  SshStream data_stream,
-					  Boolean auto_close,
-					  Boolean close_on_eof,
-					  size_t window_size,
-					  const unsigned char *data,
-					  size_t len,
-					  SshConnChannelRequestProc request,
-					  SshConnChannelDestroyProc destroy,
-					  void *callback_context,
-					  void *completion_context);
+                                          SshStream data_stream,
+                                          Boolean auto_close,
+                                          Boolean close_on_eof,
+                                          size_t window_size,
+                                          const unsigned char *data,
+                                          size_t len,
+                                          SshConnChannelRequestProc request,
+                                          SshConnChannelDestroyProc destroy,
+                                          void *callback_context,
+                                          void *completion_context);
 
 /* Callback function called whenever a channel open request is
    received from the other side.  These are registered together with
@@ -117,12 +118,12 @@ typedef void (*SshConnOpenCompletionProc)(int result,
      `completion_context' argument to ``completion''
      `context'    context argument registered with the callback */
 typedef void (*SshConnChannelOpenProc)(const char *type,
-				       int channel_id,
-				       const unsigned char *data,
-				       size_t len,
-				       SshConnOpenCompletionProc completion,
-				       void *completion_context,
-				       void *context);
+                                       int channel_id,
+                                       const unsigned char *data,
+                                       size_t len,
+                                       SshConnOpenCompletionProc completion,
+                                       void *completion_context,
+                                       void *context);
 
 /* Callback function called when a disconnect message or EOF is
    received from the other side.  The connection protocol should be
@@ -133,11 +134,14 @@ typedef void (*SshConnChannelOpenProc)(const char *type,
    other side, ``reason'' will be SSH_DISCONNECT_CONNECTION_LOST,
    and ``msg'' an appropriate descriptive message.
      `reason'     numeric reason for disconnection (usable for localization)
+     `locally_generated' whether this end initiated the disconnect
      `msg'        disconnect message in English
-     `context'    context argument from when the callback was registered */
+     `context'    context argument from when the callback was registered
+*/
 typedef void (*SshConnDisconnectProc)(int reason,
-				      const char *msg,
-				      void *context);
+                                      Boolean locally_generated,
+                                      const char *msg,
+                                      void *context);
 
 /* Callback function called when a debug message is received from the other
    side.  If this is NULL, all received debug messages are ignored.
@@ -145,8 +149,8 @@ typedef void (*SshConnDisconnectProc)(int reason,
      `msg'     the debugging message (normally English)
      `context' context argument passed when the callback was registered. */
 typedef void (*SshConnDebugProc)(int type,
-				 const char *msg,
-				 void *context);
+                                 const char *msg,
+                                 void *context);
 
 /* Callback function to be called whenever a SSH_CROSS_STARTUP,
    SSH_CROSS_ALGORITHMS, or SSH_CROSS_AUTHENTICATED packet is received
@@ -157,9 +161,9 @@ typedef void (*SshConnDebugProc)(int type,
      `len'      length of the cross-layer packet data
      `context'  context argument from when the callback was registered. */
 typedef void (*SshConnSpecialProc)(SshCrossPacketType type,
-				   const unsigned char *data,
-				   size_t len,
-				   void *context);
+                                   const unsigned char *data,
+                                   size_t len,
+                                   void *context);
 
 /* Data structure for global request functions.  An array of these is passed
    to the connection protocol.  The last entry in the array should have
@@ -186,26 +190,32 @@ typedef struct SshConnRec *SshConn;
    This will call the given request functions to process global requests,
    and the given open functions to process channel opens.
      `auth_stream'         the underlying lower-level stream
-     `service_name'	   service name to accept if SERVICE_REQUEST received
-     `requests'		   array of supported requests, or NULL for none
-     `opens'		   array of supported channel types, or NULL for none
-     `disconnect'	   called if disconnect msg or EOF received, or NULL
+     `service_name'        service name to accept if SERVICE_REQUEST received
+     `requests'            array of supported requests, or NULL for none
+     `opens'               array of supported channel types, or NULL for none
+     `disconnect'          called if disconnect msg or EOF received, or NULL
      `debug'               called if debug msg received, or NULL
-     `special'		   called when a special packet is received, or NULL
-     `context'		   argument to give to request/open functions.
+     `special'             called when a special packet is received, or NULL
+     `compat_flags'        compatibility flags. Is used to enable or disable
+                           compatibilty code with older versions. Has
+                           to be a valid struct, and it is not
+                           otherwise touched by the conn-stream,
+                           ie. has to freed by the caller, if need be.     
+     `context'             argument to give to request/open functions.
    Any of the callbacks may be NULL to specify that it will be ignored.
    The ``auth_stream'' will be automatically closed when the SshConn object
    is destroyed.  The SshConn object is typically destoyed from the
    ``disconnect'' callback, though a reasonably place may also sometimes be
    the destroy callback of some channel. */
 SshConn ssh_conn_wrap(SshStream auth_stream,
-		      const char *service_name,
-		      SshConnGlobalRequest *requests,
-		      SshConnChannelOpen *opens,
-		      SshConnDisconnectProc disconnect,
-		      SshConnDebugProc debug,
-		      SshConnSpecialProc special,
-		      void *context);
+                      const char *service_name,
+                      SshConnGlobalRequest *requests,
+                      SshConnChannelOpen *opens,
+                      SshConnDisconnectProc disconnect,
+                      SshConnDebugProc debug,
+                      SshConnSpecialProc special,
+                      SshTransportCompat compat_flags,
+                      void *context);
 
 /* Destroys the conn protocol, closes the underlying stream, calls the
    destroy function for all open channels and closes the all channel streams.
@@ -233,7 +243,7 @@ void ssh_conn_send_debug(SshConn conn, Boolean display, const char *fmt, ...);
      `success'     TRUE indicates request successfully processed, FALSE failed
      `context'     context argument from sending the request. */
 typedef void (*SshConnSendGlobalRequestCallback)(Boolean success,
-						 void *context);
+                                                 void *context);
 
 /* Sends a global request to the stream.  ``data'' is type-specific
    part of the request.  The completion procedure will be called when
@@ -241,16 +251,16 @@ typedef void (*SshConnSendGlobalRequestCallback)(Boolean success,
    about whether the request completed successfully or not.
      `conn'      the connection protocol object
      `type'      request type
-     `data'	 type-specific request data, NULL if none
+     `data'      type-specific request data, NULL if none
      `len'       length of type-specific request data, 0 if none
      `completion' completion procedure to call when reply received, or NULL
      `context'   context to pass to completion procedure */
 void ssh_conn_send_global_request(SshConn conn,
-				  const char *type,
-				  const unsigned char *data,
-				  size_t len,
-				  SshConnSendGlobalRequestCallback completion,
-				  void *completion_context);
+                                  const char *type,
+                                  const unsigned char *data,
+                                  size_t len,
+                                  SshConnSendGlobalRequestCallback completion,
+                                  void *completion_context);
 
 /* Callback function to be called when a channel open request has been
    processed and a reply has been received from the remote side.
@@ -263,20 +273,20 @@ void ssh_conn_send_global_request(SshConn conn,
    channel failed.  The specified ``destroy'' function will be called if the
    request fails. */
 typedef void (*SshConnSendChannelOpenCallback)(int result,
-					       int channel_id,
-					       const unsigned char *data,
-					       size_t len,
-					       void *context);
+                                               int channel_id,
+                                               const unsigned char *data,
+                                               size_t len,
+                                               void *context);
 
 /* Sends a channel open request to the stream.
      `conn'        the connection protocol object
-     `type'	   channel type to open
+     `type'        channel type to open
      `data_stream' data stream for the channel
      `auto_close'  if TRUE, close ``data_stream'' when channel closed
      `close_on_eof' if TRUE, automatically close channel when EOF from stream
      `window_size' maximum window size for the channel
      `max_packet_size' max size of data packet to send
-     `data'	   type-specific data for the request, NULL if none
+     `data'        type-specific data for the request, NULL if none
      `len'         length of type-specific data, 0 if none
      `request'     function to process channel requests, or NULL
      `destroy'     function to process channel destroy, or NULL
@@ -286,30 +296,30 @@ typedef void (*SshConnSendChannelOpenCallback)(int result,
    If the open fails, the ``destroy'' function will be called for the
    ``callback_context''. */
 void ssh_conn_send_channel_open(SshConn conn,
-				const char *type,
-				SshStream data_stream,
-				Boolean auto_close,
-				Boolean close_on_eof,
-				size_t window_size,
-				size_t max_packet_size,
-				const unsigned char *data,
-				size_t len,
-				SshConnChannelRequestProc request,
-				SshConnChannelDestroyProc destroy,
-				void *callback_context,
-				SshConnSendChannelOpenCallback completion,
-				void *completion_context);
+                                const char *type,
+                                SshStream data_stream,
+                                Boolean auto_close,
+                                Boolean close_on_eof,
+                                size_t window_size,
+                                size_t max_packet_size,
+                                const unsigned char *data,
+                                size_t len,
+                                SshConnChannelRequestProc request,
+                                SshConnChannelDestroyProc destroy,
+                                void *callback_context,
+                                SshConnSendChannelOpenCallback completion,
+                                void *completion_context);
 
 /* Callback function to be called when a reply has been received for a
    channel request.
      `success'    TRUE indicates that the request was processed successfully
-     `data'	  request-specific part of reply
+     `data'       request-specific part of reply
      `len'        length of request-specific part
      `context'    context argument from sending the request. */
 typedef void (*SshConnSendChannelRequestCallback)(Boolean success,
-						  const unsigned char *data,
-						  size_t len,
-						  void *context);
+                                                  const unsigned char *data,
+                                                  size_t len,
+                                                  void *context);
 
 /* Sends a channel request to the stream.  ``data'' is type-specific part
    of the request.  The callback ``cb'' will be called when a reply has
@@ -324,12 +334,12 @@ typedef void (*SshConnSendChannelRequestCallback)(Boolean success,
      `cb'          callback to be called when reply received, or NULL
      `context'     context to pass to ``cb''. */
 void ssh_conn_send_channel_request(SshConn conn,
-				   int channel_id,
-				   const char *type,
-				   const unsigned char *data,
-				   size_t len,
-				   SshConnSendChannelRequestCallback cb,
-				   void *context);
+                                   int channel_id,
+                                   const char *type,
+                                   const unsigned char *data,
+                                   size_t len,
+                                   SshConnSendChannelRequestCallback cb,
+                                   void *context);
 
 /* Registers a separate stream for extended data of the specified type.
      `conn'         the connection protocol object
@@ -345,11 +355,11 @@ void ssh_conn_send_channel_request(SshConn conn,
    this from a channel open callback, but only after calling the completion
    procedure. */
 void ssh_conn_channel_register_extended(SshConn conn,
-					int channel_id,
-					int extended_type,
-					SshStream stream,
-					Boolean write_only,
-					Boolean auto_close);
+                                        int channel_id,
+                                        int extended_type,
+                                        SshStream stream,
+                                        Boolean write_only,
+                                        Boolean auto_close);
 
 /* Registers a callback that is to be called when EOF is received from the
    primary channel stream.  This is called before the stream is closed (if
@@ -359,9 +369,9 @@ void ssh_conn_channel_register_extended(SshConn conn,
      `callback'       function to call when EOF received
      `context'        argument to pass to ``callback''. */
 void ssh_conn_channel_register_eof_callback(SshConn conn,
-					    int channel_id,
-					    void (*callback)(void *context),
-					    void *context);
+                                            int channel_id,
+                                            void (*callback)(void *context),
+                                            void *context);
 
 /* Closes the channel.  The channel will be destroyed (and the destroy
    callback called) when we receive a response from the remote side.
@@ -374,7 +384,7 @@ void ssh_conn_channel_close(SshConn conn, int channel_id);
    ssh_cross_down_send_encode.
      `conn'        the connection protocol object
      `type'        cross-layer packet type to send
-     `...'	   varialble length SSH_FORMAT_* list. */
+     `...'         varialble length SSH_FORMAT_* list. */
 void ssh_conn_send_encode(SshConn conn, SshCrossPacketType type, ...);
 
 #endif /* SSHCONN_H */
