@@ -8,10 +8,10 @@ sshcross.c
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-Functions for encoding cross-layer packets.
+  Functions for encoding cross-layer packets.
 
 */
 
@@ -27,7 +27,7 @@ Functions for encoding cross-layer packets.
    by arguments of the appropriate type, and the list must end with
    SSH_FORMAT_END.  This returns the number of bytes added to the buffer. */
 
-size_t ssh_cross_encode_packet(SshBuffer *buffer, SshCrossPacketType type, ...)
+size_t ssh_cross_encode_packet(SshBuffer buffer, SshCrossPacketType type, ...)
 {
   va_list ap;
 
@@ -43,7 +43,7 @@ size_t ssh_cross_encode_packet(SshBuffer *buffer, SshCrossPacketType type, ...)
    by arguments of the appropriate type, and the list must end with
    SSH_FORMAT_END.  This returns the number of bytes added to the buffer. */
 
-size_t ssh_cross_encode_packet_va(SshBuffer *buffer,
+size_t ssh_cross_encode_packet_va(SshBuffer buffer,
                                   SshCrossPacketType type,
                                   va_list ap)
 {
@@ -61,7 +61,7 @@ size_t ssh_cross_encode_packet_va(SshBuffer *buffer,
                     SSH_FORMAT_END);
 
   /* Encode the packet payload. */
-  payload_size = ssh_encode_va(buffer, ap);
+  payload_size = ssh_encode_buffer_va(buffer, ap);
 
   /* Update the packet header to contain the correct payload size. */
   p = ssh_buffer_ptr(buffer);
