@@ -9,13 +9,13 @@ sshcrup.h
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-Helper functions for the cross-layer protocol.  This file contains
-functions to make it very easy to implement a cross layer stream (the
-"Up" direction, which implements the stream functionality and looks up
-like a normal stream).
+  Helper functions for the cross-layer protocol.  This file contains
+  functions to make it very easy to implement a cross layer stream (the
+  "Up" direction, which implements the stream functionality and looks up
+  like a normal stream).
 
 */
 
@@ -29,12 +29,14 @@ like a normal stream).
 #include "sshencode.h"
 #include "sshcross.h"
 
+#define SSH_DEBUG_MODULE "SshProtoSshCrup"
+
 #define ALLOW_AFTER_BUFFER_FULL         (10000 + 5)
 #define BUFFER_MAX_SIZE                 50000
 
 typedef struct SshCrossUpRec {
   /* SshBuffer for a partial incoming packet. */
-  SshBuffer incoming;
+  SshBufferStruct incoming;
 
   /* Flag indicating whether the callback may be called (determines whether
      we can receive more data). */
@@ -53,16 +55,16 @@ typedef struct SshCrossUpRec {
 
   /* This flag is TRUE if ssh_cross_up_can_send has returned FALSE. */
   Boolean send_blocked;
-  
+
   /* SshBuffer for outgoing data. */
-  SshBuffer outgoing;
+  SshBufferStruct outgoing;
 
   /* Outgoing EOF. */
   Boolean outgoing_eof;
 
   /* Shortcircuit stream.  This is NULL if shortcircuiting not in effect. */
   SshStream shortcircuit_stream;
-  
+
   /* Callbacks to the actual protocol implementation code. */
 
   SshCrossPacketProc received_packet;
@@ -158,7 +160,7 @@ int ssh_cross_up_read(void *context, unsigned char *buf, size_t size)
 {
   SshCrossUp up = (SshCrossUp)context;
   size_t len;
-  
+
   /* Compute the number of bytes we can transmit. */
   len = ssh_buffer_len(&up->outgoing);
   if (len > size)
@@ -180,7 +182,7 @@ int ssh_cross_up_read(void *context, unsigned char *buf, size_t size)
           return -1;
         }
     }
-  
+
   /* Move data to the caller's buffer. */
   memcpy(buf, ssh_buffer_ptr(&up->outgoing), len);
   ssh_buffer_consume(&up->outgoing, len);
@@ -188,11 +190,11 @@ int ssh_cross_up_read(void *context, unsigned char *buf, size_t size)
   /* Wake up the sender if appropriate. */
   if (ssh_buffer_len(&up->outgoing) == 0)
     ssh_cross_up_restart_send(up);
-  
+
   return len;
 }
 
-/* This function is called when the upper layer writes to the stream. 
+/* This function is called when the upper layer writes to the stream.
    Note that there are essentially two very different cases: an entire
    packet is received at once, and a partial packet is received.  */
 
@@ -206,7 +208,7 @@ int ssh_cross_up_write(void *context, const unsigned char *buf,
   /* If shortcircuiting, direct the write down. */
   if (up->shortcircuit_stream)
     {
-      assert(ssh_buffer_len(&up->incoming) == 0);
+      SSH_ASSERT(ssh_buffer_len(&up->incoming) == 0);
       return ssh_stream_write(up->shortcircuit_stream, buf, size);
     }
 
@@ -219,7 +221,7 @@ normal:
       /* If already processing a partial packet, continue it now. */
       if (ssh_buffer_len(&up->incoming) > 0)
         goto partial;
-      
+
       /* If only partial packet available, do special proccessing. */
       if (size - offset < 4)
         goto partial;  /* Need partial packet processing. */
@@ -227,7 +229,7 @@ normal:
 
       if (size - offset < 4 + payload_len)
         goto partial;  /* Need partial packet processing. */
-      
+
       /* The entire packet is available; pass it to the callback. */
       if (up->received_packet)
         (*up->received_packet)((SshCrossPacketType)buf[offset + 4],
@@ -276,10 +278,10 @@ partial:
   /* The entire packet is now in buffer. */
   ucp = ssh_buffer_ptr(&up->incoming);
   if (up->received_packet)
-    (*up->received_packet)((SshCrossPacketType)ucp[4], ucp + 5, 
+    (*up->received_packet)((SshCrossPacketType)ucp[4], ucp + 5,
                            payload_len - 1,
                            up->context);
-  
+
   /* Clear the incoming partial packet buffer and resume normal processing. */
   ssh_buffer_clear(&up->incoming);
   goto normal;
@@ -297,7 +299,7 @@ void ssh_cross_up_output_eof(void *context)
       ssh_stream_output_eof(up->shortcircuit_stream);
       return;
     }
-  
+
   /* Mark that we have received EOF. */
   up->incoming_eof = TRUE;
 
@@ -348,7 +350,7 @@ void ssh_cross_up_destroy(void *context)
   ssh_cancel_timeouts(ssh_cross_up_signal_output_proc, (void *)up);
   ssh_cancel_timeouts(ssh_cross_up_signal_input_proc, (void *)up);
   ssh_cancel_timeouts(ssh_cross_up_signal_send_proc, (void *)up);
-  
+
   /* Uninitialize the buffers. */
   ssh_buffer_uninit(&up->outgoing);
   ssh_buffer_uninit(&up->incoming);
@@ -390,7 +392,7 @@ SshStream ssh_cross_up_create(SshCrossPacketProc received_packet,
                               void *context)
 {
   SshCrossUp up;
-  
+
   /* Allocate and initialize the context. */
   up = ssh_xcalloc(1, sizeof(*up));
   ssh_buffer_init(&up->incoming);
@@ -401,7 +403,7 @@ SshStream ssh_cross_up_create(SshCrossPacketProc received_packet,
   up->up_write_blocked = FALSE;
   up->up_read_blocked = FALSE;
   up->send_blocked = TRUE; /* Cause a callback immediately. */
-  
+
   /* Save the callback functions. */
   up->received_packet = received_packet;
   up->received_eof = received_eof;
@@ -415,13 +417,13 @@ SshStream ssh_cross_up_create(SshCrossPacketProc received_packet,
   /* Cause the send callback to be called if non-NULL.  Note that it isn't
      called until from the bottom of the event loop. */
   ssh_cross_up_restart_send(up);
-  
+
   /* Wrap it into a stream and return the stream. */
   return ssh_stream_create(&ssh_cross_up_methods, (void *)up);
 }
 
 /* Informs the cross layer code leyer about whether the more packets
-   from up can be received (i.e., whether `received_packet' may be called). 
+   from up can be received (i.e., whether `received_packet' may be called).
    Initially, packets cannot be received. */
 
 void ssh_cross_up_can_receive(SshStream up_stream, Boolean status)
@@ -487,8 +489,8 @@ Boolean ssh_cross_up_can_send(SshStream up_stream)
     up->send_blocked = TRUE;
 
   return status;
-}    
-  
+}
+
 /* Sends a cross-layer packet up, encoding the contents of the packet as
    specified for ssh_encode_cross_packet. */
 
@@ -503,14 +505,14 @@ void ssh_cross_up_send_encode_va(SshStream up_stream,
     ssh_fatal("ssh_cross_up_can_receive: not a SshCrossUp stream");
   /* Get the internal context. */
   up = (SshCrossUp)ssh_stream_get_context(up_stream);
-  
+
   /* Wrap the data into a cross layer packet and append to the outgoing
      stream. */
   ssh_cross_encode_packet_va(&up->outgoing, type, va);
 
   /* Restart reads by upper level. */
   ssh_cross_up_restart_input(up);
-  
+
   /* Sanity check that we didn't exceed max buffer size. */
   if (ssh_buffer_len(&up->outgoing) > BUFFER_MAX_SIZE)
     ssh_debug("ssh_cross_up_send: buffer max size exceeded: size %ld",
@@ -598,7 +600,7 @@ void ssh_cross_up_send_debug_va(SshStream up_stream, Boolean always_display,
   /* Wrap the data into a cross layer packet and append to the outgoing
      stream. */
   ssh_cross_up_send_encode(up_stream, SSH_CROSS_DEBUG,
-                           SSH_FORMAT_BOOLEAN, always_display, 
+                           SSH_FORMAT_BOOLEAN, always_display,
                            SSH_FORMAT_UINT32_STR, buf, strlen(buf),
                            SSH_FORMAT_UINT32_STR, lang, strlen(lang),
                            SSH_FORMAT_END);
@@ -638,8 +640,8 @@ void ssh_cross_up_shortcircuit_now(SshStream up_stream, SshStream down_stream)
   up->shortcircuit_stream = down_stream;
 
   /* We currently require there to be no partial incoming packet. */
-  assert(ssh_buffer_len(&up->incoming) == 0);
-  
+  SSH_ASSERT(ssh_buffer_len(&up->incoming) == 0);
+
   /* If it is non-NULL, make it use application callbacks directly. */
   if (down_stream)
     ssh_stream_set_callback(down_stream, up->up_callback, up->up_context);
