@@ -8,16 +8,16 @@ sshtrans.c
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-SSH transport layer protocol implementation.  This layer performs
-encryption, integrity, server host authentication, and compression.
+  SSH transport layer protocol implementation.  This layer performs
+  encryption, integrity, server host authentication, and compression.
 
 */
 
 /*
- * $Id: sshtrans.c,v 1.13 1999/04/16 14:41:23 sjl Exp $
+ * $Id: sshtrans.c,v 1.21 2000/05/22 10:39:00 sjl Exp $
  * $Log: sshtrans.c,v $
  * $EndLog$
  */
@@ -29,11 +29,12 @@ encryption, integrity, server host authentication, and compression.
 #include "namelist.h"
 #include "sshcipherlist.h"
 
-#define DEFAULT_CIPHERS         "3des-cbc,idea-cbc,blowfish-cbc,none"
-#define DEFAULT_MACS            "hmac-sha,hmac-md5,sha-8,md5-8,sha,none"
+#define SSH_DEBUG_MODULE "Ssh2Trans"
+
+#define DEFAULT_CIPHERS         SSH_STD_CIPHERS
+#define DEFAULT_MACS            SSH_STD_MACS
 #define DEFAULT_COMPRESSIONS    "none,zlib"
-#define DEFAULT_KEXS            "diffie-hellman-group1-sha1,"\
-                                "double-encrypting-sha1"
+#define DEFAULT_KEXS            "diffie-hellman-group1-sha1"
 
 /* Creates default transport protocol parameter structure.  This structure
    can be modified to choose different parameters. */
@@ -46,11 +47,12 @@ SshTransportParams ssh_transport_create_params(void)
   params = ssh_xmalloc(sizeof(*params));
   params->kex_algorithms = ssh_xstrdup(DEFAULT_KEXS);
   hlp = ssh_public_key_get_supported();
-  params->host_key_algorithms = ssh_public_key_list_canonialize(hlp);
+  params->host_key_algorithms = ssh_public_key_list_canonicalize(hlp);
   ssh_xfree(hlp);
   params->hash_algorithms = ssh_hash_get_supported();
   params->compressions_c_to_s = ssh_xstrdup(DEFAULT_COMPRESSIONS);
   params->compressions_s_to_c = ssh_xstrdup(DEFAULT_COMPRESSIONS);
+  params->compression_level = -1;
   params->ciphers_c_to_s = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
   params->ciphers_s_to_c = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
   params->macs_c_to_s = ssh_name_list_intersection_mac(DEFAULT_MACS);
@@ -82,7 +84,6 @@ void ssh_transport_destroy_params(SshTransportParams params)
  * a SshStream object representing the transport layer. */
 
 SshStream ssh_transport_client_wrap(SshStream stream,
-                                    SshRandomState random_state,
                                     const char *application_version,
                                     const char *service,
                                     SshTransportParams params,
@@ -99,7 +100,7 @@ SshStream ssh_transport_client_wrap(SshStream stream,
 
   tr = ssh_tr_create(stream, FALSE, TRUE, version_callback != NULL,
                      application_version,
-                     random_state, params);
+                     params);
   tr->version_callback = version_callback;
   tr->version_context = version_context;
 
@@ -113,7 +114,6 @@ SshStream ssh_transport_client_wrap(SshStream stream,
    a SshStream object representing the transport layer. */
 
 SshStream ssh_transport_server_wrap(SshStream stream,
-                                    SshRandomState random_state,
                                     const char *application_version,
                                     SshTransportParams params,
                                     SshPrivateKey private_host_key,
@@ -130,7 +130,7 @@ SshStream ssh_transport_server_wrap(SshStream stream,
 
   tr = ssh_tr_create(stream, TRUE, TRUE, version_callback != NULL,
                      application_version,
-                     random_state, params);
+                     params);
   tr->version_callback = version_callback;
   tr->version_context = version_context;
 
@@ -145,17 +145,23 @@ SshStream ssh_transport_server_wrap(SshStream stream,
 void ssh_transport_get_statistics(SshStream stream,
                                   SshTransportStatistics *stats)
 {
-  SshTransportCommon tr;
-
   /* Verify that this is a transport stream. */
   if (ssh_stream_get_methods(stream) != &ssh_tr_methods)
     {
       memset(stats, 0, sizeof(*stats));
       return;
     }
+  
+  ssh_transport_get_statistics_tr(ssh_stream_get_context(stream),
+                                  stats);
+}
 
-  /* Get the real object. */
-  tr = (SshTransportCommon)ssh_stream_get_context(stream);
+void ssh_transport_get_statistics_tr(void *tr_context,
+                                     SshTransportStatistics *stats)
+{
+  SshTransportCommon tr = (SshTransportCommon)tr_context;
+
+  SSH_PRECOND(tr);
   
   /* Return statistics data. */
   stats->compressed_incoming_bytes = tr->compressed_incoming_bytes;
@@ -163,7 +169,23 @@ void ssh_transport_get_statistics(SshStream stream,
   stats->compressed_outgoing_bytes = tr->compressed_outgoing_bytes;
   stats->uncompressed_outgoing_bytes = tr->uncompressed_outgoing_bytes;
   stats->incoming_packets = tr->incoming_sequence_number;
-  stats->outgoing_packets = tr->outgoing_sequence_number;
+  stats->outgoing_packets = tr->outgoing_sequence_number;  
+
+  stats->kex_name = tr->kex_name;
+  /* Negotiated `default' public key algorithm */
+  stats->host_key_name = tr->host_key_name;
+  /* List of common host key algorithms */
+  stats->host_key_names = tr->host_key_names;
+
+  /* Client to server. */
+  stats->cipher_c_to_s = tr->c_to_s.cipher_name;
+  stats->mac_c_to_s = tr->c_to_s.mac_name;
+  stats->compression_c_to_s = tr->c_to_s.compression_name;
+
+  /* Server to client. */
+  stats->cipher_s_to_c = tr->s_to_c.cipher_name;
+  stats->mac_s_to_c = tr->s_to_c.mac_name;
+  stats->compression_s_to_c = tr->s_to_c.compression_name;
 }
 
 /* Return application level compatibility flags. Note that this must
@@ -187,8 +209,32 @@ void ssh_transport_get_compatibility_flags(SshStream stream,
   tr = (SshTransportCommon)ssh_stream_get_context(stream);
 
   rec = ssh_xcalloc(1, sizeof(*rec));
-  rec->publickey_draft_incompatility =
+  rec->publickey_draft_incompatibility =
     &(tr->ssh_old_publickey_bug_compat);
 
+  rec->publickey_service_name_draft_incompatibility =
+    &(tr->ssh_old_publickey_service_name_bug_compat);
+  
+  rec->x11_channel_open_draft_incompatibility =
+    &(tr->ssh_old_x11_channel_open_bug_compat);
+
+  rec->channel_open_failure_draft_incompatibility =
+    &(tr->ssh_old_channel_open_failure_bug_compat);
+
+  rec->publickey_pk_ok_draft_incompatibility =
+    &(tr->ssh_old_publickey_pk_ok_msg_bug_compat);
+  
+  rec->hostbased_service_name_draft_incompatibility =
+    &(tr->ssh_old_hostbased_service_name_bug_compat);
+
+  rec->publickey_session_id_encoding_draft_incompatibility =
+    &(tr->ssh_old_publickey_session_id_encoding_bug_compat);
+
+  rec->malformed_signatures_draft_incompatibility =
+    &(tr->ssh_old_malformed_signatures_bug_compat);
+
+  rec->deprecated_disconnect_codes_draft_incompatibility =
+    &(tr->ssh_old_deprecated_disconnect_codes_compat);
+  
   *compat_flags = rec;
 }
