@@ -19,13 +19,15 @@ Test program for the user authentication protocol.
 #include "sshmsgs.h"
 #include "sshauth.h"
 #include "sshtimeouts.h"
-#include "sshunixeloop.h"
+#include "ssheloop.h"
 #include "sshcipherlist.h"
+#include "ssh2pubkeyencode.h"
+
+#define SSH_DEBUG_MODULE "SshTUserAuth"
 
 #undef DEBUG
 #define TEST_VERSION "t-userauth"
 
-SshRandomState random_state;
 SshPrivateKey hostkey, serverkey;
 unsigned char *hostkey_blob;
 unsigned int hostkey_blob_len;
@@ -39,20 +41,20 @@ void create_server_keys(SshPrivateKey *host, SshPrivateKey *server,
 #ifdef DEBUG
   ssh_debug("generating host key");
 #endif
-  if (ssh_private_key_generate(random_state, host,
-                               SSH_CRYPTO_RSA, SSH_PKF_SIZE, 768, 
+  if (ssh_private_key_generate(host,
+                               SSH_CRYPTO_RSA, SSH_PKF_SIZE, 768,
                                SSH_PKF_END) != SSH_CRYPTO_OK)
     ssh_fatal("Generating host key failed");
 #ifdef DEBUG
   ssh_debug("generating server key");
 #endif
 
-  if (ssh_private_key_generate(random_state, server,
-                               SSH_CRYPTO_RSA, SSH_PKF_SIZE, 512, 
+  if (ssh_private_key_generate(server,
+                               SSH_CRYPTO_RSA, SSH_PKF_SIZE, 512,
                                SSH_PKF_END) != SSH_CRYPTO_OK)
     ssh_fatal("Generating server key failed");
 
-#ifdef DEBUG  
+#ifdef DEBUG
   ssh_debug("deriving public host key blob");
 #endif
   public_host_key = ssh_private_key_derive_public_key(*host);
@@ -68,16 +70,16 @@ void create_test_setup(SshStream *client_return, SshStream *server_return)
 {
   SshStream client, server;
   SshStream s1, s2;
-  
+
   ssh_stream_pair_create(&s1, &s2);
 
   /* Initialize server side. */
-  server = ssh_transport_server_wrap(s1, random_state, TEST_VERSION, NULL,
+  server = ssh_transport_server_wrap(s1, TEST_VERSION, NULL,
                                      hostkey, serverkey,
-                                     hostkey_blob, hostkey_blob_len, 
+                                     hostkey_blob, hostkey_blob_len,
                                      NULL, NULL);
 
-  client = ssh_transport_client_wrap(s2, random_state, TEST_VERSION,
+  client = ssh_transport_client_wrap(s2, TEST_VERSION,
                                      SSH_USERAUTH_SERVICE, NULL,
                                      "localhost", NULL, NULL, NULL, NULL);
 
@@ -102,7 +104,7 @@ Boolean pingpong_success;
 
 SshAuthServerResult simple_password_server(SshAuthServerOperation op,
                                            const char *user,
-                                           SshBuffer *packet,
+                                           SshBuffer packet,
                                            const unsigned char *session_id,
                                            size_t session_id_len,
                                            void **state_placeholder,
@@ -116,7 +118,7 @@ SshAuthServerResult simple_password_server(SshAuthServerOperation op,
 #ifdef DEBUG
   ssh_debug("simple_password_server: op %d", (int)op);
 #endif
-  
+
   switch (op)
     {
     case SSH_AUTH_SERVER_OP_START:
@@ -138,7 +140,7 @@ SshAuthServerResult simple_password_server(SshAuthServerOperation op,
                                 SSH_FORMAT_UINT32_STR, &password, NULL,
                                 SSH_FORMAT_END) == 0)
             ssh_debug("simple_password_server: bad passwd request");
-          
+
           if (strcmp(user, CORRECT_USER) == 0)
             {
               if (!simple_user_accept)
@@ -152,7 +154,7 @@ SshAuthServerResult simple_password_server(SshAuthServerOperation op,
               }
             else
               ssh_fatal("simple_password_server: bad user");
-          
+
           if (strcmp(password, CORRECT_PASS) == 0)
             {
               if (!simple_password_accept)
@@ -166,7 +168,7 @@ SshAuthServerResult simple_password_server(SshAuthServerOperation op,
               }
             else
               ssh_fatal("simple_password_server: bad pass");
-          
+
           if (strcmp(user, CORRECT_USER) == 0 &&
               strcmp(password, CORRECT_PASS) == 0)
             {
@@ -176,7 +178,7 @@ SshAuthServerResult simple_password_server(SshAuthServerOperation op,
 #endif
               return SSH_AUTH_SERVER_ACCEPTED;
             }
-          
+
           ssh_xfree(password);
 #ifdef DEBUG
           ssh_debug("simple_password_server: rejected");
@@ -276,7 +278,7 @@ typedef struct ClientPingPongDataRec
 void simple_password_client(SshAuthClientOperation op,
                             const char *user,
                             unsigned int packet_type,
-                            SshBuffer *packet_in,
+                            SshBuffer packet_in,
                             const unsigned char *session_id,
                             size_t session_id_len,
                             void **state_placeholder,
@@ -285,13 +287,13 @@ void simple_password_client(SshAuthClientOperation op,
                             void *method_context)
 {
   char *pass;
-  SshBuffer *b;
+  SshBuffer b;
   ClientPingPongData *pd;
   SshUInt32 value;
 
 #ifdef DEBUG
   ssh_debug("simple_password_client: op %d", (int)op);
-#endif  
+#endif
 
   switch (op)
     {
@@ -299,7 +301,7 @@ void simple_password_client(SshAuthClientOperation op,
 
       simple_user_accept = random() % 2;
       simple_password_accept = random() % 2;
-      
+
       if (simple_password_may_fail && pingpong_success &&
           random() % 5 == 0)
         {
@@ -311,7 +313,7 @@ void simple_password_client(SshAuthClientOperation op,
                              completion_context);
           return;
         }
-      
+
       if (simple_user_accept)
         user = CORRECT_USER;
       else
@@ -326,7 +328,7 @@ void simple_password_client(SshAuthClientOperation op,
       if (pingpong_count > 0)
         {
           /* Send pingpong reply */
-          assert(*state_placeholder == NULL);
+          SSH_ASSERT(*state_placeholder == NULL);
           pd = ssh_xmalloc(sizeof(*pd));
           *state_placeholder = pd;
           pd->count = 0;
@@ -363,14 +365,14 @@ void simple_password_client(SshAuthClientOperation op,
         }
       ssh_buffer_free(b);
       break;
-      
+
     case SSH_AUTH_CLIENT_OP_START_NONINTERACTIVE:
 #ifdef DEBUG
       ssh_debug("simple_password_client: noninteractive failing");
 #endif
       (*completion_proc)(SSH_AUTH_CLIENT_FAIL, user, NULL, completion_context);
       break;
-      
+
     case SSH_AUTH_CLIENT_OP_CONTINUE:
 #ifdef DEBUG
       ssh_debug("simple_password_client: OP_CONTINUE");
@@ -405,7 +407,7 @@ void simple_password_client(SshAuthClientOperation op,
         }
       ssh_buffer_free(b);
       break;
-      
+
     case SSH_AUTH_CLIENT_OP_ABORT:
       if (*state_placeholder)
         {
@@ -413,7 +415,7 @@ void simple_password_client(SshAuthClientOperation op,
           *state_placeholder = NULL;
         }
       break;
-      
+
     default:
       ssh_fatal("simple_password_client: unknown op %d", (int)op);
     }
@@ -421,7 +423,7 @@ void simple_password_client(SshAuthClientOperation op,
 
 SshAuthServerResult serveronly_server(SshAuthServerOperation op,
                                       const char *user,
-                                      SshBuffer *packet,
+                                      SshBuffer packet,
                                       const unsigned char *session_id,
                                       size_t session_id_len,
                                       void **state_placeholder,
@@ -439,7 +441,7 @@ SshAuthServerResult serveronly_server(SshAuthServerOperation op,
 void clientonly_client(SshAuthClientOperation op,
                        const char *user,
                        unsigned int packet_type,
-                       SshBuffer *packet_in,
+                       SshBuffer packet_in,
                        const unsigned char *session_id,
                        size_t session_id_len,
                        void **state_placeholder,
@@ -496,17 +498,17 @@ void disconnect_test()
   SshStream s1, s2;
   unsigned char buf[8192];
   int len, i;
-  
+
   ssh_stream_pair_create(&s1, &s2);
 
   /* Initialize server side. */
-  server = ssh_transport_server_wrap(s1, random_state, TEST_VERSION, NULL,
+  server = ssh_transport_server_wrap(s1, TEST_VERSION, NULL,
                                      hostkey, serverkey,
                                      hostkey_blob, hostkey_blob_len,
                                      NULL, NULL);
 
   ssh_event_loop_run();
-  
+
   if (random() % 5 == 0)
     len = 0;
   else
@@ -531,7 +533,7 @@ void disconnect_test()
   ssh_event_loop_run();
 }
 
-  
+
 
 SshCrossDown client_down, server_down;
 
@@ -624,7 +626,7 @@ void server_received_packet(SshCrossPacketType type,
                                        "Service not available");
       ssh_xfree(service);
       break;
-                                 
+
     default:
       ssh_fatal("server_received_packet: type %d", (int)type);
     }
@@ -659,7 +661,7 @@ void simple_password_test(SshAuthPolicyProc policy_proc)
 
   if (!simple_password_may_fail && simple_password_failed)
     ssh_fatal("simple_pasword_test: failed without permission");
-  
+
   switch (random() % 4)
     {
     case 0:
@@ -715,8 +717,7 @@ int main()
   srandom(ssh_time());
 
   ssh_event_loop_initialize();
-  
-  random_state = ssh_random_allocate();
+
   create_server_keys(&hostkey, &serverkey, &hostkey_blob, &hostkey_blob_len);
 
   pingpong_success = FALSE;
@@ -732,7 +733,7 @@ int main()
     ssh_fatal("main: no successful pingpong");
 
   pingpong_success = FALSE;
-  
+
   for (pass = 0; pass < 100; pass++)
     {
 #ifdef DEBUG
@@ -744,11 +745,10 @@ int main()
 
   if (!pingpong_success)
     ssh_fatal("main: no successful pingpong");
-  
+
   ssh_private_key_free(hostkey);
   ssh_private_key_free(serverkey);
   ssh_xfree(hostkey_blob);
-  ssh_random_free(random_state);
 
   ssh_event_loop_uninitialize();
   return 0;
