@@ -8,15 +8,15 @@ trcommon.h
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-Common (client+server) internal definitions for the transport layer protocol.
+  Common (client+server) internal definitions for the transport layer protocol.
 
 */
 
 /*
- * $Id: trcommon.h,v 1.22 1999/04/29 13:38:42 huima Exp $
+ * $Id: trcommon.h,v 1.31 2000/05/28 16:21:05 kivinen Exp $
  * $Log: trcommon.h,v $
  * $EndLog$
  */
@@ -80,7 +80,6 @@ typedef struct
   SshSentState sent_state;          /* Send state. */
   SshReceivedState received_state;  /* Receive state. */
   SshStream connection;             /* Connection to other side. */
-  SshRandomState random_state;      /* Random state to use. */
   SshTransportParams params;        /* Configuration parameters. */
 
   /* Packet sequence numbers. */
@@ -88,11 +87,11 @@ typedef struct
   unsigned long outgoing_sequence_number;
   
   /* State for data going out to the connection. */
-  SshBuffer outgoing;               /* Pending outgoing data. */
+  SshBufferStruct outgoing;               /* Pending outgoing data. */
   Boolean outgoing_eof;             /* Send EOF when buffer empty. */
 
   /* State for packets coming from the connection. */
-  SshBuffer *incoming_packet;       /* Received packet. */
+  SshBuffer incoming_packet;       /* Received packet. */
   size_t incoming_packet_index;     /* How much data has been received? */
   size_t incoming_packet_len;       /* Total length of incoming packet. */
 
@@ -100,9 +99,9 @@ typedef struct
   SshStream up_stream;              /* The upward stream (ourself). */
   SshStreamCallback up_callback;    /* The application callback. */
   void *up_context;                 /* The application context. */
-  SshBuffer up_outgoing;            /* Pending outgoing data upwards. */
+  SshBufferStruct up_outgoing;            /* Pending outgoing data upwards. */
   Boolean up_outgoing_eof;          /* Send EOF after current data. */
-  SshBuffer up_incoming;            /* SshBuffer for incoming packets.*/
+  SshBufferStruct up_incoming;            /* SshBuffer for incoming packets.*/
   Boolean up_write_blocked;         /* Write from up has failed. */
   Boolean up_read_blocked;          /* Read from up has failed. */
   
@@ -152,7 +151,7 @@ typedef struct
   
   /* Shared buffer for all types of compression.  This is always allocated,
      even if no compression is in use. */
-  SshBuffer *compression_buffer;
+  SshBuffer compression_buffer;
 
   /* Statistics for compression. */
   /* XXX need 64 bit its here! */
@@ -164,10 +163,10 @@ typedef struct
   /* Copies of key exchange packets.  These are used in computing the session
      identifier (which, in turn, is used to check the integrity of the
      key exchange). */
-  SshBuffer *client_kexinit_packet;
-  SshBuffer *server_kexinit_packet;
-  SshBuffer *client_kex1_packet;
-  SshBuffer *server_kex1_packet;
+  SshBuffer client_kexinit_packet;
+  SshBuffer server_kexinit_packet;
+  SshBuffer client_kex1_packet;
+  SshBuffer server_kex1_packet;
 
   /* The unique session identifier for this session.  This is not changed
      even if keys are re-exchanged. */
@@ -184,7 +183,7 @@ typedef struct
   char *service_name;
 
   /* This is used by both server and client. */
-  SshBuffer *public_host_key_blob;
+  SshBuffer public_host_key_blob;
   
   /* For client only. */
   SshPublicKey public_host_key;
@@ -199,7 +198,7 @@ typedef struct
   /* For server only. */
   SshPrivateKey private_host_key;
   SshPrivateKey private_server_key;
-  SshBuffer *public_server_key_blob;
+  SshBuffer public_server_key_blob;
 
   /* For dh methods: the group, "secret" and the exchange buffer */
 
@@ -224,7 +223,46 @@ typedef struct
   /* Draft incompatibility bug in publickey authentication, which is
      in versions ssh-2.0.12 and earlier. */
   Boolean ssh_old_publickey_bug_compat;
-  
+
+  /* Draft incompatibility bug in handling SSH_MSG_SERVICE_ACCEPT,
+     which is in versions ssh-2.0.13 and earlier. */
+  Boolean ssh_old_msg_service_accept_bug_compat;
+
+  /* Draft incompatibility bug in publickey authentication, which is
+     in versions ssh-2.0.13 and earlier. */
+  Boolean ssh_old_publickey_service_name_bug_compat;
+
+  /* Draft incompatibility bug in SSH_MSG_CHANNEL_OPEN ("x11"), where
+     the originator address sent was a message, and the originator
+     port field was missing. This bug is in ssh-2.0.13 and earlier. */
+  Boolean ssh_old_x11_channel_open_bug_compat;
+
+  /* Draft incompatibility bug in
+     SSH_MSG_CHANNEL_OPEN_FAILURE. "Additional textual information"
+     and "language code" fields were missing. This bug is in
+     ssh-2.0.13 and earlier.*/
+  Boolean ssh_old_channel_open_failure_bug_compat;
+
+  /* Draft incompatibility bug in SSH_MSG_USERAUTH_PK_OK. "Public key
+     algorithm name from the request" field was omitted. This bug is
+     in ssh-2.0.13 and earlier.*/
+  Boolean ssh_old_publickey_pk_ok_msg_bug_compat;
+
+  /* Draft incompatibility bug in hostbased authentication (wrong
+     service name), which is in versions ssh-2.1.0 and earlier. */
+  Boolean ssh_old_hostbased_service_name_bug_compat;
+
+  /* Draft incompatibility bug in pubkey authentication (wrong
+     session_id encoding), which is in versions ssh-2.1.0 and earlier. */
+  Boolean ssh_old_publickey_session_id_encoding_bug_compat;
+
+  /* Draft incompatibility bug in signature encoding, which is in
+     versions ssh-2.1.0 and earlier. */
+  Boolean ssh_old_malformed_signatures_bug_compat;
+
+  /* draft incompatibility with deperecated disconnect code, ssh-2.1.0
+     and earlier. (draft changed in draft-ietf-secsh-transport-07.txt) */
+  Boolean ssh_old_deprecated_disconnect_codes_compat;
 } *SshTransportCommon;
 
 
@@ -236,7 +274,6 @@ SshTransportCommon ssh_tr_create(SshStream connection, Boolean server,
                                  Boolean compatibility,
                                  Boolean fake_old_version,
                                  const char *application_version,
-                                 SshRandomState random_state,
                                  SshTransportParams params);
 
 /* Prepares the client side for key exchange. */
