@@ -8,15 +8,15 @@ trkex.c
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-Key exchange methods.
+  Key exchange methods.
 
 */
 
 /*
- * $Id: trkex.c,v 1.31 1999/05/04 19:21:01 kivinen Exp $
+ * $Id: trkex.c,v 1.39 2000/05/28 16:21:05 kivinen Exp $
  * $Log: trkex.c,v $
  * $EndLog$
  */
@@ -32,6 +32,8 @@ Key exchange methods.
 #include "sshcipherlist.h"
 #include "sshdebug.h"
 
+#define SSH_DEBUG_MODULE "SshProtoTrKex"
+
 /* forward definitions */
 
 Boolean ssh_kex_derive_keys(SshTransportCommon tr);
@@ -63,7 +65,7 @@ void ssh_tr_server_init_kex(SshTransportCommon tr,
   unsigned char *buf;
   size_t len;
 
-  assert(tr->private_host_key == NULL);
+  SSH_ASSERT(tr->private_host_key == NULL);
   if (ssh_private_key_copy(private_host_key, &tr->private_host_key) !=
       SSH_CRYPTO_OK)
     ssh_fatal("ssh_tr_server_init_kex: private host key copy failed");
@@ -104,7 +106,7 @@ void ssh_tr_server_init_kex(SshTransportCommon tr,
    value.
    returns a SshCryptoStatus. */
 
-SshCryptoStatus ssh_kexdh_make_group(SshTransportCommon tr, 
+SshCryptoStatus ssh_kexdh_make_group(SshTransportCommon tr,
                                      const char *group_name)
 {
   int i;
@@ -126,35 +128,35 @@ SshCryptoStatus ssh_kexdh_make_group(SshTransportCommon tr,
   /* we currently accept only this one group */
 
   if (strcmp(group_name, "diffie-hellman-group1") != 0)
-    {  
+    {
       ssh_debug("ssh_kexdh_make_group: unsupported type %s",
                 group_name);
       return SSH_CRYPTO_UNKNOWN_GROUP_TYPE;
     }
 
   /* read the p and the g */
-  
+
   ssh_mp_set_str(tr->dh_p, group1_p, 16);
   ssh_mp_set_str(tr->dh_g, group1_g, 16);
 
   /*
     Randomize our secret value (x or y) and public value (e or f).
-    We'll use only 192 bits of entropy for faster exponentiation. 
+    We'll use only 192 bits of entropy for faster exponentiation.
 
     For discussion, see:
       P. C. van Oorschot and M. J. Wiener, "On Diffie-Hellman Key Agreement
       with Short Exponents", proc. Eurocrypt 96
   */
 
-  ssh_mp_set_ui(tr->dh_secret, 1);  
+  ssh_mp_set_ui(tr->dh_secret, 1);
   for (i = 0; i < 24; i++)
     {
       ssh_mp_mul_2exp(tr->dh_secret, tr->dh_secret, 8);
-      ssh_mp_add_ui(tr->dh_secret, tr->dh_secret, 
-                 ssh_random_get_byte(tr->random_state));
+      ssh_mp_add_ui(tr->dh_secret, tr->dh_secret,
+                 ssh_random_get_byte());
     }
-  
-  ssh_mp_powm(tr->server ? tr->dh_f : tr->dh_e, 
+
+  ssh_mp_powm(tr->server ? tr->dh_f : tr->dh_e,
            tr->dh_g, tr->dh_secret, tr->dh_p);
 
   return SSH_CRYPTO_OK;
@@ -165,8 +167,8 @@ SshCryptoStatus ssh_kexdh_make_group(SshTransportCommon tr,
 Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
 {
   SshIntC t;
-  SshBuffer *buf;
-  
+  SshBuffer buf;
+
   /* check that the public value is within the range */
 
   if (ssh_mp_cmp_ui(tr->server ? tr->dh_e : tr->dh_f, 2) <= 0)
@@ -183,40 +185,40 @@ Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
 
   /* compute the shared secret */
 
-  ssh_mp_powm(tr->dh_k, tr->server ? tr->dh_e : tr->dh_f, 
+  ssh_mp_powm(tr->dh_k, tr->server ? tr->dh_e : tr->dh_f,
            tr->dh_secret, tr->dh_p);
 
   /* ok, compute the exchange hash */
-  
+
   buf = ssh_buffer_allocate();
 
   if (tr->server)
     {
-      ssh_encode_buffer(buf, 
-                        SSH_FORMAT_UINT32_STR, 
+      ssh_encode_buffer(buf,
+                        SSH_FORMAT_UINT32_STR,
                           tr->remote_version, strlen(tr->remote_version),
-                        SSH_FORMAT_UINT32_STR, 
+                        SSH_FORMAT_UINT32_STR,
                           tr->own_version, strlen(tr->own_version),
                         SSH_FORMAT_END);
     }
   else
     {
-      ssh_encode_buffer(buf, 
-                        SSH_FORMAT_UINT32_STR, 
+      ssh_encode_buffer(buf,
+                        SSH_FORMAT_UINT32_STR,
                           tr->own_version, strlen(tr->own_version),
-                        SSH_FORMAT_UINT32_STR, 
+                        SSH_FORMAT_UINT32_STR,
                           tr->remote_version, strlen(tr->remote_version),
                         SSH_FORMAT_END);
     }
 
   ssh_encode_buffer(buf,
-                    SSH_FORMAT_UINT32_STR, 
+                    SSH_FORMAT_UINT32_STR,
                       ssh_buffer_ptr(tr->client_kexinit_packet),
                       ssh_buffer_len(tr->client_kexinit_packet),
                     SSH_FORMAT_UINT32_STR,
                       ssh_buffer_ptr(tr->server_kexinit_packet),
                       ssh_buffer_len(tr->server_kexinit_packet),
-                    SSH_FORMAT_UINT32_STR, 
+                    SSH_FORMAT_UINT32_STR,
                       ssh_buffer_ptr(tr->public_host_key_blob),
                       ssh_buffer_len(tr->public_host_key_blob),
                     SSH_FORMAT_END);
@@ -243,15 +245,15 @@ Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
   ssh_debug("ssh_kexdh_compute_h (%s)", tr->server ? "server" : "client");
   printf(" e = ");
   ssh_mp_out_str(stdout, 16, tr->dh_e);
-  printf("\n f = ");    
+  printf("\n f = ");
   ssh_mp_out_str(stdout, 16, tr->dh_f);
-  printf("\n secret = ");    
+  printf("\n secret = ");
   ssh_mp_out_str(stdout, 16, tr->dh_secret);
-  printf("\n k = ");    
+  printf("\n k = ");
   ssh_mp_out_str(stdout, 16, tr->dh_k);
-  printf("\n g = ");    
+  printf("\n g = ");
   ssh_mp_out_str(stdout, 16, tr->dh_g);
-  printf("\n p = ");    
+  printf("\n p = ");
   ssh_mp_out_str(stdout, 16, tr->dh_p);
 
   printf("\n -- exchange hash computed over -- \n");
@@ -271,17 +273,17 @@ Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
 
 /* client makes the diffie-hellman kex1, which is a SSH_MSG_KEXDH_INIT */
 
-SshBuffer *ssh_kexdh_client_make_kex1(SshTransportCommon tr)
+SshBuffer ssh_kexdh_client_make_kex1(SshTransportCommon tr)
 {
-  SshBuffer *packet;
+  SshBuffer packet;
 
   /* generate group1 & the exchange value for the client */
   if (ssh_kexdh_make_group(tr, "diffie-hellman-group1") != SSH_CRYPTO_OK)
-    return NULL;  
+    return NULL;
 
   /* Allocate and construct a KEXDH_INIT packet. */
 
-  packet = ssh_buffer_allocate();  
+  packet = ssh_buffer_allocate();
   buffer_put_char(packet, SSH_MSG_KEXDH_INIT);
   buffer_put_mp_int_ssh2style(packet, tr->dh_e);
 
@@ -290,7 +292,7 @@ SshBuffer *ssh_kexdh_client_make_kex1(SshTransportCommon tr)
 
 /* server recieves client's kex1, which is a SSH_MSG_KEXDH_INIT */
 
-Boolean ssh_kexdh_server_input_kex1(SshTransportCommon tr, SshBuffer *input)
+Boolean ssh_kexdh_server_input_kex1(SshTransportCommon tr, SshBuffer input)
 {
   unsigned char code;
 
@@ -310,19 +312,22 @@ Boolean ssh_kexdh_server_input_kex1(SshTransportCommon tr, SshBuffer *input)
 
   if (ssh_kexdh_make_group(tr, "diffie-hellman-group1") != SSH_CRYPTO_OK)
     return FALSE;  
-  buffer_get_mp_int_ssh2style(input, tr->dh_e);
+  if (!buffer_get_mp_int_ssh2style(input, tr->dh_e))
+    return FALSE;
 
   return TRUE;
 }
 
 /* server creates a SSH_MSG_KEXDH_REPLY (kex2) */
 
-SshBuffer *ssh_kexdh_server_make_kex2(SshTransportCommon tr)
+SshBuffer ssh_kexdh_server_make_kex2(SshTransportCommon tr)
 {
-  SshBuffer *packet;
+  SshBuffer packet;
   unsigned char *signature;
   size_t sig_len;
-
+  unsigned char *buffer;  
+  size_t buffer_len;
+  
   /* compute the exchange hash H */
 
   if (ssh_kexdh_compute_h(tr))
@@ -332,28 +337,52 @@ SshBuffer *ssh_kexdh_server_make_kex2(SshTransportCommon tr)
   signature = ssh_xmalloc(sig_len);
 
   /* do the private key operation to sign H */
-
-  if (ssh_private_key_sign(tr->private_host_key, 
+  if (ssh_private_key_sign(tr->private_host_key,
                            tr->exchange_hash, tr->exchange_hash_len,
-                           signature, sig_len, &sig_len,
-                           tr->random_state) != SSH_CRYPTO_OK)
+                           signature, sig_len, &sig_len) != SSH_CRYPTO_OK)
     return NULL;
 
-  /* construct the packet */
+  if (!tr->ssh_old_malformed_signatures_bug_compat)
+    {
+      char *pubkeytype =
+        ssh_pubkeyblob_type(ssh_buffer_ptr(tr->public_host_key_blob),
+                            ssh_buffer_len(tr->public_host_key_blob));
 
+      if (!pubkeytype)
+        return NULL;
+      
+      buffer_len =
+        ssh_encode_array_alloc(&buffer,
+                               SSH_FORMAT_UINT32_STR, pubkeytype,
+                               strlen(pubkeytype),
+                               SSH_FORMAT_UINT32_STR, signature, sig_len,
+                               SSH_FORMAT_END);
+
+      memset(signature, 0, sig_len);
+      ssh_xfree(signature);
+      ssh_xfree(pubkeytype);
+    }
+  else
+    {
+      buffer = signature;
+      buffer_len = sig_len;
+      signature = NULL;
+    }
+  
+  /* construct the packet */
   packet = ssh_buffer_allocate();
-  ssh_encode_buffer(packet, 
+  ssh_encode_buffer(packet,
                     SSH_FORMAT_CHAR, (unsigned int) SSH_MSG_KEXDH_REPLY,
-                    SSH_FORMAT_UINT32_STR, 
+                    SSH_FORMAT_UINT32_STR,
                       ssh_buffer_ptr(tr->public_host_key_blob),
                       ssh_buffer_len(tr->public_host_key_blob),
                     SSH_FORMAT_END);
   buffer_put_mp_int_ssh2style(packet, tr->dh_f);
-  buffer_put_uint32_string(packet, signature, sig_len);
-
-  memset(signature, 0, sig_len);
-  ssh_xfree(signature);
+  buffer_put_uint32_string(packet, buffer, buffer_len);
 
+  memset(buffer, 0, buffer_len);
+  ssh_xfree(buffer);
+  
   /* we're ready to derive the keys */
   ssh_kex_derive_keys(tr);
 
@@ -366,26 +395,27 @@ SshBuffer *ssh_kexdh_server_make_kex2(SshTransportCommon tr)
 /* Internal struct for the callback, and the calling function. */
 typedef struct SshKex2KeyCheckCallbackContextRec
 {
-  SshBuffer *input;
+  SshBuffer input;
   size_t pubkey_len;
   unsigned char *pubkey;
   size_t sig_len;
   unsigned char *signature;
   SshKex2CompletionProc completion;
-  
+
 } *SshKex2KeyCheckCallbackContext;
 
 /* This function is called by the registered key check function. (in
    ssh2 it is ssh_client_key_check in sshclient.c) */
+/* XXX We should be able to pass error message from application level. */
 void ssh_kex_keycheck_callback(Boolean result, void *ctx)
 {
   SshTransportCommon tr;
   SshKex2KeyCheckCallbackContext callback_context;
-  
+
   tr = (SshTransportCommon) ctx;
 
   callback_context = tr->key_check_callback_context;
-  
+
   tr->key_check_returned = TRUE;
   tr->key_check_result = result;
 
@@ -396,38 +426,119 @@ void ssh_kex_keycheck_callback(Boolean result, void *ctx)
     }
   else
     {
+      unsigned char *real_signature;
+      size_t real_sig_len;
+      
       tr->public_host_key_blob = ssh_buffer_allocate();
       ssh_buffer_append(tr->public_host_key_blob, callback_context->pubkey,
                         callback_context->pubkey_len);
-      ssh_xfree(callback_context->pubkey);
 
-      buffer_get_mp_int_ssh2style(callback_context->input, tr->dh_f);  
+      if (!buffer_get_mp_int_ssh2style(callback_context->input, tr->dh_f))
+        {
+          ssh_xfree(callback_context->pubkey);
+          tr->key_check_result = FALSE;
+          goto error;
+        }
+      
       callback_context->signature =
         buffer_get_uint32_string(callback_context->input,
                                  &(callback_context->sig_len));
 
+
+      if (!tr->ssh_old_malformed_signatures_bug_compat)
+        {
+          char *recv_pubkeytype = NULL;
+          size_t decoded_len;
+          
+          char *pubkeytype =
+            ssh_pubkeyblob_type(callback_context->pubkey,
+                                callback_context->pubkey_len);
+
+          if (!pubkeytype)
+            {
+              ssh_warning("Received malformed pubkey during server "
+                          "keycheck.");
+              tr->key_check_result = FALSE;
+              goto error;
+            }
+          
+          decoded_len =
+            ssh_decode_array(callback_context->signature,
+                             callback_context->sig_len,
+                             SSH_FORMAT_UINT32_STR, &recv_pubkeytype, NULL,
+                             SSH_FORMAT_UINT32_STR,
+                             &real_signature, &real_sig_len,
+                             SSH_FORMAT_END);
+          
+          if (decoded_len == 0 || decoded_len != callback_context->sig_len)
+            {
+              SSH_DEBUG(2, ("decoded_len: %ld, sig_len: %ld",
+                            decoded_len, callback_context->sig_len));
+              ssh_warning("Received malformed signature during server "
+                          "keycheck.");
+              ssh_xfree(pubkeytype);
+              memset(callback_context->signature, 0,
+                     callback_context->sig_len);
+              ssh_xfree(callback_context->signature);
+              tr->key_check_result = FALSE;
+              goto error;
+            }
+          
+          memset(callback_context->signature, 0,
+                 callback_context->sig_len);
+          ssh_xfree(callback_context->signature);
+
+          if (strcmp(recv_pubkeytype, pubkeytype) != 0)
+            {
+              ssh_warning("Received malformed signature during server "
+                          "keycheck. (public key type doesn't match the one "
+                          "in signature.)");
+              ssh_xfree(pubkeytype);
+              ssh_xfree(recv_pubkeytype);
+              memset(real_signature, 0, real_sig_len);
+              ssh_xfree(real_signature);
+              tr->key_check_result = FALSE;
+              goto error;              
+            }
+          ssh_xfree(pubkeytype);
+          ssh_xfree(recv_pubkeytype);
+        }
+      else
+        {
+          real_signature = callback_context->signature;
+          real_sig_len = callback_context->sig_len;
+          callback_context->signature = NULL;
+        }
+      
+      ssh_xfree(callback_context->pubkey);      
+
       /* ok, verify the signature */
-
       if (ssh_kexdh_compute_h(tr))
-        tr->key_check_result = FALSE;
-
+        {
+          tr->key_check_result = FALSE;
+          goto error;
+        }
+      
       if (ssh_public_key_verify_signature(tr->public_host_key,
-                                          callback_context->signature,
-                                          callback_context->sig_len,
-                                          tr->exchange_hash, 
+                                          real_signature,
+                                          real_sig_len,
+                                          tr->exchange_hash,
                                           tr->exchange_hash_len) == FALSE)
         {
-          ssh_xfree(callback_context->signature);
+          memset(real_signature, 0, real_sig_len);
+          ssh_xfree(real_signature);
           tr->key_check_result = FALSE;
+          goto error;
         }
 
-      memset(callback_context->signature, 0, callback_context->sig_len);
-      ssh_xfree(callback_context->signature);
+      memset(real_signature, 0, real_sig_len);
+      ssh_xfree(real_signature);
 
       /* we're ready to derive the keys */
       ssh_kex_derive_keys(tr);
     }
-  
+ error:
+
   /* Call the supplied callback.*/
   (*callback_context->completion)(tr);
 
@@ -436,15 +547,15 @@ void ssh_kex_keycheck_callback(Boolean result, void *ctx)
 
 /* client parses the server's SSH_MSG_KEXDH_REPLY (kex2) */
 
-void ssh_kexdh_client_input_kex2(SshTransportCommon tr, SshBuffer *input,
+void ssh_kexdh_client_input_kex2(SshTransportCommon tr, SshBuffer input,
                                  SshKex2CompletionProc finalize_callback)
 {
   unsigned int code;
-  unsigned char *pubkey;  
+  unsigned char *pubkey;
   size_t pubkey_len;
   char *pubkeytype;
   SshKex2KeyCheckCallbackContext callback_context;
-  
+
   if ((code = buffer_get_char(input)) != SSH_MSG_KEXDH_REPLY)
     {
       ssh_debug("ssh_kexdh_client_input_kex2: received illegal packet %d",
@@ -483,7 +594,7 @@ void ssh_kexdh_client_input_kex2(SshTransportCommon tr, SshBuffer *input,
       return;
     }
 
-  
+
   /* Check the host key */
   tr->key_check_returned = FALSE;
   if (tr->key_check)
@@ -515,7 +626,7 @@ void ssh_kex_derive_key(SshTransportCommon tr,
 {
   unsigned char buf[SSH_MAX_HASH_DIGEST_LENGTH];
   size_t len, hash_len, i;
-  SshBuffer buffer;
+  SshBufferStruct buffer;
 
   hash_len = ssh_hash_digest_length(tr->hash);
 
@@ -526,18 +637,18 @@ void ssh_kex_derive_key(SshTransportCommon tr,
     {
       ssh_buffer_init(&buffer);
       buffer_put_mp_int_ssh2style(&buffer, tr->dh_k);
-      ssh_hash_update(tr->hash, 
+      ssh_hash_update(tr->hash,
                       ssh_buffer_ptr(&buffer),
                       ssh_buffer_len(&buffer));
     }
   ssh_hash_update(tr->hash, tr->exchange_hash, tr->exchange_hash_len);
   ssh_hash_update(tr->hash, &id, 1);
-  ssh_hash_update(tr->hash, 
-                  tr->session_identifier, 
+  ssh_hash_update(tr->hash,
+                  tr->session_identifier,
                   tr->session_identifier_len);
   ssh_hash_final(tr->hash, buf);
 
-  /* Expand the key. Note that this will not increase entropy in 
+  /* Expand the key. Note that this will not increase entropy in
      any real sense. */
 
   for (i = 0; i < key_len; /*NOTHING*/)
@@ -552,7 +663,7 @@ void ssh_kex_derive_key(SshTransportCommon tr,
           ssh_hash_reset(tr->hash);
           if (! tr->ssh_old_keygen_bug_compat)
             {
-              ssh_hash_update(tr->hash, 
+              ssh_hash_update(tr->hash,
                               ssh_buffer_ptr(&buffer),
                               ssh_buffer_len(&buffer));
             }
@@ -576,13 +687,13 @@ Boolean ssh_kex_derive_keys(SshTransportCommon tr)
 {
   ssh_kex_derive_key(tr, 'A', tr->c_to_s.iv, sizeof(tr->c_to_s.iv));
   ssh_kex_derive_key(tr, 'B', tr->s_to_c.iv, sizeof(tr->s_to_c.iv));
-  ssh_kex_derive_key(tr, 'C', tr->c_to_s.encryption_key, 
+  ssh_kex_derive_key(tr, 'C', tr->c_to_s.encryption_key,
                      sizeof(tr->c_to_s.encryption_key));
-  ssh_kex_derive_key(tr, 'D', tr->s_to_c.encryption_key, 
+  ssh_kex_derive_key(tr, 'D', tr->s_to_c.encryption_key,
                      sizeof(tr->s_to_c.encryption_key));
-  ssh_kex_derive_key(tr, 'E', tr->c_to_s.integrity_key, 
+  ssh_kex_derive_key(tr, 'E', tr->c_to_s.integrity_key,
                      sizeof(tr->c_to_s.integrity_key));
-  ssh_kex_derive_key(tr, 'F', tr->s_to_c.integrity_key, 
+  ssh_kex_derive_key(tr, 'F', tr->s_to_c.integrity_key,
                      sizeof(tr->s_to_c.integrity_key));
 
   return TRUE;
@@ -591,7 +702,7 @@ Boolean ssh_kex_derive_keys(SshTransportCommon tr)
 /* Returns NULL.  This is used as a kex1/kex2 packet generator when no such
    packet is sent for the kex type. */
 
-SshBuffer *ssh_kex_return_no_packet(SshTransportCommon tr)
+SshBuffer ssh_kex_return_no_packet(SshTransportCommon tr)
 {
   return NULL;
 }
@@ -607,7 +718,7 @@ const struct SshKexTypeRec ssh_kex_algorithms[] =
     NULL, ssh_kexdh_server_input_kex1,
     ssh_kexdh_client_input_kex2, NULL },
 
-#if 0  
+#if 0
   { "double-encrypting-sha1", "sha1",
     TRUE, FALSE,
     ssh_kex_return_no_packet, ssh_kexde_server_make_hostkey,
@@ -615,7 +726,7 @@ const struct SshKexTypeRec ssh_kex_algorithms[] =
     ssh_kexde_client_input_kex1, NULL,
     NULL, ssh_kexde_server_input_kex2 },
 #endif
-  
+
   { NULL }
 };
 
@@ -624,7 +735,7 @@ const struct SshKexTypeRec ssh_kex_algorithms[] =
 
 char *ssh_kex_get_supported()
 {
-  SshBuffer buffer;
+  SshBufferStruct buffer;
   int i;
   char *cp;
 
@@ -660,16 +771,16 @@ SshHash ssh_kex_allocate_hash(const char *name)
 {
   int i;
   SshHash hash;
-  
+
   for (i = 0; ssh_kex_algorithms[i].name != NULL; i++)
     {
       if (strcmp(ssh_kex_algorithms[i].name, name) == 0)
-        {  
+        {
           if (ssh_hash_allocate(ssh_kex_algorithms[i].hash_name, &hash)
               != SSH_CRYPTO_OK)
             return NULL;
           return hash;
         }
-    }  
-  return NULL;  
+    }
+  return NULL;
 }
