@@ -8,15 +8,15 @@ trkex.h
         Timo J. Rinne <tri@ssh.fi>
         Sami Lehtinen <sjl@ssh.fi>
 
-  Copyright (C) 1996-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
-Key exchange methods.
+  Key exchange methods.
 
 */
 
 /*
- * $Id: trkex.h,v 1.5 1999/04/07 09:34:40 sjl Exp $
+ * $Id: trkex.h,v 1.7 2000/05/28 16:21:06 kivinen Exp $
  * $Log: trkex.h,v $
  * $EndLog$
  */
@@ -29,10 +29,10 @@ Key exchange methods.
 
 /* Data types for declaring key exchange methods. */
 
-typedef SshBuffer *(*SshMakeKexProc)(SshTransportCommon tr);
+typedef SshBuffer (*SshMakeKexProc)(SshTransportCommon tr);
 typedef void (*SshKex2CompletionProc) (SshTransportCommon tr);
-typedef Boolean (*SshInputKexProc)(SshTransportCommon tr, SshBuffer *buffer);
-typedef void (*SshInputKex2Proc) (SshTransportCommon tr, SshBuffer *buffer,
+typedef Boolean (*SshInputKexProc)(SshTransportCommon tr, SshBuffer buffer);
+typedef void (*SshInputKex2Proc) (SshTransportCommon tr, SshBuffer buffer,
                                   SshKex2CompletionProc completion);
 
 struct SshKexTypeRec
