@@ -6,14 +6,14 @@
 /*
  *        Program: sshreadline
  *        $Source: /ssh/CVS/src/lib/sshreadline/sshreadline.c,v $
- *        $Author: tmo $
+ *        $Author: kivinen $
  *
  *        Creation          : 19:47 Mar 12 1997 kivinen
- *        Last Modification : 18:07 Oct  8 1998 kivinen
- *        Last check in     : $Date: 1999/05/06 08:17:59 $
- *        Revision number   : $Revision: 1.17 $
+ *        Last Modification : 18:05 May 28 2000 kivinen
+ *        Last check in     : $Date: 2000/05/28 16:21:08 $
+ *        Revision number   : $Revision: 1.29 $
  *        State             : $State: Exp $
- *        Version           : 1.665
+ *        Version           : 1.668
  *
  *        Description       : Readline library
  *
@@ -47,7 +47,11 @@
 #include <sys/ioctl.h>
 
 #include "sshstream.h"
-#include "sshunixfdstream.h"
+#include "sshfdstream.h"
+#include "sshbuffer.h"
+#include "sshdllist.h"
+
+#define SSH_DEBUG_MODULE "SshReadLine"
 
 #define SSH_READLINE_MAX_UNDO_DEPTH 256
 
@@ -92,14 +96,23 @@ typedef enum {
 } ReadLineKeyMap;
 
 typedef struct ReadLineRec {
-  int fd;
+  int read_fd;
+  int write_fd;
   const unsigned char *prompt;
   int prompt_len;
   unsigned char *line;
   int line_alloc;
   unsigned char *display_line;
-  struct termios term;
-  int fcntl_flags;
+  struct termios read_term;
+  Boolean got_tty_modes;
+
+#if 0
+  struct termios write_term;  
+#endif
+  int read_fcntl_flags;
+#if 0
+  int write_fcntl_flags;
+#endif
   unsigned char *yank;
   unsigned char **undo;
   int *undo_cursors;
@@ -116,46 +129,118 @@ typedef struct ReadLineRec {
   ReadLineKeyMap keymap;
   Term *tc;
   Boolean eloop;
+  Boolean echo;
   SshStream stream;
   SshRLCallback callback;
+  SshRLExtCallback ext_callback;
+  Boolean extended_mode;
+  unsigned int wanted_flags, flags;
+  SshBuffer out_buffer;
+  void *user_context; /* Context for the above callback */
+
+  /* Command history state */
+  SshDlList cmd_history; /* null-terminated xmallocated strings */
+  SshDlListNode cmd_history_current; /* The last list item shown to the user */
+  Boolean cmd_history_end; 
+  Boolean cmd_history_line_entered; /* Whether the last line in the list was added
+                                       after user pressed enter or because user
+                                       is just browsing the command history. */
 } ReadLine;
 
-/*
- * Set tty to raw mode. Return old flags in store_old_term.
- * Return 0 if success and -1 if failure.
- */
-int ssh_rl_set_tty_modes(ReadLine *rl)
+int ssh_rl_set_tty_modes(ReadLine *rl);
+int ssh_rl_restore_tty_modes(ReadLine *rl);
+
+static RETSIGTYPE (*old_sigint_handler)(int) = NULL;
+ReadLine *global_sigint_rl;
+
+void ssh_readline_eloop_internal(const unsigned char *prompt,
+                                 const unsigned char *def,
+                                 Boolean echo,
+                                 SshReadLineCtx rl,
+                                 SshRLCallback callback,
+                                 SshRLExtCallback ext_callback,
+                                 unsigned int flags,
+                                 void *context);
+
+RETSIGTYPE ssh_rl_int_handler(int sig)
+{
+  /* Restore terminal modes. */
+  ssh_rl_restore_tty_modes(global_sigint_rl);
+
+  (void)signal(SIGINT, old_sigint_handler);
+
+#if 1
+  /* Resend the signal, with the old handler. */
+  kill(getpid(), sig);
+#else
+  ssh_register_timeout(0L, 0L, kill_me_with_signal, (void *) sig);
+#endif
+}
+
+
+int ssh_rl_set_tty_modes_for_fd(int fd, ReadLine *rl, int *fcntl_flags,
+                                struct termios *fd_tio)
 {
   struct termios new_term;
   struct winsize win;
-
+  int new_fcntl_flags;
+  
   /* Set tty modes */
-  if (tcgetattr(rl->fd, &rl->term) < 0)
+  if (tcgetattr(fd, fd_tio) < 0)
     {
-      ssh_warning("Warning: tcgetattr failed in ssh_rl_set_tty_modes: %.200s",
-                  strerror(errno));
+      ssh_warning("Warning: tcgetattr failed in "
+                  "ssh_rl_set_tty_modes_for_fd: fd %d: %.200s",
+                  fd, strerror(errno));
       return -1;
     }
-  new_term = rl->term;
+  new_term = *fd_tio;
   new_term.c_iflag &= ~(ISTRIP);
   new_term.c_oflag &= ~(ONLCR);
+  /* XXX Allrighty. The ISIG is here to protect the user from ending
+     with a raw terminal. But, we need to be able to terminate
+     etc. What to do? */
+#if 0
   new_term.c_lflag &= ~(ECHO | ICANON | ISIG);
+#else
+  new_term.c_lflag &= ~(ECHO | ICANON);
+#endif
   new_term.c_cc[VMIN] = 1;
   new_term.c_cc[VTIME] = 1;
-  if (tcsetattr(rl->fd, TCSAFLUSH, &new_term) < 0)
+  if (tcsetattr(fd, TCSAFLUSH, &new_term) < 0)
     {
-      ssh_warning("Warning: tcsetattr failed in ssh_rl_set_tty_modes: %.200s",
-                  strerror(errno));
+      ssh_warning("Warning: tcsetattr failed in "
+                  "ssh_rl_set_tty_modes_for_fd: fd %d: %.200s",
+                  fd, strerror(errno));
       return -1;
     }
-  rl->fcntl_flags = fcntl(rl->fd, F_GETFL, 0);
-#ifdef O_ASYNC
-  fcntl(rl->fd, F_SETFL, rl->fcntl_flags & ~(O_ASYNC | O_NONBLOCK));
-#else
-  fcntl(rl->fd, F_SETFL, rl->fcntl_flags & ~(O_NONBLOCK));
-#endif
 
-  if (ioctl(rl->fd, TIOCGWINSZ, &win) != -1)
+  new_fcntl_flags = fcntl(fd, F_GETFL, 0);
+  *fcntl_flags = new_fcntl_flags;
+  
+/*   (void)signal(SIGINT, ssh_rl_int_handler); */
+
+  if (!rl->eloop)
+    {
+#ifdef O_ASYNC
+      fcntl(fd, F_SETFL, new_fcntl_flags & ~(O_ASYNC | O_NONBLOCK));
+#else /* O_ASYNC */
+      fcntl(fd, F_SETFL, new_fcntl_flags & ~(O_NONBLOCK));
+#endif /* O_ASYNC */
+    }
+  else
+    {
+#ifdef O_ASYNC
+      new_fcntl_flags &= ~(O_ASYNC);
+#endif /* O_ASYNC */      
+#if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
+      new_fcntl_flags |= O_NONBLOCK;
+#else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+      new_fcntl_flags |= O_NDELAY;
+#endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+      (void)fcntl(fd, F_SETFL, new_fcntl_flags);      
+    }
+    
+  if (ioctl(fd, TIOCGWINSZ, &win) != -1)
     {
       rl->row_length = win.ws_col;
     }
@@ -167,21 +252,49 @@ int ssh_rl_set_tty_modes(ReadLine *rl)
   return 0;
 }
 
+/*
+ * Set tty to raw mode. Return old flags in store_old_term.
+ * Return 0 if success and -1 if failure.
+ */
+int ssh_rl_set_tty_modes(ReadLine *rl)
+{
+  if (ssh_rl_set_tty_modes_for_fd(rl->read_fd, rl,
+                                  &rl->read_fcntl_flags,
+                                  &rl->read_term) == -1)
+    return -1;
+
+  rl->got_tty_modes = TRUE;
+  
+  return 0;
+}
+
+int ssh_rl_restore_tty_modes_for_fd(int fd, ReadLine *rl, int fcntl_flags,
+                                    struct termios *saved_tio)
+{
+  fcntl(fd, F_SETFL, fcntl_flags);
+  
+  if (tcsetattr(fd, TCSAFLUSH, saved_tio) < 0)
+    {
+      ssh_warning("Warning: tcsetattr failed in "
+                  "ssh_rl_restore_tty_modes_for_fd: fd %d: %.200s",
+                  fd, strerror(errno));
+      return -1;
+    }
+  return 0;
+}
+
 /*
  * Restore terminal modes
  * Return 0 if success and -1 if failure.
  */
 int ssh_rl_restore_tty_modes(ReadLine *rl)
 {
-  fcntl(rl->fd, F_SETFL, rl->fcntl_flags);
-
-  if (tcsetattr(rl->fd, TCSAFLUSH, &rl->term) < 0)
-    {
-      ssh_warning("Warning: tcsetattr failed in ssh_rl_restore_tty_modes: %.200s",
-                  strerror(errno));
-      return -1;
-    }
-  return 0;
+  if (!rl->got_tty_modes)
+    return 0;
+  
+  return ssh_rl_restore_tty_modes_for_fd(rl->read_fd, rl,
+                                         rl->read_fcntl_flags,
+                                         &rl->read_term);  
 }
 
 /*
@@ -206,6 +319,19 @@ void ssh_rl_store_undo(ReadLine *rl)
   rl->undo_direction = -1;
 }
 
+/*
+ * Delete the undo information
+ */
+void ssh_rl_delete_undo(ReadLine *rl)
+{
+  int i;
+  for(i = 0; i < rl->undo_length; i++)
+    ssh_xfree(rl->undo[i]);
+  rl->undo_length = 0;
+  rl->undo_position = -1;
+  rl->undo_direction = -1;
+}
+
 /*
  * Enlarge line buffer so it can hold length characters
  */
@@ -225,10 +351,28 @@ void ssh_rl_enlarge_to(ReadLine *rl, int length)
 void ssh_rl_send_string(ReadLine *rl, const unsigned char *txt,
                         int len)
 {
-  if (write(rl->fd, txt, len) != len)
+  if (rl->eloop)
     {
-      ssh_warning("Warning: write failed in ssh_rl_send_string: %.200s",
-                  strerror(errno));
+      int ret;
+      ssh_buffer_append(rl->out_buffer, txt, len);
+      if ((ret = ssh_stream_write(rl->stream,
+                                  ssh_buffer_ptr(rl->out_buffer),
+                                  ssh_buffer_len(rl->out_buffer))))
+        {
+          if (ret == 0)
+            ssh_warning("Warning: write failed in ssh_rl_send_string: "
+                        "EOF received");
+          if (ret > 0)
+            ssh_buffer_consume(rl->out_buffer, ret);
+        }
+    }
+  else
+    { 
+      if (write(rl->write_fd, txt, len) != len)
+        {
+          ssh_warning("Warning: write failed in ssh_rl_send_string: %.200s",
+                      strerror(errno));
+        }
     }
 }
 
@@ -479,6 +623,9 @@ void ssh_rl_redraw_display(ReadLine *rl)
   int common_start, common_end, line_len, display_len;
   int i;
 
+  if (! rl->echo)
+    return;
+  
   line_len = rl->end;
   rl->line[rl->end] = '\0';
   display_len = strlen((char *) rl->display_line);
@@ -761,10 +908,15 @@ int ssh_rl_loop(ReadLine *rl)
 
   do {
     ssh_rl_redraw_display(rl);
-    ret = read(rl->fd, buffer, sizeof(buffer));
+    
+    if (rl->eloop)
+      ret = ssh_stream_read(rl->stream, buffer, sizeof (buffer));
+    else
+      ret = read(rl->read_fd, buffer, sizeof(buffer));
+    
     if (ret == 0)
       return -1;
-
+    
     if (ret < 0)
       {
         if (rl->eloop)
@@ -772,6 +924,7 @@ int ssh_rl_loop(ReadLine *rl)
         else
           return -1;
       }
+
     for(i = 0; i < ret; i++)
       {
         if (rl->keymap == SSH_READLINE_NORMAL_KEYMAP)
@@ -835,29 +988,37 @@ int ssh_rl_loop(ReadLine *rl)
                 rl->last_command_cut = -1;
                 break;
               case 011: /* Ctrl-I: Tab */
-                ssh_rl_store_undo(rl);
-                len = (rl->cursor + rl->prompt_len) % 8;
-                len = 8 - len;
-                ssh_rl_enlarge_to(rl, rl->end + len);
-                if (rl->end != rl->cursor)
-                  memmove(rl->line + rl->cursor + len, rl->line + rl->cursor,
-                          rl->end - rl->cursor);
-                if (rl->mark > rl->cursor)
-                  rl->mark += len;
-                for(; len > 0; len--)
+                if (rl->extended_mode &&
+                    rl->wanted_flags & SSH_READLINE_CB_FLAG_TAB)
                   {
-                    rl->line[rl->cursor] = ' ';
-                    rl->end++;
-                    rl->cursor++;
+                    rl->flags = rl->flags | SSH_READLINE_CB_FLAG_TAB;
+                    return 0;
+                  }
+                else
+                  {
+                    ssh_rl_store_undo(rl);
+                    len = (rl->cursor + rl->prompt_len) % 8;
+                    len = 8 - len;
+                    ssh_rl_enlarge_to(rl, rl->end + len);
+                    if (rl->end != rl->cursor)
+                      memmove(rl->line + rl->cursor + len, rl->line + rl->cursor,
+                              rl->end - rl->cursor);
+                    if (rl->mark > rl->cursor)
+                      rl->mark += len;
+                    for(; len > 0; len--)
+                      {
+                        rl->line[rl->cursor] = ' ';
+                        rl->end++;
+                        rl->cursor++;
+                      }
+                    rl->last_command_cut = -1;
                   }
-                rl->last_command_cut = -1;
                 break;
-              case 012: /* Ctrl-J: Accept line */
-                return 0;
               delete_end_of_line:
               case 013: /* Ctrl-K: Delete to end of line */
                 if (rl->cursor == rl->end)
                   break;
+                ssh_rl_store_undo(rl);
                 len = rl->end - rl->cursor;
                 if (rl->yank)
                   {
@@ -894,17 +1055,99 @@ int ssh_rl_loop(ReadLine *rl)
                 ssh_rl_move_cursor(rl, 0);
                 ssh_rl_write_string(rl, rl->prompt, rl->prompt_len);
                 break;
+              case 012: /* Ctrl-J: Accept line */
               case 015: /* Ctrl-M: Accept line */
+                /* Store the line into the end of command history list.
+                   Current position will move to the end of the command history. */
+                if (rl->cmd_history != NULL)
+                  {
+                    rl->line[rl->end] = '\0';
+                    ssh_dllist_add_item(rl->cmd_history, ssh_xstrdup(rl->line),
+                                        SSH_DLLIST_END);
+                    ssh_dllist_end(rl->cmd_history);
+                    rl->cmd_history_current = ssh_dllist_current_node(rl->cmd_history);
+                    rl->cmd_history_end = TRUE;
+                    rl->cmd_history_line_entered = TRUE;
+                  }
                 return 0;
               case 016: /* Ctrl-N: Next line */
                 if (rl->cursor <= rl->end - rl->row_length)
-                  rl->cursor += rl->row_length;
+                  {
+                    rl->cursor += rl->row_length;
+                  }
+                else
+                  {
+                    if (rl->cmd_history != NULL)
+                      {
+                        ssh_rl_delete_undo(rl);
+                        ssh_dllist_fw(rl->cmd_history, 1);
+                        /* If we went past the beginning, go back to the beginning. */
+                        if (!ssh_dllist_is_current_valid(rl->cmd_history))
+                          {
+                            ssh_dllist_end(rl->cmd_history);
+                            rl->cmd_history_end = TRUE;
+                          }
+                        /* Check whether we are at the end */
+                        if (ssh_dllist_node_get_next(ssh_dllist_current_node(rl->cmd_history)) == NULL)
+                          {
+                            rl->cmd_history_end = TRUE;
+                          }
+                        
+                        ssh_rl_enlarge_to(rl, strlen(ssh_dllist_current(rl->cmd_history)));
+                        memmove(rl->line, ssh_dllist_current(rl->cmd_history),
+                                strlen(ssh_dllist_current(rl->cmd_history)));
+                        rl->cursor = strlen(ssh_dllist_current(rl->cmd_history));
+                        rl->end = rl->cursor;
+                      }
+                  }
                 break;
               case 017: /* Ctrl-O: Ignored */
                 break;
               case 020: /* Ctrl-P: Previous line */
                 if (rl->cursor >= rl->row_length)
-                  rl->cursor -= rl->row_length;
+                  {
+                    rl->cursor -= rl->row_length;
+                  }
+                else
+                  {
+                    if (rl->cmd_history != NULL &&
+                        rl->cmd_history_current != NULL)
+                      {
+                        ssh_rl_delete_undo(rl);
+                        
+                        /* If we are starting to move backwards from the end,
+                           store the current line - it would be otherwise lost. */
+                        if (rl->cmd_history_end == TRUE)
+                          {
+                            rl->line[rl->end] = '\0';
+                            /* Add the line if different */
+                            if (strcmp(rl->line, ssh_dllist_current(rl->cmd_history)))
+                              {
+                                if (rl->cmd_history_line_entered == FALSE)
+                                  {
+                                    /* Replace instead of adding. */
+                                    ssh_xfree(ssh_dllist_delete_current(rl->cmd_history));
+                                  }
+                                ssh_dllist_add_item(rl->cmd_history,
+                                                    ssh_xstrdup(rl->line),
+                                                    SSH_DLLIST_END);
+                                rl->cmd_history_line_entered = FALSE;
+                                ssh_dllist_end(rl->cmd_history);
+                              }
+                          }                     
+                        ssh_dllist_bw(rl->cmd_history, 1);
+                        rl->cmd_history_end = FALSE;
+                        /* If we went past the beginning, go back to the beginning. */
+                        if (!ssh_dllist_is_current_valid(rl->cmd_history))
+                          ssh_dllist_rewind(rl->cmd_history);
+
+                        ssh_rl_enlarge_to(rl, strlen(ssh_dllist_current(rl->cmd_history)));
+                        memmove(rl->line, ssh_dllist_current(rl->cmd_history),
+                                strlen(ssh_dllist_current(rl->cmd_history)));
+                        rl->cursor = strlen(ssh_dllist_current(rl->cmd_history));
+                        rl->end = rl->cursor;
+                      }
+                  }
                 break;
               case 021: /* Ctrl-Q: Ignored */
               case 022: /* Ctrl-R: Ignored */
@@ -1038,6 +1281,8 @@ int ssh_rl_loop(ReadLine *rl)
                 break;
               undo:
               case 037: /* Ctrl-_: Undo */
+                if (rl->undo_length == 0)
+                  break;
                 if (rl->undo_position == -1)
                   {
                     ssh_rl_store_undo(rl);
@@ -1075,6 +1320,7 @@ int ssh_rl_loop(ReadLine *rl)
                 break;
               insert_char:
               default:
+                ssh_rl_store_undo(rl);
                 ssh_rl_enlarge_to(rl, rl->end + 1);
                 if (rl->end != rl->cursor)
                   memmove(rl->line + rl->cursor + 1, rl->line + rl->cursor,
@@ -1085,8 +1331,6 @@ int ssh_rl_loop(ReadLine *rl)
                 rl->end++;
                 rl->cursor++;
                 rl->last_command_cut = -1;
-                if (buffer[i] == ' ')
-                  ssh_rl_store_undo(rl);
                 break;
               }
           }
@@ -1255,7 +1499,7 @@ int ssh_rl_loop(ReadLine *rl)
               }
           }
       }
-  } while (rl->eloop == FALSE);
+  } while (TRUE);
 
   if (rl->eloop)
     ssh_rl_redraw_display(rl);
@@ -1268,9 +1512,10 @@ int ssh_rl_loop(ReadLine *rl)
  */
 void ssh_rl_initialize_termcap(ReadLine *rl)
 {
-  char tcbuffer[1024], *term, *term_buffer_ptr;
   Boolean use_builtin = FALSE;
-
+#if defined(HAVE_LIBTERMCAP) || defined(HAVE_LIBNCURSES) || defined(HAVE_LIBXCURSES)
+  char *term;
+  
   if (ssh_last_term == NULL)
     {
       ssh_last_term = ssh_xmalloc(sizeof(Term));
@@ -1285,9 +1530,11 @@ void ssh_rl_initialize_termcap(ReadLine *rl)
     }
   else if (term != ssh_last_term->term_type)
     {
+      char tcbuffer[1024], *term_buffer_ptr;
       if (tgetent(tcbuffer, term) < 0)
         {
-          ssh_warning("Warning: No termcap entry for `%.50s' found, using vt100",
+          ssh_warning("Warning: No termcap entry for `%.50s' found, "
+                      "using vt100",
                       term);
           use_builtin = TRUE;
           ssh_last_term->term_type = term;
@@ -1348,6 +1595,16 @@ void ssh_rl_initialize_termcap(ReadLine *rl)
             }
         }
     }
+#else /* HAVE_LIBTERMCAP || HAVE_LIBNCURSES || HAVE_LIBXCURSES */
+  if (ssh_last_term == NULL)
+    {
+      ssh_last_term = ssh_xmalloc(sizeof(Term));
+      ssh_last_term->term_buffer = ssh_xmalloc(1024);
+      ssh_last_term->term_type = NULL;
+      ssh_warning("Need basic cursor movement capablity, using vt100");
+      use_builtin = TRUE;
+    }
+#endif /* HAVE_LIBTERMCAP || HAVE_LIBNCURSES || HAVE_LIBXCURSES */
   if (use_builtin)
     {
       /* Built-in VT100 */
@@ -1379,87 +1636,158 @@ void ssh_rl_initialize_termcap(ReadLine *rl)
 void iocb(SshStreamNotification notify, void *context)
 {
   ReadLine *rl = (ReadLine *)context;
-  int lr, rm, i;
+  int lr;
 
   switch (notify)
     {
     case SSH_STREAM_DISCONNECTED:
+      lr = -1;
       goto disconnect;
-
+      break;
+      
     case SSH_STREAM_CAN_OUTPUT:
+      if (ssh_buffer_len(rl->out_buffer))
+        {        
+          int ret;
+          if ((ret = ssh_stream_write(rl->stream,
+                                      ssh_buffer_ptr(rl->out_buffer),
+                                      ssh_buffer_len(rl->out_buffer))))
+            {
+              if (ret == 0)
+                ssh_warning("Warning: write failed in ssh_rl_send_string: "
+                            "EOF received");
+              if (ret > 0)
+                ssh_buffer_consume(rl->out_buffer, ret);
+            }
+        }
+          
       ssh_rl_redraw_display(rl);
       break;
 
     case SSH_STREAM_INPUT_AVAILABLE:
       if ((lr = ssh_rl_loop(rl)) <= 0)
         {
-          rm = ssh_rl_restore_tty_modes(rl);
+        disconnect:
+          ssh_stream_set_callback(rl->stream, NULL, NULL);
+          
+#ifndef SSH_RL_ELOOP_SET_TTY_MODES_ONCE
+          ssh_rl_restore_tty_modes(rl);
+#endif /* SSH_RL_ELOOP_SET_TTY_MODES_ONCE */
 
-          if ((lr < 0) || (rm < 0))
+          ssh_rl_delete_undo(rl);
+
+          if (!rl->extended_mode)
             {
-            disconnect:
-              for(i = 0; i < rl->undo_length; i++)
-                ssh_xfree(rl->undo[i]);
-              rl->undo_length = 0;
-              if (rl->yank != NULL)
-                ssh_xfree(rl->yank);
-              ssh_xfree(rl->undo); rl->undo = NULL;
-              ssh_xfree(rl->undo_cursors); rl->undo_cursors = NULL;
-              ssh_xfree(rl->line);
-              if (rl->callback)
-                (*rl->callback)(rl->fd, NULL);
-              ssh_rl_redraw_display(rl);
-              ssh_xfree(rl);
-              return;
+              if (lr == 0 && rl->callback)
+                {
+                  rl->line[rl->end] = '\0';
+                  (*rl->callback)((const char *)rl->line,
+                                  rl->user_context);
+                }
+              
+              if (lr == -1)
+                {
+                  if (rl->callback)
+                    (*rl->callback)(NULL, rl->user_context);
+                }
+            }
+          else
+            {
+              if (lr == 0 && rl->ext_callback)
+                {
+                  rl->line[rl->end] = '\0';
+                  (*rl->ext_callback)((const char *)rl->line,
+                                      rl->flags,
+                                      rl->user_context);
+                }
+              
+              if (lr == -1)
+                {
+                  if (rl->ext_callback)
+                    (*rl->ext_callback)(NULL, 0, rl->user_context);
+                }
             }
-          for(i = 0; i < rl->undo_length; i++)
-            ssh_xfree(rl->undo[i]);
-          rl->undo_length = 0;
-          if (rl->yank != NULL)
-            ssh_xfree(rl->yank);
-          ssh_xfree(rl->undo); rl->undo = NULL;
-          ssh_xfree(rl->undo_cursors); rl->undo_cursors = NULL;
-          rl->line[rl->end] = '\0';
-          ssh_stream_destroy(rl->stream);
-          if (rl->callback)
-            (*rl->callback)(rl->fd, rl->line);
-          ssh_rl_redraw_display(rl);
-          ssh_xfree(rl->line);
-          ssh_xfree(rl);
         }
-      else
-        ssh_stream_set_callback(rl->stream, iocb, (void *)rl);
       break;
     }
 }
 
+void ssh_readline_eloop(const unsigned char *prompt,
+                        const unsigned char *def,
+                        SshReadLineCtx rl,
+                        SshRLCallback callback,
+                        void *context)
+{
+  ssh_readline_eloop_internal(prompt,
+                              def,
+                              TRUE,
+                              rl,
+                              callback,
+                              NULL,
+                              0,
+                              context);
+}
 
-int ssh_readline_eloop(const unsigned char *prompt,
-                       const unsigned char *def,
-                       int fd,
-                       SshRLCallback callback)
+/* Version with extended callback type. 'flags' should be set to indicate
+   which flags are supported. */
+void ssh_readline_eloop_ext(const unsigned char *prompt,
+                            const unsigned char *def,
+                            SshReadLineCtx rl,
+                            SshRLExtCallback ext_callback,
+                            unsigned int flags,
+                            void *context)
+{
+  ssh_readline_eloop_internal(prompt,
+                              def,
+                              TRUE,
+                              rl,
+                              NULL,
+                              ext_callback,
+                              flags,
+                              context);
+}
+
+void ssh_readpass_eloop(const unsigned char *prompt,
+                        const unsigned char *def,
+                        SshReadLineCtx rl,
+                        SshRLCallback callback,
+                        void *context)
+{
+  ssh_readline_eloop_internal(prompt,
+                              def,
+                              FALSE,
+                              rl,
+                              callback,
+                              NULL,
+                              0,
+                              context);
+}
+
+
+void ssh_readline_eloop_internal(const unsigned char *prompt,
+                                 const unsigned char *def,
+                                 Boolean echo,
+                                 SshReadLineCtx rl,
+                                 SshRLCallback callback,
+                                 SshRLExtCallback ext_callback,
+                                 unsigned int flags,
+                                 void *context)
 {
-  ReadLine *rl;
   const unsigned char *tmp, *nl, *cr;
 
-  rl = ssh_xcalloc(1, sizeof(ReadLine));
+  SSH_PRECOND(rl);
 
-  rl->callback = callback;
-  rl->eloop = TRUE;
-  rl->fd = fd;
-  rl->mark = -1;
-  rl->last_command_cut = 0;
-  rl->yank = NULL;
-  rl->keymap = SSH_READLINE_NORMAL_KEYMAP;
-  rl->max_undo_depth = SSH_READLINE_MAX_UNDO_DEPTH;
-  rl->undo = ssh_xcalloc(rl->max_undo_depth, sizeof(char *));
-  rl->undo_cursors = ssh_xcalloc(rl->max_undo_depth, sizeof(int));
-  rl->undo[0] = ssh_xstrdup("");
-  rl->undo_cursors[0] = 0;
-  rl->undo_length = 1;
-  rl->undo_position = 0;
-  rl->undo_direction = -1;
-  rl->row_length = 80;
+  rl->echo = echo;
+#ifndef SSH_RL_ELOOP_SET_TTY_MODES_ONCE
+  if (ssh_rl_set_tty_modes(rl) < 0)
+    {
+      if (rl->callback)
+        (*rl->callback)(NULL, rl->user_context);
+      SSH_TRACE(2, ("Setting tty modes failed."));
+      return;
+    }
+#endif /* SSH_RL_ELOOP_SET_TTY_MODES_ONCE */
+  
   if (prompt == NULL)
     prompt = (unsigned char *) "";
   rl->prompt = prompt;
@@ -1471,15 +1799,6 @@ int ssh_readline_eloop(const unsigned char *prompt,
     rl->prompt = tmp + 1;
   rl->prompt_len = strlen((char *) rl->prompt);
 
-  ssh_rl_initialize_termcap(rl);
-  if (ssh_rl_set_tty_modes(rl) < 0)
-    {
-      ssh_xfree(rl->undo_cursors);
-      ssh_xfree(rl->undo);
-      ssh_xfree(rl);
-      return -1;
-    }
-
   rl->display_cursor = 0;
   tmp = prompt;
   while (1)
@@ -1518,11 +1837,12 @@ int ssh_readline_eloop(const unsigned char *prompt,
           tmp = cr + 1;
         }
     }
-  ssh_rl_write_string(rl, tmp, rl->prompt_len);
 
+  ssh_rl_write_string(rl, tmp, rl->prompt_len);
+  
   if (def)
     {
-      rl->line_alloc = strlen(def) + 1;
+      rl->line_alloc = strlen((const char *) def) + 1;
       rl->line = ssh_xstrdup(def);
     }
   else
@@ -1531,15 +1851,117 @@ int ssh_readline_eloop(const unsigned char *prompt,
       rl->line = ssh_xmalloc(rl->line_alloc);
       rl->line[0] = '\0';
     }
-
+  
   rl->display_line = ssh_xmalloc(rl->line_alloc);
   rl->display_line[0] = '\0';
   rl->end = rl->cursor = strlen((char *) rl->line);
 
-  rl->stream = ssh_stream_fd_wrap(fd, FALSE);
-  ssh_stream_set_callback(rl->stream, iocb, (void *)rl);
+  ssh_rl_redraw_display(rl);
 
-  return 0;
+  rl->callback = callback;
+  rl->ext_callback = ext_callback;
+  rl->wanted_flags = flags;     /* Note what flags the client accepts */
+  rl->flags = 0;                /* Zero the flags for this call */
+  if (rl->ext_callback)
+    rl->extended_mode = TRUE;
+  rl->user_context = context;
+
+  ssh_stream_set_callback(rl->stream, iocb, (void *)rl);
+}
+
+SshReadLineCtx ssh_readline_eloop_initialize(SshStream stream)
+{
+  ReadLine *rl;
+
+  rl = ssh_xcalloc(1, sizeof(struct ReadLineRec));
+
+  rl->eloop = TRUE;
+  rl->echo = TRUE;
+  /* When calling these, the stream must be a valid fdstream. */
+  rl->read_fd = ssh_stream_fd_get_readfd(stream);
+  rl->write_fd = ssh_stream_fd_get_writefd(stream);
+  rl->mark = -1;
+  rl->last_command_cut = 0;
+  rl->yank = NULL;
+  rl->keymap = SSH_READLINE_NORMAL_KEYMAP;
+  rl->max_undo_depth = SSH_READLINE_MAX_UNDO_DEPTH;
+  rl->undo = ssh_xcalloc(rl->max_undo_depth, sizeof(char *));
+  rl->undo_cursors = ssh_xcalloc(rl->max_undo_depth, sizeof(int));
+  rl->undo[0] = ssh_xstrdup("");
+  rl->undo_cursors[0] = 0;
+  rl->undo_length = 0;
+  rl->undo_position = -1;
+  rl->undo_direction = -1;
+  rl->row_length = 80;
+
+  rl->cmd_history = ssh_dllist_allocate();
+  rl->cmd_history_current = NULL;
+  rl->cmd_history_end = TRUE;
+  rl->cmd_history_line_entered = FALSE;
+  
+  ssh_rl_initialize_termcap(rl);
+#ifdef SSH_RL_ELOOP_SET_TTY_MODES_ONCE
+  if (ssh_rl_set_tty_modes(rl) < 0)
+    {
+      ssh_xfree(rl->undo_cursors);
+      ssh_xfree(rl->undo);
+      ssh_xfree(rl);
+      return NULL;
+    }
+#endif /* SSH_RL_ELOOP_SET_TTY_MODES_ONCE */
+  
+  rl->out_buffer = ssh_buffer_allocate();
+  rl->stream = stream;
+
+  old_sigint_handler = signal(SIGINT, ssh_rl_int_handler);
+  global_sigint_rl = rl;
+
+#if 0
+  ssh_stream_set_callback(rl->stream, iocb, (void *)rl);
+#endif
+  ssh_stream_set_callback(rl->stream, NULL, NULL);
+  
+  return rl;
+}
+
+void *
+ssh_rl_mapcar_xfree(void *item, void *ctx)
+{
+  ssh_xfree(item);
+  return NULL;
+}
+
+void ssh_readline_eloop_unitialize(SshReadLineCtx rl)
+{
+  int i;
+  
+  for(i = 0; i < rl->undo_length; i++)
+    ssh_xfree(rl->undo[i]);
+  rl->undo_length = 0;
+  if (rl->yank != NULL)
+    ssh_xfree(rl->yank);
+  ssh_xfree(rl->undo); rl->undo = NULL;
+  ssh_xfree(rl->undo_cursors); rl->undo_cursors = NULL;
+  if (rl->line)
+    {
+      rl->line[rl->end] = '\0';
+      ssh_rl_redraw_display(rl);
+    }
+#ifdef SSH_RL_ELOOP_SET_TTY_MODES_ONCE
+  rm = ssh_rl_restore_tty_modes(rl);
+#endif /* SSH_RL_ELOOP_SET_TTY_MODES_ONCE */
+  (void)signal(SIGINT, old_sigint_handler);
+
+  if (rl->cmd_history)
+    {
+      ssh_dllist_rewind(rl->cmd_history);
+      ssh_dllist_mapcar(rl->cmd_history, ssh_rl_mapcar_xfree, NULL);
+      ssh_dllist_free(rl->cmd_history);
+    }
+  
+  ssh_buffer_free(rl->out_buffer);
+  ssh_xfree(rl->line);
+  ssh_xfree(rl);
 }
 
 /*
@@ -1555,7 +1977,8 @@ int ssh_readline_eloop(const unsigned char *prompt,
  */
 int ssh_readline(const unsigned char *prompt,
                  unsigned char **line,
-                 int fd)
+                 int read_fd,
+                 int write_fd)
 {
   ReadLine *rl;
   int i;
@@ -1568,7 +1991,9 @@ int ssh_readline(const unsigned char *prompt,
   rl = ssh_xcalloc(1, sizeof(ReadLine));
 
   rl->eloop = FALSE;
-  rl->fd = fd;
+  rl->echo = TRUE;
+  rl->read_fd = read_fd;
+  rl->write_fd = write_fd;
   rl->mark = -1;
   rl->last_command_cut = 0;
   rl->yank = NULL;
@@ -1578,7 +2003,7 @@ int ssh_readline(const unsigned char *prompt,
   rl->undo_cursors = ssh_xcalloc(rl->max_undo_depth, sizeof(int));
   rl->undo[0] = ssh_xstrdup("");
   rl->undo_cursors[0] = 0;
-  rl->undo_length = 1;
+  rl->undo_length = 0;
   rl->undo_position = 0;
   rl->undo_direction = -1;
   rl->row_length = 80;
@@ -1605,6 +2030,8 @@ int ssh_readline(const unsigned char *prompt,
       return -1;
     }
 
+  rl->cmd_history = NULL;
+
   rl->display_cursor = 0;
   tmp = prompt;
   while (1)
