@@ -15,6 +15,7 @@ This is a simple implementation for generic unix platforms.
 
 #include "sshsessionincludes.h"
 #include "sshuser.h"
+#include "sshdsprintf.h"
 
 #ifdef HAVE_SCO_ETC_SHADOW
 # include <sys/security.h>
@@ -25,6 +26,7 @@ This is a simple implementation for generic unix platforms.
 # endif
 #else /* HAVE_SCO_ETC_SHADOW */
 # ifdef HAVE_HPUX_TCB_AUTH
+#  undef ctime
 #  include <sys/types.h>
 #  include <hpsecurity.h>
 #  include <prot.h>
@@ -48,12 +50,11 @@ This is a simple implementation for generic unix platforms.
 #include <netdb.h>
 
 #ifdef HAVE_USERSEC_H
+#define SSH_ALLOW_CPLUSPLUS_KEYWORDS
 #include <usersec.h>
 #endif /* HAVE_USERSEC_H */
 
-#ifdef HAVE_OSF1_C2_SECURITY
-#include "tcbc2.h"
-#endif /* HAVE_OSF1_C2_SECURITY */
+#define SSH_DEBUG_MODULE "SshUnixUser"
 
 extern char *crypt(const char *key, const char *salt);
 
@@ -62,6 +63,7 @@ extern char *crypt(const char *key, const char *salt);
 struct SshUserRec
 {
   char *name;
+  char *group;
   char *dir;
   char *shell;
   char *correct_encrypted_passwd;
@@ -85,15 +87,16 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   {
     char *expiration, current_time[100], normalized[100];
     int rlogin_permitted;
-    time_t t;
-    struct tm *tm;
+    SshTime t;
+    struct SshCalendarTimeRec tm[1];
     int account_is_locked;
     
     if (setuserdb(S_READ) < 0)
       {
         if (getuid() == 0) /* It's OK to fail here if we are not root */
           {
-            ssh_debug("setuserdb S_READ failed: %.200s.", strerror(errno));
+            SSH_DEBUG(2, ("setuserdb S_READ failed: %.200s.", 
+                          strerror(errno)));
           }
         return FALSE;
       }
@@ -102,37 +105,39 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
       {
         if (getuid() == 0) /* It's OK to fail here if we are not root */
           {
-            ssh_debug("getuserattr S_RLOGINCHK failed: %.200s",
-                      strerror(errno));
+            SSH_DEBUG(2, ("getuserattr S_RLOGINCHK failed: %.200s",
+                          strerror(errno)));
           }
         enduserdb();
         return FALSE;
       }
     if (getuserattr((char *)user, S_EXPIRATION, &expiration, SEC_CHAR) < 0)
       {
-        ssh_debug("getuserattr S_EXPIRATION failed: %.200s.", strerror(errno));
+        SSH_DEBUG(2, ("getuserattr S_EXPIRATION failed: %.200s.", 
+                      strerror(errno)));
         enduserdb();
         return FALSE;
       }
 #ifdef S_LOCKED
     if (getuserattr(user, S_LOCKED, &account_is_locked, SEC_BOOL) < 0)
       {
-        ssh_debug("getuserattr S_LOCKED failed: %.200s.", strerror(errno));
+        SSH_DEBUG(2, ("getuserattr S_LOCKED failed: %.200s.", 
+                      strerror(errno)));
         enduserdb();
         return FALSE;
       }
     if (account_is_locked)
       {
-        ssh_debug("Account %.100s is locked.", user);
+        SSH_DEBUG(2, ("Account %.100s is locked.", user));
         enduserdb();
         return FALSE;
       }
 #endif /* S_LOCKED */
     if (!rlogin_permitted)
       {
-        ssh_debug("Remote logins to account %.100s not permitted by "
-                  "user profile.",
-                  user);
+        SSH_DEBUG(2, ("Remote logins to account %.100s not permitted by "
+                      "user profile.",
+                      user));
         enduserdb();
         return FALSE;
       }
@@ -144,15 +149,16 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
       }
     if (strlen(expiration) != 10)
       {
-        ssh_debug("Account %.100s expiration date is in wrong format.", user);
+        SSH_DEBUG(2, ("Account %.100s expiration date is in wrong format.", 
+                      user));
         enduserdb();
         return FALSE;
       }
-    t = time(NULL);
-    tm = localtime(&t);
+    t = ssh_time();
+    ssh_calendar_time(t, tm, TRUE);
     snprintf(current_time, sizeof(current_time), "%04d%02d%02d%02d%02d",
-             tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-             tm->tm_hour, tm->tm_min);
+             tm->year, tm->month + 1, tm->monthday,
+             tm->hour, tm->minute);
     if (expiration[8] < '7') /* Assume year < 70 is 20YY. */
       strcpy(normalized, "20");
     else
@@ -162,7 +168,7 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
     normalized[12] = '\0';
     if (strcmp(normalized, current_time) < 0)
       {
-        ssh_debug("Account %.100s has expired - access denied.", user);
+        SSH_DEBUG(2, ("Account %.100s has expired - access denied.", user));
         enduserdb();
         return FALSE;
       }
@@ -200,20 +206,22 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
         if ((stat(SHADOW, &sbf) == 0) &&
             strcmp(uc->correct_encrypted_passwd, "x") == 0)
           {
-            ssh_debug("Can't find %.100s's shadow - access denied.", user);
+            SSH_DEBUG(2, ("Can't find %.100s's shadow - access denied.", 
+                          user));
             endspent();
             return FALSE;
           }
       }
     else
       {
-        time_t today = time((time_t *)NULL)/24/60/60; /* what a day! */
+        SshTime today = ssh_time()/24/60/60; /* what a day! */
 
 #ifdef HAVE_STRUCT_SPWD_EXPIRE
         /* Check for expiration date */
         if (sp->sp_expire > 0 && today > sp->sp_expire)
           {
-            ssh_debug("Account %.100s has expired - access denied.", user);
+            SSH_DEBUG(2, ("Account %.100s has expired - access denied.", 
+                          user));
             endspent();
             return FALSE;
           }
@@ -224,13 +232,13 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
         if (sp->sp_inact > 0)
           {
             char buf[64];
-            unsigned long llt;
+            SshTime llt;
             
             llt = ssh_user_get_last_login_time(uc, buf, sizeof(buf));
             if (llt && (today - llt/24/60/60) > sp->sp_inact)
               {
-                ssh_debug("Account %.100s was inactive for more than %d days.",
-                          user, sp->sp_inact);
+                SSH_DEBUG(2, ("Account %.100s was inactive for more than %d days.",
+                              user, sp->sp_inact));
                 endspent();
                 return FALSE;
               }
@@ -241,8 +249,8 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
         if (sp->sp_lstchg == 0 ||
             (sp->sp_max > 0 && today > sp->sp_lstchg + sp->sp_max))
           {
-            ssh_debug("Account %.100s's password is too old - forced to change.",
-                      user);
+            SSH_DEBUG(2, ("Account %.100s's password is too old - forced to change.",
+                          user));
             uc->password_needs_change = TRUE;
           }
         strncpy(passwd, sp->sp_pwdp, sizeof(passwd));
@@ -258,7 +266,7 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   {
     if (strncmp(passwd,"*LK*", 4) == 0)
       {
-        ssh_debug("Account %.100s is locked.", user);
+        SSH_DEBUG(2, ("Account %.100s is locked.", user));
         return FALSE;
       }
   }
@@ -271,12 +279,12 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
       shell = DEFAULT_SHELL;
     
     while (invalid && (etc_shell = getusershell()))
-      invalid = strcmp(etc_shell,shell);
+      invalid = strcmp(etc_shell, shell);
     endusershell();
     
     if (invalid)
       {
-        ssh_debug("Account %.100s doesn't have valid shell", user);
+        SSH_DEBUG(2, ("Account %.100s doesn't have valid shell", user));
         return FALSE;
       }
   }
@@ -285,6 +293,9 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   return TRUE;
 }
 
+/* Forward declaration. */
+SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged);
+
 /* Allocates and initializes a context for the user.  The context is used
    to cache information about the particular user.  Returns NULL if the
    user does not exist. If `user' is NULL, use getuid(). 'privileged'
@@ -294,25 +305,40 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
 
 SshUser ssh_user_initialize(const char *user, Boolean privileged)
 {
-
-  SshUser uc;
-
   struct passwd *pw;
-  char correct_passwd[200];
-
-  /* Get the encrypted password for the user. */
 
   if (user == NULL)
     pw = getpwuid(getuid());
   else 
     pw = getpwnam(user);
 
+  return ssh_user_initialize_with_pw(pw, privileged);
+}
+
+/* As above, but we explicitely want to use uid (instead of name). */
+SshUser ssh_user_initialize_with_uid(uid_t uid, Boolean privileged)
+{
+  struct passwd *pw;
+
+  pw = getpwuid(uid);
+
+  return ssh_user_initialize_with_pw(pw, privileged);
+}
+
+/* As above, but uses struct passwd. This function does all the work. */
+SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
+{
+  SshUser uc;
+
+  char correct_passwd[200];
+
   if (!pw)
     return NULL;
-
+  
   uc = ssh_xcalloc(1, sizeof(*uc));
 
   uc->name = ssh_xstrdup(pw->pw_name);
+  uc->group = NULL;
   uc->dir = ssh_xstrdup(pw->pw_dir);
   uc->uid = pw->pw_uid;
   uc->gid = pw->pw_gid;
@@ -332,10 +358,6 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
       /* Save the encrypted password. */
       strncpy(correct_passwd, pw->pw_passwd, sizeof(correct_passwd));
 
-#ifdef HAVE_OSF1_C2_SECURITY
-      tcbc2_getprpwent(correct_passwd, ssh_user_name(uc),
-                       sizeof(correct_passwd));
-#else /* HAVE_OSF1_C2_SECURITY */
       /* If we have shadow passwords, lookup the real encrypted password from
          the shadow file, and replace the saved encrypted password with the
          real encrypted password. */
@@ -381,8 +403,8 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
         f = fopen("/etc/security/passwd", "r");
         if (f)
           {
-            /* XXX: user next line was server_user, is this OK? */
-            snprintf(looking_for_user, sizeof(looking_for_user), "%.190s:", user);
+            snprintf(looking_for_user, sizeof(looking_for_user), 
+                     "%.190s:", ssh_user_name(uc));
             while (fgets(line, sizeof(line), f))
               {
                 if (strchr(line, '\n'))
@@ -397,7 +419,9 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
                       {
                         for (cp = line; *cp == ' ' || *cp == '\t'; cp++)
                           ;
-                        if (strncmp(cp, "password = ", strlen("password = ")) == 0)
+                        if (strncmp(cp, 
+                                    "password = ", 
+                                    strlen("password = ")) == 0)
                           {
                             strncpy(correct_passwd, cp + strlen("password = "), 
                                     sizeof(correct_passwd));
@@ -413,7 +437,6 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
 #endif /* HAVE_ETC_SECURITY_PASSWD_ADJUNCT */
 #endif /* HAVE_ETC_SHADOW */
 #endif /* HAVE_SCO_ETC_SHADOW */
-#endif /* HAVE_OSF1_C2_SECURITY */
 
       uc->correct_encrypted_passwd = ssh_xstrdup(correct_passwd);
 
@@ -442,6 +465,7 @@ Boolean ssh_user_free(SshUser uc, Boolean undo)
   /* XXX undoing kerberos / secure rpc state. */
 
   ssh_xfree(uc->name);
+  ssh_xfree(uc->group);
   ssh_xfree(uc->dir);
   ssh_xfree(uc->shell);
   if (uc->correct_encrypted_passwd)
@@ -479,7 +503,8 @@ Boolean ssh_user_validate_local_password(SshUser uc,
     svp = getsvc();
     if (svp == NULL)
       {
-        error("getsvc() failed in ultrix code in auth_passwd");
+        SSH_DEBUG(2, ("getsvc() failed in ultrix code in auth_passwd",
+                      strerror(errno)));
         return FALSE;
       }
     if ((svp->svcauth.seclevel == SEC_UPGRADE &&
@@ -489,12 +514,8 @@ Boolean ssh_user_validate_local_password(SshUser uc,
   }
 #endif /* HAVE_ULTRIX_SHADOW_PASSWORDS */
 
+
   /* Encrypt the candidate password using the proper salt. */
-#ifdef HAVE_OSF1_C2_SECURITY
-  encrypted_password = (char *)tcbc2_crypt(password,
-                                           (correct_passwd[0] && correct_passwd[1]) ?
-                                           correct_passwd : "xx");
-#else /* HAVE_OSF1_C2_SECURITY */
 #if defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH)
   encrypted_password = bigcrypt(password, 
                                 (correct_passwd[0] && correct_passwd[1]) ?
@@ -504,7 +525,6 @@ Boolean ssh_user_validate_local_password(SshUser uc,
                              (correct_passwd[0] && correct_passwd[1]) ?
                              correct_passwd : "xx");
 #endif /* HAVE_SCO_ETC_SHADOW */
-#endif /* HAVE_OSF1_C2_SECURITY */
 
   /* Authentication is accepted if the encrypted passwords are identical. */
   return strcmp(encrypted_password, correct_passwd) == 0;
@@ -528,7 +548,7 @@ Boolean ssh_user_change_password(SshUser uc,
                                  const char *old_password,
                                  const char *new_password)
 {
-  ssh_debug("ssh_user_change_password: XXX changing not yet implemented");
+  SSH_DEBUG(2, ("ssh_user_change_password: XXX changing not yet implemented"));
   return FALSE;
 }
 
@@ -539,7 +559,7 @@ Boolean ssh_user_change_password(SshUser uc,
 Boolean ssh_user_validate_kerberos_password(SshUser uc,
                                             const char *password)
 {
-  ssh_debug("ssh_user_validate_kerberos_password: not yet implemented");
+  SSH_DEBUG(2, ("ssh_user_validate_kerberos_password: not yet implemented"));
   return FALSE;
 }
 
@@ -550,7 +570,7 @@ Boolean ssh_user_validate_kerberos_password(SshUser uc,
 Boolean ssh_user_validate_secure_rpc_password(SshUser uc,
                                               const char *password)
 {
-  ssh_debug("ssh_user_validate_secure_rpc_password: not yet implemented");
+  SSH_DEBUG(2, ("ssh_user_validate_secure_rpc_password: not yet implemented"));
   return FALSE;
 }
 
@@ -595,7 +615,7 @@ int ssh_cray_setup(uid, username)
   err = setudb();    /* open and rewind the Cray User DataBase */
   if(err != 0)
     {
-      ssh_debug("ssh_cray_setup: UDB open failure");
+      SSH_DEBUG(2, ("ssh_cray_setup: UDB open failure"));
       return(-1);
     }
   naccts = 0;
@@ -614,7 +634,7 @@ int ssh_cray_setup(uid, username)
   endudb();        /* close the udb */
   if (naccts == 0 || accts[0] == 0)
     {
-      ssh_debug("ssh_cray_setup: No Cray accounts found");
+      SSH_DEBUG(2, ("ssh_cray_setup: No Cray accounts found"));
       return(-1);
     }
  
@@ -623,8 +643,8 @@ int ssh_cray_setup(uid, username)
      the account to the first entry. */
   if (acctid(0, accts[0]) < 0) 
     {
-      ssh_debug("ssh_cray_setup: System call acctid failed, accts[0]=%d",
-                accts[0]);
+      SSH_DEBUG(2, ("ssh_cray_setup: System call acctid failed, accts[0]=%d",
+                accts[0]));
       return(-1);
     } 
  
@@ -634,7 +654,7 @@ int ssh_cray_setup(uid, username)
   jid = setjob(uid, 0);
   if (jid < 0) 
     {
-      ssh_debug("ssh_cray_setup: System call setjob failure");
+      SSH_DEBUG(2, ("ssh_cray_setup: System call setjob failure"));
       return(-1);
     }
 
@@ -646,13 +666,13 @@ int ssh_cray_setup(uid, username)
   sr = setlimits(username, C_PROC, pid, UDBRC_INTER);
   if (sr != NULL) 
     {
-      ssh_debug("%.100s", sr);
+      SSH_DEBUG(2, ("%.100s", sr));
       return(-1);
     }
   sr = setlimits(username, C_JOB, jid, UDBRC_INTER);
   if (sr != NULL) 
     {
-      ssh_debug("%.100s", sr);
+      SSH_DEBUG(2, ("%.100s", sr));
       return(-1);
     }
 
@@ -676,8 +696,8 @@ Boolean ssh_user_become(SshUser uc)
 #ifdef HAVE_SETLOGIN
   /* Set login name in the kernel.  Warning: setsid() must be called before
      this. */
-  if (setlogin(ssh_user_name(uc)) < 0)
-    ssh_debug("setlogin failed: %.100s", strerror(errno));
+  if (setlogin((char *)ssh_user_name(uc)) < 0)
+    SSH_DEBUG(2, ("setlogin failed: %.100s", strerror(errno)));
 #endif /* HAVE_SETLOGIN */
 
 #ifdef HAVE_USERSEC_H
@@ -685,27 +705,31 @@ Boolean ssh_user_become(SshUser uc)
      includes, but it seems to be important.  This also does setuid
      (but we do it below as well just in case). */
   if (setpcred(ssh_user_name(uc), NULL))
-    ssh_debug("setpcred %.100s: %.100s", strerror(errno));
+    SSH_DEBUG(2, ("setpcred %.100s: %.100s", strerror(errno)));
 #endif /* HAVE_USERSEC_H */
 
   /* Close any extra file descriptors.  Note that there may still be
      descriptors left by system functions.  They will be closed later. */
   endpwent();
-  endhostent();
 
+#if 0
+  endhostent();
+#endif
+  
   /* Close any extra open file descriptors so that we don\'t have them
      hanging around in clients.  Note that we want to do this after
      initgroups, because at least on Solaris 2.3 it leaves file descriptors
      open. */
   endgrent();
+
   for (i = 3; i < 1000; i++)
     close(i);
 
 #ifdef CRAY   /* set up accounting account number, job, limits, permissions  */
   if (cray_setup(ssh_user_uid(uc), ssh_user_name(uc)) < 0)
     {
-      ssh_debug("ssh_user_become: Failure in Cray job setup for user %d.",
-                (int)ssh_user_uid(uc));
+      SSH_DEBUG(2, ("ssh_user_become: Failure in Cray job setup for user %d.",
+                    (int)ssh_user_uid(uc)));
       return FALSE;
     }
 #endif
@@ -715,49 +739,48 @@ Boolean ssh_user_become(SshUser uc)
     { 
       if (setgid(ssh_user_gid(uc)) < 0)
         {
-          ssh_debug("ssh_user_become: setgid: %s", strerror(errno));
+          SSH_DEBUG(2, ("ssh_user_become: setgid: %s", strerror(errno)));
           return FALSE;
         }
 #ifdef HAVE_INITGROUPS
       /* Initialize the group list. */
-      if (initgroups(ssh_user_name(uc), ssh_user_gid(uc)) < 0)
+      if (initgroups((char *)ssh_user_name(uc), ssh_user_gid(uc)) < 0)
         {
-          ssh_debug("ssh_user_become: initgroups: %s", strerror(errno));
+          SSH_DEBUG(2, ("ssh_user_become: initgroups: %s", strerror(errno)));
           return FALSE;
         }
 #endif /* HAVE_INITGROUPS */
       endgrent();
           
 #ifdef HAVE_SETLUID
-      /* Set login uid, if we have setluid(). */
+  /* Set login uid, if we have setluid(). */
       if (setluid(ssh_user_uid(uc)) < 0)
         {
-          ssh_debug("ssh_user_become; setluid %d: %s",
-                    (int)ssh_user_uid(uc), strerror(errno));
+          SSH_DEBUG(2, ("ssh_user_become); setluid %d: %s",
+                        (int)ssh_user_uid(uc), strerror(errno));
           return FALSE;
         }
 #endif /* HAVE_SETLUID */
-          
       /* Permanently switch to the desired uid. */
       if (setuid(ssh_user_uid(uc)) < 0)
         {
-          ssh_debug("ssh_user_become: setuid %d: %s",
-                    (int)ssh_user_uid(uc), strerror(errno));
+          SSH_DEBUG(2, ("ssh_user_become: setuid %d: %s",
+                        (int)ssh_user_uid(uc), strerror(errno)));
+          return FALSE;
+        }
+      if (getuid() != ssh_user_uid(uc) || geteuid() != ssh_user_uid(uc))
+        {
+          SSH_DEBUG(2, ("ssh_user_become: failed to set uids to %d.",
+                        (int)ssh_user_uid(uc)));
           return FALSE;
         }
     }
   
-  if (getuid() != ssh_user_uid(uc) || geteuid() != ssh_user_uid(uc))
-    {
-      ssh_debug("ssh_user_become: failed to set uids to %d.",
-                (int)ssh_user_uid(uc));
-      return FALSE;
-    }
-
   /* We are now running with the user's privileges. */
   return TRUE;
 }
 
+
 /* Returns the login name of the user. */
 
 const char *ssh_user_name(SshUser uc)
@@ -765,6 +788,21 @@ const char *ssh_user_name(SshUser uc)
   return uc->name;
 }
 
+/* Returns the group name of the user. */
+const char *ssh_group_name(SshUser uc)
+{
+  if (uc->group == NULL)
+    {
+      struct group *grp = getgrgid(ssh_user_gid(uc));
+      
+      if (grp != NULL)
+        uc->group = ssh_xstrdup(grp->gr_name);
+      else
+        ssh_dsprintf(&uc->group, "%lu", (unsigned long)(ssh_user_gid(uc)));
+    }
+  return uc->group;
+}
+
 /* Returns the uid of the user.  This is unix-specific. */
 
 uid_t ssh_user_uid(SshUser uc)
