@@ -15,11 +15,11 @@ Test program for stream pairs.
 #include "sshbuffer.h"
 #include "sshstreampair.h"
 #include "sshtimeouts.h"
-#include "sshunixeloop.h"
+#include "ssheloop.h"
 
 SshStream ts1, ts2;
-SshBuffer *testdata = NULL;
-SshBuffer *received_data;
+SshBuffer testdata = NULL;
+SshBuffer received_data;
 int break_test;
 int test_data_index;
 int destroy_count;
@@ -40,7 +40,7 @@ void create_testdata(void)
   len = random() % 100000;
   for (i = 0; i < len; i++)
     {
-      ch = random();
+      ch = (unsigned char)(random() % 256);
       ssh_buffer_append(testdata, &ch, 1);
     }
 }
@@ -176,7 +176,7 @@ int main()
   int pass;
   SshStream s1, s2;
 
-  srandom(ssh_time());
+  srandom((SshUInt32)ssh_time());
   
   ssh_event_loop_initialize();
   
