@@ -8,19 +8,19 @@
 /*
  *        Program: Urlparse
  *        $Source: /ssh/CVS/src/lib/sshutil/tests/t-url.c,v $
- *        $Author: mtr $
+ *        $Author: kivinen $
  *
  *        Creation          : 10:45 Jul 10 1998 kivinen
- *        Last Modification : 17:55 Jan 28 1999 kivinen
- *        Last check in     : $Date: 1999/01/29 13:13:38 $
- *        Revision number   : $Revision: 1.3 $
+ *        Last Modification : 02:36 Apr 14 2000 kivinen
+ *        Last check in     : $Date: 2000/04/14 01:02:09 $
+ *        Revision number   : $Revision: 1.6 $
  *        State             : $State: Exp $
- *        Version           : 1.175
+ *        Version           : 1.180
  *
  *        Description       : Test program for library to parse urls
  */
 /*
- * $Id: t-url.c,v 1.3 1999/01/29 13:13:38 mtr Exp $
+ * $Id: t-url.c,v 1.6 2000/04/14 01:02:09 kivinen Exp $
  * $EndLog$
  */
 
@@ -43,6 +43,10 @@ typedef struct TestUrlRec {
 struct TestUrlRec tests[] = {
   { "http://www.ssh.fi/testing/host",
     "http", "www.ssh.fi", NULL, NULL, NULL, "testing/host", TRUE },
+  { "http:///testing/host",
+    "http", "", NULL, NULL, NULL, "testing/host", TRUE },
+  { "scheme1:///scheme2:///scheme3://data",
+    "scheme1", "", NULL, NULL, NULL, "scheme2:///scheme3://data", TRUE},
   { "ftp://kivinen:foobar@ftp.ssh.fi:21/hidden",
     "ftp", "ftp.ssh.fi", "kivinen", "foobar", "21", "hidden", TRUE },
   { "scheme://username:password@host:2222/path",
@@ -108,7 +112,7 @@ struct TestFormRec form_tests[] = {
     "/aksjdklasjdlkasjdlkasjdkla", TRUE,
     { { "kukkuu", "reset" } } },
   { "!@#$%25^&*()_+][|\":%3f><,./'\\{}`1234567890-=?a=b&c=d&e=f",
-    "!@#$%^&*()_+][|\":?><,./'\\{}`1234567890-=", TRUE,
+    "!@#$%^&*()_ ][|\":?><,./'\\{}`1234567890-=", TRUE,
     { { "a", "b" }, { "c", "d" }, { "e", "f" } } },
   { "%20%21%22?kukkuu=reset&zappa=bar", " !\"", TRUE,
     { { "kukkuu", "reset" }, { "zappa", "bar" } } },
@@ -153,7 +157,9 @@ struct TestFormRec form_tests[] = {
   { "/foo?a=b&c=d%3", "/foo", FALSE,
     { { "a", "b" }, { "c", "d%3" } } },
   { "/foo?na%6de=Tero%20&nam%65=T%20&n%61me=Kivinen&bar=zappa", "/foo", TRUE,
-    { { "name", "Tero \nT \nKivinen" }, { "bar", "zappa" } } }
+    { { "name", "Tero \nT \nKivinen" }, { "bar", "zappa" } } },
+  { "/foo?first+name=Tero+Tapani&Last%2bName=Kivinen%2b%2b", "/foo", TRUE,
+    { { "first name", "Tero Tapani" }, { "Last+Name", "Kivinen++" } } }
 };
 
 void mapping_print(SshMapping mapping)
@@ -266,7 +272,7 @@ int main(int argc, char **argv)
                         key, value, form_tests[i].url);
             }
         }
-      ssh_mapping_free(mapping);
+      ssh_url_free_mapping(mapping);
     }
   return 0;
 }
